# - Find CLUB installation
# This module tries to find the CLUB installation on your system.
# It tries to find the club-config script which gives you all the needed information.
# If the system variable ROOTSYS is set this is straight forward.
# If not the module uses the pathes given in ROOT_CONFIG_SEARCHPATH.
# If you need an other path you should add this path to this varaible.  
# The club-config script is then used to detect basically everything else.
# This module defines a number of key variables and macros.


MESSAGE(STATUS "Looking for Club...")

SET(CLUB_DEFINITIONS "")

SET(CLUB_INSTALLED_VERSION_TOO_OLD FALSE)

SET(CLUB_CONFIG_EXECUTABLE CLUB_CONFIG_EXECUTABLE-NOTFOUND)

FIND_PROGRAM(CLUB_CONFIG_EXECUTABLE NAMES club-config PATHS PATH)
    
IF (${CLUB_CONFIG_EXECUTABLE} MATCHES "CLUB_CONFIG_EXECUTABLE-NOTFOUND")
  MESSAGE( STATUS "CLUB not installed in the searchpath and CLUBSYS is not set. Please
 set CLUBSYS or add the path to your CLUB installation in the Macro FindCLUB.cmake in the
 subdirectory cmake/modules.")
ELSE (${CLUB_CONFIG_EXECUTABLE} MATCHES "CLUB_CONFIG_EXECUTABLE-NOTFOUND")
  STRING(REGEX REPLACE "(^.*)/bin/club-config" "\\1" test ${CLUB_CONFIG_EXECUTABLE}) 
  SET( ENV{CLUBSYS} ${test})
  set( CLUBSYS ${test})
ENDIF (${CLUB_CONFIG_EXECUTABLE} MATCHES "CLUB_CONFIG_EXECUTABLE-NOTFOUND")

IF (CLUB_CONFIG_EXECUTABLE)

  SET(CLUB_FOUND TRUE)

  EXEC_PROGRAM(${CLUB_CONFIG_EXECUTABLE} ARGS "--version" OUTPUT_VARIABLE CLUBVERSION)

  MESSAGE(STATUS "Looking for Club... - version ${CLUBVERSION} ")
ENDIF (CLUB_CONFIG_EXECUTABLE)


IF (CLUB_FOUND)
        # ask club-config for the library dir
        # Set CLUB_LIBRARY_DIR
        EXEC_PROGRAM( ${CLUB_CONFIG_EXECUTABLE}
            ARGS "--ldflags"
            OUTPUT_VARIABLE CLUB_LIBRARY_DIR_TMP )
        SET(CLUB_LIBRARY_DIR_TMP ${CLUBSYS}/lib)
        
        IF(EXISTS "${CLUB_LIBRARY_DIR_TMP}")
            SET(CLUB_LIBRARY_DIR ${CLUB_LIBRARY_DIR_TMP} )
        ELSE(EXISTS "${CLUB_LIBRARY_DIR_TMP}")
            MESSAGE("Warning: CLUB_CONFIG_EXECUTABLE reported ${CLUB_LIBRARY_DIR_TMP} as library path,")
            MESSAGE("Warning: but ${CLUB_LIBRARY_DIR_TMP} does NOT exist, CLUB must NOT be installed correctly.")
        ENDIF(EXISTS "${CLUB_LIBRARY_DIR_TMP}")
            
        # ask club-config for the include dir
        EXEC_PROGRAM( ${CLUB_CONFIG_EXECUTABLE}
            ARGS "--cppflags" 
            OUTPUT_VARIABLE root_headers )
        SET(CLUB_INCLUDE_DIR ${root_headers})
    STRING(REPLACE "-I" "" CLUB_INCLUDE_DIR ${CLUB_INCLUDE_DIR})


    # ask club-config for the library variables
        EXEC_PROGRAM( ${CLUB_CONFIG_EXECUTABLE}
            ARGS "--libs" 
            OUTPUT_VARIABLE root_flags )
        
        #  STRING(REGEX MATCHALL "([^ ])+"  root_libs_all ${root_flags})
        #  STRING(REGEX MATCHALL "-L([^ ])+"  root_library ${root_flags})
        #  REMOVE_FROM_LIST(root_flags "${root_libs_all}" "${root_library}")
        SET(root_flags "${root_flags}-${CLUBVERSION}")
        SET(CLUB_LIBRARIES ${root_flags})
    
    # Make variables changable to the advanced user
        MARK_AS_ADVANCED( CLUB_LIBRARY_DIR CLUB_INCLUDE_DIR CLUB_DEFINITIONS)
        
        # Set CLUB_INCLUDES
        SET( CLUB_INCLUDES ${CLUB_INCLUDE_DIR})
        SET(INCLUDE_DIRECTORIES ${INCLUDE_DIRECTORIES} ${CLUB_INCLUDE_DIR})
        
        
        SET(LD_LIBRARY_PATH ${LD_LIBRARY_PATH} ${CLUB_LIBRARY_DIR})
ENDIF (CLUB_FOUND)
