//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#include "TCut.h"
#include "TClass.h"
#include "TFile.h"
#include "TMath.h"
#include "TRandom.h"
#include <iostream>
#include <ctime>

using std::cout;
using std::endl;

// Uranie
#include "TPatternsEventList.h"
#include "UExceptions.h"

ClassImp (URANIE::DataServer::TPatternsEventList)

namespace URANIE
{
namespace DataServer
{

//______________________________________________________________________________
TPatternsEventList::TPatternsEventList() :
        TEventList("_pel_", "Default TPatternsEventList"), _nSeed(0), _blog(
                kFALSE)
{
}

TPatternsEventList::TPatternsEventList(const char *name, const char *title,
        Int_t initsize) :
        TEventList(name, title, initsize), _nSeed(0), _blog(kFALSE)
{
//     for(Int_t i=0; i< initsize; i++)
//       Enter(i);
}

//______________________________________________________________________________
TPatternsEventList::~TPatternsEventList()
{
    if (_blog)
        cout << " ** Destructor TPatternsEventList Name[" << GetName()
                << "] Title[" << GetTitle() << "] fN[" << fN << "]" << endl;
}

//______________________________________________________________________________
void TPatternsEventList::Enter(Long64_t entry)
{
    //          Enter element entry into the list
    if (!fList)
    {
        fList = new Long64_t[fSize];
        fList[0] = entry;
        fN = 1;
        return;
    }
    //if (entry==fList[fN-1]) return;
    if (fN >= fSize)
    {
        Int_t newsize = TMath::Max(2 * fSize, fN + fDelta);
        Resize(newsize - fSize);
    }

    fList[fN] = entry;
    ++fN;

    //   if(entry>fList[fN-1]) {
    //     fList[fN] = entry;
    //     ++fN;
    //   } else {
    //     Int_t pos = TMath::BinarySearch(fN, fList, entry);
    //     //if(pos>=0 && entry==fList[pos])
    //     //  return;
    //     ++pos;
    //     memmove( &(fList[pos+1]), &(fList[pos]), 4*(fN-pos));
    //     fList[pos] = entry;
    //     ++fN;
    //   }
}
//       {
//     //          Enter element entry into the list

//     if (!fList) {
//       fList = new Long64_t[fSize];
//       fList[0] = entry;
//       fN = 1;
//       return;
//     }
//     //if (entry==fList[fN-1]) return;
//     if (fN >= fSize) {
//       Int_t newsize = TMath::Max(2*fSize,fN+fDelta);
//       Resize(newsize-fSize);
//     }
//     if(entry>fList[fN-1]) {
//       fList[fN] = entry;
//       ++fN;
//     } else {
//       Int_t pos = Int_t(TMath::BinarySearch(fN, fList, entry));
//       //if(pos>=0 && entry==fList[pos])
//       //  return;
//       ++pos;
//       memmove( &(fList[pos+1]), &(fList[pos]), 4*(fN-pos));
//       fList[pos] = entry;
//       ++fN;
//     }
//       }

//______________________________________________________________________________
void TPatternsEventList::Print(Option_t *option) const
{
    //          Print contents of this list

    printf("TPatternsEventList:%s/%s, number of entries =%d, size=%d\n",
            GetName(), GetTitle(), fN, fSize);
    if (!strstr(option, "all"))
        return;
    Int_t i;
    Int_t nbuf = 0;
    char element[10];
    char *line = new char[100];
    sprintf(line, "%5d : ", 0);
    for (i = 0; i < fN; i++)
    {
        nbuf++;
        if (nbuf > 10)
        {
            printf("%s\n", line);
            sprintf(line, "%5d : ", i);
            nbuf = 1;
        }
        sprintf(element, "%7lld ", fList[i]);
        strcat(line, element);
    }
    if (nbuf)
        printf("%s\n", line);
    delete[] line;
    line = NULL;
}

void TPatternsEventList::setSeed(Int_t ind)
{
    if (ind == 0)
    {
        time_t secondes;
        time(&secondes);
        _nSeed = (Int_t)(secondes);
    }
    else
    {
        _nSeed = ind;
    }
    gRandom->SetSeed(_nSeed);
}

void TPatternsEventList::Mix()
{
    try
    {
        if (fN == 0)
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    "TPatternsEventList::Mix No pattern were entered");
        Int_t indtmp, valtmp;
        // Create a permutation with the Devroye Algorithm
        for (Int_t i = fN - 1; i > 0; i--)
        {
            indtmp = (Int_t) gRandom->Integer(i);
            valtmp = Int_t(fList[indtmp]);
            //       cout<< "** i["<<i << "] val [" << fList[i] << "] indtmp["<<indtmp<<"] val["<<valtmp << "]";
            fList[indtmp] = fList[i];
            fList[i] = valtmp;
            //       cout<< "************* i["<<i << "] val [" << fList[i] << "] indtmp["<<indtmp<<"] val["<<fList[indtmp] << "]"<<endl;;
        }
    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
        throw ue;
        exit(-1);
    }
}

} // Fin du namespace DataServer
} // Fin du namespace URANIE

