//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TStochAttribut.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $date$

 \brief  Implementation of the class URANIE::DataServer::TPossibilityAttribute

 Created: 2010-07-07 15:52:43

 */
#include <iostream>

#include "TPossibilityAttribute.h"

ClassImp (URANIE::DataServer::TPossibilityAttribute)
namespace URANIE
{
namespace DataServer
{
// -----------------------------------------------------------------------
//  Constructors
// -----------------------------------------------------------------------
TPossibilityAttribute::TPossibilityAttribute(TString str) :
        TAttribute(str, 0.0, 1.0), _PossibilityType(kUnknown), _MonotonousRelationship(
                kNotDefined)
{
}
TPossibilityAttribute::TPossibilityAttribute(TString str,
        EPossibilityType value, EMonotonousRelationship monotonous) :
        TAttribute(str, 0.0, 1.0), _PossibilityType(value), _MonotonousRelationship(
                monotonous)
{
}

TPossibilityAttribute::TPossibilityAttribute(TString str,
        EPossibilityType value, Double_t min, Double_t max,
        EMonotonousRelationship monotonous) :
        TAttribute(str, min, max), _PossibilityType(value), _MonotonousRelationship(
                monotonous)
{
}

TPossibilityAttribute::~TPossibilityAttribute()
{
    if (_blog)
    {
        cout << "Destructeur::TPossibilityAttribute[" << this << "] sname["
                << GetName() << "]" << endl;
    }

}

// -----------------------------------------------------------------------
//  printLog
// -----------------------------------------------------------------------
void TPossibilityAttribute::printLog()
{
    TAttribute::printLog();
    cout << "*******************************" << endl;
    cout << "** TPossibilityAttribute::printLog  ******" << endl;
    cout << "**   Possibility [" << _PossibilityType << "]" << endl;

    switch (_PossibilityType)
    {
    case kUniform:
        cout << "**   Possibility : kUniform" << endl;
        break;
    case kTriangular:
        cout << "**   Possibility : kTriangular" << endl;
        break;
    case kTrapezium:
        cout << "**   Possibility : kTrapezium" << endl;
        break;
    case kPolygonal:
        cout << "**   Possibility : kPolygonal" << endl;
        break;
    default:
        cout << "**   Possibility : kUnknown" << endl;
        break;
    }
    switch (_MonotonousRelationship)
    {
    case kIncreasing:
        cout << "**   Relationship : kIncreasing" << endl;
        break;
    case kDecreasing:
        cout << "**   Relationship : kDecreasing" << endl;
        break;
    default:
        cout << "**   Relationship : kNotDefined" << endl;
        break;
    }
    cout << "*******************************" << endl;

}

} // Fin du namespace DataServer
} // Fin du namespace URANIE
