//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestMain.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   jeu jan 24 15:56:01 2008
 
 $Revision$
 $Date$
 
 \brief Programme principal pour les tests unitiares sur la librairie DataServer.
 
 */
#include "cppunit/CompilerOutputter.h"
#include "cppunit/TestResult.h"
#include "cppunit/TestResultCollector.h"
#include "cppunit/TextTestProgressListener.h"
#include "cppunit/BriefTestProgressListener.h"
#include "cppunit/extensions/TestFactoryRegistry.h"
#include "cppunit/TestRunner.h"
#include "cppunit/XmlOutputter.h"

#include <stdexcept>

#include <iostream>
#include <fstream>

//uranie
#include "../UVersion.h"
// Liste des Tests Unitaires d'uranie
#include "TestTAttribute.h"
#include "TestTDataServer.h"
//#include "TestTMultinomialDistribution.h"
#include "TestTCustomDistribution.h"
#include "TestTGeneralizedNormalDistribution.h"
#include "TestTGeneralizedNormalV2Distribution.h"

#include "TestTUniformByPartsDistribution.h"
#include "TestTBetaDistribution.h"
#include "TestTCauchyDistribution.h"
#include "TestTWeibullDistribution.h"
#include "TestTGammaDistribution.h"
#include "TestTInvGammaDistribution.h"
#include "TestQuantiles.h"
#include "TestTPatternsEventList.h"
#include "TestTDataSpecification.h"
#include "TestTDSNtupleD.h"
#include "TestTAttributeFormula.h"
#include "TestTLogTriangularDistribution.h"
#include "TestTTriangularDistribution.h"
#include "TestTTrapeziumDistribution.h"
#include "TestTGenParetoDistribution.h"
#include "TestTUniformDistribution.h"
#include "TestTExponentialDistribution.h"
#include "TestTLogUniformDistribution.h"
#include "TestTGumbelMaxDistribution.h"
#include "TestTLogNormalDistribution.h"
#include "TestTNormalDistribution.h"
#include "TestTComposedDistribution.h"
#include "TestTTokenizer.h"
#include "TestQuantilesEstimation.h"
#include "TestTPCA.h"

CPPUNIT_TEST_SUITE_REGISTRATION (TDataServerTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TAttributeTest);
//CPPUNIT_TEST_SUITE_REGISTRATION( TMultinomialDistributionTest );
CPPUNIT_TEST_SUITE_REGISTRATION (TCustomDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TPatternsEventListTest);
CPPUNIT_TEST_SUITE_REGISTRATION (QuantilesTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TDSNtupleDTest); //28
CPPUNIT_TEST_SUITE_REGISTRATION (TBetaDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TUniformByPartsDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TDataSpecificationTest); //1
CPPUNIT_TEST_SUITE_REGISTRATION (TCauchyDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TLogUniformDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TTriangularDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TTrapeziumDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TGenParetoDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TUniformDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TGumbelMaxDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TTokenizerTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TAttributeFormulaTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TLogTriangularDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TWeibullDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TGammaDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TInvGammaDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TGeneralizedNormalDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TGeneralizedNormalV2DistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TComposedDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TLogNormalDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TNormalDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TExponentialDistributionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (QuantilesEstimationTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TPCATest);

bool uranie_test_suite(const char *module, const char *xml, const char *xsl);

int main()
{

    bool wasSuccessful;
#ifdef WIN32
    wasSuccessful = uranie_test_suite("DataServer",
                                      "testDataServerResults.xml", "..\\..\\..\\testReport.xsl");
#else
    wasSuccessful = uranie_test_suite("DataServer",
                                      "testDataServerResults.xml", "../../../testReport.xsl");
#endif

    // ---  Run the tests.

    TDataServerTest tdst;
    tdst.cleanWorkingDirectories("pouet");
    gDirectory->ls();
    // ---  Return error code 1 if the one of test failed.

    return wasSuccessful ? 0 : 1;
}
