//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestQuantilesEstimation.h
 \author Maud Salvoldi <AREVA/OCC>
 \date   $Date$
 
 \brief  Fichier de test des nouvelles methodes d'estimation du quantile
 implement�es dans la classe URANIE::DataServer::TDataServer 
 
 Created le : jeu jui  7 10:32:30 2011
 
 */
#ifndef CPPUNITEST_TQUANTILESESTIMATION_H
#define CPPUNITEST_TQUANTILESESTIMATION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TDataServer.h"
#include "TUniformDistribution.h"
#include "TNormalDistribution.h"
#include "UExceptions.h"

/*!
 @class QuantilesEstimationTest
 @brief Unit tests for the new methods of quantile estimation developped  TDataServer.
 
 */

class QuantilesEstimationTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (QuantilesEstimationTest);

    CPPUNIT_TEST (throwExceptioncomputeQuantileBadAlpha);
    CPPUNIT_TEST (throwExceptioncomputeQuantileFalseYname);
    CPPUNIT_TEST (throwExceptioncomputeQuantileFalseZname);
    CPPUNIT_TEST (computeQuantileCV);
//   FGA::WARNING 2012-03-13 11:23:54
//   Because desactived method with SR
//   CPPUNIT_TEST( throwExceptioncomputeQuantileSRBadAlpha);
//   CPPUNIT_TEST( throwExceptioncomputeQuantileSRFalseYname);
//   CPPUNIT_TEST( throwExceptioncomputeQuantileFalseSR);
//   CPPUNIT_TEST( computeQuantileCVwithSR);
//   End Of FGA::WARNING 2012-03-13 11:23:54

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    QuantilesEstimationTest();

    //! Default destructor
    virtual ~QuantilesEstimationTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and Tear Down methods      
     **/
    //@{
    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test of the method computeQuantileCV(TString yname, Double_t alpha, TString zname, Double_t zapha, Double_t &quantile, Double_t & rho)
     **/
    //@{
    //! Unit test to check the parameter alpha
    /*!
     \brief This test checks that the code returns an error message if the parameter alpha is no include between 0 and 1.
     */
    void throwExceptioncomputeQuantileBadAlpha();
    //! Unit test to checck the parameter yname
    /*!
     \brief This test checks that the code returns an error message if the parameter yname is unknown of the tds.
     */
    void throwExceptioncomputeQuantileFalseYname();
    //! Unit test to check the parameter zname
    /*!
     \brief This test checks that the code returns an error message if the parameter zname is unknown of the tds.
     */
    void throwExceptioncomputeQuantileFalseZname();
    //! Unit test to check computation of the quantile
    /*!
     \brief This test verifies that the code compute correclty the alpha-quantile 
     */
    void computeQuantileCV();

    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test of the method computeQuantileCV(TString yname, Double_t alpha, void *SR, Double_t &quantile, Double_t & rho)
     **/
    //@{
    //! Unit test to check the parameter alpha
    /*!
     \brief This test checks that the code returns an error message if the parameter alpha is no include between 0 and 1.
     */
    void throwExceptioncomputeQuantileSRBadAlpha();
    //! Unit test to checck the parameter yname
    /*!
     \brief This test checks that the code returns an error message if the parameter yname is unknown of the tds.
     */
    void throwExceptioncomputeQuantileSRFalseYname();
    //! Unit test to check the parameter zname
    /*!
     \brief This test checks that the code returns an error message if the file SR doesn't exist.
     */
    void throwExceptioncomputeQuantileFalseSR();
    //! Unit test to check computation of the quantile
    /*!
     \brief This test verifies that the code compute correclty the alpha-quantile
     */
    void computeQuantileCVwithSR();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds;
    URANIE::DataServer::TDataServer *_tdsza;

    Double_t _dZa, _dYa, _dRho;
};

#endif 
