//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTAttribute.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TAttribute
 
 Cr�e le : mer nov 21 07:52:44 2007
 
 */
#ifndef CPPUNITEST_TATTRIBUTE_H
#define CPPUNITEST_TATTRIBUTE_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TAttributeTest
 * \brief Unit tests for TAttribute.
 */
class TAttributeTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TAttributeTest);
    CPPUNIT_TEST (testBasisAttributes);
// CFO::Warning NOT INCLUDED TU
//   CPPUNIT_TEST( testClone );
    CPPUNIT_TEST (testNames);
    CPPUNIT_TEST (testLabels);
    CPPUNIT_TEST (testKeyMethod);
    CPPUNIT_TEST (testKeyCardinal);
    CPPUNIT_TEST (testKeyDefaultValues);
    CPPUNIT_TEST (testKeyMethod1);
    CPPUNIT_TEST (testSetFieldOfKey1);
    CPPUNIT_TEST (testSetFieldOfKey2);
    CPPUNIT_TEST (testGetSetOrigin);
    CPPUNIT_TEST (testGetSetDataType);
    CPPUNIT_TEST (testGetSetTitle);
    CPPUNIT_TEST (testGetSetBounds);
    CPPUNIT_TEST (testGetSetMeanStd);
    CPPUNIT_TEST (testGetSetUnity);
    CPPUNIT_TEST (testGetLegend);
    CPPUNIT_TEST (testGetSetNote);
    CPPUNIT_TEST (testGetSetMinimum);
    CPPUNIT_TEST (testGetSetMaximum);
    CPPUNIT_TEST (testGetSetQuantile);
    CPPUNIT_TEST (testGetInputOutput);
    CPPUNIT_TEST (testGetKeyList);
    CPPUNIT_TEST (testGetSetFormatSubstitue);
    CPPUNIT_TEST (testIsInputOutput);
    CPPUNIT_TEST (testShareMethods);
    CPPUNIT_TEST (testGetSetDefault);
    CPPUNIT_TEST (testGetSetDefaultValue);
    CPPUNIT_TEST (testGetSetDefaultVector);
    CPPUNIT_TEST (testGetSetDefaultString);
    CPPUNIT_TEST (testGetSetStepValue);
    CPPUNIT_TEST (throwExceptionWithNegativeStd);
    CPPUNIT_TEST (throwExceptionWithBadMean);
    CPPUNIT_TEST (throwExceptionSetBounds);
    CPPUNIT_TEST (throwExceptionSetLowerBound);
    CPPUNIT_TEST (throwExceptionSetUpperBound);
    CPPUNIT_TEST (throwExceptionSetDefault);
    CPPUNIT_TEST (throwExceptionSetGetHasDefaultValue);
    CPPUNIT_TEST (throwExceptionSetGetHasDefaultVector);
    CPPUNIT_TEST (throwExceptionSetGetHasDefaultString);
    CPPUNIT_TEST (throwExceptionTestKeyMethod);
    CPPUNIT_TEST (throwExceptionWithNegativeElementForStatistic);
    CPPUNIT_TEST (throwExceptionTooBigElementForStatistic);
    CPPUNIT_TEST (throwExceptionWithNegativeElementForQuantile);
    CPPUNIT_TEST (throwExceptionTooBigElementForQuantile);
    CPPUNIT_TEST (throwExceptionWarningQuantileNotComputed);

    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TAttributeTest object.
     */
    TAttributeTest();

    /// Destructor.
    virtual ~TAttributeTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void testNames();

    void testLabels();
    //! Tests the containt of attribute structure.
    void testKeyMethod();

    //! Tests set/getFieldOfKey methods.
    void testFieldOfKeyMethod();

    //! Tests when the TDataServer is not filled with setKey() that KeyList.size is null.
    void testKeyCardinal();

    //! Tests when the TDataServer is filled with a setKey() with juste a file and key. 
    void testKeyDefaultValues();

    //! Test 
    void testKeyMethod1();

    //! By using for a first time a setKey(), the TEventList _sEventList is NULL.
    void testSetFieldOfKey1();

    void testSetFieldOfKey2();

    //! Tests the getDataType and setDataType() methods.
    void testGetSetDataType();

    //! Tests the setTitle methods.
    void testGetSetTitle();

    //! Tests getLowerBound, setLowerBound, getUpperBound, isUpperBounded(), setUpperBound, setBounds and isBounded methods
    void testGetSetBounds();

    //! Tests the getMean(), setMean, getStd() and setStd() methods.
    void testGetSetMeanStd();

    //! Tests the getUnity() method
    void testGetSetUnity();

    //! Tests the getLegend() method.
    void testGetLegend();

    //! Tests the getNote() method.
    void testGetSetNote();

    //! Tests the get and set Minimums.
    void testGetSetMinimum();

    //! Tests the get and set Maximums.
    void testGetSetMaximum();

    //! Tests the get and set Quantiles.
    void testGetSetQuantile();

    //! Tests the Clone() method.
    // void  testClone();

    //! Tests basic of the constructors.
    void testBasisAttributes();

    //! Tests the getInputOutput() method.
    void testGetInputOutput();

    //! Tests the getKeyList() method.
    void testGetKeyList();

    //! Tests the getFormatSubstitue and setFormatSubstitue methods.
    void testGetSetFormatSubstitue();

    //! Tests the isInput() and isOutput() methods and setFormatSubstitue methods.
    void testIsInputOutput();

    //! Tests the getShare(), delShare(), initShare().
    void testShareMethods();

    //! Tests the getDefault() and setDefault().
    void testGetSetDefault();

    //! Tests the getDefaultValue() and setDefaultValue().
    void testGetSetDefaultValue();

    //! Tests the getDefaultVector() and setDefaultVector().
    void testGetSetDefaultVector();
    
    //! Tests the getDefaultString() and setDefaultString().
    void testGetSetDefaultString();

    //! Tests the getStepValue() and setStepValue().
    void testGetSetStepValue();

    //! Tests the good behavior of exceptions in setkey method.
    void throwExceptionTestKeyMethod();

    //! Throws an exception when using negative standard deviation.
    void throwExceptionWithNegativeStd();

    //! Tests getOrigin() et setOrigin() methods.
    void testGetSetOrigin();

    //!  Tests that a deprecated method rises up an exception.
    void throwExceptionWithBadMean();

    //! Throws an exception when upperbound < lowerbound. 
    void throwExceptionSetBounds();

    //! Throws an exception when predefined upperbound < given lowerbound. 
    void throwExceptionSetLowerBound();

    //! Throws an exception when predefined lowerbound < given upperbound. 
    void throwExceptionSetUpperBound();

    //! Throws an exception when negative element are requested in stat getters
    void throwExceptionWithNegativeElementForStatistic();
    
    //! Throws an exception when too big  element are requested in stat getters
    void throwExceptionTooBigElementForStatistic();
    
    //! Throws an exception when negative element are requested in quantile getters
    void throwExceptionWithNegativeElementForQuantile();
    
    //! Throws an exception when too big  element are requested in quantile getters
    void throwExceptionTooBigElementForQuantile();
    
    //! Throws an exception when quantile is not computed
    void throwExceptionWarningQuantileNotComputed(); 
    
    //! Throws an exception when given default value is not comprised between bounds. 
    void throwExceptionSetDefault();

    //! Throws an exception when given default value is not comprised between bounds. 
    void throwExceptionSetGetHasDefaultValue();

    //! Throws an exception when given default value is not comprised between bounds. 
    void throwExceptionSetGetHasDefaultVector();
    
    //! Throws an exception when given default value is not comprised between bounds. 
    void throwExceptionSetGetHasDefaultString();
    //@}

private:
    URANIE::DataServer::TAttribute *att1, *att1bis, *att2, *att3, *att4, *attV, *attS;
    URANIE::DataServer::TDataServer * tds1;
};

#endif  // CPPUNITEST_TATTRIBUTE_H
