//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTDataServer.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TDataServer
 
 Created le : mer nov 21 07:52:44 2007
 
 */
#ifndef CPPUNITEST_TDATASERVER_H
#define CPPUNITEST_TDATASERVER_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TDataServer.h"
#include "TUniformDistribution.h"
#include "TNormalDistribution.h"
#include "UExceptions.h"
#include "TNtuple.h"
#include "TDSNtupleD.h"

/*!
 @class TDataServerTest
 @brief Unit tests for TDataServer.
 
 The units tests are :
 \li Names \ref testNames
 \li Title \ref testTitles
 
 */

class TDataServerTest: public CPPUNIT_NS::TestFixture
{

    CPPUNIT_TEST_SUITE (TDataServerTest);
    CPPUNIT_TEST (testNames);
    CPPUNIT_TEST (testNameAttributes);
    CPPUNIT_TEST (testTitles);
    CPPUNIT_TEST (testFileDataRead);
    CPPUNIT_TEST (testExportData);
    CPPUNIT_TEST (testReadAndExportDataWithVectorAndString);
    CPPUNIT_TEST (testReloadFileVectorAndString);
    CPPUNIT_TEST (testPreaddFileVectorAndString);
    CPPUNIT_TEST (testExportDataHeader);
    CPPUNIT_TEST (testExportDataNeMo);
    CPPUNIT_TEST (testExportDataContext);
    CPPUNIT_TEST (testPatternsCardinalWithCarriageReturn);
    CPPUNIT_TEST (testPatternsCardinalWithoutCarriageReturn);
    CPPUNIT_TEST (testPatternsCardinalWithWhiteLineAtEof);
    CPPUNIT_TEST (testPatternsCardinalWithWhiteLineInsideData);
    CPPUNIT_TEST (testComputeCorrelationMatrix1);
    CPPUNIT_TEST (testComputeCorrelationMatrix2);
    CPPUNIT_TEST (testComputeCorrelationMatrix3);
    CPPUNIT_TEST (testComputeCorrelationMatrix4);
    CPPUNIT_TEST (testComputeCAH);
    CPPUNIT_TEST (testFillDataServer);
    CPPUNIT_TEST (testAddAndDelAttributes);
    CPPUNIT_TEST (testAddAttributeWithData);
    CPPUNIT_TEST (testAddAndDelAttributesWithVectors);

#ifdef WITH_JSONCPP
    CPPUNIT_TEST (testFileDataReadJSon);
    CPPUNIT_TEST (testExportDataJson);
    CPPUNIT_TEST (testReadAndExportDataJsonMultiType);
    CPPUNIT_TEST (testThrowExceptionsInJSon);
#endif
    
    // -- ngi: 2013-04-24 --
    CPPUNIT_TEST (testFillAttribute);
    // ----
    CPPUNIT_TEST (testBootstrap);
    CPPUNIT_TEST (testfillConstantAttribute);
    CPPUNIT_TEST (testGetValue1);
    CPPUNIT_TEST (testGetValue2);
    CPPUNIT_TEST (testGetValueRange);
    CPPUNIT_TEST (testLeafIteratorPositionCreateTuple);
    CPPUNIT_TEST (testLeafIteratorPosition_FileWithoutIterator);
    CPPUNIT_TEST (testLeafIteratorPosition_FileWithIterator);
    CPPUNIT_TEST (testLeafIteratorPosition_Merge2Files);
    // CFO::Warning NOT INCLUDED TU
    CPPUNIT_TEST( testdrawMethods);
    CPPUNIT_TEST( testdrawMethodsNoCanvases);
    CPPUNIT_TEST (testDrawWithSelection);
    CPPUNIT_TEST (testGetNPatterns);
    CPPUNIT_TEST (testGetIteratorName);
    CPPUNIT_TEST (testGetDataTreeName);
    CPPUNIT_TEST (testGetArchiveFileName);
    CPPUNIT_TEST (testGetOrigin);
    CPPUNIT_TEST (testGetNAttributes);
    CPPUNIT_TEST (testGetListOfAttributes);
    CPPUNIT_TEST (testGetInputOutputAttributes);
    CPPUNIT_TEST (testGetCut);
    CPPUNIT_TEST (testSelect);
    CPPUNIT_TEST (testGetDataSpecification);
    CPPUNIT_TEST (testNtupleDataRead);
    CPPUNIT_TEST (testNtupleDataReadStringAndVector);
    CPPUNIT_TEST (testLoadTree);
    CPPUNIT_TEST (testGetAttributeIndex);
    CPPUNIT_TEST (testGetAttributeFromIndex);
    CPPUNIT_TEST (testMerge);
    CPPUNIT_TEST (testMergeWithVectors);
    CPPUNIT_TEST (testMergeWithIterators);
    CPPUNIT_TEST (testIsAttribute);
    CPPUNIT_TEST (testSetIterator);
    CPPUNIT_TEST (testVariousNormalisation); 
    CPPUNIT_TEST (testComputeStatistic);
    CPPUNIT_TEST (testComputeStatisticOnABranchWithSeveralLeaves);
    CPPUNIT_TEST (testComputeIndexQuantileWilks);
    CPPUNIT_TEST (testComputeInverseNormalCDF);
    CPPUNIT_TEST (testComputeQuantile);
    CPPUNIT_TEST (testComputeQuantilesVector);
    CPPUNIT_TEST (testEstimateQuantilesVector);
    CPPUNIT_TEST (testCreateTuple);
    CPPUNIT_TEST (testGetTuple);
    CPPUNIT_TEST (testSetTuple);
    CPPUNIT_TEST (testComputeRank);
    CPPUNIT_TEST (testGetVIssues);

    CPPUNIT_TEST (throwExceptionFileDataReadDataServerNotEmpty1);
    CPPUNIT_TEST (throwExceptionFileDataReadDataServerNotEmpty2);
    CPPUNIT_TEST (throwExceptionFileDataReadFileFileNotExist);
    CPPUNIT_TEST (throwExceptionFileDataReadNoLineHeader);
    CPPUNIT_TEST (throwExceptionFileDataReadTypeHeaderNotCompleteType);
    CPPUNIT_TEST (throwExceptionFileDataReadTypeHeaderMissingField);
    CPPUNIT_TEST (throwExceptionFileDataReadTypeHeaderTooMuchField);

    CPPUNIT_TEST (throwExceptionfillConstantAttribute);
    CPPUNIT_TEST (throwExceptionSetIterator);
    CPPUNIT_TEST (throwExceptionGetNInputAttributes);
    CPPUNIT_TEST (throwExceptionGetNOutputAttributes);
    CPPUNIT_TEST (throwExceptionNtupleDataReadWithInexistingRootFile);
    CPPUNIT_TEST (throwExceptionNtupleDataReadWithNoExistingTuple);
    CPPUNIT_TEST (throwExceptionNtupleDataReadWithEmptyExistingTree);
    CPPUNIT_TEST (throwExceptionNtupleDataReadWithNotEmptyTDataspecification);
  
    /* *** FGA::MODIFCATION 2011-12-21 11:37:46 ne marche pas sous Windows
    //   CPPUNIT_TEST( throwExceptionExportDataWithTDSNull );
    //   *** End Of FGA::MODIFCATION 2011-12-21 11:37:46 */
    CPPUNIT_TEST (throwExceptionExportDataWithEmptyTDataserver);
    CPPUNIT_TEST (throwExceptionExportDataHeaderWithEmptyTDataserver);
    CPPUNIT_TEST (throwExceptionExportDataHeaderWithNoPatterns);
    CPPUNIT_TEST (throwExceptionExportDataNeMoWithEmptyTDataserver);
    CPPUNIT_TEST (throwExceptionExportDataNeMoWithNoPatterns);
    CPPUNIT_TEST (throwExceptionExportDataContextWithEmptyTDataserver);
    CPPUNIT_TEST (throwExceptionExportDataContextWithNoPatterns);

    /* /\* *** FGA::MODIFCATION 2011-12-21 11:37:46 */
    /*    CPPUNIT_TEST( throwExceptionAddAttributeWithFormula ); */
    /*    *** End Of FGA::MODIFCATION 2011-12-21 11:37:46 *\/ */
    /* // -- ngi: 2013-04-24 -- */
    CPPUNIT_TEST (throwExceptionAddAttributeWithFormula);
    CPPUNIT_TEST (throwExceptionAddAttributeWithData);
    CPPUNIT_TEST (throwExceptionFillAttribute);
    // ----

    CPPUNIT_TEST (throwExceptionDelAttribute);
    CPPUNIT_TEST (throwExceptionGetValueWithStringOrVector);
    CPPUNIT_TEST (throwExceptionGetStringWithVectorOrDouble);
    CPPUNIT_TEST (throwExceptionGetVectorWithStringOrDouble);
    CPPUNIT_TEST (throwExceptionGetAttributeIndexFromName);
    CPPUNIT_TEST (throwExceptionGetAttributeIndexFromAttribute);
    CPPUNIT_TEST (throwExceptionGetAttributeWithEmptyDataspecification);
    CPPUNIT_TEST (throwExceptionGetAttributeWithIndexSuperiorToNbAttributes);
    CPPUNIT_TEST (throwExceptionGetAttributeFromIndexWithNegativeIndex);
    CPPUNIT_TEST (throwExceptionMerge2filesOfDifferentNbPatterns);
    CPPUNIT_TEST (throwExceptionComputeStatistic);
    CPPUNIT_TEST (throwExceptionComputeRank);
    CPPUNIT_TEST (throwExceptionComputeInverseNormalCDF);
    CPPUNIT_TEST (throwExceptionComputeIndexQuantileWilksForBadProbability);
    CPPUNIT_TEST (throwExceptionComputeIndexQuantileWilksForInapropriateConfidenceLevel);
    // Visualization
    CPPUNIT_TEST (throwExceptionScan);
    CPPUNIT_TEST (throwExceptionDraw);
    CPPUNIT_TEST (throwExceptionDrawWithOrderTooHigh);
    CPPUNIT_TEST (throwExceptionDrawWithOrderAndEmptyTds);
    CPPUNIT_TEST (throwExceptionDrawHistogram);
    CPPUNIT_TEST (throwExceptionDrawIter);
    CPPUNIT_TEST (throwExceptionDrawECDF);
    CPPUNIT_TEST (throwExceptionDrawECDFWithMultiVariables);
    CPPUNIT_TEST (throwExceptionDrawCDF);
    CPPUNIT_TEST (throwExceptionDrawCDFWithMultiVariables);
    CPPUNIT_TEST (throwExceptionDrawBoxPlot);
    CPPUNIT_TEST (throwExceptionDrawTufte);
    CPPUNIT_TEST (throwExceptionDrawTufteWithBadNumberOfVariables);
    CPPUNIT_TEST (throwExceptionDrawScatterPlotWithBadNumberOfVariables);
    CPPUNIT_TEST (throwExceptionDrawProfile);
    CPPUNIT_TEST (throwExceptionStartViewer);

#ifdef WITH_MATHMORE
    CPPUNIT_TEST (throwExceptionDrawPPPlot);
    CPPUNIT_TEST (throwExceptionDrawQQPlot);
#endif
    // CFO::Warning NOT INCLUDED TU
    CPPUNIT_TEST (throwExceptionGetTuple);
	
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TDataServerTest();

    //! Default destructor
    virtual ~TDataServerTest();

    void setUp();
    void tearDown();

    //! the clean tempory directories
    void cleanWorkingDirectories(const char *option = "without");
    //@}

    //---------------------------------------------
    /** @name Unit Tests
	The list of unit test
    **/
    //@{ 
    //! Test of the names
    void testNames();

    //! Test of the titles
    /*!
      The title given by the constructor, the default title, and the title given with the setter method
      \em setTitle.
    */
    void testTitles();

    //! Tests that iterator is in first place in the TNtupleD of a TDataServer, created trough a the methiod createTuple().
    void testLeafIteratorPositionCreateTuple();
    //! Tests that iterator is in first place in the TNtupleD of a TDataServer, reading from a file without an iterator.
    void testLeafIteratorPosition_FileWithoutIterator();
    //! Tests that iterator is in first place in the TNtupleD of a TDataServer, reading from a file with an iterator.
    void testLeafIteratorPosition_FileWithIterator();
    //! Tests that iterator is in first place in the TNtupleD of a TDataServer, reading from a 2 files, one with iterator, other withou iterator.
    void testLeafIteratorPosition_Merge2Files();

    //! Test of several draw methods
    void testdrawMethods();
    
    //! Test of several draw methods when no canvas are provided
    void testdrawMethodsNoCanvases();

    //! Tests the draw method with selection
    void testDrawWithSelection();

    //! Tests the getNPatterns();
    void testGetNPatterns();

    //! Tests the getIteratorName() method.
    void testGetIteratorName();

    //! Tests the getDataTreeName() method. 
    void testGetDataTreeName();

    //! Tests the getArchiveFileName() method.
    void testGetArchiveFileName();

    //! Tests the getOrigin() method. 
    void testGetOrigin();

    //! Tests the getListOfAttributes() method
    void testGetListOfAttributes();

    //! Tests the getNAttributes() method
    void testGetNAttributes();

    //! Tests the getNInputAttributes and getNOutputAttributes methods.
    void testGetInputOutputAttributes();

    //! Tests the getDataSpecification() method.
    void testGetDataSpecification();

    //! Tests the ntupleDataRead method
    void testNtupleDataRead();

    //! Tests the ntupleDataRead method with string and vectors
    void testNtupleDataReadStringAndVector();

    //! Tests the loadTree() method.
    void testLoadTree();

    //! Tests the getAttributeIndex methods.
    void testGetAttributeIndex();

    //! Tests the getAttribute(Int_t) method.
    void testGetAttributeFromIndex();

    //! Tests the merge()  method.
    void testMerge();

    //! Tests the merge()  method. with complicated objects
    void testMergeWithVectors();

    //! Tests the merge()  method. with spotable iterator and mixed orders => Should realign the second file on the iterator number of the first one.
    void testMergeWithIterators();
    
    //! Tests the isAttribute() method.
    void testIsAttribute();

    //! Tests the setIterator() method.
    void testSetIterator();

    //! Test the various possible normalisation methods
    void testVariousNormalisation();
  
    //! Tests computeStatistic() method.
    void testComputeStatistic();
    
    //! Tests computeStatistic() method on a branch with several leaves.
    void testComputeStatisticOnABranchWithSeveralLeaves();

    //! Tests computeIndexQuantileWilks() method.
    void testComputeIndexQuantileWilks();

    //! Tests the computeInverseNormalCDF() method.
    void testComputeInverseNormalCDF();

    //! Tests the computeQuantile methods
    void testComputeQuantile();

    //! Tests the computeQuantile methods
    void testComputeQuantilesVector();
  
    //! Tests the estimateQuantile methods
    void testEstimateQuantilesVector();
  
    //! Tests the createTuple() method.
    void testCreateTuple();

    //! Tests the getTuple() method.
    void testGetTuple();

    //! Tests the setTuple() method.
    void testSetTuple();

    //! Tests the computeRank() method.
    void testComputeRank();

    //! Tests the GetVIssues() method.
    void testGetVIssues();
    //@}
    //---------------------------------------------
    /** @name Unit Tests for loading data
	The list of unit test for loading process from ASCII file, ROOT file, add attribute
    **/
    //@{ 
    //! Test the loading process from ASCII File
    /*!
      Read a good ASCII File 
    */
    //! Test the fileDataRead method for files
    /*!
      \li classical files ( with COLUMN_NAMES line)
      \li without COLUMN_NAMES line but with COLUMN_TITLES instead
      \li when the number of attributes is greater than 1000
    */
    void testFileDataRead();
    //! Test the reloading of a database
    void testReloadFileVectorAndString();
    //! Test preadding attribute before doing a fileDataRead
    void testPreaddFileVectorAndString();    
    //! Tests the create and fill dataserver
    void testFillDataServer();
    //! Tests that number of patterns are well computed with a file with a carriage return at the end of the data (geyser.dat).
    void testPatternsCardinalWithCarriageReturn();
    //! Tests that number of patterns are well computed with a file without any carriage return at the end of the data.
    void testPatternsCardinalWithoutCarriageReturn();

    //! Tests number of patterns when datafile contains a white line at the end of data;
    void testPatternsCardinalWithWhiteLineAtEof();
    //! Tests number of patterns when datafile contains a white line inside data;
    void testPatternsCardinalWithWhiteLineInsideData();

    //! Throw an exception when reading twice a ASCII File and the dataserver isn't not empty 
    void throwExceptionFileDataReadDataServerNotEmpty1();
    //! Throw an exception when reading a ASCII File and the dataserver isn't not empty after adding attribute
    void throwExceptionFileDataReadDataServerNotEmpty2();
    //! Throw an exception when reading a ASCII File which doesn't exist
    void throwExceptionFileDataReadFileFileNotExist();
    //! Throw an exception when reading a file if types contains a different number of elements than Names
    void throwExceptionFileDataReadTypeHeaderNotCompleteType();
    //! Throw an exception when reading a file if there is a missing field
    void throwExceptionFileDataReadTypeHeaderMissingField();
    //! Throw an exception when reading a file if there is too much field
    void throwExceptionFileDataReadTypeHeaderTooMuchField(); 
    //! Throw an exceptions when reading a ASCII File with no line header
    void throwExceptionFileDataReadNoLineHeader();
    //! Tests that an exception is returned by the getNInputAttributes() method when TTree is empty.
    void throwExceptionGetNInputAttributes();
    //! Tests that an exception is returned by the getNOutputAttributes() method when TTree is empty.
    void throwExceptionGetNOutputAttributes();
    //! Tests that an exception is returned when the file to load doesn't exist.
    void throwExceptionNtupleDataReadWithInexistingRootFile();
    //! Tests that an exception is returned when the required TDSNtupleD doesn't exist.
    void throwExceptionNtupleDataReadWithNoExistingTuple();
    //! Tests that an exception is returned when the required TDSNtupleD is empty
    void throwExceptionNtupleDataReadWithEmptyExistingTree();
    //! Tests that an exception is returned when the TDataSpecification is not empty.
    void throwExceptionNtupleDataReadWithNotEmptyTDataspecification();
    //! Tests that an exception is returned when trying to export a TDataServer with TDS NULL with exportData().
    void throwExceptionExportDataWithTDSNull();
    //! Tests that an exception is returned when trying to export an empty TDataServer with exportData().
    void throwExceptionExportDataWithEmptyTDataserver();
    //! Tests that an exception is returned when trying to export an empty TDataServer with exportDataHeader().
    void throwExceptionExportDataHeaderWithEmptyTDataserver();
    //! Tests that an exception is returned when trying to export a TDataServer with no patterns() with exportDataHeader().
    void throwExceptionExportDataHeaderWithNoPatterns();
    //! Tests that an exception is returned when attribute exists already of UEntry is not specified
    void throwExceptionfillConstantAttribute();

    //! Tests that an exception is returned when trying to export an empty TDataServer with exportDataNeMo().
    void throwExceptionExportDataNeMoWithEmptyTDataserver();
    //! Throws an exception when trying to export a tds with no patterns.
    void throwExceptionExportDataNeMoWithNoPatterns();
    //! Tests that an exception is returned when trying to export an empty TDataServer with exportDataContext().
    void throwExceptionExportDataContextWithEmptyTDataserver();
    //! Throws an exception when trying to export a tds with no patterns.
    void throwExceptionExportDataContextWithNoPatterns();
    //! Throws exceptions related to the addition of a TAttributeFormula.
    void throwExceptionAddAttributeWithFormula();
    //! Throws exceptions related to the addition of a TAttribute when providing data
    void throwExceptionAddAttributeWithData();
    //! Throws exceptions from the fillAttribute function.
    void throwExceptionFillAttribute();
    //! Throws an exception when deleting a non existing attribute. 
    void throwExceptionDelAttribute();

    //! Test that an exception is returned wwhen using getValue on vector or string 
    void throwExceptionGetValueWithStringOrVector();
    //! Test that an exception is returned wwhen using getString on vector or double
    void throwExceptionGetStringWithVectorOrDouble();
    //! Test that an exception is returned wwhen using getVector on double or string 
    void throwExceptionGetVectorWithStringOrDouble();
    //! Tests that an exception is returned when trying to get index of an inexisting attribute with the prototype getAttributeIndex(TString ).
    void throwExceptionGetAttributeIndexFromName();
    //! Tests that an exception is returned when trying to get index of an inexisting attribute with the prototype getAttributeIndex(TAttribute * ).
    void throwExceptionGetAttributeIndexFromAttribute();
    //! Tests that an exception is returned when trying to get an attribute from its index when TDataspecification is empty.
    void throwExceptionGetAttributeWithEmptyDataspecification();
    //! Tests that an exception is returned when trying to get an attribute from its index when index is superior to the number of attributes du TDataServer.
    void throwExceptionGetAttributeWithIndexSuperiorToNbAttributes();
    //! Tests that an exception is returned when trying to get an attribute from its index when index is superior to the number of attributes of the TDataServer.
    void throwExceptionGetAttributeWhenAccessingToNoExistingIndexOfAttributeInDataspecification();
    //! Throws an exception when using a negative index. 
    void throwExceptionGetAttributeFromIndexWithNegativeIndex();
    //! Tests that an exception is returned when trying to merge 2 files with different number of patterns.
    void throwExceptionMerge2filesOfDifferentNbPatterns();
  
    //! Throws an exception when using omputeStatistic() method on an empty TDS.
    void throwExceptionComputeStatistic();

    //! Throws an exception when using computeRank() method on an empty TDS.
    void throwExceptionComputeRank();

    //! Throws an exception when using scan() method on an empty TDataServer.
    void throwExceptionScan();

    //! Throws an exception when using scan() method on an empty TDataServer.
    void throwExceptionComputeQuantileProba();

    //! Throws an exception when using scan() method on an empty TDataServer.
    void throwExceptionComputeIndexQuantileWilksForBadProbability();

    //! Throws an exception when using scan() method on an empty TDataServer.
    void throwExceptionComputeIndexQuantileWilksForInapropriateConfidenceLevel();

    //! Throws an exception when giving a value as probability not in [0,1[.
    void throwExceptionComputeInverseNormalCDF();

    //! Throws an exception when using draw() method on an empty TDataServer.
    void throwExceptionDraw();

    //! Throws an exception when using draw() (with order) method with an order superior to total number of values.
    void throwExceptionDrawWithOrderTooHigh();

    //! Throws an exception when using draw() (with order) method on an empty TDataServer.
    void throwExceptionDrawWithOrderAndEmptyTds();

    //! Throws an exception when using drawHistogram() method on an empty TDataServer.
    void throwExceptionDrawHistogram();

    //! Throws an exception when using drawIter() method on an empty TDataServer.
    void throwExceptionDrawIter();

    //! Throws an exception when using drawECDF() method on an empty TDataServer.
    void throwExceptionDrawECDF();

    //! Throws an exception when using multivariate drawECDF() method on an empty TDataServer.
    void throwExceptionDrawECDFWithMultiVariables();

    //! Throws an exception when using drawCDF() method on an empty TDataServer.
    void throwExceptionDrawCDF();

    //! Throws an exception when using multivariate drawCDF() method on an empty TDataServer.
    void throwExceptionDrawCDFWithMultiVariables();

    //! Throws an exception when using drawBoxPlot() method on an empty TDataServer.
    void throwExceptionDrawBoxPlot();

    //! Throws an exception when using drawTufte() method on an empty TDataServer.
    void throwExceptionDrawTufte();

    //! Throws an exception when using drawTufte() method with a number of variables >< 2.
    void throwExceptionDrawTufteWithBadNumberOfVariables();

    //! Throws an exception when using drawScatterPlot() method with a number of variables >< 2.
    void throwExceptionDrawScatterPlotWithBadNumberOfVariables();

    //! Throws an exception when using drawProfile() method on an empty TDataServer.
    void throwExceptionDrawProfile();

#ifdef WITH_MATHMORE
    //! Throws an exception when using drawPPPlot() method on an empty TDataServer.
    void throwExceptionDrawPPPlot();
        //! Throws an exception when using drawQQPlot() method on an empty TDataServer.
    void throwExceptionDrawQQPlot();
#endif
    
    //! Throws an exception when using startViewer() method on an empty TDataServer.
    void throwExceptionStartViewer();

    //! Throws an exception when using getTuple() method on an empty TDataServer.
    void throwExceptionGetTuple();
    //@}

    //---------------------------------------------
    /** @name Unit Tests for export Data
     **/
    //@{ 
    //! Test the exportData  method for files
    /*!
     */
    void testExportData();

    //! Test the exportData method with string and vector... Re-read the exported file, with few checks
    void testReadAndExportDataWithVectorAndString();    

    //! Tests the exportDataHeader() method;
    void testExportDataHeader();

    //! Tests the exportDataNemo() method;
    void testExportDataNeMo();

    //! Tests the exportDataContext method;
    void testExportDataContext();
    //@}

    //---------------------------------------------
    /** @name Unit Tests for statistic computation
     **/
    //@{
    //! Test the computation of the correlation matrix on values
    void testComputeCorrelationMatrix1();
    //! Test the computation of the correlation matrix on ranks with full list of variables.
    void testComputeCorrelationMatrix2();
    //! Test the computation of the correlation matrix on ranks with full explicit list of variables.
    void testComputeCorrelationMatrix3();
    //! Test the computation of the correlation matrix on ranks with a sub-list of variables.
    void testComputeCorrelationMatrix4();
    //@}

    //---------------------------------------------
    /** @name Unit Tests for statistic computation
     **/
    //@{
    //! Test the computation of the CAH from Birth-Death database
    void testComputeCAH();
    //@}

    //---------------------------------------------
    /** @name Unit Tests for manipulate patterns
     **/
    //@{
    //---------------------------------------------
    /** @name Unit Tests for boostrap analysis
     **/
    //@{
    //! Test the bootstrap database generation
    void testBootstrap();
    //@}

    //---------------------------------------------
    /** @name Unit Tests for manipulate attribute
     **/
    //@{
    //! Throw an exceptions when we specifie a variable which doesn't exists
    void throwExceptionSetIterator();
    //! Test to set/get names to attributes
    void testNameAttributes();
    //! Add and delete attributes of a TDataServer
    void testAddAndDelAttributes();
    //! Add and delete attributes of a TDataServer with data
    void testAddAttributeWithData();
    //! Add and delete attributes of a TDataServer playing with vectors
    void testAddAndDelAttributesWithVectors();
    //! Fill attributes of a TDataServer
    void testFillAttribute();
    //@}

    //---------------------------------------------
    /** @name Unit Tests for manipulate attribute
     **/
    //@{
     //!Tests the fillConstantAttribute method
    void testfillConstantAttribute();
    //!Tests the getValue method with a file without iterator
    void testGetValue1();
    //!Tests the getValue method with a file with iterator  
    void testGetValue2();
    //!Tests the getValue method with a file with iterator  
    void testGetValueRange();
    //@}

    //---------------------------------------------
    /** @name Unit Tests for Selection/cut patterns
	We apply Select/Cut methods
    **/
    //@{
    //! Tests the getCut() method.
    void testGetCut();
    //! Test the selection of patterns
    void testSelect();
    //@}

#ifdef WITH_JSONCPP
    //! tests the JSon read
    void testFileDataReadJSon();
    //! tests the JSon export
    void testExportDataJson();
    //! tests the JSon read and export with vectors and strings
    void testReadAndExportDataJsonMultiType();
    //! tests the exception raised with bad JSon files
    void testThrowExceptionsInJSon();    
#endif

    
private:
    TString _workingTestPath;
    URANIE::DataServer::TDataServer *tds1, *tds2, *tds3, *tds4, *tds5,
	*tdsgeyser, *tdsJSON;
    Bool_t _bLog;

};

#endif  // CPPUNITEST_TDATASERVER_H
