//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTGumbelDistribution.h
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TGumbelDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 // *****************************************************************************

 */
#ifndef CPPUNITEST_TGUMBELDISTRIBUTION_H
#define CPPUNITEST_TGUMBELDISTRIBUTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TGumbelDistribution.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TGumbelDistributionTest
 * \brief Unit tests for TGumbelDistribution.
 */
class TGumbelDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TGumbelDistributionTest);
    CPPUNIT_TEST (testLaw);
    CPPUNIT_TEST (testGetSetScale);
    CPPUNIT_TEST (testGetSetLocation);
    CPPUNIT_TEST (testGumbelTruncation);
    CPPUNIT_TEST (throwExceptionWithNegativeBeta);CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TGumbelDistribution object.
     */
    TGumbelDistributionTest();

    /// Destructor.
    virtual ~TGumbelDistributionTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests gumbel distribution with set parameters (set mu and beta).
    void testLaw();

    //! Tests the getScale() and setScale() methods.
    void testGetSetScale();

    //! Tests the getLocation() and setLocation() methods.
    void testGetSetLocation();

    //! Tests truncation on gumbel distribution.
    void testGumbelTruncation();

    //! Throws an exception when the given beta is negative.
    void throwExceptionWithNegativeBeta();

    //@}

private:
    URANIE::DataServer::TGumbelDistribution *_dproba;
    URANIE::DataServer::TAttribute * _att;
};
#endif  // CPPUNITEST_TGUMBELDISTRIBUTION_H
