#include <cppunit/extensions/HelperMacros.h>
#include "TestTInfiniteDistribution.h"
#include "TInvGammaDistribution.h"

const double DBL_PRECISION_COMPARE = 1.e-12;
const double DBL_PRECISION_INVCDF = 1.e-4;

void check_lower_bound(URANIE::DataServer::TInfiniteDistribution& distribution,
    const std::unordered_map<string, double>& references) {

    CPPUNIT_ASSERT_DOUBLES_EQUAL_MESSAGE("getCDF",
        0., distribution.getCDF(distribution.getLowerBound()), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT(references.find("pmin") != references.end());
    CPPUNIT_ASSERT_DOUBLES_EQUAL_MESSAGE("getPMin",
        references.at("pmin"), distribution.getPMin(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT(references.find("x_min") != references.end());
    URANIE::DataServer::TInvGammaDistribution* test=dynamic_cast<URANIE::DataServer::TInvGammaDistribution *>(&distribution);
    if ( test == 0 )
        CPPUNIT_ASSERT_DOUBLES_EQUAL_MESSAGE("getInvCDF",
                                             references.at("x_min"), distribution.getInvCDF(0.), DBL_PRECISION_INVCDF);
    else
        CPPUNIT_ASSERT_DOUBLES_EQUAL_MESSAGE("getInvCDF",
                                             references.at("x_min"), distribution.getInvCDF(0.), 1e-2);

    CPPUNIT_ASSERT(references.find("x_min_prev") != references.end());
    CPPUNIT_ASSERT_DOUBLES_EQUAL_MESSAGE("getPDF",
        0., distribution.getPDF(references.at("x_min_prev")), DBL_PRECISION_COMPARE);
}

void check_upper_bound(URANIE::DataServer::TInfiniteDistribution& distribution,
    const std::unordered_map<string, double>& references) {

    CPPUNIT_ASSERT_DOUBLES_EQUAL_MESSAGE("getCDF",
        1., distribution.getCDF(distribution.getUpperBound()), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT(references.find("pmax") != references.end());
    CPPUNIT_ASSERT_DOUBLES_EQUAL_MESSAGE("getPMax",
        references.at("pmax"), distribution.getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT(references.find("x_max") != references.end());
    CPPUNIT_ASSERT_DOUBLES_EQUAL_MESSAGE("getInvCDF",
        references.at("x_max"), distribution.getInvCDF(1.), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT(references.find("x_max_next") != references.end());
    CPPUNIT_ASSERT_DOUBLES_EQUAL_MESSAGE("getPDF",
        0., distribution.getPDF(references.at("x_max_next")), DBL_PRECISION_COMPARE);
}

void test_truncation(URANIE::DataServer::TInfiniteDistribution& distribution,
    const std::unordered_map<string, double>& references) {
    if (distribution.isLowerBounded())
        check_lower_bound(distribution, references);
    if (distribution.isUpperBounded())
        check_upper_bound(distribution, references);
}

void TestTInfiniteDistribution::test_bounds(
    URANIE::DataServer::TInfiniteDistribution& distribution,
    const std::unordered_map<string, double>& references) {
    distribution.setBounds(references.at("x_min"), references.at("x_max"));
    test_truncation(distribution, references);
    distribution.setBounds(references.at("x_min"), references.at("x_max"));
    test_truncation(distribution, references);
}

void TestTInfiniteDistribution::test_lower_bound(
    URANIE::DataServer::TInfiniteDistribution& distribution,
    const std::unordered_map<string, double>& references) {
    std::unordered_map<string, double> ref_lbound;
    ref_lbound["x_min"] = references.at("x_min");
    ref_lbound["x_min_prev"] = references.at("x_min_prev");
    ref_lbound["pmin"] = references.at("pmin");

    distribution.setLowerBound(ref_lbound.at("x_min"));
    test_truncation(distribution, ref_lbound);
    distribution.setLowerBound(ref_lbound.at("x_min"));
    test_truncation(distribution, ref_lbound);
}

void TestTInfiniteDistribution::test_upper_bound(
    URANIE::DataServer::TInfiniteDistribution& distribution,
    const std::unordered_map<string, double>& references) {
    std::unordered_map<string, double> ref_ubound;
    if (references.find("x_min_default") != references.end()) {
        ref_ubound["x_min"] = references.at("x_min_default");
        ref_ubound["x_min_prev"] = references.at("x_min_default_prev");
        ref_ubound["pmin"] = references.at("pmin_default");
    }
    ref_ubound["x_max"] = references.at("x_max");
    ref_ubound["x_max_next"] = references.at("x_max_next");
    ref_ubound["pmax"] = references.at("pmax");

    distribution.setUpperBound(ref_ubound.at("x_max"));
    test_truncation(distribution, ref_ubound);
    distribution.setUpperBound(ref_ubound.at("x_max"));
    test_truncation(distribution, ref_ubound);
}
