//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTLogUniformDistribution.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TLogUniformDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.  
 \li Cedric Forgues 2010/11/17
 Test testLogUniformTruncation() on truncature was added.
 \li Cedric Forgues (C.S.) 2010/06/16
 Ajout des m�thodes DI49
 // *****************************************************************************

 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

#include "TestTLogUniformDistribution.h"
#include "TString.h"

#include <iostream>

TLogUniformDistributionTest::TLogUniformDistributionTest()
{
}

TLogUniformDistributionTest::~TLogUniformDistributionTest()
{
}

void TLogUniformDistributionTest::setUp()
{
    _att = new URANIE::DataServer::TAttribute("logUniform", 0.01, 10.0);
    _dproba = new URANIE::DataServer::TLogUniformDistribution("logUniform");
}

void TLogUniformDistributionTest::tearDown()
{
    delete _att;
    _att = NULL;
    delete _dproba;
    _dproba = NULL;
}

void TLogUniformDistributionTest::testDefaultLaw()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.01, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kLogUniform);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kLegendre);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.44620062473783, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.01, _dproba->getTheoreticalMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0720223021398428, _dproba->getPDF(2.01),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0289529654602168, _dproba->getPDF(5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(15.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.767732019140163, _dproba->getCDF(2.01),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.899656668112006, _dproba->getCDF(5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(15.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0199526231496888, _dproba->getInvCDF(0.10),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.158489319246111, _dproba->getInvCDF(0.40),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.316227766016838, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.630957344480193, _dproba->getInvCDF(0.60),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.01187233627272, _dproba->getInvCDF(0.90),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TLogUniformDistribution("loguniform", 0.01,
            10.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.01, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kLogUniform);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kLegendre);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.44620062473783, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.01, _dproba->getTheoreticalMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0720223021398428, _dproba->getPDF(2.01),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0289529654602168, _dproba->getPDF(5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(15.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.767732019140163, _dproba->getCDF(2.01),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.899656668112006, _dproba->getCDF(5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(15.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0199526231496888, _dproba->getInvCDF(0.10),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.158489319246111, _dproba->getInvCDF(0.40),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.316227766016838, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.630957344480193, _dproba->getInvCDF(0.60),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.01187233627272, _dproba->getInvCDF(0.90),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TLogUniformDistribution(_att);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.01, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.44620062473783, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.01, _dproba->getTheoreticalMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPMax(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kLegendre);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0720223021398428, _dproba->getPDF(2.01),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0289529654602168, _dproba->getPDF(5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(15.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.767732019140163, _dproba->getCDF(2.01),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.899656668112006, _dproba->getCDF(5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(15.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0199526231496888, _dproba->getInvCDF(0.10),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.158489319246111, _dproba->getInvCDF(0.40),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.316227766016838, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.630957344480193, _dproba->getInvCDF(0.60),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.01187233627272, _dproba->getInvCDF(0.90),
            DBL_PRECISION_COMPARE);
}

void TLogUniformDistributionTest::testLogUniformTruncation()
{
    _dproba->setBounds(2., 4.);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPMax(), DBL_PRECISION_COMPARE);
}

/////////////////////////////////////////////////////////////////
////////////////////      Exceptions         ////////////////////
/////////////////////////////////////////////////////////////////

void TLogUniformDistributionTest::throwExceptionWithNegativeLowerBound()
{
    URANIE::DataServer::TAttribute * att2 = new URANIE::DataServer::TAttribute(
            "logUniform", -1.0, 10.0);
    cout << endl;
    CPPUNIT_ASSERT_THROW( new URANIE::DataServer::TLogUniformDistribution("loguniform",-1.0,10.0);,URANIE::Exceptions::UErrorExceptions);
    cout << endl;
    CPPUNIT_ASSERT_THROW(new URANIE::DataServer::TLogUniformDistribution(att2),
            URANIE::Exceptions::UErrorExceptions);
    delete att2;
    att2 = NULL;
}

void TLogUniformDistributionTest::throwExceptionWithBadBoundaries()
{
    cout << endl;
CPPUNIT_ASSERT_THROW( new URANIE::DataServer::TLogUniformDistribution("loguniform",10.0, 0.01);,URANIE::Exceptions::UErrorExceptions);
}

