//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTMultinomialDistribution.h
 \author Nicolas Gilardi <nicolas.gilardi@cea.fr>
 \date   $Date$

 \brief  Fichier de test de la classe URANIE::DataServer::TMultinomialDistribution

 */
#ifndef CPPUNITEST_TMULTINOMIALDISTRIBUTION_H
#define CPPUNITEST_TMULTINOMIALDISTRIBUTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TMultinomialDistribution.h"
#include "UExceptions.h"

/*! \class TMultinomialDistributionTest
 * \brief Unit tests for TMultinomialDistribution.
 */
class TMultinomialDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TMultinomialDistributionTest);
    CPPUNIT_TEST (testNbValues);
    CPPUNIT_TEST (testValues);
    CPPUNIT_TEST (testFrequencies);
    CPPUNIT_TEST (testProba);
    CPPUNIT_TEST (testSetDistribution);
    CPPUNIT_TEST (testNewValues);
    CPPUNIT_TEST (testNewFrequencies);
    CPPUNIT_TEST (testGetPDF);
    CPPUNIT_TEST (testGetInvCDF);
    CPPUNIT_TEST (throwExceptionGetAllValues);
    CPPUNIT_TEST (throwExceptionGetValue);
    CPPUNIT_TEST (throwExceptionGetAllFrequencies);
    CPPUNIT_TEST (throwExceptionGetFrequency);
    CPPUNIT_TEST (throwExceptionGetProba);
    CPPUNIT_TEST (throwExceptionSetDistributionWithNegativeFrequency);
    CPPUNIT_TEST (throwExceptionSetValue);
    CPPUNIT_TEST (throwExceptionSetFrequency);
    CPPUNIT_TEST (throwExceptionComputeMean);
    CPPUNIT_TEST (throwExceptionGetCDF);CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TMultinomialDistributionTest object.
     */
    TMultinomialDistributionTest();

    /// Destructor.
    virtual ~TMultinomialDistributionTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void testNbValues();
    void testValues();
    void testFrequencies();
    void testProba();
    void testSetDistribution();
    void testNewValues();
    void testNewFrequencies();
    void testGetPDF();
    void testGetInvCDF();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Throws an exception when values vector of distribution is empty.
    void throwExceptionGetAllValues();

    //! Throws an exception when values vector of distribution is empty OR when given index is negative or superior to the max number of values.
    void throwExceptionGetValue();

    //! Throws an exception when the frequencies vector is empty.
    void throwExceptionGetAllFrequencies();

    //! Throws an exception when the frequencies vector is empty OR when given index is negative or superior to the max number of values.
    void throwExceptionGetFrequency();

    //! Throws an exception when the frequencies vector is empty OR when given index is negative or superior to the max number of values.
    void throwExceptionGetProba();

    //! Throws an exception when one or more given frequency is negative.
    void throwExceptionSetDistributionWithNegativeFrequency();

    //! Throws an exception when given index is negative or bigger than the number of values in distribution.
    void throwExceptionSetValue();

    //! Throws an exception when given index is negative or bigger than the number of values in distribution. 
    void throwExceptionSetFrequency();

    //! Throws a warning exception when using the computeMean() method.
    void throwExceptionComputeMean();

    //! Throws an exception when when using the computeMean() method.
    void throwExceptionGetCDF();
    //@}

private:
    URANIE::DataServer::TMultinomialDistribution *dproba;
};

#endif  // CPPUNITEST_TATTRIBUTE_H
