//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTPatternsEventList.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TPatternsEventList
 
 Cr�e le : mar avri 26 10:00:00 2010

 \par Modifications
 \li Cedric Forgues (C.S.) 2010/04/26
 Ajout des m�thodes
 // *****************************************************************************

 */

#include "TestTPatternsEventList.h"
#include "TString.h"

#include <iostream>
#include <ctime>

TPatternsEventListTest::TPatternsEventListTest()
{
}

TPatternsEventListTest::~TPatternsEventListTest()
{
}

void TPatternsEventListTest::setUp()
{
    _tpe = new URANIE::DataServer::TPatternsEventList("TPatternsEventList",
            "Title of TPatternsEventList", 0);
}

void TPatternsEventListTest::tearDown()
{
    delete _tpe;
    _tpe = NULL;
}

////////////////////////////////
///                          ///
///         Tests            ///
///                          ///
////////////////////////////////

void TPatternsEventListTest::testTPatternsEventListAttributes()
{
    CPPUNIT_ASSERT(!strcmp(_tpe->GetName(), TString("TPatternsEventList")));
    CPPUNIT_ASSERT(
            !strcmp(_tpe->GetTitle(), TString("Title of TPatternsEventList")));
    CPPUNIT_ASSERT_EQUAL(0, _tpe->getSeed());
    time_t secondesBefore;
    time(&secondesBefore);
    _tpe->setSeed();
    time_t secondesAfter;
    time(&secondesAfter);
    CPPUNIT_ASSERT(_tpe->getSeed() >= secondesBefore);
    CPPUNIT_ASSERT(_tpe->getSeed() <= secondesAfter);

    URANIE::DataServer::TPatternsEventList *tpe2 =
            new URANIE::DataServer::TPatternsEventList();
    CPPUNIT_ASSERT(!strcmp(tpe2->GetName(), TString("_pel_")));
    CPPUNIT_ASSERT(
            !strcmp(tpe2->GetTitle(), TString("Default TPatternsEventList")));
    CPPUNIT_ASSERT_EQUAL(0, tpe2->getSeed());
    CPPUNIT_ASSERT(!tpe2->getLog());
    tpe2->setLog();
    CPPUNIT_ASSERT(tpe2->getLog());
    tpe2->unsetLog();
    CPPUNIT_ASSERT(!tpe2->getLog());
    tpe2->changeLog();
    CPPUNIT_ASSERT(tpe2->getLog());
    delete tpe2;
    tpe2 = NULL;
}

void TPatternsEventListTest::testEnter()
{
    CPPUNIT_ASSERT_EQUAL(_tpe->GetSize(), 100); // For a specified size inferior to 100, size is set to 100
    _tpe->enter(4);
    _tpe->enter(2);
    _tpe->enter(10);
    _tpe->enter(4);
    _tpe->enter(9);
    CPPUNIT_ASSERT_EQUAL(5, _tpe->GetN());
    CPPUNIT_ASSERT_EQUAL((Int_t)(4), (Int_t)(_tpe->GetEntry(0)));
    CPPUNIT_ASSERT_EQUAL((Int_t)(2), (Int_t)(_tpe->GetEntry(1)));
    CPPUNIT_ASSERT_EQUAL((Int_t)(10), (Int_t)(_tpe->GetEntry(2)));
    CPPUNIT_ASSERT_EQUAL((Int_t)(4), (Int_t)(_tpe->GetEntry(3)));
    CPPUNIT_ASSERT_EQUAL((Int_t)(9), (Int_t)(_tpe->GetEntry(4)));

    Int_t nsize = 110;

    URANIE::DataServer::TPatternsEventList *tpe2 =
            new URANIE::DataServer::TPatternsEventList("tpe2",
                    "Title of TPatternsEventList", nsize);
    CPPUNIT_ASSERT_EQUAL(nsize, tpe2->GetSize()); // For a specified size strictly superior to 100, size is the specified one
    for (Int_t ii = 0; ii < 200; ii++)
        tpe2->enter(ii + 1);
    CPPUNIT_ASSERT_EQUAL(200, tpe2->GetN());
    CPPUNIT_ASSERT_EQUAL(2 * nsize, tpe2->GetSize());
    CPPUNIT_ASSERT_EQUAL((Int_t)(1), (Int_t)(tpe2->GetEntry(0)));
    CPPUNIT_ASSERT_EQUAL((Int_t)(200), (Int_t)(tpe2->GetEntry(199)));

    delete tpe2;
}

void TPatternsEventListTest::testMix()
{
    Int_t nseed = 123456;
    _tpe->setSeed(nseed);

    _tpe->enter(4);
    _tpe->enter(2);
    _tpe->enter(10);
    _tpe->enter(7);
    _tpe->enter(5);

    CPPUNIT_ASSERT_EQUAL(nseed, _tpe->getSeed());
    _tpe->mix();
    CPPUNIT_ASSERT_EQUAL(Int_t(10), Int_t(_tpe->GetEntry(0)));
    CPPUNIT_ASSERT_EQUAL(Int_t(5), Int_t(_tpe->GetEntry(1)));
    CPPUNIT_ASSERT_EQUAL(Int_t(7), Int_t(_tpe->GetEntry(2)));
    CPPUNIT_ASSERT_EQUAL(Int_t(2), Int_t(_tpe->GetEntry(3)));
    CPPUNIT_ASSERT_EQUAL(Int_t(4), Int_t(_tpe->GetEntry(4)));
    _tpe->mix();
    CPPUNIT_ASSERT_EQUAL(Int_t(2), Int_t(_tpe->GetEntry(0)));
    CPPUNIT_ASSERT_EQUAL(Int_t(10), Int_t(_tpe->GetEntry(1)));
    CPPUNIT_ASSERT_EQUAL(Int_t(4), Int_t(_tpe->GetEntry(2)));
    CPPUNIT_ASSERT_EQUAL(Int_t(7), Int_t(_tpe->GetEntry(3)));
    CPPUNIT_ASSERT_EQUAL(Int_t(5), Int_t(_tpe->GetEntry(4)));
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TPatternsEventListTest::throwExceptionMix()
{
    CPPUNIT_ASSERT_THROW(_tpe->mix(), URANIE::Exceptions::UErrorExceptions);
}
