//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTTokenizer.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TTokenizer.
 
 Cr�e le : jeu jun 16 10:00:00 2010

 // *****************************************************************************

 */

#define DBL_PRECISION_COMPARE 1.e-12
#include "TestTTokenizer.h"
#include "TString.h"

#include <iostream>

TTokenizerTest::TTokenizerTest()
{
}

TTokenizerTest::~TTokenizerTest()
{
}

void TTokenizerTest::setUp()
{
    token = new TTokenizer();

}

void TTokenizerTest::tearDown()
{
    delete token;
    token = NULL;
}

////////////////////////////////
///                          ///
///         Tests            ///
///                          ///
////////////////////////////////

void TTokenizerTest::testGetTokens()
{
    std::string stringtokens[3] =
    { "Element1", "Element2", "Element3" };
    std::vector < std::string > tokens = token->getTokens(
            "Element1 Element2\tElement3");
    Int_t i = 0;
    for (std::vector<std::string>::iterator it = tokens.begin();
            it != tokens.end(); it++)
    {
        CPPUNIT_ASSERT((*it) == stringtokens[i]);
        i++;
    }

    std::string stringtokens2[10] =
    { "#COLUMN_NAMES:", "rw", "r", "tu", "tl", "hu", "hl", "l", "kw", "ystar" };
    token->setDelimiters(" |-");
    tokens = token->getTokens(
            "#COLUMN_NAMES: rw| r- tu| tl- hu| hl- l| kw- ystar");
    i = 0;
    for (std::vector<std::string>::iterator it = tokens.begin();
            it != tokens.end(); it++)
    {
        CPPUNIT_ASSERT((*it) == stringtokens2[i]);
        i++;
    }
    token->setDelimiters(" |");
    tokens = token->getTokens(
            "#COLUMN_NAMES: rw| r| tu| tl| hu| hl| l| kw| ystar");
    i = 0;
    for (std::vector<std::string>::iterator it = tokens.begin();
            it != tokens.end(); it++)
    {
        CPPUNIT_ASSERT((*it) == stringtokens2[i]);
        i++;
    }
}

void TTokenizerTest::testSetComment()
{
    std::string stringtokens[12] =
    { "#NAME:", "flowrateborehole", "#COLUMN_NAMES:", "rw", "r", "tu", "tl",
            "hu", "hl", "l", "kw", "ystar" };
    token->setComment(47);
    std::vector < std::string > tokens =
            token->getTokens(
                    "#NAME: flowrateborehole // Ent�te NAME\n// Commentaire inutile\n#COLUMN_NAMES: rw| r| tu| tl| hu| hl| l| kw | ystar\n");
    Int_t i = 0;
    for (std::vector<std::string>::iterator it = tokens.begin();
            it != tokens.end(); it++)
    {
        CPPUNIT_ASSERT((*it) == stringtokens[i]);
        i++;
    }
}

void TTokenizerTest::testSetDelimiters()
{
    CPPUNIT_ASSERT(token->isDelimiter(32)); //32: code ASCII de " "
    CPPUNIT_ASSERT(token->isDelimiter(44)); //44: code ASCII de ","
    CPPUNIT_ASSERT(token->isDelimiter(9)); //9 : code ASCII de "\t"
    CPPUNIT_ASSERT(token->isDelimiter(13)); //13: code ASCII de "\n"
    CPPUNIT_ASSERT(token->isDelimiter(10)); //10: code ASCII de "\r"

    // CPPUNIT_ASSERT( token->isDelimiter(" ,") );//CPPUNIT_ASSERT( token->isDelimiter(" ,") );
    // CPPUNIT_ASSERT( !token->isDelimiter(",|") );//CPPUNIT_ASSERT( !token->isDelimiter(",|") );
    token->setDelimiters("|");
    CPPUNIT_ASSERT(token->isDelimiter(124));  //124: code ASCII de "|"
    //CPPUNIT_ASSERT( !token->isDelimiter(" ,\t\n\r") );//CPPUNIT_ASSERT( !token->isDelimiter(" ,\t\n\r") );
}

void TTokenizerTest::testIsSetQuotes()
{
    //CPPUNIT_ASSERT( token->isQuote("\"") );
    //CPPUNIT_ASSERT( token->isQuote("\'") );

    token->setQuotes("+");
    CPPUNIT_ASSERT(token->isQuote(43));  //43: code ASCII de "+"
    //CPPUNIT_ASSERT( !token->isQuote("\"") );
    //CPPUNIT_ASSERT( !token->isQuote("\'") );
}

