//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTTrapeziumDistribution.h
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TTrapeziumDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 // *****************************************************************************

 */
#ifndef CPPUNITEST_TTRAPEZIUMDISTRIBUTION_H
#define CPPUNITEST_TTRAPEZIUMDISTRIBUTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TTrapeziumDistribution.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TTrapeziumDistributionTest
 * \brief Unit tests for TTrapeziumDistribution.
 */
class TTrapeziumDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TTrapeziumDistributionTest);
    CPPUNIT_TEST (testDefaultLaw);
    CPPUNIT_TEST (throwExceptionWithBadBoundaries);
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TTrapeziumDistributionTest object.
     */
    TTrapeziumDistributionTest();

    /// Destructor.
    virtual ~TTrapeziumDistributionTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests the default parameters for trapezium distribution.
    void testDefaultLaw();

    //! Throws an exception when upper bound is inferior to lower bound.
    void throwExceptionWithBadBoundaries();

    //@}

private:
    URANIE::DataServer::TTrapeziumDistribution *_dproba;
    URANIE::DataServer::TAttribute * _att;
};
#endif  // CPPUNITEST_TTRAPEZIUMDISTRIBUTION_H
