//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TWeibullDistribution.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TWeibullDistribution

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.  
 Created: 2009-12-08 09:14:10
 
 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTInfiniteDistribution.h"
#include "TestTWeibullDistribution.h"

TWeibullDistributionTest::TWeibullDistributionTest()
{
}

TWeibullDistributionTest::~TWeibullDistributionTest()
{
}

void TWeibullDistributionTest::setUp()
{
    _dproba = new URANIE::DataServer::TWeibullDistribution("weibull");
}

void TWeibullDistributionTest::tearDown()
{
    delete _dproba;
    _dproba = NULL;
}

////////////////////////////////
///                          ///                            
///         Tests            ///
///                          ///
////////////////////////////////

void TWeibullDistributionTest::testDefaultLaw()
{

    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kWeibull);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_THROW(_dproba->getTheoreticalMode(), URANIE::Exceptions::UWarningExceptions);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterK(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.606530659712633, _dproba->getPDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.367879441171442, _dproba->getPDF(1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.22313016014843, _dproba->getPDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.393469340287367, _dproba->getCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.632120558828558, _dproba->getCDF(1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.77686983985157, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.105360515657826, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.510825623765991, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.693147180559945, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.916290731874155, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.30258509299405, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TWeibullDistribution("weibull", 0.5);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kWeibull);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_THROW(_dproba->getTheoreticalMode(), URANIE::Exceptions::UWarningExceptions);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterK(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0359059298790057, _dproba->getPDF(2.01),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.07998595249697e-05, _dproba->getPDF(5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.67881148442009e-14, _dproba->getPDF(15.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.982047035060497, _dproba->getCDF(2.01),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999954600070238, _dproba->getCDF(5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999999999999906, _dproba->getCDF(15.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0526802578289131, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.255412811882995, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.346573590279973, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.458145365937077, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.15129254649702, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    _dproba = new URANIE::DataServer::TWeibullDistribution(att, 0.5);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kWeibull);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_THROW(_dproba->getTheoreticalMode(), URANIE::Exceptions::UWarningExceptions);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterK(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0359059298790057, _dproba->getPDF(2.01),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.07998595249697e-05, _dproba->getPDF(5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.67881148442009e-14, _dproba->getPDF(15.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.982047035060497, _dproba->getCDF(2.01),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999954600070238, _dproba->getCDF(5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999999999999906, _dproba->getCDF(15.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0526802578289131, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.255412811882995, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.346573590279973, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.458145365937077, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.15129254649702, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete att;
    att = NULL;
}

void TWeibullDistributionTest::testOtherLaw()
{
  delete _dproba;
    _dproba = new URANIE::DataServer::TWeibullDistribution("weibull", 3.0, 2.0,
            -2.0);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kWeibull);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.658680776358262,
            _dproba->getTheoreticalMean(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.38975412552831,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.121320343559643,
            _dproba->getTheoreticalMode(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3., _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterK(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.284969061524424, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.150234058138725, _dproba->getPDF(2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00672037251525743, _dproba->getPDF(5.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.358819611570045, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.830986684593934, _dproba->getCDF(2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.995679760525906, _dproba->getCDF(5.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getParameterXmin(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.0262214620765, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.144161984061353, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.497663833473093, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.871692286242973, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.55228138815544, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    _dproba = new URANIE::DataServer::TWeibullDistribution(att, 3.0, 2.0, -2.0);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kWeibull);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.658680776358262,
            _dproba->getTheoreticalMean(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.38975412552831,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.121320343559643,
            _dproba->getTheoreticalMode(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3., _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterK(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.284969061524424, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.150234058138725, _dproba->getPDF(2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00672037251525743, _dproba->getPDF(5.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.358819611570045, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.830986684593934, _dproba->getCDF(2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.995679760525906, _dproba->getCDF(5.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.0262214620765, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.144161984061353, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.497663833473093, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.871692286242973, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.55228138815544, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete att;
    att = NULL;
}

static void truncation_set(std::unordered_map<string, double>& references) {
    references["scale"] = 1.;
    references["shape"] = 2.;

    references["x_min_default"] = -0.1;
    references["x_min_default_prev"] = nextafter(references.at("x_min_default"), -HUGE_VAL);
    references["x_min"] = 0.2;
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = 1.8;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);

    // scipy.stats.weibull_min.cdf()
    references["pmin_default"] = 0.;
    references["pmin"] = 8.6068814728771842e-02;
    references["pmax"] = 9.7294815313364957e-01;
}

void TWeibullDistributionTest::testTruncationBounds(){
    std::unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TWeibullDistribution distribution("weibull", references.at("scale"),
        references.at("shape"), references.at("x_min_default"));
    TestTInfiniteDistribution::test_bounds(distribution, references);
}
void TWeibullDistributionTest::testTruncationLowerBound(){
    std::unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TWeibullDistribution distribution("weibull", references.at("scale"),
        references.at("shape"), references.at("x_min_default"));
    TestTInfiniteDistribution::test_lower_bound(distribution, references);
}
void TWeibullDistributionTest::testTruncationUpperBound(){
    std::unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TWeibullDistribution distribution("weibull", references.at("scale"),
        references.at("shape"), references.at("x_min_default"));
    TestTInfiniteDistribution::test_upper_bound(distribution, references);
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TWeibullDistributionTest::throwExceptionWithNegativeLambda()
{
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TWeibullDistribution("weibull", -1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TWeibullDistribution(att, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TWeibullDistributionTest::throwExceptionWithNegativeK()
{
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TWeibullDistribution("weibull", 1.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TWeibullDistribution(att, 1.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TWeibullDistributionTest::throwExceptionSetParametersWithNegativeLambda()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(-1.0, 2.0, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TWeibullDistributionTest::throwExceptionSetParametersWithNegativeK()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(1.0, -2.0, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TWeibullDistributionTest::throwExceptionSetParameterLambdaWithNegativeLambda()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterLambda(-1.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TWeibullDistributionTest::throwExceptionSetParameterKWithNegativeK()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterK(-1.0),
            URANIE::Exceptions::UErrorExceptions);
}

