//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TInputFile.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mon oct 9th 14:50:04 CEST 2006
 
 \brief Implementation of the class URANIE::DLauncher::TInputFile

 \par Modifications
 \li F. Gaudier (2010-12-14 11:38:09)
 - resolve the DI111 and DI113

 
 */
#ifndef WIN32
//  #include <linux/limits.h>
#endif
#ifdef OPEN_MAX
#undef OPEN_MAX
#define OPEN_MAX         1000      /* # open files a process may have */
#else
#define OPEN_MAX         1000      /* # open files a process may have */
#endif

// URANIE
#include "TInputFile.h"

// ROOT
#include "TRegexp.h"
#include "TSystem.h"

#define MAX_LENGTH 256
int bdebug = 0;

//static int ncrt = 0;

ClassImp (URANIE::Launcher::TInputFile)

namespace URANIE
{
namespace Launcher
{

TInputFile::TInputFile(TString str) :
    TCodeFile(str)
{
    if (_blog)
        cout << "constructing TInputFile file[" << str << "]" << endl;
}
TInputFile::~TInputFile()
{
    if (_blog)
        cout << " ** deletes TInputFile [" << getFileName() << "]" << endl;
}

Bool_t TInputFile::isExist()
{
    if (_blog)
        cout << " ** Method [TInputFile::" << __func__  << "] for file[" << getFileName() << "]" << endl;

    return !gSystem->AccessPathName(getFileName().Data(), kFileExists );
}

void TInputFile::addAttribute(URANIE::DataServer::TAttribute* att)
{
    try
    {
	if ((att->getKeyList().size() == 0))
	    throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
						       "TInputFile::addAttribute the attribute has keys or positions (lines and fields) not defined");
    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
	ue.printMessage();
	throw ue;
    }

    if( find(_attributes.begin(), _attributes.end(), att) == _attributes.end() ) 
    {
	searchIndexeAttribute(att);
	TCodeFile::addAttribute(att);
    }
}

void TInputFile::init()
{
    if (_blog)
        cout << "URANIE::Launcher::init()" << endl;
}

void TInputFile::terminate()
{
    if (_blog)
        cout << "URANIE::Launcher::terminate()" << endl;
}

void TInputFile::printLog(Option_t *option)
{
    cout << "TInputFile::printLog[" << option << "]" << endl;
    cout << "_sFileName[" << _sFileName << "]" << endl;
    cout << "_cComments[" << _cComments << "]" << endl;
    cout << "_cSeparators[" << _cSeparators << "]" << endl;
    Int_t nAttribute = (Int_t)(_attributes.size());
    for (Int_t indx = 0; indx < nAttribute; indx++)
    {
        cout << "index[" << indx + 1 << "/" << nAttribute << "]" << endl;
        _attributes[indx]->printLog();
    }
}

} // Fin du namespace Launcher
} // Fin du namespace URANIE

