//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TLocalHostCluster.cxx
 \author gilles arnaud <gilles.arnaud@cea.fr>
 \date   mer. mai  5 13:33:10 CEST 2010
 
 \brief Implementation of the class URANIE::Launcher::TLocalHostCluster

 \par Modifications

 \li C. Forgues (2011-01-06 19:02:59)
 - Getenv("HOSTNAME") is replaced by HostName().
 */

#include "TLocalHostCluster.h"

#include "TSystem.h"

ClassImp (URANIE::Launcher::TLocalHostCluster)

namespace URANIE
{
namespace Launcher
{
// class method
TRegexp TLocalHostCluster::__Cre("localhost=[0-9]+");
int TLocalHostCluster::__Cnnode = 1;
TString TLocalHostCluster::__Chost = "";

bool TLocalHostCluster::eligible(Option_t *opt)
{
    TString soption = opt;
    soption.ToLower();
    soption.ReplaceAll(" ", "");
    TString sopt = soption(__Cre);
    if (sopt[0])
    {
        TString nopt = sopt.Remove(0, 10);
        __Cnnode = nopt.Atoi();
        cout << " ** nproc [" << __Cnnode << "]" << endl;
        __Chost = gSystem->HostName();
    }
    return sopt[0];
}

// Constructor
TLocalHostCluster::TLocalHostCluster() :
    TCluster("TLocalHostCluster", __Chost, __Cnnode)
{
    if (_blog)
        cout << " ** Constructor TLocalHostCluster::TLocalHostCluster ["
                << GetName() << "] Title[" << GetTitle() << "]" << endl;
    _bMultiDump=true;
    init();
}
TLocalHostCluster::~TLocalHostCluster()
{
    if (_blog)
        cout << " ** Destroy TLocalHostCluster [" << GetName() << "] Title["
                << GetTitle() << "]" << endl;
}

///////////////////////////////////////////////////////////
//  Construct and return the list of node
///////////////////////////////////////////////////////////
void TLocalHostCluster::init()
{
    TString host = GetTitle();

    if (_blog)
        cout << " ** TLocalHostCluster::getListOfNodes Node[" << host << "]"
                << endl;

    addNodeName(host.Data());

    TCluster::getListOfNodes(); // ??

}

void TLocalHostCluster::getCommandPrefix(char *mpiexe, char *procs)
{
    (void)(procs);
    if( _bMultiDump)
        cout << " ** LocalHostCluster : do not need mpirun" << endl;
    strcpy(mpiexe, "");
}

///////////////////////////////////////////////////////////
//  Log
///////////////////////////////////////////////////////////
void TLocalHostCluster::printLog(Option_t *option)
{
    cout << "TLocalHostCluster::printLog[" << option << "]" << endl;
    TCluster::printLog(option);
}

} // Fin du namespace Launcher
} // Fin du namespace URANIE
