//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
// TLocalServer lun aoû 4 14:19:19 CEST 2008
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TLocalServer.cxx
 \author gilles arnaud <gilles.arnaud@cea.fr>
 \date   mer. mai  5 13:33:10 CEST 2010
 
 \brief Implementation of the class URANIE::Launcher::TLocalServer

 \par Modifications
 \li C. Forgues (2011-01-06 19:02:59)
 - Getenv("HOSTNAME") is replaced by HostName().
 
 */

#include "TLocalServer.h"

#include "TSystem.h"

ClassImp (URANIE::Launcher::TLocalServer)

namespace URANIE
{
namespace Launcher
{
// class method
TRegexp TLocalServer::__Cre("local=[0-9]+");
int TLocalServer::__Cnnode = 1;
TString TLocalServer::__Chost = "";

bool TLocalServer::eligible(Option_t *opt)
{
    TString soption = opt;
    // soption.ToLower();
    // soption.ReplaceAll(" ","");
    TString sopt = soption(__Cre);
    if (sopt[0])
    {
        TString nopt = sopt.Remove(0, 6);
        __Cnnode = nopt.Atoi();
        __Chost = gSystem->HostName();
    }
    return sopt[0];
}

// Constructor
TLocalServer::TLocalServer() :
        TCluster("LocalServer", __Chost, __Cnnode)
{
    if (_blog)
        cout << " ** Constructor TLocalServer::TLocalServer [" << GetName()
                << "] Title[" << GetTitle() << "]" << endl;
    init();
}
TLocalServer::~TLocalServer()
{
    if (_blog)
        cout << " ** Destroy TLocalServer [" << GetName() << "] Title["
                << GetTitle() << "]" << endl;
}

///////////////////////////////////////////////////////////
//  Construct and return the list of node
///////////////////////////////////////////////////////////
void TLocalServer::init()
{
    TString host = GetTitle();

    if (_blog)
        cout << " ** TLocalServeur::getListOfNodes Node[" << host << "]"
                << endl;

    addNodeName(host.Data());

    TCluster::getListOfNodes(); // ??

}

void TLocalServer::getCommandPrefix(char *mpiexe, char *procs)
{
    cout << " ** TLocalServer : do not need mpirun" << endl;
    strcpy(mpiexe, "");
}

///////////////////////////////////////////////////////////
//  Log
///////////////////////////////////////////////////////////
void TLocalServer::printLog(Option_t *option)
{
    cout << "TLocalServer::printLog[" << option << "]" << endl;
    TCluster::printLog(option);
}

} // Fin du namespace Launcher
} // Fin du namespace URANIE
