//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TDATASERVER_H
#define CPPUNITEST_TDATASERVER_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TLauncher.h"
#include "TLauncherByStep.h"
#include "TDataServer.h"
#include "UExceptions.h"
#include "TCode.h"
#include "TInputFileRecreate.h"
#include "TInputFileKey.h"
// #ifdef WITH_CLUB
#include "TOutputFileColumn.h"
#include "TOutputFileRow.h"
#include "TOutputFileKey.h"
// #endif

#ifdef WITH_FMI
#include "TCodeFMU.h"
#include "TOutputFileFMU.h"
#endif

#include "TOutputFileDataServer.h"
#include "TOutputFileXML.h"

#ifdef WITH_LIBXSLT
#include "TInputFileXML.h"
#endif

#include "TCodeFile.h"

#ifndef OLDCLUB
#include "TamponTexte.h"
#else
#include "club/TamponTexte.h"
#endif

class TLauncherTest: public CPPUNIT_NS::TestFixture{
    CPPUNIT_TEST_SUITE (TLauncherTest);

    CPPUNIT_TEST (testNames);
    CPPUNIT_TEST (testTitles);
    CPPUNIT_TEST (testWorkingdirectory);
    CPPUNIT_TEST (testRunDefault);
    CPPUNIT_TEST (testRunDefaultMultiInputKK);
    CPPUNIT_TEST (testRunDefaultMultiInputKF);
    CPPUNIT_TEST (testRunDefaultMultiInputFF);
    CPPUNIT_TEST (testRunDefaultMultiInputFollowingKK);
    CPPUNIT_TEST (testRunDefaultMultiInputFollowingKF);
    CPPUNIT_TEST (testRunDefaultMultiInputFollowingFF);
    CPPUNIT_TEST (testNumberOfCore);
    CPPUNIT_TEST (testAllocatedSeconds);
    CPPUNIT_TEST (testClusterName);
    CPPUNIT_TEST (testProjectName);
    CPPUNIT_TEST (testRunByStepCurie);
    CPPUNIT_TEST (testRunDefaultMultithread);
    // CPPUNIT_TEST (testRunFlowrateMultithreadAbove4000); Resource consuming. Should we have Dedicated new release tests ?
    CPPUNIT_TEST (testInputFileKeyRecreate);
    CPPUNIT_TEST (testInputFileFlag);
#ifdef WITH_LIBXSLT
    CPPUNIT_TEST(testInputFileXML_WithoutLIBXML2 );
#endif

#ifdef WITH_LIBXML2
#ifdef WITH_LIBXSLT
    CPPUNIT_TEST(testInputFileXML );
    CPPUNIT_TEST(testInputFileXMLLauncherFailure );
    CPPUNIT_TEST(testOutputFileXML_yhat );
    CPPUNIT_TEST(testOutputFileXML_d );
    CPPUNIT_TEST(testOutputFileXML_yhat_d );
    CPPUNIT_TEST(testOutputFileXML_d_yhat );
    CPPUNIT_TEST(testOutputFileDataServer_yhat_XML_d );
    CPPUNIT_TEST( throwExceptionOutputFileXMLNotFound );
    CPPUNIT_TEST( throwExceptionOutputFileXMLXPathNotFound );
#endif
    CPPUNIT_TEST( testOutputFileXML_yhat_d_with_keys_input );
    CPPUNIT_TEST_EXCEPTION( throwExceptionTwoOutputFilesForOneAttribute, URANIE::Exceptions::UExceptions );
    CPPUNIT_TEST_EXCEPTION( throwExceptionOutputFileTypeUnauthorized, URANIE::Exceptions::UExceptions );
#endif

    CPPUNIT_TEST (testNewKey);
    CPPUNIT_TEST (testNewLine);
    CPPUNIT_TEST (testNewColumn);
    CPPUNIT_TEST (testSearchIndexAttributeRecreate);

    CPPUNIT_TEST (testLauncherOutputFileKey_yhat_Relative);
    CPPUNIT_TEST (testLauncherInputOutputKey_WithBlank);
    CPPUNIT_TEST (testLauncherOutputFileKey_yhat);
    CPPUNIT_TEST (testLauncherOutputFileKey_d);
    CPPUNIT_TEST (testLauncherOutputFileKey_yhat_d);
    CPPUNIT_TEST (testLauncherOutputFileKey_d_yhat);
    CPPUNIT_TEST (testLauncherOutputFileKey_d_yhat_zWithDefaultValue);
    CPPUNIT_TEST (testLauncherOutputFileKey_d_yhat_zWithoutDefaultValue);
    CPPUNIT_TEST (testLauncherSeveralOutputFiles);
    CPPUNIT_TEST (testLauncherInputRowRecreateOutputFileDataServer);
    CPPUNIT_TEST (testLauncherHighNbSamplesWithoutSave);
    CPPUNIT_TEST (testLauncherHighNbSamplesWithSave1);
    CPPUNIT_TEST (testLauncherHighNbSamplesWithSave2);
    CPPUNIT_TEST (testLauncherHighNbSamplesWithSave3);
    CPPUNIT_TEST (testLauncherWithNotModifiedInputFiles);
    CPPUNIT_TEST (testLauncherAttributeWithSeveralInputFiles);
    CPPUNIT_TEST (testLauncherWithNoSpecifiedWorkingDirectory);
    CPPUNIT_TEST (testLauncherWithCurrentDirectorySpecifiedAsWorkingDirectory);
    CPPUNIT_TEST (testLauncherFailure);
    CPPUNIT_TEST (testlaunchCodeAttributeNameDifferentKey);

#ifdef WITH_LEVELE
    CPPUNIT_TEST(testLauncherLevelEOutputFileDataServer);
    CPPUNIT_TEST(testLauncherLevelEOutputFileColumn);
    CPPUNIT_TEST(testLauncherLevelEOutputFileRow);
    CPPUNIT_TEST(testLauncherLevelEOutputFileRowMultithread);
    CPPUNIT_TEST(testLauncherLevelEOutputFileKeys);
#ifdef WITH_LIBXML2
    CPPUNIT_TEST(testLauncherLevelEOutputFileXML);
#endif
    CPPUNIT_TEST(testLauncherLevelEOutputFileColumnByStep);
#endif

#ifdef WITH_FMI
    CPPUNIT_TEST(teFMIv1BoundingBall);
    CPPUNIT_TEST(teFMIv2BoundingBall);
#endif

#ifndef OLDCLUB
    CPPUNIT_TEST(testLauncherMultiTypeOutputFileDataServer);
    CPPUNIT_TEST(testLauncherMultiTypeOutputFileColumn);
#ifdef WITH_LIBXML2
    CPPUNIT_TEST(testLauncherMultiTypeOutputFileXML);
#endif
    CPPUNIT_TEST(testLauncherMultiTypeOutputFileRow);
    CPPUNIT_TEST(testLauncherMultiTypeOutputFileRowMultithread);
    CPPUNIT_TEST(testLauncherMultiTypeOutputFileRowMultithreadWithInputs);
    CPPUNIT_TEST(testLauncherMultiTypeOutputFileRowBySteps);
    CPPUNIT_TEST(testLauncherMultiTypeOutputFileKey);
    CPPUNIT_TEST(testLauncherMultiTypeOutputFileKeyCondensate);
    CPPUNIT_TEST(testLauncherMultiTypeInputFileDataServer);
    CPPUNIT_TEST(testLauncherMultiTypeInputFileRow);
#ifdef WITH_LIBXML2
#ifdef WITH_LIBXSLT
    CPPUNIT_TEST(testLauncherMultiTypeInputFileXML);
#endif
#endif
    CPPUNIT_TEST(testLauncherMultiTypeInputFileRowMultithread);
    CPPUNIT_TEST(testLauncherMultiTypeInputFileRowMultithreadWithInputs);
    CPPUNIT_TEST(testLauncherMultiTypeInputFileRowBySteps);
    CPPUNIT_TEST(testLauncherMultiTypeInputFileColumn);
    CPPUNIT_TEST(testLauncherMultiTypeInputFileKey);
    
    CPPUNIT_TEST(testLauncherFormattingMultiTypeInputFile);
#endif

    CPPUNIT_TEST (throwExceptionCodeAttributeWithSeveralTOutputFiles);
    CPPUNIT_TEST (throwExceptionCodeAttributeWithDifferentNames);
    CPPUNIT_TEST (throwExceptionTCodeWithoutOutputFile);
    CPPUNIT_TEST (throwExceptionCodeFileSetPointSeparator);
    CPPUNIT_TEST (throwExceptionInputFileNotFound);
    CPPUNIT_TEST (throwExceptionInputAndOutputAttributeWithSameName);

    CPPUNIT_TEST (testRunDefaultWithDrawings);
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TLauncherTest();

    //! Default destructor
    virtual ~TLauncherTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
// #ifdef WITH_CLUB
    //! Sets up Dataserver1 with TinputFileKey as input files
    void setUp1(bool nodata=false);
// #endif
// #ifdef WITH_CLUB
    //! Sets up Dataserver2 with InputfileRecreate/kNewKey as input files and TOutputFileRow 
    void setUp2();
// #endif

    //! Sets up Dataserver3 with TinputFileKey as input files
// #ifdef WITH_CLUB
    void setUp3();
// #endif

    //! Sets up Dataserver4 with XML input
#ifdef WITH_LIBXML2
// #ifdef WITH_LIBXSLT
    void setUp4();
// #endif
#endif
    
    //! Sets up Dataserver with several input file for flowrate
    void setUpMultiInput(TString type);
    
    //! the clean tempory directories
    void cleanWorkingDirectories(const char *option = "without");
    void cleanTestDirectories();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
	The list of unit test
    **/
    //@{ 
    //! Test of the names
    void testNames();

    //! Test of the titles
    /*!
      The title given by the constructor, the default title, and the title given with the setter method
      \em setTitle.
    */
    void testTitles();
    //@}

// #ifdef WITH_CLUB
    //---------------------------------------------
    /** @name Unit Tests of run
	The list of unit test of the "run" method
    **/
    //@{
    //! Tests if after a launch of a code and when working directory is not current directory, the directory UranieLauncher_1 doesn't exist.
    void testWorkingdirectory();

    //! Tests the run command with defaut argument
    void testRunDefault();

    //! Tests the run command with defaut argument multiinputfile kk
    void testRunDefaultMultiInputKK();

    //! Tests the run command with defaut argument multiinputfile kf
    void testRunDefaultMultiInputKF();
    
    //! Tests the run command with defaut argument multiinputfile ff
    void testRunDefaultMultiInputFF();

    //! Tests the run command with defaut argument multiinputfile kk
    void testRunDefaultMultiInputFollowingKK();

    //! Tests the run command with defaut argument multiinputfile kf
    void testRunDefaultMultiInputFollowingKF();
    
    //! Tests the run command with defaut argument multiinputfile ff
    void testRunDefaultMultiInputFollowingFF();

    //! Tests the run command with defaut argument
    void testRunDefaultWithDrawings();

    //! Test Number of Core    
    void testNumberOfCore();

    //! Test Allocated seconds
    void testAllocatedSeconds();

    //! Test Set Cluster Name
    void testClusterName();

    //! Test Set Project Name
    void testProjectName();

    //! Basic test for the irene TGCC machine
    void testRunByStepCurie();

    //! Tests the run command with defaut argument and multithreaded
    void testRunDefaultMultithread();
    void testRunFlowrateMultithreadAbove4000();
    //@}
// #endif

    //---------------------------------------------
    /** @name Unit Tests of run
	The list of unit test for input file creation
    **/
    //@{
// #ifdef WITH_CLUB
    //! Tests the run command with an inputfilerecreate file
    void testInputFileKeyRecreate();

    //! Tests run command with an inputfileflag file
    void testInputFileFlag();
// #endif

#ifdef WITH_LIBXSLT
    //! Test an inputfileXML when the libXml2 library is not found
    void testInputFileXML_WithoutLIBXML2();

#endif

#ifdef WITH_LIBXML2
#ifdef WITH_LIBXSLT
    //! Tests run command with an inputfileXML file
    void testInputFileXML();

    //! Tests run command with an inputfileXML file and code failure
    void testInputFileXMLLauncherFailure();
	
    //! Tests XML output for Yhat value
    void testOutputFileXML_yhat();

    //! Tests XML output for D value
    void testOutputFileXML_d();

    //! Tests XML output for Yhat and then D values
    void testOutputFileXML_yhat_d();
	
    //! Tests XML output for D and then Yhat values
    void testOutputFileXML_d_yhat();	
	
    //! Tests XML output for D and DataServer for Yhat
    void testOutputFileDataServer_yhat_XML_d();

    //! Test exception throwing if file not found
    void throwExceptionOutputFileXMLNotFound();

    //! Test exception throwing if XPath not found in file
    void throwExceptionOutputFileXMLXPathNotFound();	
#endif
#endif

#ifdef WITH_LIBXML2
    //! Tests XML output for Yhat and then D values with key=value inputs
    void testOutputFileXML_yhat_d_with_keys_input();
	
    //! Test exception throwing if output type not XML type
    void throwExceptionOutputFileTypeUnauthorized();

    //! Test exception throwing if two files defined for one single attribute
    void throwExceptionTwoOutputFilesForOneAttribute();

#endif

    //! Tests the good behaviour of TInpuFileRecreate::substitue for a kNewKey file.
    void testNewKey();

    //! Tests the good behaviour of TInpuFileRecreate::substitue for a kNewLine file.
    void testNewLine();

    //! Tests the good behaviour of TInpuFileRecreate::substitue for a kNewColumn file.
    void testNewColumn();

    //! Tests that the SearchIndexAttribute of the TInputfileRecreate class doesn't modify the attribute.
    void testSearchIndexAttributeRecreate();

    //@}

    //---------------------------------------------
    /** @name Unitary Tests of TLauncher when the output file is a "DataServer" format
     **/
    //@{
    //! input file : TinputFileRecreate(kRow) and output : OutputFileDataServer
    void testLauncherInputRowRecreateOutputFileDataServer();

    //@}

// #ifdef WITH_CLUB
    //---------------------------------------------
    /** @name Unitary Tests of TLauncher when the output file is a "key" format
     
	Tests the TOuputFileKey classe with lanching the flowrate code on the uniform design (32 patterns)
	We use the \em flowrateUniformDesign.dat dataset with the ouput file \em _output_flowrate_withKey_.dat.
	\code
	yhat = 9.750797e+01;
	d = 1.751812e+06;
	\endcode
	We test the following cases:
	\li when only one attribute was request to store (first and second attribute)
	\li when the output attributes are in the same order that in the output file
	\li when the output attributes are not in the same order that in the output file
	\li when requested one attributes that did not exist in the output file but contains a default value
	\li when requested one attributes that did not exist in the output file and dit not contains a default value

    **/
    //@{
    //! Test to run key files with blnk in names
    void testLauncherInputOutputKey_WithBlank();
    //! Tests that running succeeds with the input file is relative and the output file is of "key" format and only yhat
    void testLauncherOutputFileKey_yhat_Relative();
    //! Tests that running succeeds with the output file is of "key" format and only yhat
    void testLauncherOutputFileKey_yhat();
    //! Tests that running succeeds with the output file is of "key" format and only d
    void testLauncherOutputFileKey_d();
    //! Tests that running succeeds with the output file is of "key" format and yhat firstly and d
    void testLauncherOutputFileKey_yhat_d();
    //! Tests that running succeeds with the output file is of "key" format and d firstly and yhat 
    void testLauncherOutputFileKey_d_yhat();
    //! Tests that running succeeds with the output file is of "key" formats for d and yhat and z not present but with a default value
    void testLauncherOutputFileKey_d_yhat_zWithDefaultValue();
    //! Tests that running succeeds with the output file is of "key" formats for d and yhat and z not present and without a default value
    void testLauncherOutputFileKey_d_yhat_zWithoutDefaultValue();
    //! Test that runnin succeeds when several output files are treated
    void testLauncherSeveralOutputFiles();
    //@}
// #endif

// #ifdef WITH_CLUB
    //---------------------------------------------
    /** @name Unitary Tests of TLauncher with number of samples > 1024
	The list of unitary tests of TLauncher with number of samples > 1024
    **/
    //@{
    //! Tests that running succeeds with a high number of samples without save
    void testLauncherHighNbSamplesWithoutSave();

    //! Tests that running succeeds without specifiying any number of saves
    void testLauncherHighNbSamplesWithSave1();

    //! Tests that running succeeds with specifying a number of saves inferior to the maximum number of patterns.
    void testLauncherHighNbSamplesWithSave2();

    //! Tests that running succeeds with the maximum number of available saves.
    void testLauncherHighNbSamplesWithSave3();

    //! Tests when one put an input file which is not modified
    void testLauncherWithNotModifiedInputFiles();

    //! Tests when an attribute have several input files
    void testLauncherAttributeWithSeveralInputFiles();
    //@}
// #endif

// #ifdef WITH_CLUB
    //---------------------------------------------
    /** @name Unitary Tests of TLauncher on good behaviour
	The list of unitary tests of TLauncher allowing to test methods existing to avoid oversight's user 
    **/
    //@{  
    //! Tests that a URANIE directory is created to be the new working directory when the user didn't specify it.
    void testLauncherWithNoSpecifiedWorkingDirectory();
    //! Tests that a URANIE directory is created to be the new working directory when the user specifies the current directory as the working directory.
    void testLauncherWithCurrentDirectorySpecifiedAsWorkingDirectory();

    //! Tests the good behaviour of a code launched without Save
    void testLauncherFailure();

    //@}
// #endif

#ifdef WITH_LEVELE
    //---------------------------------------------
    /** @name levelE Unit Tests 
	These TU treat the temporal output features
    **/
    //@{  
    //! Set up for the levelE use case
    void setUpLevelE();
    //! levelE with DataServer output file
    void testLauncherLevelEOutputFileDataServer();
    //! levelE with Column output file
    void testLauncherLevelEOutputFileColumn();
    //! levelE with Row output file
    void testLauncherLevelEOutputFileRow();
    //! levelE with Row output file
    void testLauncherLevelEOutputFileRowMultithread();
    //! levelE with Row output file
    void testLauncherLevelEOutputFileKeys();
#ifdef WITH_LIBXML2
    //! levelE with XML output file
    void testLauncherLevelEOutputFileXML();
#endif
    //! levelE with Column output file and by steps
    void testLauncherLevelEOutputFileColumnByStep();
    //! levelE results checker
    void checkLauncherLevelEResults(TString sOutputfileTmp);
    //@}
#endif
    
#ifdef WITH_FMI
    //---------------------------------------------
    /** @name Tests for FMI
        These Tests (Test Units and ThrowExceptions) devoted to FMI
    **/
    //@{
    void setUpFMI();
    void tearDownFMI();

    void teFMIv1BoundingBall();

    void teFMIv2BoundingBall();

    void checkValuesFMI(URANIE::DataServer::TDataServer *tds, Int_t nSizeY = 101);
    //@}
#endif

    //---------------------------------------------
    /** @name Handliing multitype in output
     **/
    //@{  
    //! Set up for the MultiType use case
    void setUpMultiType();
    //! MultiType with DataServer output file
    void testLauncherMultiTypeOutputFileDataServer();
    //! MultiType with Column output file
    void testLauncherMultiTypeOutputFileColumn();
#ifdef WITH_LIBXML2
    //! MultiType with XML output file
    void testLauncherMultiTypeOutputFileXML();
#endif    
    //! MultiType with Row output file
    void testLauncherMultiTypeOutputFileRow();
    //! MultiType with Row output file
    void testLauncherMultiTypeOutputFileRowMultithread();
    //! MultiType with Row output file wth inputs
    void testLauncherMultiTypeOutputFileRowMultithreadWithInputs();    
    //! MultiType with Row output file
    void testLauncherMultiTypeOutputFileRowBySteps();
    //! MultiType with Key output file
    void testLauncherMultiTypeOutputFileKey();
    //! MultiType with Key condensate output file
    void testLauncherMultiTypeOutputFileKeyCondensate();
    //! MultiType results checker
    void checkLauncherMultiTypeResults(int shift=0);
    //@}


    //---------------------------------------------
    /** @name Handliing multitype in input
     **/
    //@{  
    //! Set up for the MultiType input use case
    void setUpMultiTypeAsInput();
    //! MultiType with DataServer input file
    void testLauncherMultiTypeInputFileDataServer();
#ifdef WITH_LIBXML2
#ifdef WITH_LIBXSLT  
    //! MultiType with XML input file
    void testLauncherMultiTypeInputFileXML();
#endif
#endif
    //! MultiType with Row input file
    void testLauncherMultiTypeInputFileRow();
    //! MultiType with Row input file
    void testLauncherMultiTypeInputFileRowMultithread();
    //! MultiType with Row input file with and without inputs
    void testLauncherMultiTypeInputFileRowMultithreadWithInputs();
    //! MultiType with Row input file
    void testLauncherMultiTypeInputFileRowBySteps();
    //! MultiType with Column input file
    void testLauncherMultiTypeInputFileColumn();
    //! MultiType with Row input file
    void testLauncherMultiTypeInputFileKey();
    //! MultiType results checker
    void checkLauncherMultiTypeInputResults(int shift=0);    
    //! Changing the way the input file looks like
    void testLauncherFormattingMultiTypeInputFile();
    //@}
    
// #ifdef WITH_CLUB
    //---------------------------------------------
    /** @name Unitary Tests of TLauncher on flowrate UniformDesign dataset
	The list of unitary tests of TLauncher allowing to test methods existing to avoid oversight's user 
    **/
    //@{  
    //! Test when we use attribute which name is different than key
    void testlaunchCodeAttributeNameDifferentKey();
    //! Throw an exception when attributes is associated with more one TOutputFileKey
    void throwExceptionCodeAttributeWithSeveralTOutputFiles();
    //! Throw an exception when attributes is associated with more one TOutputFileKey
    void throwExceptionCodeAttributeWithDifferentNames();
    //@}
// #endif
    //! Throw an exception when using a TCode without TOutputFile declared
    void throwExceptionTCodeWithoutOutputFile();

    //! Throw an exception when setting "." as a separator character
    void throwExceptionCodeFileSetPointSeparator();
    
    //! Throw an exception when the Input file cannot be opened
    void throwExceptionInputFileNotFound();
    
    //! Throw an exception when input and output attributes have the same name
    void throwExceptionInputAndOutputAttributeWithSameName();
    
private:
#ifdef WITH_LIBXML2
    void validateTDS4ValueYhat();
    void validateTDS4ValueD();
#endif

private:
    TString _workingTestPath;

    URANIE::DataServer::TDataServer *tds1;
    URANIE::Launcher::TCode *tc1;
    URANIE::Launcher::TLauncher *tl1;

    URANIE::DataServer::TDataServer *tds2;
    URANIE::Launcher::TCode *tc2;
    URANIE::Launcher::TLauncher *tl2;

    URANIE::DataServer::TDataServer * tds3;
    URANIE::Launcher::TCode * tc3;
    URANIE::Launcher::TLauncher * tl3;

    URANIE::DataServer::TDataServer * tds4;
    URANIE::Launcher::TCode * tc4;
    URANIE::Launcher::TLauncher * tl4;

    URANIE::Launcher::TLauncherByStep* tlbs;
    
    //! Output file Column
    URANIE::Launcher::TOutputFileColumn *_foutCol;
    //! Output file Row
    URANIE::Launcher::TOutputFileRow *_foutRow;
    //! Output file Key
    URANIE::Launcher::TOutputFileKey *_foutKey;
// #endif
    URANIE::Launcher::TOutputFileDataServer *_foutDataServer;

#ifdef WITH_FMI
    URANIE::Launcher::TOutputFileFMU *_foutFMI1, *_foutFMI2;
    URANIE::Launcher::TCodeFMU *_tcFMI1, *_tcFMI2;
#endif

#ifdef WITH_LIBXML2
    //! Output file XML
    URANIE::Launcher::TOutputFileXML *_foutXML;
#endif

    URANIE::DataServer::TAttribute *ptrHat;
    URANIE::DataServer::TAttribute *ptrD;

    URANIE::Launcher::TInputFileRecreate * tifr;

    Bool_t _bLog;
};

#endif  // CPPUNITEST_TDATASERVER_H
