{
  // Load the function flowrateCalib1D
  gROOT->LoadMacro("UserFunctions.C");
  
  // Input reference file   
  TString ExpData="linReg_Database.dat";
  
  // Input reference file loaded
  TDataServer *tdsRef = new TDataServer("tdsRef","tdsRef");
  tdsRef->fileDataRead(ExpData.Data());
  
  // Define the uncertainty model wih a guess
  double sd_exp=0.2;    
  tdsRef->addAttribute("wei_exp",Form("1./(%g*%g)",sd_exp,sd_exp));
  
  // Define the parameters
  TDataServer *tdsPar = new TDataServer("tdsPar","poute");
  double binf_search=-2.0, bsup_search=2.0;
  tdsPar->addAttribute(new TUniformDistribution("t0", binf_search, bsup_search) );
  tdsPar->addAttribute(new TUniformDistribution("t1", binf_search, bsup_search)) ;
  
  // Create the output attribute
  TAttribute *out = new TAttribute("out");
  
  // Create interface to assessors
  TCIntEval eval("Toy");
  eval.addInput(tdsRef->getAttribute("x"));
  eval.addInput(tdsPar->getAttribute("t0"));
  eval.addInput(tdsPar->getAttribute("t1"));
  eval.addOutput(out);
  
  // Set the runner
  TSequentialRun run(&eval);

  // Set the calibration object
  // Providing wild guess for value and variation range
  vector<double> inval={0.8, -0.6}, std={0.4, 0.5};
  int ns=12000; 
  TMetropHasting *cal = new TMetropHasting(tdsPar, &run, ns,"");
  cal->setDistanceAndReference("weightedLS",tdsRef,"x","yExp","wei_exp");
  cal->setNbDump(4000);
  cal->setInitialisation(inval, std);
  cal->estimateParameters();
  
  // Quality assessment :  Draw the trace the MCMC
  TCanvas *canTr = new TCanvas("CanTr","CanTr",1200,800);
  TPad *padTr = new TPad("padTr","padTr",0, 0.03, 1, 1);  padTr->Draw();  padTr->cd();
  cal->drawTrace("Trace title","*","","nonewcanvas");
  
  // Quality assessment :  Draw the trace the MCMC
  TCanvas *canAcc = new TCanvas("CanAcc","CanAcc",1200,800);
  TPad *padAcc = new TPad("padAcc","padAcc",0, 0.03, 1, 1);  padAcc->Draw();  padAcc->cd();
  cal->drawAcceptationRatio("AcceptRatio title","*","","nonewcanvas");

  int burn=100;
  // Compute the auto-correlation
  vector<int> lag={1,3,6,10,20};
  vector<double> autoCorr;
  cal->getAutoCorrelation(lag, &autoCorr, burn);
  
  cout<<"Autocorrelation are:"<<endl;
  for(unsigned il=0; il<lag.size(); il++)
  {
    cout<<"*** for lag="<<lag.at(il)<<": ";
    for(unsigned ip=0; ip<cal->getNPar(); ip++)            
    cout<<autoCorr.at(ip*lag.size()+il)<<";  ";
    cout<<endl;
  } 

  int mylag=6;
  //Define a selection based on burn-in and lag
  TString mycut=Form("(%s > %d) && ((%s %% %d) == 0)", tdsPar->getIteratorName(), burn,  tdsPar->getIteratorName(), mylag);
  // Draw the parameters
  TCanvas *canPar = new TCanvas("CanPar","CanPar",1200,800);
  TPad *padPar = new TPad("padPar","padPar",0, 0.03, 1, 1);  padPar->Draw();  padPar->cd();
  cal->drawParameters("Parameter title","*",mycut.Data(),"nonewcanvas");
  
  // Draw the residuals
  TCanvas *canRes = new TCanvas("CanRes","CanRes",1200,800);
  TPad *padRes = new TPad("padRes","padRes",0, 0.03, 1, 1);  padRes->Draw();  padRes->cd();
  cal->drawResidues("Residual title","*","","nonewcanvas"); 
}