"""
Example of code launching when keys might contain blank
"""
from rootlogon import ROOT, DataServer, Launcher, Sampler

# Create dataserver and add attributes
tds = DataServer.TDataServer("pouet", "foo")
tds.addAttribute(DataServer.TUniformDistribution("pressure", 0, 10))
tds.addAttribute(DataServer.TLogUniformDistribution("density", 0.01, 100))

# Define the input file and the keys for
sFileName = ROOT.TString("withblank_input.dat")
pressName = "water model.brewery pit pressure "
tds.getAttribute("pressure").setFileKey(sFileName, pressName)
densName = "residual brew density model.factor for glass "
tds.getAttribute("density").setFileKey(sFileName, densName)

# Create input file interface before done by the TCode to precise information
inF = Launcher.TInputFileKey(ROOT.gSystem.PrependPathName(ROOT.gSystem.pwd(),
                                                          sFileName))
# Set separator to '=' so that lines could contain blank spaces (specific)
inF.setFieldSeparatorCharacter("=")
inF.setSeparatorCharacter(" ")
inF.addAttribute(tds.getAttribute("pressure"))
inF.addAttribute(tds.getAttribute("density"))

# Generate a doe
sampl = Sampler.TBasicSampling(tds, "srs", 10)
sampl.generateSample()

# Define the output
sOutFileName = "withblank_output.dat"
fout = Launcher.TOutputFileKey(sOutFileName)
# Set separator to '=' so that lines could contain blank spaces (specific)
fout.setFieldSeparatorCharacter("=")
fout.setSeparatorCharacter("")

# Define the attributes and their keys
outAtt = DataServer.TAttribute("outpressure")
outAttName = "Test doubling.water model.brewery pit pressure "
outAtt.setFileKey(sOutFileName, outAttName)
outAtt2 = DataServer.TAttribute("outdensity")
outAtt2Name = "Chocolat doubling.residual brew density model.factor for glass "
outAtt2.setFileKey(sOutFileName, outAtt2Name)
fout.addAttribute(outAtt)
fout.addAttribute(outAtt2)

# Create TCode with the TDS and the command to execute
mycode = Launcher.TCode(tds, "withblank")
# Add the output file
mycode.addInputFile(inF)
mycode.addOutputFile(fout)

# Launcher of the code on the Design of experiments (DoE) in the TDS
tlch = Launcher.TLauncher(tds, mycode)
tlch.setSave(-1)
tlch.setClean()
tlch.run()

tds.Scan("pressure:outpressure:density:outdensity")
