  
{

  Int_t nS = 300;
  // Define the DataServer
  TDataServer *tds = new TDataServer("tdsflowrate", "DataBase flowrate");
  // Add the eight attributes of the study with uniform law
  tds->addAttribute( new TUniformDistribution("rw", 0.05, 0.15));
  tds->addAttribute( new TUniformDistribution("r", 100.0, 50000.0));
  tds->addAttribute( new TUniformDistribution("tu", 63070.0, 115600.0));
  tds->addAttribute( new TUniformDistribution("tl", 63.1, 116.0));
  tds->addAttribute( new TUniformDistribution("hu", 990.0, 1110.0));
  tds->addAttribute( new TUniformDistribution("hl", 700.0, 820.0));
  tds->addAttribute( new TUniformDistribution("l", 1120.0, 1680.0));
  tds->addAttribute( new TUniformDistribution("kw", 9855.0, 12045.0));

  // The input file
  TString sFileName = TString("flowrate_input_with_flags.in");

  // Set the input file with flag format and the flag for each input attributes
  tds->getAttribute("rw")->setFileFlag(sFileName, "@Rw@");
  tds->getAttribute("r")->setFileFlag(sFileName, "@R@");
  tds->getAttribute("tu")->setFileFlag(sFileName, "@Tu@");
  tds->getAttribute("tl")->setFileFlag(sFileName, "@Tl@");
  tds->getAttribute("hu")->setFileFlag(sFileName, "@Hu@");
  tds->getAttribute("hl")->setFileFlag(sFileName, "@Hl@");
  tds->getAttribute("l")->setFileFlag(sFileName, "@L@");
  tds->getAttribute("kw")->setFileFlag(sFileName, "@Kw@");

  
  // Generate the sample
  TSampling *sampling = new TSampling(tds, "lhs", 400); 
  sampling->generateSample();   
  
  // The output file of the code where values are stored in row
  TOutputFileRow *fout = new TOutputFileRow("_output_flowrate_withRow_.dat");
  // The attribute in the output file
  TAttribute * tyhat = new TAttribute("yhat");
  tyhat->setDefaultValue(-200.0);
  fout->addAttribute(tyhat);
  
  // Create a TCode object
  TCode *mycode = new TCode(tds, "flowrate -s -ff");
  // Add the output file where to find the output attributes
  mycode->addOutputFile( fout );
  
  // Create a TLauncher object from a TDataServer and a TCode objects
  TLauncher *tlch = new TLauncher(tds, mycode);
  tlch->setSave();
  // tlch->setClean();
  tlch->setVarDraw("hu:hl","yhat>0","");
  // tlch->setWorkingDirectory(gSystem->Getenv("PWD") + TString("/tmpLanceurUranie/flowrate"));

  tlch->run();

  // Visualisation
  TCanvas  *Canvas = new TCanvas("c1", "Graph for the Macro launchCodeFlowrateFlagFailure",5,64,1270,667);
  tds->draw("hu:hl","yhat>0","");
  
  
  tds->exportData("_flowrate_sampler_launcher_.dat");
}
