 
void PlanOATMinMax(TDataServer *tds)
{
  // Create the data tuple
  tds->createTuple();
  
  Int_t natt = tds->getNAttributes();
  // The number of attributes is incremented by one for the iterator
  const Int_t nn = natt+1;
  
  // Allocate the data vector to fill the tuple
  Double_t value[nn];
  
  // Init the iterator. It is always the first value.
  value[0] = 0;
  // Init the data vector with the nominal/mean values of attributes
  for(Int_t i=0;i<natt;i++)
    value[1+i] = tds->getAttribute(i)->getMean();
  
  // Fill the nominal/mean values : nominal pattern
  tds->getTuple()->Fill(value);
  
  // Loop on each attributes
  for(Int_t i=0;i<natt;i++) {
    // Case of min values
    value[1+i] = tds->getAttribute(i)->getLowerBound();
    value[0] += 1.0;
    // Fill the min value value for xi attribute and nominal values for other attributes
    tds->getTuple()->Fill(value);
    // Case of the max value
    value[1+i] = tds->getAttribute(i)->getUpperBound();
    value[0] += 1.0;
    // Fill the max value value for xi attribute and nominal values for other attributes
    tds->getTuple()->Fill(value);
    // Reset the nominal/mean value
    value[1+i] = tds->getAttribute(i)->getMean();
  }  
}

void launchCodeFlowrateFlagOATMinMax()
{
  // Define the DataServer
  TDataServer *tds = new TDataServer("tdsOATMinMaxFlowrateFlag", "OATMinMax Design of experiments for Flowrate");

  // Add the eight attributes of the study with min, max and nominal values
  tds->addAttribute( new TAttribute("rw", 0.05, 0.15));
  tds->getAttribute("rw")->setMean(0.10);

  tds->addAttribute( new TAttribute("r", 100.0, 50000.0));
  tds->getAttribute("r")->setMean(25050.);

  tds->addAttribute( new TAttribute("tu", 63070.0, 115600.0));
  tds->getAttribute("tu")->setMean(89335.);

  tds->addAttribute( new TAttribute("tl", 63.1, 116.0));
  tds->getAttribute("tl")->setMean(89.55);

  tds->addAttribute( new TAttribute("hu", 990.0, 1110.0));
  tds->getAttribute("hu")->setMean(1050.);

  tds->addAttribute( new TAttribute("hl", 700.0, 820.0));
  tds->getAttribute("hl")->setMean(760.);

  tds->addAttribute( new TAttribute("l", 1120.0, 1680.0));
  tds->getAttribute("l")->setMean(1400.);

  tds->addAttribute( new TAttribute("kw", 9855.0, 12045.0));
  tds->getAttribute("kw")->setMean(10950.);

  // The input file
  TString sFileName = TString("flowrate_input_with_flags.in");
  
  // Set the input file with flag format and the flag for each input attributes
  tds->getAttribute("rw")->setFileFlag(sFileName, "@Rw@");
  tds->getAttribute("r")->setFileFlag(sFileName, "@R@");
  tds->getAttribute("tu")->setFileFlag(sFileName, "@Tu@");
  tds->getAttribute("tl")->setFileFlag(sFileName, "@Tl@");
  tds->getAttribute("hu")->setFileFlag(sFileName, "@Hu@");
  tds->getAttribute("hl")->setFileFlag(sFileName, "@Hl@");
  tds->getAttribute("l")->setFileFlag(sFileName, "@L@");
  tds->getAttribute("kw")->setFileFlag(sFileName, "@Kw@");

  // Build the Design of Experiments (DoE)
   PlanOATMinMax(tds);
        
  // Scan the data ( 17 = 1 + 2*8 patterns)
  tds->Scan("*","","colsize=5 col=3:4::6::4:3:4:");
  
  // Save the Design of experiments in an ASCII file
  tds->exportData("_flowrate_sampler_oat_.dat");

  // The output file of the code
  TOutputFileRow *fout = new TOutputFileRow("_output_flowrate_withRow_.dat");
  // The attribute in the output file
  fout->addAttribute(new TAttribute("yhat"));
  
  // Instanciation de mon code
  TCode *mycode = new TCode(tds, "flowrate -s -f");
  //mycode->setLog();
  // on ajoute le fichier de sortie du code
  mycode->addOutputFile( fout );
  
  // Lancement du code
  TLauncher *tlch = new TLauncher(tds, mycode);
  tlch->setSave();
  tlch->setClean();
  tlch->setDrawProgressBar(kFALSE);
  tlch->setVarDraw("yhat:rw","","");

  tlch->run();

  
  
  tds->exportData("_flowrate_sampler_launcher_oat_.dat");

  // Visualisation
  TCanvas  *Canvas = new TCanvas("c1", "Graph for the Macro launchCodeFlowrateFlagOATMinMax",5,64,1270,667);
  TPad *pad = new TPad("pad","pad",0, 0.03, 1, 1); pad->Draw();
  gStyle->SetPalette(1);
  pad->Divide(2, 2);
  pad->cd(1);
  tds->draw("yhat:rw");
  
  pad->cd(3); tds->draw("yhat");
  pad->cd(2); tds->draw("yhat:rw","","colz");
  
  pad->cd(4);
  // The parallel plot
  tds->draw("rw:r:tu:tl:hu:hl:l:kw:yhat","","para");
  TParallelCoord* para = (TParallelCoord*)gPad->GetListOfPrimitives()->FindObject("ParaCoord");
  
  // The output attribute
  TParallelCoordVar* axis = (TParallelCoordVar*)para->GetVarList()->FindObject("yhat");
  axis->AddRange(new TParallelCoordRange(axis,15.0,20.0));
  para->AddSelection("blue");
  para->GetCurrentSelection()->SetLineColor(kBlue);
  
  axis->AddRange(new TParallelCoordRange(axis,155.0,160.0));
  para->AddSelection("blue");
     para->GetCurrentSelection()->SetLineColor(kBlue);
  
}
