"""
Example of flowrate code launching in key file and failure mode
"""
from rootlogon import ROOT, DataServer, Launcher, Sampler

nS = 400
# Define the DataServer
tds = DataServer.TDataServer("tdsflowrate", "DataBase flowrate")
tds.addAttribute(DataServer.TUniformDistribution("rw", 0.05, 0.15))
tds.addAttribute(DataServer.TUniformDistribution("r", 100.0, 50000.0))
tds.addAttribute(DataServer.TUniformDistribution("tu", 63070.0, 115600.0))
tds.addAttribute(DataServer.TUniformDistribution("tl", 63.1, 116.0))
tds.addAttribute(DataServer.TUniformDistribution("hu", 990.0, 1110.0))
tds.addAttribute(DataServer.TUniformDistribution("hl", 700.0, 820.0))
tds.addAttribute(DataServer.TUniformDistribution("l", 1120.0, 1680.0))
tds.addAttribute(DataServer.TUniformDistribution("kw", 9855.0, 12045.0))

# The reference input file
sIn = ROOT.gSystem.pwd() + "/"
sIn += "flowrate_input_with_keys.in"
tds.getAttribute("rw").setFileKey(sIn, "Rw")
tds.getAttribute("r").setFileKey(sIn, "R")
tds.getAttribute("tu").setFileKey(sIn, "Tu")
tds.getAttribute("tl").setFileKey(sIn, "Tl")
tds.getAttribute("hu").setFileKey(sIn, "Hu")
tds.getAttribute("hl").setFileKey(sIn, "Hl")
tds.getAttribute("l").setFileKey(sIn, "L")
tds.getAttribute("kw").setFileKey(sIn, "Kw")

# Generate the sample
sampling = Sampler.TSampling(tds, "lhs", nS)
sampling.generateSample()

# The output file of the code
fout = Launcher.TOutputFileRow("_output_flowrate_withRow_.dat")
# The attribute in the output file
tyhat = DataServer.TAttribute("yhat")
tyhat.setDefaultValue(-200.0)
fout.addAttribute(tyhat)

# Create a Launcher.TCode object
mycode = Launcher.TCode(tds, "flowrate -s -kf")
# Add the output file where to find the output attributes
# mycode.addInputFile(Launcher.TInputFile(sIn))
mycode.addOutputFile(fout)

# Create a TLauncher object from a TDataServer and a TCode objects
laun = Launcher.TLauncher(tds, mycode)
laun.setSave()
# laun.setClean()
laun.setVarDraw("hu:hl", "yhat>0", "")
# laun.setWorkingDirectory(ROOT.gSystem.pwd() + "/tmpLaunUranie/flowrate")

laun.run()

# Visualisation
Canvas = ROOT.TCanvas("c1", "Graph launchCodeFlowratestop", 5, 64, 1270, 667)
tds.draw("hu:hl", "yhat>0")

tds.exportData("_flowrate_sampler_launcher_.dat")
