 
void launchCodeFlowrateKeyRecreateSamplingOutputDataServer(Int_t nS = 100)
{
  
  // Define the DataServer
  TDataServer *tds = new TDataServer("tdsFlowrate", "Design of experiments for Flowrate");

  // Add the study attributes ( min, max and nominal values)
  tds->addAttribute( new TUniformDistribution("rw", 0.05, 0.15));
  tds->addAttribute( new TUniformDistribution("r", 100.0, 50000.0));
  tds->addAttribute( new TUniformDistribution("tu", 63070.0, 115600.0));
  tds->addAttribute( new TUniformDistribution("tl", 63.1, 116.0));
  tds->addAttribute( new TUniformDistribution("hu", 990.0, 1110.0));
  tds->addAttribute( new TUniformDistribution("hl", 700.0, 820.0));
  tds->addAttribute( new TUniformDistribution("l", 1120.0, 1680.0));
  tds->addAttribute( new TUniformDistribution("kw", 9855.0, 12045.0));

  // The reference input file
  TString sIn = TString("flowrate_input_with_keys.in");
  
  // Set the reference input file and the key for each input attributes
  tds->getAttribute("rw")->setFileKey(sIn, "Rw","",TAttributeFileKey::kNewKey);
  tds->getAttribute("r")->setFileKey(sIn, "R","",TAttributeFileKey::kNewKey);
  tds->getAttribute("tu")->setFileKey(sIn, "Tu","",TAttributeFileKey::kNewKey);
  tds->getAttribute("tl")->setFileKey(sIn, "Tl","",TAttributeFileKey::kNewKey);
  tds->getAttribute("hu")->setFileKey(sIn, "Hu","",TAttributeFileKey::kNewKey);
  tds->getAttribute("hl")->setFileKey(sIn, "Hl","",TAttributeFileKey::kNewKey);
  tds->getAttribute("l")->setFileKey(sIn, "L","",TAttributeFileKey::kNewKey);
  tds->getAttribute("kw")->setFileKey(sIn, "Kw","",TAttributeFileKey::kNewKey);

  // Generate the design of experiments
  TSampling *sampling = new TSampling(tds, "lhs", nS); 
  sampling->generateSample();   
  
  // The output file of the code
  TOutputFileDataServer *fout = new TOutputFileDataServer("_output_flowrate_withRow_.dat");
  // The attribute in the output file
  fout->addAttribute("yhat"); // fout->addAttribute(new TAttribute("y"));
  
  // Create a TCode object from a TDS and the command line
  TCode *mycode = new TCode(tds, "flowrate -s -k");
  // Add the output file
  mycode->addOutputFile( fout );
  
  // Create a launcher of code
  TLauncher *lanceur = new TLauncher(tds, mycode);
  lanceur->setSave();
  lanceur->setClean();
  //lanceur->setWorkingDirectory(gSystem->Getenv("PWD") + TString("/tmpLanceurUranie/flowrate"));
  
  lanceur->setVarDraw("yhat:rw","","");
  
  lanceur->run();

  
  
  tds->exportData("_flowrate_sampler_launcher_.dat");
  // Visualisation
  TCanvas  *Canvas = new TCanvas("c1", "Graph for the Macro launchCodeFlowrateKeyRecreateSamplingOutputDataServer",565,62,560,619);
  TPad *pad = new TPad("pad","pad",0, 0.03, 1, 1); pad->Draw();
  gStyle->SetPalette(1);
  pad->Divide(1, 3); 
  
  // Scatterplot y versus rw 
  pad->cd(1); tds->draw("yhat:rw");
  
  // Histogramm of the output attribute
  pad->cd(2); tds->draw("yhat");
  
  // The parallel plot
  pad->cd(3);
  tds->draw("rw:r:tu:tl:hu:hl:l:kw:yhat","","para");
  TParallelCoord* para = (TParallelCoord*)gPad->GetListOfPrimitives()->FindObject("ParaCoord");
  
  // The output attribute
  TParallelCoordVar* axis = (TParallelCoordVar*)para->GetVarList()->FindObject("yhat");
  axis->AddRange(new TParallelCoordRange(axis,15.0,20.0));
  para->AddSelection("blue");
  para->GetCurrentSelection()->SetLineColor(kBlue);
  
  axis->AddRange(new TParallelCoordRange(axis,155.0,160.0));
  para->AddSelection("blue");
  para->GetCurrentSelection()->SetLineColor(kBlue);
         
}
