 
void launchCodeFlowrateRowRecreateSamplingOutputDataServer(Int_t nS = 100)
{
  
  // Define the DataServer
  TDataServer *tds = new TDataServer("tdsFlowrate", "Design of experiments for Flowrate");

  // Add the study attributes ( min, max and nominal values)
  tds->addAttribute( new TUniformDistribution("rw", 0.05, 0.15));
  tds->addAttribute( new TUniformDistribution("r", 100.0, 50000.0));
  tds->addAttribute( new TUniformDistribution("tu", 63070.0, 115600.0));
  tds->addAttribute( new TUniformDistribution("tl", 63.1, 116.0));
  tds->addAttribute( new TUniformDistribution("hu", 990.0, 1110.0));
  tds->addAttribute( new TUniformDistribution("hl", 700.0, 820.0));
  tds->addAttribute( new TUniformDistribution("l", 1120.0, 1680.0));
  tds->addAttribute( new TUniformDistribution("kw", 9855.0, 12045.0));

  // The input file
  TString sFileName = TString("flowrate_input_with_values_rows.in");
  
  // Set the input file with "key = value" format CREATED ON FLY with row for each input attributes
  tds->getAttribute("rw")->setFileKey(sFileName, "Rw","",TAttributeFileKey::kNewRow);
  tds->getAttribute("r")->setFileKey(sFileName, "R","",TAttributeFileKey::kNewRow);
  tds->getAttribute("tu")->setFileKey(sFileName, "Tu","",TAttributeFileKey::kNewRow);
  tds->getAttribute("tl")->setFileKey(sFileName, "Tl","",TAttributeFileKey::kNewRow);
  tds->getAttribute("hu")->setFileKey(sFileName, "Hu","",TAttributeFileKey::kNewRow);
  tds->getAttribute("hl")->setFileKey(sFileName, "Hl","",TAttributeFileKey::kNewRow);
  tds->getAttribute("l")->setFileKey(sFileName, "L","",TAttributeFileKey::kNewRow);
  tds->getAttribute("kw")->setFileKey(sFileName, "Kw","",TAttributeFileKey::kNewRow);

  // Generate a Design of experiments (DoE) from the TDataServer
  TSampling *sampling = new TSampling(tds, "lhs", nS);
  sampling->generateSample();   
  
  // The output file of the code based on the same format that a TDataServer can load (#COLUMN_NAMES: yhat | d)
  TOutputFileDataServer *fout = new TOutputFileDataServer("_output_flowrate_withRow_.dat");
  // The attributes in the output file
  fout->addAttribute("d"); // fout->addAttribute(new TAttribute("d"));
  fout->addAttribute("yhat"); // fout->addAttribute(new TAttribute("yhat"));
  
  // Create a TCode object with the TDS (attribute and input files) and the command to execute
  TCode *mycode = new TCode(tds, "flowrate -s -r");
  // Add the output file
  mycode->addOutputFile( fout );
  
  // Launcher of the code on the Design of experiments (DoE) in the TDS
  TLauncher *tlch = new TLauncher(tds, mycode);
  tlch->setSave();
  tlch->setClean();
  //tlch->setWorkingDirectory(gSystem->Getenv("PWD") + TString("/tmpLanceurUranie/flowrate"));
  tlch->setVarDraw("yhat:rw","","");
  
  // Evaluate the DoE
  tlch->run();

  
  
//   tds->exportData("_flowrate_sampler_launcher_.dat");

  // Visualisation
  TCanvas  *Canvas = new TCanvas("c1", "Graph for the Macro launchCodeFlowrateRowRecreateSamplingOutputDataServer.C",565,62,560,619);
  TPad *pad = new TPad("pad","pad",0, 0.03, 1, 1); pad->Draw();
  gStyle->SetPalette(1);
  pad->Divide(1, 3);
  pad->cd(1); tds->draw("yhat:rw");

  pad->cd(2);
  // Histogramm of the output attribute
  tds->draw("yhat");
  
  pad->cd(3);
  // The parallel plot
  tds->draw("rw:r:tu:tl:hu:hl:l:kw:yhat","","para");
  TParallelCoord* para = (TParallelCoord*)gPad->GetListOfPrimitives()->FindObject("ParaCoord");
  
  // The output attribute
  TParallelCoordVar* axis = (TParallelCoordVar*)para->GetVarList()->FindObject("yhat");
  axis->AddRange(new TParallelCoordRange(axis,15.0,20.0));
  para->AddSelection("blue");
  
  para->GetCurrentSelection()->SetLineColor(kRed);
  axis->AddRange(new TParallelCoordRange(axis,155.0,160.0));
      
}
