"""
Example of multitype code launching with XML output format
"""
from rootlogon import ROOT, DataServer, Launcher, Sampler

# Create dataserver with the seed attribute
tds = DataServer.TDataServer("foo", "pouet")
tds.addAttribute(DataServer.TUniformDistribution("seed", 0, 100000))
tds.getAttribute("seed").setFileKey("multitype_input.dat", "",
                                    "%e", DataServer.TAttributeFileKey.kNewRow)

# Create DOE
tsam = Sampler.TSampling(tds, "lhs", 100)
tsam.generateSample()

# Precise output and create Code
out = Launcher.TOutputFileXML("_output_multitype_mt_.xml")
AttIn = DataServer.TAttributeFileKey.kXMLAttribute
FieIn = DataServer.TAttributeFileKey.kXMLField
out.addAttribute(DataServer.TAttribute("w1", DataServer.TAttribute.kString),
                 "/multitypeMT/w1/@value", AttIn)
out.addAttribute(DataServer.TAttribute("w2", DataServer.TAttribute.kString),
                 "/multitypeMT/w2/@value", AttIn)
out.addAttribute(DataServer.TAttribute("v1", DataServer.TAttribute.kVector),
                 "/multitypeMT/v1/values", FieIn)
out.addAttribute(DataServer.TAttribute("v2", DataServer.TAttribute.kVector),
                 "/multitypeMT/v2/values", FieIn)
out.addAttribute(DataServer.TAttribute("f1"),
                 "/multitypeMT/f1/@value", AttIn)

myc = Launcher.TCode(tds, "multitype -mtXML")
myc.addOutputFile(out)

# Create Launcher.TLauncher and run it
tlau = Launcher.TLauncher(tds, myc)
tlau.run()

# Produce control plot
Can = ROOT.TCanvas("Can", "Can", 10, 10, 1000, 800)
pad = ROOT.TPad("pad", "pad", 0, 0.03, 1, 1)
pad.Draw()
pad.cd()
tds.drawPairs("w1:v1:v2:f1:w2")
