"""
Example of readmultitype code launching with dataserver output format
"""
from rootlogon import ROOT, DataServer, Launcher

# Create dataserver with a database of 10 runs of the "multitype -mt" code
tds = DataServer.TDataServer("foo", "pouet")
tds.fileDataRead("readmultitype_sampling.dat")

# Explain the name of the input file and the format chosen
inFile = "_output_multitype_mt_DataServer_.dat"
inType = DataServer.TAttributeFileKey.kNewTDS
tds.getAttribute("w1").setFileKey(inFile, "", "%e", inType)
tds.getAttribute("v1").setFileKey(inFile, "", "%e", inType)
tds.getAttribute("v2").setFileKey(inFile, "", "%e", inType)
tds.getAttribute("f1").setFileKey(inFile, "", "%e", inType)
tds.getAttribute("w2").setFileKey(inFile, "", "%e", inType)

# Create the Class that will handle re-writting the input file
FullinFile = ROOT.gSystem.PrependPathName(ROOT.gSystem.pwd(), inFile)
inFile = Launcher.TInputFileRecreate(FullinFile)

# Add attribute in the correct (needed) order
inFile.addAttribute(tds.getAttribute("w1"))
inFile.addAttribute(tds.getAttribute("v1"))
inFile.addAttribute(tds.getAttribute("v2"))
inFile.addAttribute(tds.getAttribute("f1"))
inFile.addAttribute(tds.getAttribute("w2"))

# Create the output file interface and state that there will be 2 outputs
OutName = "_output_multitype_readmt_DataServer_.dat"
out = Launcher.TOutputFileDataServer(OutName)
out.addAttribute(DataServer.TAttribute("thev1"))
out.addAttribute(DataServer.TAttribute("thev2"))

# Create the corresponding interface to code
tc1 = Launcher.TCode(tds, "multitype -ReadmtDS")
tc1.addInputFile(inFile)
tc1.addOutputFile(out)

# Create the launcher and run the code
tl1 = Launcher.TLauncher(tds, tc1)
tl1.run()

tds.Scan("thev1:thev2")
