"""
Example of readmultitype code launching with XML output format
"""
from rootlogon import ROOT, DataServer, Launcher

# Create dataserver with a database of 10 runs of the "multitype -mt" code
tds = DataServer.TDataServer("foo", "pouet")
tds.fileDataRead("readmultitype_sampling.dat")

# Produce the input files
ROOT.gSystem.Exec("multitype -mtXML")

# Explain the name of the input file and the format chosen
inFile = "_output_multitype_mt_.xml"
AttType = DataServer.TAttributeFileKey.kXMLAttribute
FieType = DataServer.TAttributeFileKey.kXMLField
tds.getAttribute("w1").setFileKey(inFile, "w1/@value", "", AttType)
tds.getAttribute("v1").setFileKey(inFile, "v1/values", "", FieType)
tds.getAttribute("v2").setFileKey(inFile, "v2/values", "", FieType)
tds.getAttribute("f1").setFileKey(inFile, "f1/@value", "", AttType)
tds.getAttribute("w2").setFileKey(inFile, "w2/@value", "", AttType)

# Create the Class that will handle re-writting the input file
FullinFile = ROOT.gSystem.PrependPathName(ROOT.gSystem.pwd(), inFile)
inFile = Launcher.TInputFileXML(FullinFile)

# Add attribute in the correct (needed) order
inFile.addAttribute(tds.getAttribute("w1"))
inFile.addAttribute(tds.getAttribute("v1"))
inFile.addAttribute(tds.getAttribute("v2"))
inFile.addAttribute(tds.getAttribute("f1"))
inFile.addAttribute(tds.getAttribute("w2"))

# Create the output file interface and state that there will be 2 outputs
out = Launcher.TOutputFileXML("_output_multitype_readmt_.xml")
out.addAttribute(DataServer.TAttribute("thev1"),
                 "/flowrateREADMT/thev1/@value", AttType)
out.addAttribute(DataServer.TAttribute("thev2"),
                 "/flowrateREADMT/thev2/@value", AttType)

# Create the corresponding interface to code
tc1 = Launcher.TCode(tds, "multitype -ReadmtXML")
tc1.addInputFile(inFile)
tc1.addOutputFile(out)

# Create the launcher and run the code
tl1 = Launcher.TLauncher(tds, tc1)
tl1.run()

tds.Scan("thev1:thev2")
