"""
Example of function launching with graphical examples
"""
from rootlogon import ROOT, DataServer, Launcher, Sampler

nS = 1000
# Create a DataServer.TDataServer
tds = DataServer.TDataServer("tdsFlowrate", "TDS for flowrate")
# Add the eight attributes of the study with uniform law
tds.addAttribute(DataServer.TUniformDistribution("rw", 0.05, 0.15))
tds.addAttribute(DataServer.TUniformDistribution("r", 100.0, 50000.0))
tds.addAttribute(DataServer.TUniformDistribution("tu", 63070.0, 115600.0))
tds.addAttribute(DataServer.TUniformDistribution("tl", 63.1, 116.0))
tds.addAttribute(DataServer.TUniformDistribution("hu", 990.0, 1110.0))
tds.addAttribute(DataServer.TUniformDistribution("hl", 700.0, 820.0))
tds.addAttribute(DataServer.TUniformDistribution("l", 1120.0, 1680.0))
tds.addAttribute(DataServer.TUniformDistribution("kw", 9855.0, 12045.0))

# Generate the sampling from the DataServer.TDataServer
sampling = Sampler.TSampling(tds, "lhs", nS)
sampling.generateSample()

# Load the function in the UserFunction macros file
ROOT.gROOT.LoadMacro("UserFunctions.C")

# Create a TLauncherFunction from a TDataServer and an analytical function
# Rename the outpout attribute "ymod"
tlf = Launcher.TLauncherFunction(tds, "flowrateModel", "", "ymod")
# Evaluate the function on all the design of experiments
tlf.run()

# Graph
Canvas = ROOT.TCanvas("c1", "Graph launchFunctionSampling", 5, 90, 935, 614)
pad = ROOT.TPad("pad", "pad", 0, 0.03, 1, 1)
pad.Draw()
pad.Divide(2, 4)
pad.cd(1)
tds.drawTufte("rw:r", "", "same")
pad.cd(2)
tds.drawCDF("rw")
pad.cd(3)
tds.draw("ymod")
pad.cd(4)
tds.drawCDF("ymod", "", "ccdf")
pad.cd(5)
tds.draw("ymod:rw:hu")
pad.cd(6)
tds.drawProfile("ymod:r", "", "same")
pad.cd(7)
tds.drawProfile("ymod:rw", "", "same")
pad.cd(8)
tds.draw("ymod:rw", "", "colz")
