{
 using namespace URANIE::DataServer; // TDataServer, TLogUniformDistribution
 using namespace URANIE::Modeler;    // TGPBuilder, TGPMLCostFunction, TKriging 
 using namespace URANIE::Sampler;    // TBasicSampling
 using namespace URANIE::Relauncher; // TLauncher2
      
 // Load observations
 TDataServer *tdsObs = new TDataServer("tdsObs","");
 tdsObs->fileDataRead("utf_4D_train.dat");

 // Construct the GPBuilder        
 TGPBuilder *gpb = new TGPBuilder(tdsObs, "x1:x2:x3:x4", "y", "matern3/2");

 // construction of the cost function
 TGPMLCostFunction *cost = new TGPMLCostFunction(gpb);
      
 //----------------------------------//
 // Construction of the LHS sampling //
 //----------------------------------//
 // construction of the attributes
 TLogUniformDistribution *cl1 = new TLogUniformDistribution("cl1", 1e-6, 10.0);
 TLogUniformDistribution *cl2 = new TLogUniformDistribution("cl2", 1e-6, 10.0);
 TLogUniformDistribution *cl3 = new TLogUniformDistribution("cl3", 1e-6, 10.0);
 TLogUniformDistribution *cl4 = new TLogUniformDistribution("cl4", 1e-6, 10.0);
 
 // Construction of the data server containing the exploration 
 // of the correlation lengths' space
 TDataServer *tdsParam = new TDataServer("tdsParam", "");
 tdsParam->addAttribute(cl1);
 tdsParam->addAttribute(cl2);
 tdsParam->addAttribute(cl3);
 tdsParam->addAttribute(cl4);
 
 // Construction of the sampler and generation of the data
 Int_t screeningSize = 1000;
 TBasicSampling *s = new TBasicSampling(tdsParam, "lhs", screeningSize);
 s->generateSample();
 
 // Evaluate the cost function on the sampling
 TLauncher2 *l = new TLauncher2(tdsParam,cost,"cl1:cl2:cl3:cl4","cost:varGP");
 l->solverLoop();
 
 //-----------------------------------------------------------//
 // Search for the minimum value of the cost and retrieve the //
 // corresponding correlation lengths                         //
 //-----------------------------------------------------------//
 // Retrieve the cost values
 Double_t *costValues = tdsParam->getTuple()->getBranchData("cost");
 
 // find the index of the minimum cost
 Int_t minIndex = TMath::LocMin(screeningSize, costValues);
 Double_t optimalCost = costValues[minIndex];
 
 // retrieve the values of the optimal correlation lengths 
 Double_t optimalParams[4];
 optimalParams[0] = tdsParam->getValue("cl1", minIndex);
 optimalParams[1] = tdsParam->getValue("cl2", minIndex);
 optimalParams[2] = tdsParam->getValue("cl3", minIndex);
 optimalParams[3] = tdsParam->getValue("cl4", minIndex);
 
 // retrieve the value of the GP variance
 Double_t optimalVar = tdsParam->getValue("varGP", minIndex);  
 
 //-----------------------------------------------------------//
 // Update the GP Builder with the newly found parameters and //
 // create the model                                          //
 //-----------------------------------------------------------//
 // set the parameters of the correlation function
 gpb->getCorrFunction()->setParameters(optimalParams);
 
 // set the GP variance
 gpb->setVariance(optimalVar);
 
 // Create the model
 TKriging *krig = gpb->buildGP();
 
 // Display model information
 krig->printLog();    
      
  }