{
  //inputs
  TDataServer tds("foo","TDS for multitype");
  tds.fileDataRead("multitype_sampling.dat");

  //output attributes...
  // ... for code 1
  TAttribute *w1 = new TAttribute("w1", URANIE::DataServer::TAttribute::kString);
  TAttribute *w2 = new TAttribute("w2", URANIE::DataServer::TAttribute::kString);
  TAttribute *v1 = new TAttribute("v1", URANIE::DataServer::TAttribute::kVector);
  TAttribute *v2 = new TAttribute("v2", URANIE::DataServer::TAttribute::kVector);
  TAttribute *f1 = new TAttribute("f1");

  // ... for code 2
  TAttribute *thev1 = new TAttribute("thev1");
  TAttribute *thev2 = new TAttribute("thev2");

  // ==================================================================
  // ========================== Code 1 ================================
  // ==================================================================

  // Create the input files
  TFlatScript inputFile1("multitype_input.dat");
  inputFile1.setInputs(1, tds.getAttribute("seed"), "seed");

  // Create the output files
  TKeyResult outputFile1("_output_multitype_mt_Key_.dat");
  outputFile1.addOutput(w1, "w1");
  outputFile1.addOutput(v1, "v1");
  outputFile1.addOutput(v2, "v2");
  outputFile1.addOutput(f1, "f1");
  outputFile1.addOutput(w2, "w2");

  // Create the user's evaluation function
  TCodeEval eval1("multitype -mtKey");
  eval1.addInputFile(&inputFile1);
  eval1.addOutputFile(&outputFile1);

  // ==================================================================
  // ========================== Code 2 ================================
  // ==================================================================

  // Create the output files
  TKeyScript inputFile2("_output_multitype_mt_Key_condensate_.dat");
  inputFile2.setInputs(5, w1, "w1", v1, "v1", v2, "v2", f1, "f1", w2, "w2");

  // Create the output files
  TKeyResult outputFile2("_output_multitype_readmt_Key_.dat");
  outputFile2.addOutput(thev1, "thev1");
  outputFile2.addOutput(thev2, "thev2");

  // Create the user's evaluation function
  TCodeEval eval2("multitype -ReadmtKey");
  eval2.addInputFile(&inputFile2);
  eval2.addOutputFile(&outputFile2);

  // ==================================================================
  // ======================= Composition ==============================
  // ==================================================================

  // Create the composition
  TComposeEval eval;
  // Add the code one-by-one, in the right order
  eval.addEval(&eval1);
  eval.addEval(&eval2);  

  // Create the runner by providing the TComposeEval
  TSequentialRun runner(&eval);
    
  runner.startSlave();                                           
  if (runner.onMaster())
  {      
      
    // Create the launcher
    TLauncher2 lanceur(&tds, &runner);
    lanceur.solverLoop();
    
    // Stop the slave processes
    runner.stopSlave();
    
    tds.exportData("pouet.dat");
    
  }

  tds.Scan("thev1:thev2");
  
}