
// Function defined to generate randomly a good, highly-correlated, correlation matrix for inputs
// and defibe the proper covariance with the output to do as if this output if a perfect linear combination
// of the inputs
TMatrixD GenCorr(int _nX=8, bool correlated=true)
{

    gRandom->SetSeed(__rdtsc());
    //Define a randomly filled matrix 
    TMatrixD A(_nX,_nX);
    for (int i=0; i<_nX; i++)
        for (int j=0; j<_nX; j++)
            A(i,j)=gRandom->Gaus(0,1);

    // Compute AA^T and normalise it to get "covariance matrix"
    TMatrixD Gamma(A,TMatrixD::kMultTranspose,A);
    Gamma*=1./_nX;

    // Get the inverse of the diagonal matrix to do as if this was 1/sqrt(variance)
    TMatrixD Sig(_nX,_nX);
    for(int i=0;i<_nX;i++) Sig(i,i)=1./sqrt( Gamma(i,i) );

    // Compute the input correlation matrix 
    TMatrixD inCorr(Sig*Gamma,TMatrixD::kMult, Sig);
    if( !correlated )
        inCorr.UnitMatrix();
    double varY=inCorr.Sum();

    // Proper correlation, output included
    TMatrixD Corr(_nX+1,_nX+1);
    Corr.UnitMatrix();
    // putting already defined input
    Corr.SetSub(0,0,inCorr);
    // Adjust the covariance of the output wrt to all inputs
    for(unsigned int i=0; i<_nX; i++)
    {
        double value=0;
        for(unsigned int j=0; j<_nX; j++)
            value+=inCorr(i,j);
        Corr(_nX,i) = Corr(i,_nX) = value / sqrt(varY);
    }
        
    return Corr;
}


void sensitivityJohnsonRWJustCorrelationFakeFlowrate()
{      
  // Define the DataServer
  TDataServer *tds = new TDataServer("tdsflowrate", "DataBase flowrate");
  tds->addAttribute("rw");
  tds->addAttribute("r");
  tds->addAttribute("tu");
  tds->addAttribute("tl");
  tds->addAttribute("hu");
  tds->addAttribute("hl");
  tds->addAttribute("l");
  tds->addAttribute("kw");
  // outputs
  tds->addAttribute("flowrateModel");
  tds->getAttribute("flowrateModel")->setOutput();
  

  // Get the full correlation matrix
  TMatrixD inCorr=GenCorr(8,true);

  // Johnson definition  
  TJohnsonRW * tjrw = new TJohnsonRW(tds, "rw:r:tu:tl:hu:hl:l:kw", "flowrateModel");
  //Putting the newly defined correlation that states our output as a perfect linear combination of inputs
  tjrw->setCorrelationMatrix(inCorr);
  tjrw->computeIndexes();

  // Get the results on screen 
  tjrw->getResultTuple()->Scan("Out:Inp:Method:Value","Order==\"First\"");

  // Get the results as plots
  TCanvas *cc = new TCanvas("canhist", "histgramme");
  tjrw->drawIndexes("Flowrate", "", "nonewcanv,hist,first"); 
  cc->Print("appliUranieFakeFlowrateJohnsonRWCorrelationHistogram.png"); 

  TCanvas *ccc = new TCanvas("canpie", "TPie");
  tjrw->drawIndexes("Flowrate", "", "nonewcanv,pie"); 
  ccc->Print("appliUranieFakeFlowrateJohnsonRWCorrelationPie.png"); 
  
}
