//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!

\file   TRejectionABC.cxx
\author Rudy CHOCAT <rudy.chocat@cea.fr>
\date    2019-03-11

\brief   TRejectionABC function
*/

// Root
#include "TRegexp.h"
#include "TLeaf.h"

// Uranie
#include "TRejectionABC.h"

ClassImp (URANIE::Calibration::TRejectionABC);

namespace URANIE
{
namespace Calibration
{

// -----------------------------------------------------------------------
//  Constructors & Initialisation
// ---------------------------------------------------------------------

TRejectionABC::TRejectionABC(URANIE::DataServer::TDataServer *tds, URANIE::Relauncher::TRun *run,  int ns, Option_t *option):
    TABC(tds, run, ns, option)
{
}
    
TRejectionABC::TRejectionABC(URANIE::DataServer::TDataServer *tds, void (*fcn)(Double_t*,Double_t*), const char *varexpinput, const char *varexpoutput, int ns, Option_t *option):
    TABC(tds, fcn, varexpinput, varexpoutput, ns, option) 
{
}

TRejectionABC::TRejectionABC(URANIE::DataServer::TDataServer *tds, const char *fcn, const char *varexpinput, const char *varexpoutput, int ns, Option_t *option):
    TABC(tds, fcn, varexpinput, varexpoutput, ns, option)
{
}

TRejectionABC::TRejectionABC(URANIE::DataServer::TDataServer *tds, URANIE::Launcher::TCode *fcode, int ns, Option_t *option):
    TABC(tds, fcode, ns, option)
{
}


// -----------------------------------------------------------------------
//  Destructor
// -----------------------------------------------------------------------

TRejectionABC::~TRejectionABC()
{
    if (_blog)
        cout << "************ Destructor URANIE::Calibration::TRejectionABC" << endl;

    if (_blog)
        cout << "************ End of Destructor URANIE::Calibration::TRejectionABC"<< endl;
}

// -----------------------------------------------------------------------
//  Compute the parameters
// -----------------------------------------------------------------------

void TRejectionABC::computeParameters(Option_t *option)
{
    TABC::setUpABC();
    
    _nIterMax = (int)(_nSam / _Epsilon);
    try
    {
        if (!_boolEpsilon)
        {
            throw URANIE::Exceptions::UInfoExceptions(__FILE__, __LINE__,
                    Form( "TRejectionABC::computeParameters: the percentile is not specified : we fix it at %4.2f %% ! ", 100.0*_Epsilon));
        }
    } catch (URANIE::Exceptions::UInfoExceptions& ue)
    {
        ue.printMessage();
    }
    try
    {
        throw URANIE::Exceptions::UInfoExceptions(__FILE__, __LINE__,
                Form( "TRejectionABC::computeParameters:: %d evaluations will be performed ! ", _nIterMax));       
    } catch (URANIE::Exceptions::UInfoExceptions& ue)
    {
        ue.printMessage();
    }
    
    TString soption = TString(option);
    soption.ToLower();
    soption.ReplaceAll(" ", "");

    _tdsPar->deleteTuple();
    ////// generate and evaluate the sample
    TABC::computeABC(_nIterMax);
    
    ////// Evaluate the threshold by quantile
    _tdsPar->computeQuantile("agreement",_Epsilon,_threshold);
    if(_blog)
        cout << "The "<<_Epsilon<<"-Quantile is " << _threshold << endl;
    
    ////// Evaluate and generate the sample
    TABC::computeABCscore(_threshold);
    TABC::posteriorToPar();
}
} // End of namespace ABC
} // End of namespace URANIE
