//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestMain.cxx
 \author Jean-Baptiste Blanchard
 \date   2020/07/17
 
 \brief Programme principal pour les tests unitaires sur la librairie Calibration.
 
 */

#include "TDirectory.h"

// Liste des Tests Unitaires d'uranie
#include "TestTLinearBayesian.h"
#include "TestTMetropHasting.h"
#include "TestTMinimisation.h"
#include "TestTRejectionABC.h"
#include "TestTL1DistanceFunction.h"
#include "TestTLSDistanceFunction.h"
#include "TestTRelativeLSDistanceFunction.h"
#include "TestTWeightedLSDistanceFunction.h"
#include "TestTMahalanobisDistanceFunction.h"

CPPUNIT_TEST_SUITE_REGISTRATION (TLinearBayesianTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TMetropHastingTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TMinimisationTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TRejectionABCTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TL1DistanceFunctionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TLSDistanceFunctionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TRelativeLSDistanceFunctionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TWeightedLSDistanceFunctionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TMahalanobisDistanceFunctionTest);

bool uranie_test_suite(const char *module, const char *xml, const char *xsl);

int main()
{
    bool wasSuccessful;
#ifdef WIN32
    wasSuccessful = uranie_test_suite("Calibration",
                                      "testCalibrationResults.xml", "..\\..\\..\\..\\testReport.xsl");
#else
    wasSuccessful = uranie_test_suite("Calibration",
                                      "testCalibrationResults.xml", "../../../../testReport.xsl");
#endif
    
    gDirectory->ls();

    // ---  Return error code 1 if the one of test failed.
    return wasSuccessful ? 0 : 1;
}
