//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#ifndef CPPUNITEST_TREJECTIONABC_H
#define CPPUNITEST_TREJECTIONABC_H

// CppUnit
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/Test.h>

//URANIE
#include "TDataServer.h"
#include "TRejectionABC.h"
#include "TSequentialRun.h"
#include "TThreadedRun.h"
#include "TUniformDistribution.h"
#include "TNormalDistribution.h"
#include "TCJitEval.h"
#include "TCodeEval.h"
#include "TCode.h"
#include "TOutputFileRow.h"
#include "TAttribute.h"
#include "TKeyScript.h"
#include "TKeyResult.h"

using namespace URANIE::DataServer;
using namespace URANIE::Calibration;
using namespace URANIE::Relauncher;
using namespace URANIE::Launcher;

class TRejectionABCTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TRejectionABCTest);

    CPPUNIT_TEST ( runFlow1DCJitNoPlots );
    CPPUNIT_TEST ( runFlow1DCJitNoPlotsSmallNumber );
    CPPUNIT_TEST ( runFlow1DCJitNoPlotsHigherPercentile );
    CPPUNIT_TEST ( runFlow1DCJitNoPlotsHigherPercentileNoise );
    CPPUNIT_TEST ( runFlow1DCJitNoPlotsHigherPercentileStdNoise );
    CPPUNIT_TEST ( runFlow1DCJitNoPlotsSmallNumberHigherPercentile );
    CPPUNIT_TEST ( runFlow1DCodeEvalNoPlotsSmallNumberHigherPercentile );
    CPPUNIT_TEST ( runFlow1DCodeEvalThreadedNoPlotsSmallNumberHigherPercentile );
    CPPUNIT_TEST ( runFlow1DCodeNoPlotsSmallNumberHigherPercentile );
    CPPUNIT_TEST ( runFlow1DPointerFunctionNoPlots );
    CPPUNIT_TEST ( runFlow1DFunctionNoPlots );

    CPPUNIT_TEST ( throwExceptionGaussianNoise );
    CPPUNIT_TEST ( throwExceptionConstruction );
    CPPUNIT_TEST ( throwExceptionPercentile );
    CPPUNIT_TEST ( throwExceptionReference );

    CPPUNIT_TEST_SUITE_END();

public:
    
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TRejectionABCTest();

    //! Default destructor
    virtual ~TRejectionABCTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    void setUpException();
    void setUpFlow1D(string type="TCJitEval");
    void tearDown();

    void cleanWorkingDirectories();
    //@}
    //---------------------------------------------
    /** @name Unit Tests
        The list of unit test
    **/
    //@{
    void runFlow1DCJitNoPlots();
    void runFlow1DCJitNoPlotsSmallNumber();
    void runFlow1DCJitNoPlotsHigherPercentile();
    void runFlow1DCJitNoPlotsHigherPercentileNoise();
    void runFlow1DCJitNoPlotsHigherPercentileStdNoise();
    void runFlow1DCJitNoPlotsSmallNumberHigherPercentile();
    void runFlow1DCodeEvalNoPlotsSmallNumberHigherPercentile();
    void runFlow1DCodeEvalThreadedNoPlotsSmallNumberHigherPercentile();
    void runFlow1DCodeNoPlotsSmallNumberHigherPercentile();
    void runFlow1DPointerFunctionNoPlots();
    void runFlow1DFunctionNoPlots();
    
    void throwExceptionGaussianNoise();
    void throwExceptionConstruction();
    void throwExceptionPercentile();
    void throwExceptionReference();
    //@}

private:
    TString _workingTestPath;
   
    TDataServer *_tds;
    TDataServer *_tdsRef;
    TRejectionABC *_tabc;
    TSequentialRun *_seqRun;
    TThreadedRun *_thrRun;
    TCJitEval *_evalJit;
    TCodeEval *_evalCode;
    TCode *_code;
    TOutputFileRow *_outRow;
    TKeyScript *_inp;
    TKeyResult *_outp;
    TNormalDistribution *_norm;
    TAttribute *_out;

    bool _threaded;
};

#endif

