/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2019 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TEGO.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2019-10-22
 
 \brief  efficient global optimization algorithm
*/


#ifndef __EGO__
#define __EGO__

#define ID_EGO 0

// ROOT
#include "Rtypes.h"
#include "TNamed.h"
#include "TRandom.h"

// Uranie
#include "TOptimShare.h"

#include "TDataServer.h"
#include "TAttribute.h"
#include "TKriging.h"
#include "TRun.h"
#include "TEval.h"
#include "TOptimFit.h"
#include "TFitGetter.h"
#include "TVizir2.h"

// mark
#define EMPTY 0
#define FILL 1
#define PENDING 2
#define EVALUATED 3


namespace URANIE {
namespace MetaModelOptim {

class TEgoModeler
{
public:
    TEgoModeler() {};
    virtual ~TEgoModeler() {};
    
    virtual void build(URANIE::DataServer::TDataServer *, URANIE::Reoptimizer::TFitGetter *,
            vector<URANIE::DataServer::TAttribute*>&,
            URANIE::DataServer::TAttribute *, URANIE::DataServer::TAttribute *, 
            double *, int
        ) = 0;
    virtual URANIE::Modeler::TKriging *next(vector<char> &, 
        vector<vector<URANIE::DataServer::UEntry*>> &, int) = 0;
    
    ClassDef(URANIE::MetaModelOptim::TEgoModeler, ID_EGO)
};

class TEgoSolver
{
public:
    TEgoSolver() {};
    virtual ~TEgoSolver() {};

    virtual void significantEI(double) = 0;
        
    virtual void build(URANIE::Relauncher::TRun *,
            vector<URANIE::DataServer::TAttribute*>&, URANIE::DataServer::TAttribute *) = 0;
    virtual void next() = 0;
    virtual void more() = 0;
    virtual int getNext(long, int, vector<vector<URANIE::DataServer::UEntry*>> &, 
        vector<char> &) = 0;
    
    ClassDef(URANIE::MetaModelOptim::TEgoSolver, ID_EGO)
};

class TEGO : public URANIE::Reoptimizer::TOptimShare
{
protected:
    int _NK, _NC, _NR;
    URANIE::DataServer::TAttribute *_y;
    URANIE::Reoptimizer::TFitGetter *_crit;
    TEgoSolver *_solv;
    TEgoModeler *_model;

    // solver helper method
    int copyTDSin(vector<URANIE::DataServer::TAttribute*>& in,
        URANIE::DataServer::TDataServer *tds, 
        vector<vector<URANIE::DataServer::UEntry*>> &tab,
        vector<char> &mark);
    int copyTDSall(URANIE::DataServer::TDataServer *tds, 
        vector<vector<URANIE::DataServer::UEntry*>> &tab,
        vector<char> &mark);
    int init_sampling(vector<URANIE::DataServer::TAttribute*>& in,
        vector<vector<URANIE::DataServer::UEntry*>> &sampl, 
        vector<char> &mark);
    int reuse_sampling(vector<URANIE::DataServer::TAttribute*>& in,
        URANIE::DataServer::TDataServer *tds, 
        vector<vector<URANIE::DataServer::UEntry*>> &sampl, 
        vector<char> &mark, bool done);
    void prepareInput(vector<URANIE::DataServer::UEntry*> *sampl,
        vector<URANIE::DataServer::UEntry*> *in);
    void fillSample(long ev, vector<URANIE::DataServer::UEntry*> *in,
        vector<URANIE::DataServer::UEntry*> *out,
        vector<URANIE::DataServer::UEntry*> *sampl);
    double fillWithExpectedValue(vector<char> *mark, 
        vector<URANIE::DataServer::UEntry*> *sampl,
        URANIE::DataServer::TDataServer &tds2, URANIE::Modeler::TKriging *krig);


public :
    TEGO(URANIE::DataServer::TDataServer *tds, URANIE::Relauncher::TRun *run);
    TEGO(URANIE::DataServer::TDataServer *tds, URANIE::Relauncher::TEval *eval);
    virtual ~TEGO();

    // initialisation
    void setModeler(TEgoModeler *);
    void setSolver(TEgoSolver *);
    virtual void addObjective(URANIE::DataServer::TAttribute *, 
            URANIE::Reoptimizer::TOptimFit *f = NULL);
    void setSize(int, int);
    
    // solve
    virtual void solverLoop();

    ClassDef(URANIE::MetaModelOptim::TEGO, ID_EGO)
};

// 
int egocrit(double* in, double *out, void *info);
int krigeage(double *in, double *out, void *info);



} } //namespace

#endif
