// --- C++ ---
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2019 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TEgoDynSolver.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2019-10-31
 
 \brief  recherche dynamique du point suivant maximisant EI
*/

#include "TEgoDynSolver.h"

// ROOT

// Uranie

ClassImp(URANIE::MetaModelOptim::TEgoDynSolver);


#define DYNOBJ 1

// constructeur
URANIE::MetaModelOptim::TEgoDynSolver::TEgoDynSolver() :
    TEgoVzrSolver() 
{
}

URANIE::MetaModelOptim::TEgoDynSolver::~TEgoDynSolver()
{
}

// protected

void URANIE::MetaModelOptim::TEgoDynSolver::fitnessChanged()
{
    _viz->fitnessChanged();
    _viz->extend(_NG);
}

void URANIE::MetaModelOptim::TEgoDynSolver::extend()
{
    _viz->extend(_NG);
}

// public
void URANIE::MetaModelOptim::TEgoDynSolver::build(URANIE::Relauncher::TRun *run,
    vector<URANIE::DataServer::TAttribute*>& in, URANIE::DataServer::TAttribute *ego)
{
    int i;
    
    URANIE::MetaModelOptim::TEgoShareSolver::build(run, in, ego);
    // tds
    _tds = new URANIE::DataServer::TDataServer("eihjTDS", "tds for EI search via HJMA");
    _tds->keepFinalTuple(kFALSE);
    for (i=0; _in[i]; i++) _tds->addAttribute(_in[i]);
    // algo
    if (!_algo) { 
        // default
        _algo = new URANIE::Reoptimizer::TVizirGenetic();
        _algo->setSize(1200,120000, 400);
    }
    _algo->setModalDiversity(32);
    _algo->setStoppingCriteria(1);
    // vizir
    _viz = new URANIE::Reoptimizer::TVizir2 (_tds, _run, _algo);
    _viz->setTolerance(1.e-20);
    _viz->addObjective(_ego, &_maxim);
    _viz->addConstraint(_ego, &_signif);
}

int URANIE::MetaModelOptim::TEgoDynSolver::getNext(long ev, int nb, 
    vector<vector<URANIE::DataServer::UEntry*>> &sampl, vector<char> &mark)
{
    int i, j, siz, ret;
    int id, rank, info, exist;
    double front[4];
    vector<double> zone(_NinDat);
    vector<int> idx;
    vector<double> fit;

    siz = (nb < _many) ? nb+1 : _many ;
    idx.resize(siz); fit.resize(siz);
    
    // first
    id = 0;
    _viz->item(id, zone.data(), front, &rank, NULL, &info, NULL);
    idx[0] = 0;
    fit[0] = front[DYNOBJ] ;
    
    // remplissage
    for (i = 0, ret = 0; ret < siz; i ++) {
        exist = _viz->item(i, zone.data(), front, &rank, NULL, &info, NULL);
        if (exist && rank == 0 && info == 0) {
            if (front[DYNOBJ] > -_valid_ei) continue;
            for (j = ret;
                j > 0 && front[DYNOBJ] < fit[j-1];
                j--)
            {
                idx[j] = idx[j-1];
                fit[j] = fit[j-1];
            }
            idx[j] = i;
            fit[j] = front[DYNOBJ];
            ret ++;
        }
        else break;
    }
    
    // n meilleur
    for( ; 1 ; i++) {
        exist = _viz->item(i, zone.data(), front, &rank, NULL, &info, NULL);
        if (exist && rank == 0 && info == 0) {
            if (front[DYNOBJ] < fit[ret-1]) {
                // keep
                for (j = ret-2; j >= 0 && front[DYNOBJ] < fit[j]; j--) 
                {
                    idx[j+1] = idx[j];
                    fit[j+1] = fit[j];
                }
                idx[j+1] = i;
                fit[j+1] = front[DYNOBJ];
            }
        }
        else break;
    }
    
    std::cout << "select " << ret << " from " << i << std::endl;
    for (j=0; j<ret; j++, ev++) {
        (void) _viz->item(idx[j], zone.data(), front, NULL, NULL, NULL, NULL);
        std::cout << "EGO expect " << ev << " " << -fit[j] << std::endl;
        //copie
        mark[ev] = FILL;
        *sampl[ev][0]->val.dou = ev ;
        for (i=0; i<_NinDat; i++) *sampl[ev][i+1]->val.dou = zone[i];
    }
    
    return ret;
}

