// --- C++ ---
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2019 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TEgoHjStdSolver.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2019-12-13
 
 \brief  HJMA for statique EI search
*/

#include "TEgoHjStdSolver.h"

ClassImp(URANIE::MetaModelOptim::TEgoHjStdSolver);

// constructeur
URANIE::MetaModelOptim::TEgoHjStdSolver::TEgoHjStdSolver() :
    TEgoHjSolver()
{
}

URANIE::MetaModelOptim::TEgoHjStdSolver::~TEgoHjStdSolver() {}

// setup

// protected
void URANIE::MetaModelOptim::TEgoHjStdSolver::fitnessChanged()
{
    int i;
    
    if (_tds) delete _tds;
    if (_algo) delete _algo;
    
    // tds
    _tds = new URANIE::DataServer::TDataServer("eihjTDS", "tds for EI search via HJMA");
    _tds->keepFinalTuple(kFALSE);
    for (i=0; _in[i]; i++) _tds->addAttribute(_in[i]);
    // algo
    _algo = new THJMA(_tds, _run);
    _algo->setSize(_Nfirst, _Nstep);
    _algo->addObjective(_ego, &_maxim);
}

void URANIE::MetaModelOptim::TEgoHjStdSolver::extend() {}

// public
void URANIE::MetaModelOptim::TEgoHjStdSolver::build(URANIE::Relauncher::TRun *run,
    vector<URANIE::DataServer::TAttribute*>& in, URANIE::DataServer::TAttribute *ego)
{
    URANIE::MetaModelOptim::TEgoShareSolver::build(run, in, ego);
    fitnessChanged();
}

