/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2020 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TestTEgo.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2020-11-03
 
 \brief  test de l algorithme EGO
*/

#ifndef __TESTTEGO__
#define __TESTTEGO__

// CppUnit
#include <cppunit/extensions/HelperMacros.h>

// Uranie
#include "TEGO.h"
#include "TUniformDistribution.h"
#include "TAttribute.h"
#include "TCJitEval.h"

class TEgoTest : public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TEgoTest);
    
    CPPUNIT_TEST (modalHjma);

    CPPUNIT_TEST (rosEmptyDft);
    CPPUNIT_TEST (himEmptyDft);
    CPPUNIT_TEST (himAllHjmaStd);
    CPPUNIT_TEST (rosInHjmaDyn);
#ifdef WITH_VIZIR
    //CPPUNIT_TEST (modalVizir);
    //CPPUNIT_TEST (rosInVizirStd);
    CPPUNIT_TEST (himAllVizirDyn);
#endif

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     *      **/
    //@{
    //! Default constructor
    TEgoTest();

    //! Default destructor
    virtual ~TEgoTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    void tearDown();

    //@}
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void modalHjma();
    void rosEmptyDft();
    void himEmptyDft();
#ifdef WITH_VIZIR
    void modalVizir();
    void rosInVizirStd();
    void himAllVizirDyn();
#endif
    void himAllHjmaStd();
    void rosInHjmaDyn();
    //@}
    
protected:
    URANIE::DataServer::TUniformDistribution *_x1, *_x2, *_x3;
    URANIE::DataServer::TAttribute *_ros, *_y;
    URANIE::Relauncher::TCJitEval *_fun, *_funl;
    
private:
    URANIE::MetaModelOptim::TEgoModeler *regulationMod();
    void modalRoutine(bool);
    void commonRoutine(URANIE::Relauncher::TEval *,int pbl, int data,
        URANIE::MetaModelOptim::TEgoModeler *,
        URANIE::MetaModelOptim::TEgoSolver *);
    void validateRosen(URANIE::DataServer::TDataServer &tds);
    void validateHimmelblau(URANIE::DataServer::TDataServer &tds);

};


#endif
