#include <stdlib.h>
#include <math.h>
#include "linpack_d.h"
#ifdef WIN32
#include "libpgauss_Export.h"
#else
#define libpgauss_EXPORT
#endif

/* Calcul des erreurs par LOO                           */
/* Methode gpLoo1                                       */
/* Input Kin : matrice de variance                      */
/* Input yin : vecteur des observations                 */
/* Input n : dimension Kin[n x n], yin[n]               */
/* Output loo[2] : critere + variance estimee 		*/
/* loo[0] : eqm	et loo[1] = variance estimee		*/
/* Si loo[1] = -1 => echec Cholesky	  		*/
libpgauss_EXPORT int gpLoo1(double *Kin, double *yin, int n, double *loo) {
  void gpDposl ( double a[], int lda, int n, double b[] );
  int i,j,info;
  double *v, vty, sv, dv, determinant[2], dT, xk, *errLoo, *sg2Loo, vt;

  /* Recopie des donnees qui seront modifiees */
  double *K, *pK, *y, *py;
  K = calloc(sizeof(double),n*n);
  y = calloc(sizeof(double),n);
  errLoo = calloc(sizeof(double),n);
  sg2Loo = calloc(sizeof(double),n);
  for(pK = K,i=0;i<n*n;i++) *pK++ = *Kin++;
  for(py = y,i=0;i<n;i++)   *py++ = *yin++;
  /*******************************/

  v    = calloc(sizeof(double),n);
  info = dpofa (K, n, n);    /* decomposition Cholesky */
  if (info == 0) {
    for(i=0;i<n;i++)
	v[i]=1.;

    dposl(K,n,n,v);
    for(vty=0.,sv=0.,i=0;i<n;i++) {vty += v[i]*y[i], sv += v[i];} dv = vty/sv;
    dposl(K,n,n,y);
    for(i=0;i<n;i++) y[i] -= v[i] * dv;
    dtrdi(K,n,n,determinant,11); /* inverse d'une triangulaire superieure inv(L) */
    for(i=0;i<n;i++) {
      dT = - v[i] * v[i] / sv;
      for(j=i;j<n;j++) {
	xk = K[i+j*n];
	dT += xk * xk;
      }
      errLoo[i] = y[i] / dT;
      sg2Loo[i] = 1. / dT;
    }
    for(loo[0]=0., loo[1]=0., i=0;i<n;i++) {
      vt      = errLoo[i] * errLoo[i];
      loo[0] += vt;
      loo[1] += vt / sg2Loo[i];
      
    }
    loo[0] /= n;
    loo[1] /= n;
  }
  else {
    /* printf("gpLoo1 : The leading minor %d is not positive\n", info); */
    loo[0] = -1.;
    loo[1] = -1.;
  }
  free(v); v=NULL;
  free(K);  K=NULL;
  free(y);  y=NULL;
  free(errLoo);   errLoo = NULL;
  free(sg2Loo);   sg2Loo = NULL;
  return info;
}

