/*//////////////////////////////////////////////////////////////
 // Copyright (C) 2013-2024 CEA/DES
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Lesser General Public License as published
 // by the Free Software Foundation, either version 3 of the License, or any
 // later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 // $Id$
 // $Author$
 // $Date$
 // $Revision$
 // $State$
 //////////////////////////////////////////////////////////////*/
/*!
 \file   matern.c
 \author Jean-Marc Martinez <jean-marc.martinez@cea.fr>
 \date   2012/07/19 15:38:00

 \brief
 */
#include <stdlib.h>
#include <math.h>
#ifdef WIN32
#include "utilWIN32.h"
#endif
#ifdef WIN32
#include "libpgauss_Export.h"
#else
#define libpgauss_EXPORT
#endif

/* On borne nu entre 1.e-03 et 1.e05 */
/* Pour x < 1.e-20 on renvoie 1 */
/* Pour x > 1.e3   on renvoie 0 */

/* -- ngi: modifications du 25/10/2012 --
 ** - prise en compte des modification de la fonction bessik qui retourne un code d'erreur. Si 
 **   bessik ne converge pas (errorCode == 1), gpMaternFunction retourne une valeur negative (-1.0). 
 **   Sinon, elle retourne une valeur positive ou nulle, correspondant a la fonction de Matern.
 */
libpgauss_EXPORT double gpMaternFunction(double nu, double x)
{
    int bessik(double x, double nu, double *ri, double *rk, double *rip,
            double *rkp);
    double tgamma(double x);
    int errorCode;
    double ri, rk, rip, rkp;
    /* attention borner nu par exemple nu <= 10 */
    if (nu < 1.e-3)
        nu = 1.e-3;
    else if (nu > 1.e+2)
        nu = 1.e+2;
    if (x < 1.e-20)
        return ((double) 1.);
    if (x > 1.e3)
        return ((double) 0.);
    errorCode = bessik(2. * sqrt(nu) * x, nu, &ri, &rk, &rip, &rkp);
    if (errorCode == 0)
        return ((2. / tgamma(nu)) * pow(sqrt(nu) * x, nu) * rk);
    else
        return (-1.0);
}
