//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#include "anisp.hpp"
#include <stdexcept>

namespace Anisp
{

void anisp(void (*f)(std::vector<double*>), void (*step_by_step)(double*,std::vector<int*>),int d, int l, int m,
	   double tol,int kmin,
	   std::string loi,int max, int nomax,bool rep, bool log,
	   std::string ResultFile, std::string StochasticFile, std::string DataAnispFile,
	   std::string ErrorIndicatorFile, std::string IndicesFile) //Nisp utilisant l'algortithme CATP d'int�gration adaptative avec ensemble d'indices admissible (ANISP)
{ 
int n,nl;
double r,s;
int* ni = new int; // nombre d'indices de la cubature
int* nld = new int;// nombre de points de la cubature

n = int(pow(2.0,d+0.0))-1;

nl = int(pow(2.0,l+0.0))-1; 

*nld = 0; 

char* law = new char[d]; 
for(int i=0;i<d;i++)
  {
   if(loi[i]=='u') 
       law[i] = 'l';
   else{
      if(loi[i]=='n') 
          law[i] = 'h';
      else{
         if(loi[i]=='e') 
             law[i] = 'e';
        }
     } 
  }

double** Delta = new double*[nl];
double** Delta_n = new double*[nl];
for(int i=0;i<nl;i++)
  {
   Delta[i] = new double[l+1];
   Delta_n[i] = new double[l+1];
  }  

 //--------calcul de Delta -----------------------

for(int i=0;i<d;i++)
  {
   if(loi[i]=='n')
    {
     calcul_delta_n(Delta_n,l);//va normale
     break;
    }
  }

calcul_delta(Delta,l);//va uniforme

//--------fin du calcul de Delta ------------------


// vecteur contenant les points, les poids et les r�alisations de la fonction
std::vector<double*>* P = new std::vector<double*>;

// vecteur  
std::vector<double*>* H = new std::vector<double*>;

// vecteur
std::vector<double*>* N = new std::vector<double*>;

//vecteur contenant les indices angles de I
std::vector<int*>* V = new std::vector<int*>;

//vecteur contenant les numéros d'indices de l'indicateur d'erreur
std::vector<int>* O = new std::vector<int>;

//vecteur contenant les numéros des différentes tensorisations pertinentes
std::vector<int>* S = new std::vector<int>;

(*P).reserve(10000);
(*H).reserve(10000);
(*N).reserve(10000);
(*V).reserve(10000);

// vecteur contenant les cat�gories des �l�ments de la base de PC admissibles pour le calcul des coefficients
std::vector<int*>* Q = new std::vector<int*>;
(*Q).reserve(10000);


// Integration de f^2
try
{
    r = CATP_carre(f,step_by_step, d, l, m, max, tol, ni, nld, N, H, P, O, V, S, kmin,loi,rep,log,
            DataAnispFile, ErrorIndicatorFile, IndicesFile);
}
catch( anispException & )
{
 // Clean
    for (int i = 0; i < nl; i++)
    {
        delete[] Delta[i];
        Delta[i] = NULL;
        delete[] Delta_n[i];
        Delta_n[i] = NULL;
    }
    delete[] Delta;
    Delta = NULL;
    delete[] Delta_n;
    Delta_n = NULL;
    
    delete ni;
    ni = NULL;
    delete nld;
    nld = NULL;
    delete[] law;
    law = NULL;


    for (int idel = 0; idel < int((*P).size()); idel++)
    {
        delete[] (*P)[idel];
    }
    (*P).clear();
    delete P;

    for (int idel = 0; idel < int((*H).size()); idel++)
    {
        delete[] (*H)[idel];
    }
    (*H).clear();
    delete H;

    for (int idel = 0; idel < int((*N).size()); idel++)
    {
        delete[] (*N)[idel];
    }
    (*N).clear();
    delete N;

    for (int idel = 0; idel < int((*V).size()); idel++)
    {
        delete[] (*V)[idel];
    }
    (*V).clear();
    delete V;

    (*O).clear();
    delete O;

    (*S).clear();
    delete S;

    for (int idel = 0; idel < int((*Q).size()); idel++)
    {
        delete[] (*Q)[idel];
    }
    (*Q).clear();
    delete Q;
    
//////////////////
    
//    return;
    throw;

}

if(log){
    std::cout<<"integration succeeded "<<std::endl;
    
}
char nomfich[11] = "anisp.isf";
std::ofstream isf (nomfich,std::ios::out);
isf<<std::setw(25)<<std::setprecision(25)<<r<<std::endl;
isf.close();
// Tri de S par ordre croissant
stable_sort((*S).begin(),(*S).end());

if(log)
    {
    std::cout<<"number of indices:  "<<*ni<<std::endl;
    std::cout<<"total number of evaluations:  "<<*nld<<std::endl;
    std::cout<<std::setprecision(10)<<"square of norm L2:   "<<r<<std::endl;
    }

// Construction de Q      
 construction_Q( Q, S,H, P, nld, d, tol, Delta, Delta_n, l,loi)  ;

    // Clean
    for (int i = 0; i < nl; i++)
    {
        delete[] Delta[i];
        Delta[i] = NULL;
        delete[] Delta_n[i];
        Delta_n[i] = NULL;
    }
    delete[] Delta;
    Delta = NULL;
    delete[] Delta_n;
    Delta_n = NULL; 
 
/* for(int i=0;i<(*Q).size();i++)
   {
    for(int j=0;j<d;j++)
      cout<<(*Q)[i][j]<<"   ";
    cout<<endl;  
   } */

std::vector<int*> NO;   
int* vo = new int[d]; 

int no, npol, nk, a=0, b, c;
double r1,s1;
no = 0;
for(unsigned i=0;i<(*V).size();i++)
    {
     nk=0;
     for(int j=0;j<d;j++) //fejer
       {
        nk = nk + int(pow(2.0,(*V)[i][j]+0.0))-2;
        vo[j] = (int(pow(2.0,(*V)[i][j]+0.0))-2);
       }
     NO.push_back(vo);
     vo = new int[d];
     if(nk/2 > no) 
         no = nk/2;     
    }     

if(no>nomax)
    { 
        no = nomax;
        if(log){
            std::cout<<"no = nomax = "<<no<<std::endl;
            
        }
    }
else 
    if(log){
        std::cout<<"no = "<<no<<std::endl;
        
    }

SPC psi(d,law);
SPC psi1(d,law);

const char* sto = StochasticFile.c_str();
std::ofstream points (sto,std::ios::out); 

double* val = new double[*nld]; 
for(int i=0;i<*nld;i++)
  { 
   val[i] = (*P)[i][d+1] * (*P)[i][d];
   for(int j=0;j<d;j++)
      points<<std::setw(25)<<std::setprecision(20)<<(*P)[i][j]<<"     "; 
     
   points<<std::setw(25)<<std::setprecision(20)<<(*P)[i][d]<<std::endl; 
  } 



std::vector<int*> npt; 
//int* XX = new int[d]; 
int* X = new int[d];   
for(int j=0;j<d;j++)
  X[j] = 0;
npt.push_back(X);
X = new int[d];


int m_npt = 1;
int n_npt;
int rho;
for(unsigned i1=0;i1<(*Q).size();i1++)
 {
  for(int i=0;i<d;i++)
    {
     n_npt = npt.size();
     for(int j=0;j<d;j++)
       X[j] = 0;
     rho = rho_NO(NO,npt,i,0);
     a=1;
     for(int jj=0;jj<i;jj++)
       {
        if((X[jj]!=0 && X[jj]%2 == 0 && (*Q)[i1][jj]!=2) || (X[jj]%2 == 1 && (*Q)[i1][jj]!=1) || (X[jj]==0 &&  (*Q)[i1][jj]!=0) )
          {
           a = 0;
           break;
          }
       }
     if(a==1)
      {for(int k=1;k<=rho;k++)
       {
         if((k%2 == 0 && (*Q)[i1][i]==2) || (k%2 == 1 && (*Q)[i1][i]==1) )
          {
           X[i] = X[i] + k;

           npt.push_back(X);
           X = new int[d];
           for(int jj=0;jj<d;jj++)
             X[jj] = 0;
          }
       }
      }  
     for(int j=m_npt;j<n_npt;j++)
       {
         for(int jj=0;jj<d;jj++)
           X[jj] = npt[j][jj];

         rho = rho_NO(NO,npt,i,j);
         a=1;
         for(int jj=0;jj<i;jj++)
            {
             if((X[jj]!=0 && X[jj]%2 == 0 && (*Q)[i1][jj]!=2) || (X[jj]%2 == 1 && (*Q)[i1][jj]!=1) || (X[jj]==0 &&  (*Q)[i1][jj]!=0) )
               {
                a = 0;
                break;
               }
             }
         if(a==1)
          {for(int k=1;k<=rho;k++)
           {
            if((k%2 == 0 && (*Q)[i1][i]==2) || (k%2 == 1 && (*Q)[i1][i]==1))
             {
              X[i] = X[i] + k;

             npt.push_back(X);
             X = new int[d];
             for(int jj=0;jj<d;jj++)
               X[jj] = npt[j][jj]; 
             }
          }
         }
       }
    }
  m_npt = npt.size();
 }

// for(int i=0;i<npt.size();i++)
//   {
//    for(int j=0;j<d;j++)
//      cout<<npt[i][j]<<"     ";
//    cout<<endl;
//   }

// for(int i=0;i<d;i++)
//  { 
//  c=npt.size();cout<<"c "<<c<<endl;
//  for(int i1=0;i1<c;i1++)
//   {
//    a=0;
//    for(int i2=0;i2<NO.size();i2++)
//      {
//       b=1;
//       for(int i3=0;i3<d;i3++)
//         {
//          if(npt[i1][i3]>0 && (*V)[i2][i3]<=1)
//             b=0;
//         }
//       if(b==1)
//        {
//         a = IMAX(a,IMIN(NO[i2][i]/2,no));
//        }
//      }cout<<a<<endl;
//    for(int i2=1;i2<=a;i2++)
//     {
//      for(int i3=0;i3<d;i3++)
//        X[i3] = npt[i1][i3];
//      X[i] = X[i]+i2; 
//     cout<<"XX ";
//      for(int j=0;j<d;j++)
//       {
//        if(X[j]==0)
//         XX[j]=0;
//        else
//         {
// 	 if((X[j]%2)==0)
// 	   XX[j]=2;
// 	 else XX[j]=1;  
// 	}cout<<XX[j]<<"   ";
//       } cout<<endl;
// 
// //activation ou non du contrôle des symétries des PC     
// if(par1 != 0)
//    {
//      for(int i3=0;i3<(*Q).size();i3++)
//        {
//         b=1;
//         for(int j=0;j<d;j++)
//          {
//           if(XX[j]!=(*Q)[i3][j])
// 	    {
// 	     b = 0; 
// 	     break;
// 	    } 
//          }
//         if(b==1) break;
//        }
//      } 
// 
//       if(b==1)
//        {
//         npt.push_back(X);
//         X = new int[d];
//        }
//     }
//   }
// 
//  }

for(unsigned i=0;i<npt.size();i++)
  {
   b=0;
   for(int j=0;j<d;j++)
      b = b + npt[i][j];
   if(b<=nomax)
     psi.ajoute(1.,npt[i]);
  } 


psi.ordonne();
npol = psi.valeur_npol();

for(int i=0;i<npol;i++)
  psi[i] = 1;
//psi.affiche_SPC();

int** npt1; /*= new int*[npol];
for(int i=0;i<npol;i++) 
  npt1[i] = new int[d];*/
npt1 = psi.valeur_npt();


  for(int k=npol-1;k>=1;k--)
   {
    if(psi[k]==1)
     {
      for(int kk=1;kk<npol;kk++)
       {
       
        if(psi[kk]==1)
         { 
          a=0;
          for(unsigned i=0;i<(NO).size();i++)
            {  
             b=1;
             c=0;
             for(int j=0;j<d;j++)
                if((npt1[k][j]+npt1[kk][j])%2 == 1){ c=1;}

             if(c==0)
              {

               for(int j=0;j<d;j++)
                 {
                 if((npt1[k][j]+npt1[kk][j])>NO[i][j])
                  { 
                   b=0;
                   break;
                  }
                }
               }
             if(b==1)
               {
                a=1;
                break;
               }
            }
          if(a==0){ break;} 
         }
       }
       if(a==0){ psi[k] = 0;}
      }
   }

psi.filtre(1.0E-12);

    // Clean
for (int idel = 0; idel < npol; idel++)
{
    delete[] npt1[idel];
    npt1[idel] = NULL;
}
delete[] npt1;
npt1 = NULL;
//////////

npol = psi.valeur_npol();
if(log)
{
    std::cout<<"npol = "<<npol<<std::endl;
    
}  
//psi.affiche_SPC();


 for(int k=0;k<npol;k++)
   psi[k] = 0;
//psi.affiche_npt();
npt1 = psi.valeur_npt();
 std::vector<int> coef;
 coef.reserve(1000);
 a = 0;   
 //psi1.zeros();
 r1 = 0; b=0;

//calcul des coefficients beta_k
 for(int k=0;k<npol;k++)
   {
       //calcul beta_k
       s1 = 0;
       for(int i=0;i<*nld;i++)
         { 
          s = psi.eval_psi(k,(*P)[i]); 
          
	  s1 = s1 + val[i] * s ; 
         }
       if(fabs(s1)>1.0E-12){
       psi[k] = s1;a = a+1;
       r1 = r1 + s1*s1; 
       coef.push_back(k); 
        }
       else{psi[k] = 0;
       }
  
       if(r1>r) 
         {
             if(log){
             std::cout<<" r reached  "<<std::endl;
             
            }
          if(fabs(r1-r)<fabs(r1-s1*s1-r))
           {
             b = 0; 
             for(int i=0;i<d;i++)
               b = b + npt1[k][i];
            } 
          else
            {
             r1 = r1-s1*s1;
             b = 0; 
             if(k>0){
             for(int i=0;i<d;i++)
               b = b + npt1[k-1][i];
             psi[k] = 0;
             coef.pop_back(); 
             }
            }  
	  break;   
          
	 } 
       
   
   }   

if(log)
{
    std::cout<<"basis size = "<<a<<std::endl;
    
}
if(b==0)
{ 
    b = no;
    
}
if(log)
    {
    std::cout<<"degree of the polynomial expansion:  "<<b<<std::endl;
    std::cout<<"square L2 norm of the polynome:   "<<std::setprecision(12)<<r1<<std::endl;
    std::cout<<"difference of square L2 norms:  "<<fabs(r-r1)<<std::endl;
    }

s1 = 0;
for(int i=0;i<*nld;i++)
   { 
     s = psi((*P)[i])-(*P)[i][d]; 
     s1 = s1 + (*P)[i][d+1] * s * s ; 
    }
if(log)
{
    std::cout<<"approximation du carré de l'erreur "<<s1<<std::endl;
    
}
//npt1 = psi.valeur_npt();

const char* res = ResultFile.c_str();
std::ofstream resultat (res,std::ios::out); 

   resultat<<std::setw(4)<<0<<"    "<<std::setw(25)<<std::setprecision(20)<<psi[0]<<"           ";
       for(int j=0;j<d;j++)
          resultat<<std::setw(3)<<npt1[0][j]<<"    ";
       resultat<<std::endl;
       
   for(int i=1;i<npol;i++)
     {
// 	 cout<<i<<"    "<<psi[i]<<endl;
       if(fabs(psi[i])>1.0E-12)
         {
          resultat<<std::setw(4)<<calcul_num(npt1[i],d)<<"    "<<std::setw(25)<<std::setprecision(20)<<psi[i]<<"           ";
          for(int j=0;j<d;j++)
             resultat<<std::setw(3)<<npt1[i][j]<<"    ";
          resultat<<std::endl;
         } 
     }       
   resultat.close();


 // Clean
    delete ni;
    ni = NULL;
    delete nld;
    nld = NULL;
    delete[] law;
    law = NULL;
    delete[] val;
    val = NULL;

    for (int idel = 0; idel < int((*P).size()); idel++)
    {
        delete[] (*P)[idel];
    }
    (*P).clear();
    delete P;

    for (int idel = 0; idel < int((*H).size()); idel++)
    {
        delete[] (*H)[idel];
    }
    (*H).clear();
    delete H;

    for (int idel = 0; idel < int((*N).size()); idel++)
    {
        delete[] (*N)[idel];
    }
    (*N).clear();
    delete N;

    for (int idel = 0; idel < int((*V).size()); idel++)
    {
        delete[] (*V)[idel];
    }
    (*V).clear();
    delete V;

    (*O).clear();
    delete O;

    (*S).clear();
    delete S;

    for (int idel = 0; idel < int((*Q).size()); idel++)
    {
        delete[] (*Q)[idel];
    }
    (*Q).clear();
    delete Q;

    for (int idel = 0; idel < int(NO.size()); idel++)
    {
        delete[] NO[idel];
    }
    NO.clear();

    for (int idel = 0; idel < npol; idel++)
    {
        delete[] npt1[idel];
        npt1[idel] = NULL;
    }
    delete[] npt1;
    npt1 = NULL;

    for (int idel = 0; idel < int(npt.size()); idel++)
    {
        delete[] npt[idel];
        npt[idel] = NULL;
    }
    npt.clear();
    
    delete[] vo;
    vo = NULL;
    
    delete[] X;
    X = NULL;
   

   
/*
 int mc = 20000;
 time_t *adr= new time_t; 
 srand(int(time(adr)));
 delete adr;
 double* v = new double[d]; 
 r = 0;
 for(int i=0;i<mc;i++)
   {
    for(int j=0;j<d;j++)
      v[j] = 2*(rand()/(double)RAND_MAX)-1.0;
    //s = 0;
   // for(int k=0;k<npol;k++)
      s =  psi(chgt_var(v,d,loi));
    s1 = f(chgt_var(v,d,loi))- s;
    //s1 = f(chgt_var(v,d,loi))-psi(chgt_var(v,d,loi));  
    r = r +  s1*s1; 
   }
 r = r/mc;
 cout<<"erreur L2  "<<setprecision(12)<<r<<endl; */
  
/*
char don[20] = "donnee.dat";
ofstream donnee (don,ios::out); 
int mc = 50000;
donnee<<mc<<"     "<<d<<endl;
time_t *adr= new time_t; 
srand(int(time(adr)));
delete adr;
double* v = new double[d]; 

v = psi.ST();
for(int j=0;j<d;j++)
  cout<<v[j]<<endl;

for(int i=0;i<mc;i++)
   {
    for(int j=0;j<d;j++)
      {
       v[j] = 2*(rand()/(double)RAND_MAX)-1.0;
       donnee<<v[j]<<"     ";
      }
  r = psi(v);
    donnee<<r*1.0E-6<<endl; 
   }*/
// sleep(3);
}
//-------fin du main-----------------------------


void construction_Q(std::vector<int*>* Q, std::vector<int>* S, std::vector<double*>*H,  std::vector<double*>* P, int* nld, int d,double tol, double** Delta, double** Delta_n,int l,std::string loi)
{
    (void)(nld);
    (void)(tol);
 std::vector<int> vec;
 std::vector<int*> A;
 A.reserve(10000);
 int* va = new int[d+1];
 int* vb = new int[d+1];
 for(int j=0;j<d;j++)
    va[j]=0;
 int n,m,nk,it,itt;
 double s,s1; 
 

 int p,a,aa,b;
 double coef;
 double * K = new double[d];
 double * vh = new double[2];
 int nl = int(pow(2.0,l+0.0))-1;
 int * indi = new int[d];

   
 for(unsigned i=0;i<(*S).size();i++)
   {
    calcul_ind(indi,(*S)[i],d);
    for(int j=0;j<d;j++)
      {
       if(indi[j]==1)
        {
	 vec.push_back(j);
	}	 
      }
    m = vec.size();  
    n = int(pow(2.0,m+0.0))-1;
   
    
    for(int k=0;k<m;k++)
       va[vec[k]] = 1;
    va[d] = (*S)[i];  
    A.push_back(va);
    va = new int[d+1];
    for(int j=0;j<d;j++)
      va[j]=0;
    for(int j=0;j<n;j++)
      {
       calcul_ind(indi,j,m);
       for(int k=0;k<m;k++)
         {
          va[vec[k]] = indi[k]+1;
	 } 
       va[d] = (*S)[i];
       A.push_back(va);
       va = new int[d+1];
       for(int jj=0;jj<d;jj++)
         va[jj]=0;
      }  
    vec.clear();
   }
    

   
for(int i=A.size()-1;i>=0;i--)
  {
   it=0;
   for(unsigned j=0;j<(*Q).size();j++)
     {
      it = 0;
      for(int k=0;k<d;k++)
        {
	 if(A[i][k]==(*Q)[j][k]) 
         it=it+1;
	}
	if(it==d) 
        break;
     }
   if(it!=d)
   {
   it = int(A[i][d]);
   nk=1;
     for(int j=0;j<d;j++) //fejer
        {
         if(A[i][j]>0)
           {
	    nk = nk * 3;
            indi[j] = 2;
           }
         else 
             indi[j] = 1; 
	}
   
   s = 0;
   for(int j=0;j<nk;j++)
     {
      coef = 1;
	 p = nk;
	 for(int kk=0;kk<d;kk++)
	    {
             a = indi[kk];
	     b = int(pow(2.0,a+0.0))-1; //fejer
	     p = p/b;
	     aa = int(pow(2.0,l-a+0.0))*((j/p)%b +1)-1; // fejer
	     K[kk] = aa; 
             if(loi[kk]=='u')
              {
	       coef = coef* Delta[aa][a];
              }  
             else
              {
               coef = coef* Delta_n[aa][a];
              }
	    }
      vh[0] = h(K,d,nl);
      vh[1] = 0;
      itt = int(*(*(lower_bound((*H).begin(),(*H).end(),vh,inf2))+1));
      
      s1 = 1;
      for(int k=0;k<d;k++)
        {
	 if(A[i][k]==1)
	   s1 = s1*(*P)[itt][k]; 
	}
       s = s + s1 * coef * (*P)[itt][d];
       
     }
     if(fabs(s)>1.0E-12)
       { 
        for(int j=0;j<d;j++)
	 {
	  va[j] = A[i][j];
	  if(va[j]==2) 
          vec.push_back(j);
	 }  
	 va[d] = A[i][d];
        (*Q).push_back(va);
	m = vec.size();
	n = int(pow(2.0,m+0.0))-1;
        
	for(int j=0;j<int(pow(2.0,m+0.0))-1;j++)
	  {
	   for(int k=0;k<d;k++)
	     vb[k] = va[k];
           calcul_ind(indi,j,m);
	   for(int k=0;k<m;k++)
	     {
              if(indi[k]==1)
	        vb[vec[k]] = 0;
	     }   
	   for(unsigned jj=0;jj<(*Q).size();jj++)
             {
              it = 0;itt = 0;
              for(int k=0;k<d;k++)
                {
		 itt = itt + vb[k];
	         if(vb[k]==(*Q)[jj][k]) 
                 it=it+1;
	        }
	      if(it==d) break;
             }
           if(it!=d  && itt!=0)
           {
	      (*Q).push_back(vb); 
	      vb = new int[d+1]; 
           }
	  }
	va = new int[d+1];     
	vec.clear();
       }
   }  
  }
  

//Clean
    delete[] indi;
    indi = NULL;
    delete[] vh;
    vh = NULL;
    delete[] K;
    K = NULL;
    for (int idel = 0; idel < int(A.size()); idel++)
    {
        delete[] A[idel];
    }
    A.clear();
    delete[] va;
    va = NULL;
    delete[] vb;
    vb = NULL;  
  
  
  /*
  cout<<endl;  
  for(int i=0;i<A.size();i++)
    {
     for(int j=0;j<d;j++)
       cout<<A[i][j]<<"    ";
     cout<<endl;   
    }   
   
  cout<<endl;  
  for(int i=0;i<(*Q).size();i++)
    {
     for(int j=0;j<d;j++)
       cout<<(*Q)[i][j]<<"    ";
     cout<<endl;   
    }*/
     
}


//----------------------------------------------------------------------------


int rho_NO(std::vector<int*> NO, std::vector<int*> npt, int i, int j)
{
int rho = 0;
int b;

for(unsigned k=0;k < NO.size();k++)
  {
   b = 1;
   for(int l=0;l<i;l++)
    {
     if( npt[j][l] > NO[k][l]/2 )
      {
       b = 0;
       break;
      }
    }
   if(b==1)
    {
     rho = IMAX(rho,NO[k][i]/2);
    }
  }

return rho;
}


}


















