//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

/** ****************************************************** **
                    Engine (GPU)
 ** ****************************************************** **/

#include "cuda.h"

#ifndef __ENGINE_GPU_INCLUDE__
#define __ENGINE_GPU_INCLUDE__


namespace URANIE
{
  namespace GPU
  {    
    /**!\brief Initializes/shuts down all required subsystems.
       Doesn't do anything if CPU_PROCESSING_ONLY is defined. **/
    class Engine
    {
    public:
      static void Init();
      static void UpdateFreeMemory();
      static void Shutdown();
    public:
      static int _nMaxThreadsPerBlock;
      static float _nMemoryTotal;
      static float _nMemoryFree;
    private:
      explicit Engine(){} // No instanciation.
      static int _nShared;
      static bool systemInitialized;
    };
    
  } // namespace GPU
  
} // namespace URANIE

#endif // #ifndef __ENGINE_GPU_INCLUDE__

