//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////


/** ****************************************************** **
                     Errors (GPU) - Implementation
    ** ****************************************************** **/

#include "errors_gpu.h"
#include <sstream>

namespace URANIE {
  namespace Reporting {
  
    /* ************************************* */
    /*         Exception                     */
    /* ************************************* */

    Exception::Exception( const std::string err, const char* filename, size_t line )
      : mError(err), mFilename(filename), mLine(line)
    {
      std::stringstream errorMessage;
      errorMessage << "--Error (in \"" << mFilename << "\", l. " << mLine << ")" << std::endl;
      errorMessage << "\tDescription: " << mError << std::endl;
      mFullError = errorMessage.str();
    }
    
    
    Exception::~Exception() throw()
    {
    }
    
    const char* Exception::what() const throw()
    {
      return mFullError.c_str();
    }
    
  } // namespace Reporting

} // namespace URANIE

// End Of File
