//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

/** ****************************************************** **
                    Kernels (GPU)
 ** ****************************************************** **/


#ifndef __KERNELS_GPU_INCLUDE__
#define __KERNELS_GPU_INCLUDE__

typedef float Real;                             //!< GPU => Real (this is an implication, not a pretty arrow)

namespace URANIE
{
  namespace GPU
  {    
    namespace Wrappers
    {
      void gpuNormalizeSamples( dim3 numBlocks, dim3 threadsPerBlock,
				const Real *vecMin, const Real *vecMax, Real *dValue,
				const int dimN, const int dimM);
      
    } // namespace Wrappers
      
  } // namespace GPU

} // namespace URANIE


void kernel_kNN(Real* ref_host, int ref_width, Real* query_host, int query_width, int height, int k, Real* dist_host, int* ind_host);
void kernel_GP(Real* ref_host, Real* matLInv_host, int ref_width, Real* query_host, int query_width, int height, Real *min_host, Real *max_host, Real *lambda_host, Real *Coefficient_host, Real dVariance, bool blog, Real* yMean_host, Real* yVariance_host);
void kernel_GP_With_Texture(Real* ref_host, int ref_width, Real* query_host, int query_width, int height, int k, Real* yMean_host, Real* yVariance_host);

#endif // #ifndef __KERNELS_GPU_INCLUDE__


