//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   nisp_index.cpp
 \author Jean-Marc Martinez <jean-marc.martinez@cea.fr>
 \date   2011/04/07 14:33:00

 \brief

 // From ROOT::TMath::SortImp(Size n1, const Element *a, Index *index, Bool_t down)

 // Templated version of the Sort.
 //
 // Sort the n1 elements of the array a.of Element
 // In output the array index contains the indices of the sorted array.
 // If down is false sort in increasing order (default is decreasing order).
 // This is a translation of the CERNLIB routine sortzv (M101)
 // based on the quicksort algorithm.
 // NOTE that the array index must be created with a length >= n1
 // before calling this function.
 //
 // See also the declarations at the top of this file.
 */

void dindex(int n1, double *a, int *index, int down)
{
    int i, i1, n, i2, i3, i33, i222, iswap, n2;
    int i22 = 0;
    double ai;
    n = n1;
    if (n <= 0 || !a)
        return;
    if (n == 1)
    {
        index[0] = 0;
        return;
    }
    for (i = 0; i < n; i++)
        index[i] = i + 1;
    for (i1 = 2; i1 <= n; i1++)
    {
        i3 = i1;
        i33 = index[i3 - 1];
        ai = a[i33 - 1];
        while (1)
        {
            i2 = i3 / 2;
            if (i2 <= 0)
                break;
            i22 = index[i2 - 1];
            if (ai <= a[i22 - 1])
                break;
            index[i3 - 1] = i22;
            i3 = i2;
        }
        index[i3 - 1] = i33;
    }

    while (1)
    {
        i3 = index[n - 1];
        index[n - 1] = index[0];
        ai = a[i3 - 1];
        n--;
        if (n - 1 < 0)
        {
            index[0] = i3;
            break;
        }
        i1 = 1;
        while (2)
        {
            i2 = i1 + i1;
            if (i2 <= n)
                i22 = index[i2 - 1];
            if (i2 - n > 0)
            {
                index[i1 - 1] = i3;
                break;
            }
            if (i2 - n < 0)
            {
                i222 = index[i2];
                if (a[i22 - 1] - a[i222 - 1] < 0)
                {
                    i2++;
                    i22 = i222;
                }
            }
            if (ai - a[i22 - 1] > 0)
            {
                index[i1 - 1] = i3;
                break;
            }
            index[i1 - 1] = i22;
            i1 = i2;
        }
    }
    for (i = 0; i < n1; i++)
        index[i]--;
    if (!down)
        return;
    n2 = n1 / 2;
    for (i = 0; i < n2; i++)
    {
        iswap = index[i];
        index[i] = index[n1 - i - 1];
        index[n1 - i - 1] = iswap;
    }
}
