//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   nisp_random.cpp
 \author Jean-Marc Martinez <jean-marc.martinez@cea.fr>
 \date   2011/04/07 14:33:00

 \brief
 */
#ifndef WIN32 
#include <unistd.h> // pour le type pid_t
#else
#include <io.h>
#include <process.h>
#endif
#include <stdlib.h> // pour random
#include <math.h>
#include "nisp_util.h"

double betain ( double x, double p, double q, double beta, int *ifault );
double alngam ( double a, int *ifault );

void InitSeed(int sp)
{
#ifndef WIN32
    pid_t pidcrt;
	pidcrt = getpid();
#else
    int pidcrt;
	pidcrt = _getpid();
#endif
    
    unsigned int seed;
    if (sp)
        seed = (unsigned int) sp;
    else
        seed = (unsigned int) pidcrt;
#ifndef WIN32		
    srandom(seed);
#else
    srand(seed);
#endif
}

// Variable aleatoire uniforme entre 0 et 1
double RandUni()
{
#ifndef WIN32		
    return (((double) random()) / ((double) RAND_MAX));
#else
    return (((double) rand()) / ((double) RAND_MAX));
#endif
    
}

// Variable aleatoire exponentielle $exp(-x)$
double RandExp()
{
    double dum;
    do
        dum = RandUni();
    while (dum < 1.e-100);
    return -log(dum);
}

// Variable alatoire normale standardisee
double RandNor()
{
    static int flip = 0;
    static double y;
    double fac, rho, x1, x2;
    if (flip == 0)
    {
        do
        {
            x1 = 2.0 * RandUni() - 1.0;
            x2 = 2.0 * RandUni() - 1.0;
            rho = x1 * x1 + x2 * x2;
        } while (rho >= 1.0 || rho < 1.e-50);
        fac = sqrt(-2.0 * log(rho) / rho);
        y = x1 * fac;
        flip = 1;
        return x2 * fac;
    }
    else
    {
        flip = 0;
        return y;
    }
}

// Variable alatoire beta sur [0,1]
double RandBeta(double a, double b)
{
  int ifault;
  double beta_log =  alngam ( a, &ifault ) + alngam ( b, &ifault ) - alngam ( a + b, &ifault );
  double u = RandUni();
  return(betain (u, a, b, beta_log, &ifault));
}