//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTGLM.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TGLM
 
 Created : 2012-11-23 22:27:55
 
 */
#ifndef CPPUNITEST_TESTTGLM_H
#define CPPUNITEST_TESTTGLM_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "../TGLM.h"

/*! \class TGLMTest
 * \brief Unit tests for TGLM.
 */
class TGLMTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TGLMTest);
    CPPUNIT_TEST (testTGLMPoisson);
    CPPUNIT_TEST (testTGLMBinomial);
    CPPUNIT_TEST (testTGLMGamma);
    CPPUNIT_TEST (testExportModelCplusplus);

    CPPUNIT_TEST (throwExceptionTGLMSetterFamily);
    CPPUNIT_TEST (throwExceptionTGLMInitBeta);
    CPPUNIT_TEST (throwExceptionExportFunction);
    CPPUNIT_TEST (throwExceptionConstructorForTooMuchOutputs);
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TGLMTest object.
     */
    TGLMTest();

    /// Destructor.
    virtual ~TGLMTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Compute the GLM with Poisson Distribution and Log link function
    /*!
     The references values are validated with the R software
     \par R command
     \code
     tds<-read.table("capitalPunishement_R.dat",header=TRUE)
     attach(tds)
     tds
     model<-glm(exe~inc+pov+blk+logcri+sou+deg,family=poisson(link="log"), trace=TRUE)
     summary(model)
     \endcode
     and the log of results is
     \example
     > tds<-read.table("capitalPunishement_R.dat",header=TRUE)
     > attach(tds)
     > tds
     inc  pov  blk   logcri sou  deg exe CapitalPunishment__n__iter__
     1  34453 16.7 12.2 2.808886   1 0.16  37                            1
     2  41534 12.5 20.0 2.545307   1 0.27   9                            2
     3  35802 10.6 11.2 2.771587   0 0.21   6                            3
     4  26954 18.4 16.1 2.719331   1 0.16   4                            4
     5  31468 14.8 25.9 2.752048   1 0.19   3                            5
     6  32552 18.8  3.5 2.800717   0 0.25   2                            6
     7  40873 11.6 15.3 2.947434   0 0.25   2                            7
     8  34861 13.1 30.1 2.998695   1 0.21   2                            8
     9  42562  9.4  4.3 2.607455   0 0.31   1                            9
     10 31900 14.3 15.4 3.021603   1 0.24   1                           10
     11 37421  8.2  8.2 2.729974   0 0.19   1                           11
     12 33305 16.4  7.2 2.506505   0 0.16   1                           12
     13 32108 18.4 32.1 2.968016   1 0.18   1                           13
     14 45844  9.3 27.4 2.968950   0 0.29   1                           14
     15 34743 10.0  4.0 2.638489   0 0.24   1                           15
     16 29709 15.2  7.7 2.775974   0 0.21   1                           16
     17 36777 11.7  1.8 2.665581   0 0.25   1                           17
     > model<-glm(exe~inc+pov+blk+logcri+sou+deg,family=poisson(link="log"), trace=TRUE)
     Deviance = 23.32616 Iterations - 1
     Deviance = 18.79941 Iterations - 2
     Deviance = 18.59239 Iterations - 3
     Deviance = 18.59164 Iterations - 4
     Deviance = 18.59164 Iterations - 5
     > summary(model)

     Call:
     glm(formula = exe ~ inc + pov + blk + logcri + sou + deg, family = poisson(link = "log"),
     trace = TRUE)

     Deviance Residuals:
     Min        1Q    Median        3Q       Max
     -1.50517  -0.77001   0.05295   0.29638   2.97779

     Coefficients:
     Estimate Std. Error z value Pr(>|z|)
     (Intercept) -6.801e+00  4.147e+00  -1.640    0.101
     inc          2.611e-04  5.187e-05   5.034 4.81e-07 ***
     pov          7.782e-02  7.940e-02   0.980    0.327
     blk         -9.493e-02  2.292e-02  -4.142 3.44e-05 ***
     logcri       6.837e-01  1.007e+00   0.679    0.497
     sou          2.301e+00  4.284e-01   5.372 7.80e-08 ***
     deg         -1.872e+01  4.284e+00  -4.370 1.24e-05 ***
     ---
     Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

     (Dispersion parameter for poisson family taken to be 1)

     Null deviance: 136.573  on 16  degrees of freedom
     Residual deviance:  18.592  on 10  degrees of freedom
     AIC: 77.855

     Number of Fisher Scoring iterations: 5
     \endexample
     */
    void testTGLMPoisson();
    //! Compute the GLM with Binomial Distribution and Logistic link function
    /*!
     The references values are validated with the R software
     \par R command
     \code
     > cafards<-read.table("cafards.dat",header=TRUE)
     > attach(cafards)
     > cafards
     > y<-cbind(morts,total-morts)
     > y
     > model<-glm(y~dose,family=binomial(link="logit"))
     > summary(model)
     \endcode
     and the log of results is
     \example
     > cafards<-read.table("cafards.dat",header=TRUE)
     > attach(cafards)
     > cafards
     dose total morts
     1 1.691    59     6
     2 1.724    60    13
     3 1.755    62    18
     4 1.784    56    28
     5 1.811    63    52
     6 1.837    59    53
     7 1.861    62    61
     8 1.884    60    60
     > y<-cbind(morts,total-morts)
     > y
     morts
     [1,]     6 53
     [2,]    13 47
     [3,]    18 44
     [4,]    28 28
     [5,]    52 11
     [6,]    53  6
     [7,]    61  1
     [8,]    60  0
     > model<-glm(y~dose,family=binomial(link="logit"))
     > summary(model)

     Call:
     glm(formula = y ~ dose, family = binomial(link = "logit"))

     Deviance Residuals:
     Min       1Q   Median       3Q      Max
     -1.5878  -0.4085   0.8442   1.2455   1.5860

     Coefficients:
     Estimate Std. Error z value Pr(>|z|)
     (Intercept)  -60.740      5.182  -11.72   <2e-16 ***
     dose          34.286      2.913   11.77   <2e-16 ***
     ---
     Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

     (Dispersion parameter for binomial family taken to be 1)

     Null deviance: 284.202  on 7  degrees of freedom
     Residual deviance:  11.116  on 6  degrees of freedom
     AIC: 41.314

     Number of Fisher Scoring iterations: 4
     \endexample
     
     */
    void testTGLMBinomial();
    //! Compute the GLM with Gamma Distribution and canonical link function
    /*!
     The references values are validated with the R software
     \par R command
     \code
     tds<-read.table("scottish_R.dat",header=TRUE)
     attach(tds)
     tds
     model<-glm(yes~cou+unm+mor+act+gdp+age+inters,family=Gamma(), trace=TRUE)
     summary(model)
     \endcode
     and the log of results is
     \example
     > tds<-read.table("scottish_R.dat",header=TRUE)
     > attach(tds)
     > tds
     cou  unm mor  act  age   gdp  inters   yes scotland__n__iter__
     1  712 21.0 105 82.4 12.3 13566 14952.0 0.603                   1
     2  643 26.5  97 80.2 15.3 13566 17039.5 0.523                   2
     3  679 28.3 113 86.3 13.9  9611 19215.7 0.534                   3
     4  801 27.1 109 80.4 13.6  9483 21707.1 0.570                   4
     5  753 22.0 115 64.7 14.6  9265 16566.0 0.687                   5
     6  714 24.3 107 79.0 13.8  9555 17350.2 0.488                   6
     7  920 21.2 118 72.2 13.3  9611 19504.0 0.655                   7
     8  779 20.5 114 75.2 14.5  9483 15969.5 0.705                   8
     9  771 23.2 102 81.1 14.2  9483 17887.2 0.591                   9
     10 724 20.5 112 80.3 13.7 12656 14842.0 0.627                  10
     11 682 23.8  96 83.0 14.6  9483 16231.6 0.516                  11
     12 837 22.1 111 74.5 11.6 12656 18497.7 0.620                  12
     13 599 19.9 117 83.8 15.1  8298 11920.1 0.684                  13
     14 680 21.5 121 77.6 13.7  9265 14620.0 0.692                  14
     15 747 22.5 109 77.9 14.4  8314 16807.5 0.647                  15
     16 982 19.4 137 65.3 13.3  9483 19050.8 0.750                  16
     17 719 25.9 109 80.9 14.9  8298 18622.1 0.621                  17
     18 831 18.5 138 80.2 14.6  9483 15373.5 0.672                  18
     19 858 19.4 119 84.8 14.3 12656 16645.2 0.677                  19
     20 652 27.2 108 86.4 14.6 13566 17734.4 0.527                  20
     21 718 23.7 115 73.5 15.0  9483 17016.6 0.657                  21
     22 787 20.8 126 74.7 14.9  9483 16369.6 0.722                  22
     23 515 26.8 106 87.8 15.3  8298 13802.0 0.474                  23
     24 732 23.0 103 86.6 13.8  9611 16836.0 0.513                  24
     25 783 20.5 125 78.5 14.1  9483 16051.5 0.636                  25
     26 612 23.7 100 80.6 13.3  9033 14504.4 0.507                  26
     27 486 23.2 117 84.8 15.9  8298 11275.2 0.516                  27
     28 765 23.6 105 79.2 13.7  9483 18054.0 0.562                  28
     29 793 21.7 125 78.4 14.5  9483 17208.1 0.676                  29
     30 776 23.0 110 77.2 13.6  9265 17848.0 0.589                  30
     31 978 19.3 130 71.5 15.3  9483 18875.4 0.747                  31
     32 792 21.2 126 82.2 15.1 12656 16790.4 0.673                  32
     > model<-glm(yes~cou+unm+mor+act+gdp+age+inters,family=Gamma(), trace=TRUE)
     Deviance = 0.08769357 Iterations - 1
     Deviance = 0.08738852 Iterations - 2
     Deviance = 0.08738852 Iterations - 3
     Deviance = 0.08738852 Iterations - 4
     > summary(model)

     Call:
     glm(formula = yes ~ cou + unm + mor + act + gdp + age + inters,
     family = Gamma(), trace = TRUE)

     Deviance Residuals:
     Min         1Q     Median         3Q        Max
     -0.149533  -0.028272   0.002736   0.037625   0.094741

     Coefficients:
     Estimate Std. Error t value Pr(>|t|)
     (Intercept) -1.777e+00  1.148e+00  -1.548 0.134804
     cou          4.962e-03  1.622e-03   3.060 0.005381 **
     unm          2.034e-01  5.321e-02   3.824 0.000822 ***
     mor         -7.181e-03  2.712e-03  -2.648 0.014073 *
     act          1.119e-02  4.058e-03   2.757 0.010981 *
     gdp         -1.468e-05  1.237e-05  -1.187 0.246936
     age         -5.187e-02  2.403e-02  -2.159 0.041073 *
     inters      -2.427e-04  7.460e-05  -3.253 0.003373 **
     ---
     Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

     (Dispersion parameter for Gamma family taken to be 0.003584283)

     Null deviance: 0.536072  on 31  degrees of freedom
     Residual deviance: 0.087389  on 24  degrees of freedom
     AIC: -111.78

     Number of Fisher Scoring iterations: 4
     \endexample
     */
    void testTGLMGamma();
    
    //! Unit test of exportModelCplusplus
    void testExportModelCplusplus();
    
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test of type throwException
     **/
    //@{
    //!Throw an exectpion with setter attributs
    void throwExceptionTGLMSetterFamily();
    //!Throw an exectpion if the init vector of beta is not the good size
    void throwExceptionTGLMInitBeta();
    //!Throw an exception if the language is not accpeted
    void throwExceptionExportFunction();
    //!Throw an exception if there is too much output variables
    void throwExceptionConstructorForTooMuchOutputs();
    //@}

private:
    TString _workingTestPath;
    URANIE::DataServer::TDataServer *_tdsPoisson, *_tdsBinomial, *_tdsGamma;
};

#endif  // CPPUNITEST_TESTTGLM_H
