//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTKernel.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TKernel
 
 Created : 2012-12-02 23:37:34
 
 */
#ifndef CPPUNITEST_TESTTKERNEL_H
#define CPPUNITEST_TESTTKERNEL_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "../TKernel.h"

/*! \class TKernelTest
 * \brief Unit tests for TKernel.
 */
class TKernelTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TKernelTest);
    CPPUNIT_TEST (testTKernelSpline1);
    CPPUNIT_TEST (testTKernelBernoulli1);
    CPPUNIT_TEST (testTKernelSpline1ConstructorArchitecture);
    CPPUNIT_TEST (testTKernelSetKernel);
    CPPUNIT_TEST (testExportModelCplusplus);

    CPPUNIT_TEST (throwExceptionTKernelSetterKernel);
    CPPUNIT_TEST (throwExceptionTKernelTooManyOutputs);
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TKernelTest object.
     */
    TKernelTest();

    /// Destructor.
    virtual ~TKernelTest();

    void setUp();
    void tearDown();
    
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //
    //! Compute kernel with Spline1
    void testTKernelSpline1();
    //! Compute kernel with Bernoulli1
    void testTKernelBernoulli1();
    //! Compute kernel with Spline1 with the constructor using the architecture argument.
    void testTKernelSpline1ConstructorArchitecture();
    //! Unit test of setKernel methods
    void testTKernelSetKernel();
    //! Unit test of exportFunction c++
    void testExportModelCplusplus();
    //@}

    //---------------------------------------------
    /** @name throwException
     The list of unit test of type throwException
     **/
    //@{
    //! Throw an exectpion when setter the kernel
    void throwExceptionTKernelSetterKernel();
    
    //! Throw an exectpion when there is more than one output
    void throwExceptionTKernelTooManyOutputs();
    //@}

private:
    TString _workingTestPath;
    
    URANIE::DataServer::TDataServer *_tds1, *_tds2;
};

#endif  // CPPUNITEST_TESTTKERNEL_H
