//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTLinearRegression.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TLinearRegression
 
 Cr�e le : mer nov 21 07:52:44 2007
 
 */
#ifndef CPPUNITEST_TESTTLINEARREGRESSION_H
#define CPPUNITEST_TESTTLINEARREGRESSION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "../TLinearRegression.h"

/*! \class TLinearRegressionTest
 * \brief Unit tests for TLinearRegression.
 */
class TLinearRegressionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TLinearRegressionTest);
    CPPUNIT_TEST (testLinearRegressionSteamplantDatabaseUnivariate);
    CPPUNIT_TEST (testLinearRegressionConstructorArchitecture);
    CPPUNIT_TEST (testLinearRegressionSteamplantDatabaseMultivariate);
    CPPUNIT_TEST (testLinearRegressionCASPDatabaseMultivariateBigSize);
    //   FGA :: 12/04/2013 14:12  CPPUNIT_TEST( testLinearRegressionSteamplantDatabaseMultivariateArchiNoIntercept );
    CPPUNIT_TEST (testLinearRegressionDataQ2NoIntercept);
    CPPUNIT_TEST (testLinearRegressionEffortDatabase);
    CPPUNIT_TEST (testLinearRegressionMileageDatabase);
    CPPUNIT_TEST (testexportFunctionC);
    CPPUNIT_TEST (testexportFunctionF);
    CPPUNIT_TEST (testgetInputName);
#ifdef  WITH_LIBXML2
    CPPUNIT_TEST( testExportModelPMML);
    CPPUNIT_TEST( testExportModelPMMLBackUp);
    CPPUNIT_TEST( testLinearRegressionConstructorPmml);
#endif

    CPPUNIT_TEST (throwExceptionTLinearRegressionTdsNull);
    CPPUNIT_TEST (throwExceptionTLinearRegressionOutputFalse);
    CPPUNIT_TEST (throwExceptionTLinearRegressionInputFalse);
    CPPUNIT_TEST (throwExceptionestimateTdsMultiOutput);
    CPPUNIT_TEST (throwExceptionExportFunction);
    CPPUNIT_TEST (throwExceptiongetInputNameIndiceSup);

    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TLinearRegressionTest object.
     */
    TLinearRegressionTest();

    /// Destructor.
    virtual ~TLinearRegressionTest();

    void setUp();
    void tearDown();
    
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Compute the Linear Regression for the Steamplant database with univariate and test the R2 : 
    /*!
     The references values are validated with the R software
     \par R command
     \code
     mydata <- read.table("steamplant.dataR")
     summary(mydata)
     ssrc <-  lm(V1~V8,mydata)
     summary(ssrc)
     ssrc <-  lm(V2~V8-1,mydata)
     summary(ssrc)
     \endcode
     and the log of results is
     \example
     >   summary(ssrc)

     Call:
     lm(formula = V1 ~ V8, data = mydata)

     Residuals:
     Min      1Q  Median      3Q     Max
     -1.6789 -0.5291 -0.1221  0.7988  1.3457

     Coefficients:
     Estimate Std. Error t value Pr(>|t|)
     (Intercept) 13.62299    0.58146  23.429  < 2e-16 ***
     V8          -0.07983    0.01052  -7.586 1.05e-07 ***
     ---
     Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

     Residual standard error: 0.8901 on 23 degrees of freedom
     Multiple R-Squared: 0.7144,     Adjusted R-squared: 0.702
     F-statistic: 57.54 on 1 and 23 DF,  p-value: 1.055e-07
     \endexample
     \example
     >   summary(ssrc)

     Call:
     lm(formula = V2 ~ V8 - 1, data = mydata)

     Residuals:
     Min      1Q  Median      3Q     Max
     -3.5258 -0.9251  0.7488  1.8903  3.3953

     Coefficients:
     Estimate Std. Error t value Pr(>|t|)
     V8 0.093760   0.006834   13.72 7.45e-13 ***
     ---
     Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

     Residual standard error: 1.888 on 24 degrees of freedom
     Multiple R-Squared: 0.8869,     Adjusted R-squared: 0.8822
     F-statistic: 188.2 on 1 and 24 DF,  p-value: 7.45e-13
     \endexample
     
     */
    void testLinearRegressionSteamplantDatabaseUnivariate();

    
    //! Unit test of the constructor TLinearRegression(URANIE::DataServer::TDataServer *tds, TString architecture,       Option_t *option)
    //! This test is the same that testLinearRegressionSteamplantDatabaseUnivariate apart from the constructor
    void testLinearRegressionConstructorArchitecture();
    
    //! Compute the Linear Regression for the Steamplant database with multivariate
    /*!
     The references values are validated with the R software
     \par R command
     \code
     mydata <- read.table("steamplant.dataR")
     summary(mydata)
     ssrc <-  lm(V1~V2+V3+V4+V5+V6+V7+V8+V9+V10,mydata)
     summary(ssrc)
     \endcode
     and the log of results is
     \example
     >   summary(ssrc)

     Call:
     lm(formula = V1 ~ V2 + V3 + V4 + V5 + V6 + V7 + V8 + V9 + V10,
     data = mydata)

     Residuals:
     Min       1Q   Median       3Q      Max
     -1.23126 -0.26697  0.07533  0.30451  0.70427

     Coefficients:
     Estimate Std. Error t value Pr(>|t|)
     (Intercept)  1.89421    6.99636   0.271 0.790279
     V2           0.70541    0.56490   1.249 0.230903
     V3          -1.89372    4.14629  -0.457 0.654412
     V4           1.13422    0.74609   1.520 0.149253
     V5           0.11876    0.20461   0.580 0.570247
     V6           0.17935    0.08095   2.216 0.042611 *
     V7          -0.01818    0.02451  -0.742 0.469699
     V8          -0.07742    0.01659  -4.666 0.000304 ***
     V9          -0.08585    0.05200  -1.651 0.119529
     V10         -0.34501    0.21070  -1.637 0.122337
     ---
     Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

     Residual standard error: 0.5697 on 15 degrees of freedom
     Multiple R-Squared: 0.9237,     Adjusted R-squared: 0.8779
     F-statistic: 20.18 on 9 and 15 DF,  p-value: 7.97e-07
     \endexample
     */
    void testLinearRegressionSteamplantDatabaseMultivariate();
    
    
     //! Compute the Linear Regression for the CASP database with multivariate 
     //! Big size : npatterns >  SVD_LIMIT => choleski method used
    /*!
     The references values are validated with the R software
     \par R command
     \code
     mydata <- read.table("CASP.data")
     summary(mydata)
     ssrc <-  lm(V1~V2+V3+V4+V5+V6+V7+V8+V9+V10,mydata)
     summary(ssrc)
     \endcode
     and the log of results is
     \example
     >   summary(ssrc)

        Call:
        lm(formula = V1 ~ V2 + V3 + V4 + V5 + V6 + V7 + V8 + V9 + V10, 
            data = mydata)

        Residuals:
            Min       1Q   Median       3Q      Max 
        -24.1639  -4.0934  -0.9466   3.8620  20.0977 

        Coefficients:
                    Estimate Std. Error t value Pr(>|t|)    
        (Intercept)  6.037e+00  5.876e-01  10.274  < 2e-16 ***
        V2           1.572e-03  1.231e-04  12.764  < 2e-16 ***
        V3           1.429e-03  1.130e-04  12.653  < 2e-16 ***
        V4           1.803e+01  1.065e+00  16.927  < 2e-16 ***
        V5          -1.082e-01  1.645e-03 -65.773  < 2e-16 ***
        V6          -4.075e-06  7.550e-07  -5.397 6.79e-08 ***
        V7          -2.388e-02  1.752e-03 -13.628  < 2e-16 ***
        V8          -1.387e-04  1.481e-05  -9.361  < 2e-16 ***
        V9           1.485e-02  5.933e-04  25.035  < 2e-16 ***
        V10         -1.101e-01  1.000e-02 -11.013  < 2e-16 ***
        ---
        Signif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1 

        Residual standard error: 5.184 on 45720 degrees of freedom
        Multiple R-squared: 0.2823,     Adjusted R-squared: 0.2822 
        F-statistic:  1998 on 9 and 45720 DF,  p-value: < 2.2e-16 
     \endexample
     */
    void testLinearRegressionCASPDatabaseMultivariateBigSize();

    //! Compute the Linear Regression on the "testQ2.dat" dataset with option "nointercept".
    /*!
     Values found by Jump from SAS for the R2, R2a and Q2 values
     
     R2=0.9832
     R2a=0.9645
     Q2=0.8939
     */
    void testLinearRegressionDataQ2NoIntercept();

    //! Compute the Linear Regression on the "stemplant" dataset with option "nointercept".
    /*!
     The references values are validated with the R software
     \par R command
     \code
     mydata <- read.table("steamplant.dataR")
     summary(mydata)
     ssrc <-  lm(V1~V6+V8-1,mydata)
     summary(ssrc)
     \endcode
     and the log of results is
     \example
     >   summary(ssrc)

     Call:
     lm(formula = V1 ~ V6 + V8 - 1, data = mydata)

     Residuals:
     Min       1Q   Median       3Q      Max
     -1.53575 -0.86964 -0.03415  0.92259  3.24852

     Coefficients:
     Estimate Std. Error t value Pr(>|t|)
     V6  0.55130    0.03558  15.497 1.15e-13 ***
     V8 -0.03547    0.01317  -2.693   0.0130 *
     ---
     Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1

     Residual standard error: 1.312 on 23 degrees of freedom
     Multiple R-Squared: 0.9827,     Adjusted R-squared: 0.9812
     F-statistic: 651.7 on 2 and 23 DF,  p-value: < 2.2e-16
     \endexample
     */
    void testLinearRegressionSteamplantDatabaseMultivariateArchiNoIntercept();

    //! Compute the linear regression from the effort database in wws509 web site
    /*!
     \par The Program Effort DataSet
     The dataset comes from Mauldin and Berelson (1978) and contains an index of social setting, an index of family planning effort, and the percent decline in the crude birth rate (CBR)-the number of births per thousand population-between 1965 and 1975, for 20 countries in Latin America and the Caribbean.

     Setting 	Effort 	CBR-Decline
     Bolivia 	 46 	0 	1
     Brazil 	         74 	0 	10
     Chile 	         89 	16 	29
     Colombia 	 77 	16 	25
     CostaRica 	 84 	21 	29
     Cuba 	         89 	15 	40
     Dominican Rep 	 68 	14 	21
     Ecuador 	 70 	6 	0
     El Salvador 	 60 	13 	13
     Guatemala 	 55 	9 	4
     Haiti 	         35 	3 	0
     Honduras 	 51 	7 	7
     Jamaica 	 87 	23 	21
     Mexico 	         83 	4 	9
     Nicaragua 	 68 	0 	7
     Panama 	         84 	19 	22
     Paraguay 	 74 	3 	6
     Peru 	         73 	0 	2
     Trinidad-Tobago  84 	15 	29
     Venezuela 	 91 	7 	11

     The references values come from the \ref http://data.princeton.edu/wws509/stata/c2s5.html
     \example
     . regress change setting effort
     
     Source |       SS       df       MS              Number of obs =      20
     -------------+------------------------------           F(  2,    17) =   23.96
     Model |  1956.19433     2  978.097163           Prob > F      =  0.0000
     Residual |  694.005675    17  40.8238632           R-squared     =  0.7381
     -------------+------------------------------           Adj R-squared =  0.7073
     Total |      2650.2    19  139.484211           Root MSE      =  6.3894
     
     ------------------------------------------------------------------------------
     change |      Coef.   Std. Err.      T    P>|t|     [95% Conf. Interval]
     -------------+----------------------------------------------------------------
     setting |   .2705885   .1079405     2.51   0.023      .042854     .498323
     effort |   .9677137   .2250074     4.30   0.000     .4929895    1.442438
     _cons |   -14.4511   7.093841    -2.04   0.058    -29.41779    .5155975
     ------------------------------------------------------------------------------
     \endexample
     */
    void testLinearRegressionEffortDatabase();

    //! Compute the linear regression with second order attribute from the mileage dataset from SAS User Guide
    /*!
     The references values come from the Stat SAS User guide \href http://support.sas.com/documentation/cdl/en/statug/63033/HTML/default/viewer.htm#statug_glm_sect047.htm
     \example
     A car is tested for gas mileage at various speeds to determine at what speed the car achieves the highest gas mileage. A quadratic model is fit to the experimental data.
     The following statements produce Output 39.2.1 through Output 39.2.4.

     title 'Gasoline Mileage Experiment';
     data mileage;
     input mph mpg @@;
     datalines;
     20 15.4
     30 20.2
     40 25.7
     50 26.2  50 26.6  50 27.4
     55   .
     60 24.8
     ;

     Source 	DF 	Sum of Squares 	Mean Square 	F Value 	Pr > F
     Model 	2 	111.8086183 	55.9043091 	77.96 	0.0006
     Error 	4 	2.8685246 	0.7171311 	  	 
     Corrected Total 	6 	114.6771429 	  	  	 

     R-Square 	Coeff Var 	Root MSE 	mpg Mean
     0.974986 	3.564553 	0.846836 	23.75714

     Source 	DF 	Type I SS 	Mean Square 	F Value 	Pr > F
     mph 	1 	85.64464286 	85.64464286 	119.43 	0.0004
     mph*mph 	1 	26.16397541 	26.16397541 	36.48 	0.0038

     Source 	DF 	Type III SS 	Mean Square 	F Value 	Pr > F
     mph 	1 	41.01171219 	41.01171219 	57.19 	0.0016
     mph*mph 	1 	26.16397541 	26.16397541 	36.48 	0.0038

     Parameter 	Estimate 	Standard Error 	t Value 	Pr > |t|
     Intercept 	-5.985245902 	3.18522249 	-1.88 	0.1334
     mph 	         1.305245902 	0.17259876 	 7.56 	0.0016
     mph*mph 	-0.013098361 	0.00216852 	-6.04 	0.0038

     \endexample
     */
    void testLinearRegressionMileageDatabase();

    //!Control taht the code write corectly the export file data - C language
    void testexportFunctionC();
    //!Control taht the code write corectly the export file data - Fortran language
    void testexportFunctionF();
    //!Control that the good nae is returned
    void testgetInputName();

    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test of type throwException
     **/
    //@{
    //!Throw an exectpion if the parmaeter tds is null
    void throwExceptionTLinearRegressionTdsNull();
    //!Throw an exception if the output variable doesn't exist in tds
    void throwExceptionTLinearRegressionOutputFalse();
    //!Throw an exception if the input variable doesn't exist in tds
    void throwExceptionTLinearRegressionInputFalse();
    //!Throw an exception if the input variable doesn't exist in tds
    void throwExceptionestimateTdsMultiOutput();
    //!Throw an exception if the language is not accpeted
    void throwExceptionExportFunction();
    //!Throw an exception if the indice is superior to number of input
    void throwExceptiongetInputNameIndiceSup();

#ifdef  WITH_LIBXML2
    //! Unit test of ExportFunction in PMML format
    void testExportModelPMML();
    //! Unit test of ExportFunction in PMML format for default option backup
    void testExportModelPMMLBackUp();
    //! Unit test of constructor using PMML format
    void testLinearRegressionConstructorPmml();
#endif

    //@}
 
private:
    URANIE::DataServer::TDataServer *tds1;
    URANIE::DataServer::TDataServer *tds2;
    URANIE::Modeler::TLinearRegression *_tlr;
};

#endif  // CPPUNITEST_TESTTLINEARREGRESSION_H
