//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestTPolynomialChaos.h
 \author Maud Salvoldi (OCC)
 \date   08/04/2011
 
 \brief Unit tests ot the class TPOlynomialChaos
 
 */
#ifndef CPPUNITEST_TESTTPOLYNOMIALCHAOS_H
#define CPPUNITEST_TESTTPOLYNOMIALCHAOS_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// uranie
#include "TNisp.h"
#include "TPolynomialChaos.h"
#include "TLauncherFunction.h"
#include "TLauncher.h"
/**

 * @class TestTPolynomialChaos
 *
 * @brief Description of the class TestTPolynomialChaos.
 *
 *  The class TestTPolynomialChaos is the unit test class of the class TPolynomialChaos. 
 *  
 *
 */

class TPolynomialChaosTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TPolynomialChaosTest);
    CPPUNIT_TEST (testgetDegree);
    CPPUNIT_TEST (testsetDegree);
    CPPUNIT_TEST (throwExceptionsetDegree);
    CPPUNIT_TEST (throwExceptionTPolynomialChaosTdsNull);
    CPPUNIT_TEST (throwExceptionTPolynomialChaosTNispNull);
    CPPUNIT_TEST (throwExceptionTPolynomialChaosTdsNotSame);
    CPPUNIT_TEST (throwExceptionTPolynomialChaosOutputNotExist);
    CPPUNIT_TEST (testTPolynomialChaosListOutput);
    CPPUNIT_TEST (testComputeOutputGetOuput);
    CPPUNIT_TEST (throwExceptionGetOuputJNegative);
    CPPUNIT_TEST (throwExceptionGetOuputJExceed);
    CPPUNIT_TEST (throwExceptionGetOuputNamefalse);
    CPPUNIT_TEST (testcomputeChaosExpansionIntegration);
    CPPUNIT_TEST (testcomputeChaosExpansionIntegrationWithCopy);
    CPPUNIT_TEST (testcomputeChaosExpansionIntegrationOnCopyWithLaguerre);
    CPPUNIT_TEST (testcomputeChaosExpansionIntegrationWithTPetras);
    CPPUNIT_TEST (testcomputeChaosExpansionIntegrationWithImportedDatas);
    CPPUNIT_TEST (testcomputeChaosExpansionRegression);
    CPPUNIT_TEST (testcomputeChaosExpansionRegressionAutomatised);
    CPPUNIT_TEST (testcomputeChaosExpansionRegressionWithImportedDatas);
    CPPUNIT_TEST (throwExceptioncomputeChaosExpansion);
    CPPUNIT_TEST (testgetMean);
    CPPUNIT_TEST (throwExceptiongetMeanFalseName);
    CPPUNIT_TEST (throwExceptiongetMeanJNegative);
    CPPUNIT_TEST (throwExceptiongetMeanJExceed);
    CPPUNIT_TEST (testgetVariance);
    CPPUNIT_TEST (throwExceptiongetVarianceFalseName);
    CPPUNIT_TEST (throwExceptiongetVarianceJNegative);
    CPPUNIT_TEST (throwExceptiongetVarianceJExceed);
    CPPUNIT_TEST (testgetCorrelation);
    CPPUNIT_TEST (throwExceptiongetCorrelationJnameFalse);
    CPPUNIT_TEST (throwExceptiongetCorrelationInameFalse);
    CPPUNIT_TEST (throwExceptiongetCorrelationInegative);
    CPPUNIT_TEST (throwExceptiongetCorrelationJnegative);
    CPPUNIT_TEST (throwExceptiongetCorrelationIexceed);
    CPPUNIT_TEST (throwExceptiongetCorrelationJexceed);
    CPPUNIT_TEST (testgetDimensionInput);
    CPPUNIT_TEST (testgetDimensionOutput);
    CPPUNIT_TEST (testgetDimensionExpansion);
    CPPUNIT_TEST (testexportFunction);
    CPPUNIT_TEST (testgetCovariance);
    CPPUNIT_TEST (throwExceptiongetCovarianceJnameFalse);
    CPPUNIT_TEST (throwExceptiongetCovarianceInameFalse);
    CPPUNIT_TEST (throwExceptiongetCovarianceInegative);
    CPPUNIT_TEST (throwExceptiongetCovarianceJnegative);
    CPPUNIT_TEST (throwExceptiongetCovarianceIexceed);
    CPPUNIT_TEST (throwExceptiongetCovarianceJexceed);
    CPPUNIT_TEST (testgetIndexFirstOrder);
    CPPUNIT_TEST (throwExceptiongetIndexFirstOrderJnameFalse);
    CPPUNIT_TEST (throwExceptiongetIndexFirstOrderInameFalse);
    CPPUNIT_TEST (throwExceptiongetIndexFirstOrderInegative);
    CPPUNIT_TEST (throwExceptiongetIndexFirstOrderJnegative);
    CPPUNIT_TEST (throwExceptiongetIndexFirstOrderIexceed);
    CPPUNIT_TEST (throwExceptiongetIndexFirstOrderJexceed);
    CPPUNIT_TEST (testgetIndexTotalOrder);
    CPPUNIT_TEST (throwExceptiongetIndexTotalOrderJnameFalse);
    CPPUNIT_TEST (throwExceptiongetIndexTotalOrderInameFalse);
    CPPUNIT_TEST (throwExceptiongetIndexTotalOrderInegative);
    CPPUNIT_TEST (throwExceptiongetIndexTotalOrderJnegative);
    CPPUNIT_TEST (throwExceptiongetIndexTotalOrderIexceed);
    CPPUNIT_TEST (throwExceptiongetIndexTotalOrderJexceed);
    CPPUNIT_TEST (testgetIndex);
    CPPUNIT_TEST (throwExceptiongetIndexJnameFalse);
    CPPUNIT_TEST (throwExceptiongetIndexSInputFalse);
    CPPUNIT_TEST (testgetIndexInteraction);
    CPPUNIT_TEST (throwExceptiongetIndexInteractionJnameFalse);
    CPPUNIT_TEST (throwExceptiongetIndexInteractionSInputFalse);
    CPPUNIT_TEST (getAnovaOrdered);
    CPPUNIT_TEST (throwExceptiongetAnovaOrdered);
    CPPUNIT_TEST (throwExceptionNPatternsDifferentNispTds);
    CPPUNIT_TEST (throwExceptionConstructorByCopyWithIndisponibleLaw);
    CPPUNIT_TEST (throwExceptionSetGroupAddVarWithRankTooHigh);
    CPPUNIT_TEST (throwExceptionSetCoefficientNumberOfOutputTooHigh);
    CPPUNIT_TEST (throwExceptionSetCoefficientNumberOfPCTooHigh);
    CPPUNIT_TEST (throwExceptionGetCoefficientNumberOfOutputTooHigh);
    CPPUNIT_TEST (throwExceptionGetCoefficientNumberOfPCTooHigh);
    CPPUNIT_TEST (throwExceptiongetAnovaOrderedCoefficients);
    CPPUNIT_TEST (throwExceptiongetSampleWrongOutputName);
//     CPPUNIT_TEST (testGetSampleWithOutputName);
    CPPUNIT_TEST (throwExceptiongetQuantileWrongOutputName);
    CPPUNIT_TEST (throwExceptionGetCoefficientWithOutputName);
    CPPUNIT_TEST (testGetQuantiles);
    CPPUNIT_TEST (throwExceptionGetAnovaOrdered);
    CPPUNIT_TEST (throwExceptionGetSample);
    CPPUNIT_TEST (throwExceptionGetEqmLoo);
    CPPUNIT_TEST (throwExceptionGetErrLoo);
    CPPUNIT_TEST (throwExceptionSetAutoDegreeBoundaries);
    CPPUNIT_TEST (throwExceptionSetAutoDegreeFactor);
    CPPUNIT_TEST_SUITE_END();

public:

    /*! Constructs a TPolynomialChaosTest object.
     */
    TPolynomialChaosTest();
    /*! Destructor.
     */
    virtual ~TPolynomialChaosTest();
    //---------------------------------------------
    /** @name setUp and Tear Down methods
     **/
    //@{
    //! Build a object TNisp from a tds object
    void setUp();
    //! ! Delete the object tds and nips
    void tearDown();
    //@}

//---------------------------------------------
    /** @name Units tests methods
     **/
    //@{
    //! Unit test of method getDegree
    void testgetDegree();
    //! Unit test of method getDegree  
    void testsetDegree();
    //! Unit test of method getDegree
    /*!
     \brief: Test that the code throws an exception is degree is null or negative
     */
    void throwExceptionsetDegree();
    //! Unit test of the constructor
    /*!
     \brief Test that the code throws an exception if the parameter tds is null.
     */
    void throwExceptionTPolynomialChaosTdsNull();
    //! Unit test of the constructor TPolynomialChaos(tds,nips)
    /*!
     \brief Test that the code throws an exception if the parameter tnips is null.
     */
    void throwExceptionTPolynomialChaosTNispNull();
    //! Unit test of the constructor TPolynomialChaos(tds,nips)
    /*!
     \brief Test that the code throws an exception if the parameter tds in not the same taht the tds in the tnisp parameter.
     */
    void throwExceptionTPolynomialChaosTdsNotSame();
    //! Unit test of the constructor TPolynomialChaos(tds,nips)
    /*!
     \brief Test that the code throws an exception if the parameter ouput don't exist.
     */
    void throwExceptionTPolynomialChaosOutputNotExist();
    //! Unit test of the constructor TPolynomialChaos(tds,nips)
    /*!
     \brief Test that the code built the same PC if we give the output in parameter that if not
     */
    void testTPolynomialChaosListOutput();
    //! Unit test of the method computeOutput(v) and getOutput(j)
    /*!
     \brief Control that the method computeOuput compute correclty the output of the polynomial chaos.
     */
    void testComputeOutputGetOuput();
    //! Unit test of getOuput(j)
    /*!
     \brief Test that the code throws an exception if the ouput index is negative.
     */
    void throwExceptionGetOuputJNegative();
    //! Unit test of getOuput(j)
    /*!
     \brief Test that the code throws an exception if the ouput index is exceed.
     */
    void throwExceptionGetOuputJExceed();
    //! Unit test of getOuput(jname)
    /*!
     \brief Test that the code throws an exception if the ouput name is false.
     */
    void throwExceptionGetOuputNamefalse();
    //! Unit test of computeChaosExpansion(methodology)
    /*!
     \brief Test that the code compute correclty the coefficients of the polynomial chaos with the methodology "Integration".
     */
    void testcomputeChaosExpansionIntegration();
    
    //! Unit test of computeChaosExpansion(methodology)
    /*!
     \brief Test that the code compute correclty the coefficients of the polynomial chaos with the methodology "Integration" with a TPolynomialChaos constructed by copy.
     */
    void testcomputeChaosExpansionIntegrationWithCopy();   
    //! Unit test of computeChaosExpansion(methodology)
    /*!
     \brief Test that the code compute correclty the coefficients of the polynomial chaos with the methodology "Integration" with a TPolynomialChaos constructed by copy with one laguerre polynomial.
     */
    void testcomputeChaosExpansionIntegrationOnCopyWithLaguerre();
    
    //! Unit test of computeChaosExpansion(methodology)
    /*!
     \brief Test that the code compute correclty the coefficients of the polynomial chaos with the methodology "Integration" and a sample generated with TPetras.
     */
    void testcomputeChaosExpansionIntegrationWithTPetras();
    
    //! Unit test of computeChaosExpansion(methodology)
    /*!
     \brief Test that the code compute correclty the coefficients of the polynomial chaos with the methodology "Integration" and datas imported from a file.
     */
    void testcomputeChaosExpansionIntegrationWithImportedDatas();
    
    //! Unit test of computeChaosExpansion(methodology)
    /*!
     \brief Test that the code compute correclty the coefficients of the polynomial chaos with the methodology "Regression".
     */
    void testcomputeChaosExpansionRegression();
   //! Unit test of computeChaosExpansion for automatsed degree(methodology)
    /*!
     \brief Test that the code compute correclty the coefficients of the polynomial chaos with the methodology "Regression" when asking to scan the degree
     */
    void testcomputeChaosExpansionRegressionAutomatised();
     //! Unit test of computeChaosExpansion(methodology)
    /*!
     \brief Test that the code compute correclty the coefficients of the polynomial chaos with the methodology "Regression" with imported datas from a file.
     */
    void testcomputeChaosExpansionRegressionWithImportedDatas();
    //! Unit test of computeChaosExpansion(methodology)
    /*!
     \brief Test that the code throws an exception if the methology name is false.
     */
    void throwExceptioncomputeChaosExpansion();
    //! Unit test of getMean(j/jname)
    /*!
     \brief Test that the code compute correclty the mean of the polynomial chaos.
     */
    void testgetMean();
    //! Unit test of getMean(jname)
    /*!
     \brief Test that the code throws an exception is the name of the ouput is false. The value of the mean returned is NULL.
     */
    void throwExceptiongetMeanFalseName();
    //! Unit test of getMean(j)
    /*!
     \brief Test that the code throws an exception is the index of the ouput is negative. The value of the mean returned is NULL.
     */
    void throwExceptiongetMeanJNegative();
    //! Unit test of getMean(j)
    /*!
     \brief Test that the code throws an exception is the index of the ouput is exceed. The value of the mean returned is NULL.
     */
    void throwExceptiongetMeanJExceed();
    //! Unit test of getVariance(j/jmane)
    /*!
     \brief Test that the code compute correctly the variance of the polynomial chaos.
     */
    void testgetVariance();
    //! Unit test of getVariance(jname)
    /*!
     \brief Test that the code throws an exception is the name of the output is false. The value of the variance returned is NULL.
     */
    void throwExceptiongetVarianceFalseName();
    //! Unit test of getMean(j)
    /*!
     \brief Test that the code throws an exception is the index of the output is negative. The value of the variance returned is NULL.
     */
    void throwExceptiongetVarianceJNegative();
    //! Unit test of getVariance(j)
    /*!
     \brief Test that the code throws an exception is the index of the output is exceed. The value of the variance returned is NULL.
     */
    void throwExceptiongetVarianceJExceed();
    //! Unit test of getCorrelation(i/iname, j/jname)
    /*!
     \brief Test that the code copute correctly the correlation between two outputs.
     */
    void testgetCorrelation();
    //! Unit test of getCorrelation(iname,jname)
    /*!
     \brief Test that the code throws an exception is the name of the second output is false. The value of the correlation returned is NULL.
     */
    void throwExceptiongetCorrelationJnameFalse();
    //! Unit test of getCorrelation(iname,jname)
    /*!
     \brief Test that the code throws an exception is the name of the first output is false. The value of the correlation returned is NULL.
     */
    void throwExceptiongetCorrelationInameFalse();
    //! Unit test of getCorrelation(i,j)
    /*!
     \brief Test that the code throws an exception is the index  of the first output is negative. The value of the correlation returned is NULL.
     */
    void throwExceptiongetCorrelationInegative();
    //! Unit test of getCorrelation(i,j)
    /*!
     \brief Test that the code throws an exception is the index  of the second output is negative. The value of the correlation returned is NULL.
     */
    void throwExceptiongetCorrelationJnegative();
    //! Unit test of getCorrelation(i,j)
    /*!
     \brief Test that the code throws an exception is the index of the first output is exceed. The value of the correlation returned is NULL.
     */
    void throwExceptiongetCorrelationIexceed();
    //! Unit test of getCorrelation(i,j)
    /*!
     \brief Test the code throws an exception is the index of the second output is exceed. The value of the correlation returned is NULL.
     */
    void throwExceptiongetCorrelationJexceed();
    //! Unit test of getDimensionInput
    /*!
     \brief Test that the code gives the number of input.
     */
    void testgetDimensionInput();
    //! Unit test of getDimensionOutput
    /*!
     \brief Test that the code gives the number of output.
     */
    void testgetDimensionOutput();
    //! Unit test of getDimensionExpansion
    /*!
     \brief Test that the code gives the number of coefficients of the polynomial chaos.
     */
    void testgetDimensionExpansion();
    //! Unit test of exportFunction
    /*!
     \brief Test that the code generes a file .C which contains a back-up of the macro.
     */
    void testexportFunction();
    //! Unit test of getCovariance(i/iname, j/jname)
    /*!
     \brief Test that the code compute correctly the covariance of 2 outputs.
     */
    void testgetCovariance();
    //! Unit test of getCovariance(iname, jname)
    /*!
     \brief Test that the code throws an exception if the name of the second output is false. The value of the covariance returned is NULL.
     */
    void throwExceptiongetCovarianceJnameFalse();
    //! Unit test of getCovariance(iname, jname)
    /*!
     \brief Test that the code throws an exception if the name of the first output is false. The value of the covariance returned is NULL.
     */
    void throwExceptiongetCovarianceInameFalse();
    //! Unit test of getCovariance(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the first output is negative. The value of the covariance returned is NULL.
     */
    void throwExceptiongetCovarianceInegative();
    //! Unit test of getCovariance(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the second output is negative. The value of the covariance returned is NULL.
     */
    void throwExceptiongetCovarianceJnegative();
    //! Unit test of getCovariance(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the first output is exceed. The value of the covariance returned is NULL.
     */
    void throwExceptiongetCovarianceIexceed();
    //! Unit test of getCovariance(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the second output is exceed. The value of the covariance returned is NULL.
     */
    void throwExceptiongetCovarianceJexceed();
    //! Unit test of getIndexFirstOrder(i/iname, j/jname)
    /*!
     \brief Test taht the code computes coorectly the first order coefficient of sensitivity.
     */
    void testgetIndexFirstOrder();
    //! Unit test of getIndexFirstOrder(iname, jname)
    /*!
     \brief Test that the code throws an exception if the name of the second output is false. The value of the first order sensitivity index  returned is NULL.
     */
    void throwExceptiongetIndexFirstOrderJnameFalse();
    //! Unit test of getIndexFirstOrder(iname, jname)
    /*!
     \brief Test that the code throws an exception if the name of the first output is false. The value of the first order sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexFirstOrderInameFalse();
    //! Unit test of getIndexFirstOrder(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the first output is negative. The value of the first order sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexFirstOrderInegative();
    //! Unit test of getIndexFirstOrder(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the second output is negative. The value of thefirst order sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexFirstOrderJnegative();
    //! Unit test of getIndexFirstOrder(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the first output is exceed. The value of the first order sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexFirstOrderIexceed();
    //! Unit test of getIndexFirstOrder(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the second output is exceed . The value of the first order sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexFirstOrderJexceed();
    //! Unit test of getIndexTotalOrder(i/inazme,j/jname)
    /*!
     \brief Test taht the code computes correctly the total index of sensitivity.
     */
    void testgetIndexTotalOrder();
    //! Unit test of getIndexFirstOrder(i, j)
    /*!
     \brief Test that the code throws an exception if the name of the second output is false. The value of the total sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexTotalOrderJnameFalse();
    //! Unit test of getIndexFirstOrder(i, j)
    /*!
     \brief Test that the code throws an exception if the name of the first output is false. The value of the total sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexTotalOrderInameFalse();
    //! Unit test of getIndexFirstOrder(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the first output is negative. The value of the total sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexTotalOrderInegative();
    //! Unit test of getIndexFirstOrder(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the second output is negative. The value of the total sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexTotalOrderJnegative();
    //! Unit test of getIndexFirstOrder(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the first output is exceed. The value of the total sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexTotalOrderIexceed();
    //! Unit test of getIndexFirstOrder(i, j)
    /*!
     \brief Test that the code throws an exception if the index of the second output is exceed. The value of the total sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexTotalOrderJexceed();
    //! Unit test of  getIndex(sinput, jname)
    /*!
     \brief Test that the  code computes correctly the sensitivity index of the input variable group for one output.
     */
    void testgetIndex();
    //! Unit test of  getIndex(sinput, jname)
    /*!
     \brief Test that the code throws an exception if the name of the output is false. The value of the sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexJnameFalse();
    //! Unit test of  getIndex(sinput jname)
    /*!
     \brief Test that the code throws an exception if a name in the list of inputs is false. The code stops the calculation and exits.
     */
    void throwExceptiongetIndexSInputFalse();
    //! Unit test of  getIndexInteraction(sinput, jname)
    /*!
     \brief Test that the  code computes correctly the sensitivity index in interaction of the input variable group for one output.
     */
    void testgetIndexInteraction();

    //! Unit test of  getIndexInteraction(sinput, jname)
    /*!
     \brief  Test that the code throws an exception if the name of the output is false. The value of the interactive sensitivity index returned is NULL.
     */
    void throwExceptiongetIndexInteractionJnameFalse();
    //! Unit test of  getIndexInteraction(sinput, jname)
    /*!
     \brief Test that the code throws an exception if a name in the list of inputs is false. The code stops the calculation and exits.
     */
    void throwExceptiongetIndexInteractionSInputFalse();
    //!Unit test of getAnovaOrdered(seuil,jname)
    /*!
     \brief Test the functionning
     */
    void getAnovaOrdered();
    //!Unit test of getAnovaOrdered(seuil,jname)
    /*!
     \brief Test that the code throws an exception if jname doesn't exit.
     */
    void throwExceptiongetAnovaOrdered();

    //!Unit test of TPolynomialChaos constructor
    /*!
     \brief Test that the code throws an exception if number of simulations is different
     in the nisp object and in the tds object.
     */
    void throwExceptionNPatternsDifferentNispTds();
    
    //!Unit test of TPolynomialChaos constructor by copy
    /*!
     \brief Test that the code throws an exception if there is a non supported law.
     */
    void throwExceptionConstructorByCopyWithIndisponibleLaw();
    
    //!Unit test of setGroupAddVar(Int_t i)
    /*!
     \brief Test that the code throws an exception if the argument is stricly superior to _nx.
     */
    void throwExceptionSetGroupAddVarWithRankTooHigh();
    
    //!Unit test of setCoefficient(Int_t noutput, Int_t num, Double_t coef)
    /*!
     \brief Test that the code throws an exception if the argument noutput is stricly superior to the number of output.
     */
    void throwExceptionSetCoefficientNumberOfOutputTooHigh();
    
    //!Unit test of setCoefficient(Int_t noutput, Int_t num, Double_t coef)
    /*!
     \brief Test that the code throws an exception if the argument num is stricly superior to the number of PC.
     */
    void throwExceptionSetCoefficientNumberOfPCTooHigh();
    
    //!Unit test of getCoefficient(Int_t noutput, Int_t num)
    /*!
     \brief Test that the code throws an exception if the argument noutput is stricly superior to the number of output.
     */
    void throwExceptionGetCoefficientNumberOfOutputTooHigh();
    
    //!Unit test of getCoefficient(Int_t noutput, Int_t num)
    /*!
     \brief Test that the code throws an exception if the argument num is stricly superior to the number of PC.
     */
    void throwExceptionGetCoefficientNumberOfPCTooHigh();
    
    //!Unit test of getAnovaOrderedCoefficients
    /*!
     \brief Test that the code throws an exception if the name of the output is unknown.
     */
    void throwExceptiongetAnovaOrderedCoefficients();
    
    //!Unit test of getSample(Int_t k, TString yname)
    /*!
     \brief Test that the code throws an exception if the yname of the output is unknown.
     */
    void throwExceptiongetSampleWrongOutputName();
    
     //!Unit test of getSample(Int_t k, TString yname)
    /*!
     \brief Test that the code get the correct value.
     */
//     void testGetSampleWithOutputName();

    //!Unit test of getQuantile(Double_t alpha, TString yname)
    //!Unit test of getQuantileWilks(Double_t alpha, Double_t beta, TString yname)
    //!Unit test of getInvQuantile(Double_t seuil, TString yname)
    /*!
     \brief Test that the code throws an exception if the yname of the output is unknown.
     */
    void throwExceptiongetQuantileWrongOutputName();

    //!Unit test of getCoefficient(Int_t k, TString yname)
    /*!
     \brief Test that the code throws an exception if the yname of the output is unknown.
     */
    void throwExceptionGetCoefficientWithOutputName();
    
     //!Unit test of getQuantile(Double_t alpha, TString yname)
    //!Unit test of getQuantileWilks(Double_t alpha, Double_t beta, TString yname)
    //!Unit test of getInvQuantile(Double_t seuil, TString yname)
    void testGetQuantiles();
    
    //!Unit test of getAnovaOrdered(Double_t seuil, Int_t j)
    /*!
     \brief Test that the code throws an exception if j is incorrect
     */
    void throwExceptionGetAnovaOrdered();
    
    //!Unit test of getSample(Int_t k, Int_t j)
    /*!
     \brief Test that the code throws an exception if j is incorrect
     */
    void throwExceptionGetSample();

    //! Throw an exception when requested rank is out of scope 
    void throwExceptionGetEqmLoo();

    //! Throw an exception when requested input value and/or rank are out of scope 
    void throwExceptionGetErrLoo();

    //! Throw an exception when bad value is asked for autoDegreeFactor
    void throwExceptionSetAutoDegreeFactor();

    //! Throw an exception when bad values are asked for boundaries
    void throwExceptionSetAutoDegreeBoundaries();
    
    //@}  

private:
    TString _workingTestPath;
    
    URANIE::DataServer::TDataServer *_tds;
    URANIE::Modeler::TNisp *_nisp;
    URANIE::Modeler::TPolynomialChaos *_pc;
    URANIE::Launcher::TLauncherFunction *_tlf;

};

#endif  // CPPUNITEST_TESTTPOLYNOMIALCHAOS_H
