//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TOptimizerOpt.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   thu jan 3 14:06:39 CET 2008
 
 \brief Interface of class URANIE::Optimize::TOptimizerOpt
 
 */
/**
 
 * @class URANIE::Optimizer::TOptimizerOpt
 *
 * @brief Description of the class TOptimizerOpt.
 *
 * To be written by the developper.
 *
 */
#ifndef TOPTIMIZEROPT_H
#define TOPTIMIZEROPT_H

#include <iostream>

//ROOT
#include "Rtypes.h"
#include "TMethodCall.h"

// Uranie
#include "Optimizer.h"
#include "TDataServer.h"
// Newmat

#include "newmat.h"
#include "opt++.h"
//using namespace OPTPP;
#ifdef HAVE_NAMESPACES
using namespace NEWMAT;
#endif

// Opt++
//extern typedef void(* OPTPP::USERFCN2)(int, int, const NEWMAT::ColumnVector &, real &, NEWMAT::ColumnVector &, NEWMAT::SymmetricMatrix &, int &);

namespace URANIE
{
namespace Optimizer
{
class TOptimizerOpt: public TNamed
{
public:
    enum EOptimType
    {
        kFunction, kSumOfSquare
    };
private:
    Bool_t _blog;                          ///< Boolean to edit the log
    URANIE::DataServer::TDataServer *_tds;     //! Pointer vers un TDS

//fga 2015/01/12    void * _function;                       //! rem : Do not process by rootcint
//fga 2015/01/12    OPTPP::USERFCN2 _MyOptFunction;         //! rem : Do not process by rootcint
//fga 2015/01/12    OPTPP::OPTPPINITFCN _MyOptInitFunction; //! rem : Do not process by rootcint

    TString _sLibrary;
    TString _sFunctionName;
    TString _sInitFunctionName;

    Int_t _niter;

    //      OPTPP::USERFCN0 *_theFunction ;
    ///OPTPP::USERFCN0 *_theOptFunction ; ///< The function
    TMethodCall *_fMethodCall; ///< Pointer to MethodCall in case of interpreted function
    void (*_fMethod)(Double_t *gin, Double_t *f);
//       void         (*_fMethodOpt)(int mode, int n, const NEWMAT::ColumnVector& x, double& fx, NEWMAT::ColumnVector& g, NEWMAT::SymmetricMatrix& H, int &result);
    void (*_fMethodOpt)(int n, const ColumnVector& x, double& fx, int& result);

    TString _sname;
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TOptimizerOpt();
    //! Constructor with a dataserver
//       TOptimizerOpt (URANIE::DataServer::TDataServer *tds, void *function = NULL);
    TOptimizerOpt(URANIE::DataServer::TDataServer *tds, const char *functionname);
    TOptimizerOpt(URANIE::DataServer::TDataServer *tds, TString slibrary,
            TString sfunction, TString sinit = "");
    //! Default destructor
    virtual ~TOptimizerOpt();
    //@}

    //---------------------------------------------
    /** @name Manipulate the code
     **/
    //@{
    //! Init phase of the code
    virtual void init(Option_t *option = "");
    //! Runs phase of the code
    //      virtual void optimize(void *fj, Option_t *option ="");
    virtual void optimize(Option_t *option = "");
    //! Terminate phase of the code
    virtual void terminate(Option_t *option = "");
    //! Cleans the working directory of the code
    virtual void clean(Option_t *option = "");

    void run(Option_t *option = "");
//      void run();

    TMethodCall *getMethodCall() const
    {
        return _fMethodCall;
    }
    void incrementIteration()
    {
        _niter++;
    }
    Int_t getIteration()
    {
        return _niter;
    }
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Optimizer::TOptimizerOpt, ID_OPTIMIZER)
    //Classe de
};

} // Fin du namespace Optimizer
} // Fin du namespace URANIE

R__EXTERN URANIE::Optimizer::TOptimizerOpt *gOptOptimizer;
#endif
