//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TSumOfSquares.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mar jui 3 10:38:56 CEST 2007
 
 \brief Implementation of the object TSumOfSquares
 
 \par Modifications
 \li M. Salvoldi (Areva/OCC) 2011/09/28
 - add a IFDEF WIN32 to replace rm -fr by del /Q on Windows platform
 \li Fabrice Gaudier (2010-04-12 14:30:47)
 Save the name of the output attribute for drawing
 
 */

#include "TSumOfSquares.h"

ClassImp (URANIE::Optimizer::TSumOfSquares)

namespace URANIE
{
namespace Optimizer
{
void TSumOfSquares::init()
{

    cout << endl << "*************************************" << endl
            << " *** TSumOfSquares::init" << endl;
    if (_tds != NULL)
    {

        _ninput = _tds->getNInputAttributes();
        _noutput = _tds->getNOutputAttributes();

        cout << " ** _ninput [ " << _ninput << "]" << endl;
        cout << " ** _noutput [ " << _noutput << "]" << endl;

        // looking for the output attribute
        for (Int_t i = 0; i < _tds->getNAttributes(); i++)
        {
            URANIE::DataServer::TAttribute *att = _tds->getAttribute(i);
            if (att->isOutput())
                _sCriteria = att->GetName();
        }
        cout << " ** _sCriteria [ " << _sCriteria << "]" << endl;
        if (_tds->getTuple() == NULL)
            _tds->createTuple();

        _tds->getTuple()->Print();
    }

    if (_code != NULL)
    {

        // The size of the input vector is increased by two
        // - one for the iterator
        // - one for the criteria for drawing
        _dinputValue = new double[_ninput + 2];
        _doutputValue = new double[_noutput + 1];

        clean();

        _listOfDataOutput = new TList();

        _code->init();
    }

    cout << " *** End Of TSumOfSquares::init" << endl
            << "*************************************" << endl;
}

void TSumOfSquares::clean()
{
    TString sworkindirectory = _code->getWorkingDirectory();

    // Place in the good directory
    if (sworkindirectory.Length() != 0)
        gSystem->cd(sworkindirectory);

    // Clean the subdirectory "UranieLauncher_*"
#ifdef WIN32
    gSystem->Exec("rd /s /q UranieLauncher_*");
#else
    gSystem->Exec("rm -fr UranieLauncher_*");
#endif

}
} // Fin du namespace Optimizer
} // Fin du namespace URANIE

