//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestMain.cxx
 \author Cedric Forgues <uranietm@cea.fr>
 \date   mercredi jun 03 9:00:01 2009
 
 
 \brief Programme principal pour les tests unitaires sur la librairie Optimizer.

 \par Modifications
 \li C. Forgues (2011-01-04 15:59:27)
 - adds the flag WITH_MINUIT2 for TOptimizerTest suite registration. 
 */
#include "cppunit/CompilerOutputter.h"
#include "cppunit/TestResult.h"
#include "cppunit/TestResultCollector.h"
#include "cppunit/TextTestProgressListener.h"
#include "cppunit/BriefTestProgressListener.h"
#include "cppunit/extensions/TestFactoryRegistry.h"
#include "cppunit/TestRunner.h"
#include "cppunit/XmlOutputter.h"

#include <stdexcept>
#include <iostream>
#include <fstream>

//uranie
//#include "UVersion.h"
// Liste des Tests Unitaires d'uranie
#include "TestTOptimizer.h"
#include "TestTMultiGenAnalyticalFunction.h"
#include "TestTMultiGenCode.h"
#include "TestTMultiGenSumOfSquares.h"
#include "TestTObjective.h"
#include "TestTBestEstimate.h"
#ifdef WITH_OPT
#include "TestTOptimizerOpt.h"
#endif
#ifdef WITH_VIZIR
#include "TestTVizir.h"
#endif
#include "TObjective.h"
#include "uranie_commands.h"

#ifdef WITH_MINUIT2
CPPUNIT_TEST_SUITE_REGISTRATION( TOptimizerTest ); //2
CPPUNIT_TEST_SUITE_REGISTRATION( TMultiGenAnalyticalFunctionTest ); //0
CPPUNIT_TEST_SUITE_REGISTRATION( TMultiGenCodeTest ); //0
CPPUNIT_TEST_SUITE_REGISTRATION( TMultiGenSumOfSquaresTest ); //0
CPPUNIT_TEST_SUITE_REGISTRATION( TObjectiveTest );
CPPUNIT_TEST_SUITE_REGISTRATION( TBestEstimateTest );
#endif
 
#ifdef WITH_OPT
CPPUNIT_TEST_SUITE_REGISTRATION( TOptimizerOptTest );
#endif

#ifdef WITH_VIZIR
CPPUNIT_TEST_SUITE_REGISTRATION( TVizirTest );
#endif

bool uranie_test_suite(const char *module, const char *xml, const char *xsl);

int main()
{
    bool wasSuccessful;
#ifdef WIN32
    wasSuccessful = uranie_test_suite("Optimizer",
                                      "testOptimizerResults.xml", "..\\..\\..\\..\\testReport.xsl");
#else
    wasSuccessful = uranie_test_suite("Optimizer",
                                      "testOptimizerResults.xml", "../../../../testReport.xsl");
#endif
    ////////////////
    // Clean
    ////////////////
    UranieTests::removeFolder(gSystem->pwd(), "tmpLanceurUranie");
    UranieTests::removeFolder(gSystem->pwd(), "URANIE");
    UranieTests::removeFolder(gSystem->pwd(), "UranieResults");

    gDirectory->ls();

    // ---  Return error code 1 if the one of test failed.
    return wasSuccessful ? 0 : 1;
}
