//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTestTMultiGenSumOfSquares.cxx
 \author Maud Salvoldi (OCC/Areva group)
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::Optimizer::TestTMultiGenSumOfSquares
 
 Cree le : jeu d�c 22 11:27:34 CET 2011
 
 */

#define DBL_PRECISION_COMPARE 1.e-12
#define EPSILON 0.01 

// Uranie
#include "TestTMultiGenSumOfSquares.h"
#include "UDisableTest.h"

// ROOT
#include "TString.h"
#include "TROOT.h"
#include "TSystem.h" 
#include <iostream>

TMultiGenSumOfSquaresTest::TMultiGenSumOfSquaresTest() :
        tds(NULL), mycode(NULL), fout(NULL)
{
    _workingTestPath = gSystem->pwd();
}

TMultiGenSumOfSquaresTest::~TMultiGenSumOfSquaresTest()
{
}

void TMultiGenSumOfSquaresTest::setUp()
{
    gSystem->cd(_workingTestPath);

    tds = new URANIE::DataServer::TDataServer("testTMultiGenSumOfSquares",
            "test of classes TMultiGenSumOfSquares");
    tds->addAttribute("a", -5.0, 5.0);
    tds->addAttribute("b", -1.0, 1.0);
    tds->addAttribute("c", -5.0, 5.0);
    
    
    TString sJDDReference = "toto.in";
    tds->getAttribute("a")->setFileKey(sJDDReference, "a", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);
    tds->getAttribute("b")->setFileKey(sJDDReference, "b", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);
    tds->getAttribute("c")->setFileKey(sJDDReference, "c", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);
    tds->getAttribute("a")->setDefaultValue(0.200);
    tds->getAttribute("a")->setStepValue(0.01);
    tds->getAttribute("b")->setDefaultValue(0.200);
    tds->getAttribute("b")->setStepValue(0.01);
    tds->getAttribute("c")->setDefaultValue(0.200);
    tds->getAttribute("c")->setStepValue(0.01);

    fout = new URANIE::Launcher::TOutputFileDataServer(
            "_function_venise_output1_.dat");
    fout->addAttribute("yhat");

    mycode = new URANIE::Launcher::TCode(tds, "ls -al >> /dev/null");
    mycode->addOutputFile(fout);
}

void TMultiGenSumOfSquaresTest::tearDown()
{
    delete mycode;
    mycode = NULL;

    delete fout;
    fout = NULL;

    delete tds;
    tds = NULL;
}

////////////////////////////////
///                          /// 
///          test            ///
///                          ///
////////////////////////////////
void TMultiGenSumOfSquaresTest::testTMultiGenSumOfSquareaddObjective()
{
    URANIE::Optimizer::TMultiGenSumOfSquares* topt =
            new URANIE::Optimizer::TMultiGenSumOfSquares(tds, mycode);
    CPPUNIT_ASSERT_EQUAL(3, (Int_t) topt->NDim());
    CPPUNIT_ASSERT_EQUAL(0, topt->getNObjectives());
    CPPUNIT_ASSERT_EQUAL(0, strcmp("", topt->getCriteriaName()));
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, topt->getSumOfWeight(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(tds->isAttribute("aaa") == kFALSE);
    CPPUNIT_ASSERT(tds->isAttribute("bbb") == kFALSE);
    CPPUNIT_ASSERT(tds->isAttribute("__cost__") == kFALSE);
    CPPUNIT_ASSERT(tds->isAttribute("yhat") == kFALSE);

    topt->addObjective(new URANIE::Optimizer::TObjective("aaa"));
    CPPUNIT_ASSERT_EQUAL(1, topt->getNObjectives());
    CPPUNIT_ASSERT_EQUAL(0, strcmp("aaa", topt->getCriteriaName()));
    CPPUNIT_ASSERT_EQUAL(1, strcmp("bbb", topt->getCriteriaName()));
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, topt->getSumOfWeight(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(tds->isAttribute("aaa"));
    CPPUNIT_ASSERT(tds->isAttribute("bbb") == kFALSE);
    CPPUNIT_ASSERT(tds->isAttribute("__cost__") == kFALSE);
    CPPUNIT_ASSERT(tds->isAttribute("yhat") == kFALSE);

    topt->addObjective(new URANIE::Optimizer::TObjective("bbb"));
    CPPUNIT_ASSERT_EQUAL(2, topt->getNObjectives());
    CPPUNIT_ASSERT_EQUAL(0, strcmp("__cost__", topt->getCriteriaName()));
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, topt->getSumOfWeight(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(tds->isAttribute("__cost__"));
    CPPUNIT_ASSERT(tds->isAttribute("yhat") == kFALSE);

    topt->addObjective(new URANIE::Optimizer::TObjective("ccc"));
    CPPUNIT_ASSERT_EQUAL(3, topt->getNObjectives());
    CPPUNIT_ASSERT_EQUAL(0, strcmp("__cost__", topt->getCriteriaName()));
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.0, topt->getSumOfWeight(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(tds->isAttribute("__cost__"));
    CPPUNIT_ASSERT(tds->isAttribute("yhat") == kFALSE);

    URANIE::Optimizer::TObjective *tobj = new URANIE::Optimizer::TObjective(
            "ddd");
    tobj->setWeight(0.25);
    topt->addObjective(tobj);
    CPPUNIT_ASSERT_EQUAL(4, topt->getNObjectives());
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.25, topt->getSumOfWeight(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT(tds->isAttribute("__cost__"));
    CPPUNIT_ASSERT(tds->isAttribute("yhat") == kFALSE);

    delete topt;
    topt = NULL;
    
}

void TMultiGenSumOfSquaresTest::testTMultiGenSumOfSquareClone()
{
  URANIE_IGNORED_TEST();
    URANIE::Optimizer::TMultiGenSumOfSquares* topt =
            new URANIE::Optimizer::TMultiGenSumOfSquares(tds, mycode);
    URANIE::Optimizer::TObjective *tobj = new URANIE::Optimizer::TObjective("jjjj");
    topt->addObjective(tobj);

    URANIE::Optimizer::TMultiGenSumOfSquares* topt2 = topt->Clone();
    CPPUNIT_ASSERT_EQUAL(1, topt2->getNObjectives());
    CPPUNIT_ASSERT_EQUAL(3, (Int_t) topt2->NDim());

    delete topt;
    topt = NULL;
    delete topt2;
    topt2 = NULL;
}
void TMultiGenSumOfSquaresTest::testTMultiGenSumOfSquareNDim()
{
    URANIE::Optimizer::TMultiGenSumOfSquares* topt =
            new URANIE::Optimizer::TMultiGenSumOfSquares(tds, mycode);
    URANIE::Optimizer::TObjective *tobj = new URANIE::Optimizer::TObjective(
            "jjjj");
    topt->addObjective(tobj);

    CPPUNIT_ASSERT_EQUAL(3, (Int_t) topt->NDim());

    delete topt;
    topt = NULL;
}
void TMultiGenSumOfSquaresTest::throwExceptionTMultiGenSumOfSquaresConstructorNull()
{
    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenSumOfSquares(NULL, mycode),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenSumOfSquares(tds, NULL),
            URANIE::Exceptions::UErrorExceptions);
}

void TMultiGenSumOfSquaresTest::throwExceptionTMultiGenSumOfSquareaddObjective()
{
    URANIE::Optimizer::TMultiGenSumOfSquares* topt =
            new URANIE::Optimizer::TMultiGenSumOfSquares(tds, mycode);
    CPPUNIT_ASSERT_THROW(topt->addObjective(NULL),
            URANIE::Exceptions::UErrorExceptions);
    delete topt;
    topt = NULL;
}

