//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTOptimizerOpt.cxx
 \author Maud Salvoldi (OCC/Areva group)
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::Optimizer::TOptimizerOpt
 
 Cree le : jeu d�c 22 11:27:34 CET 2011
 
 */

#define DBL_PRECISION_COMPARE 1.e-12
#define EPSILON 0.01 

// Uranie
#include "TestTOptimizerOpt.h"
#include "TOutputFileKey.h"
#include "TOutputFileRow.h"
#include "TOutputFileDataServer.h"
#include "UDisableTest.h"
#include "uranie_commands.h"

// ROOT
#include "TString.h"
#include "TROOT.h"
#include "TSystem.h" 
#include <iostream>

TOptimizerOptTest::TOptimizerOptTest()
{
   
}

TOptimizerOptTest::~TOptimizerOptTest()
{

}

void TOptimizerOptTest::setUp()
{
    _tds = new URANIE::DataServer::TDataServer("testTOptimizerOpt",
            "test of classes TOptimizerOpt");
    _tds->addAttribute("x1", -4.5, 4.5);
    _tds->addAttribute("x2", -4.5, 5.5);
    _tds->addAttribute("x3", -5.0, 5.0);

}

void TOptimizerOptTest::tearDown()
{
     delete _tds;
//     UranieTests::removeFile(gSystem->pwd(),"_tst_opt++_hs65_2_3.out");

}
////////////////////////////////
///                          ///                            
///           test           ///
///                          ///
////////////////////////////////
void TOptimizerOptTest::testOptimizerOptConstructor()
{
    TString libraryPath = "../../teST/liBRARY/hs65/";
#ifndef WIN32   
    TString libraryName = "libdllOptpphs65.so";

#if __clang__
    URANIE_IGNORED_TEST() ;
#endif
#if __GNUC__ >= 5 || (__GNUC__ >= 4 &&  __GNUC_MINOR__ > 8 )
    URANIE_IGNORED_TEST() ;
#endif

#else
    TString libraryName = "Debug/libdllOptpphs65.dll";
#endif   
    libraryPath += libraryName;
#ifndef WIN32	
    TOptimizerOpt * topths65 = new TOptimizerOpt(_tds, gSystem->UnixPathName(libraryPath), "hs65_2", "init_hs65");
    topths65->unsetLog();
    time_t seconds;
    string line, date;
    Bool_t bStop = kFALSE;
    Int_t test[11];
    for(Int_t it = 0 ; it < 11 ; it++) test[it] = 0;
    
    Int_t ndim = _tds->getNAttributes();
    Int_t idate;
    std::fstream file(Form("_tst_opt++_%s_%d.out", "hs65_2", ndim));
    time(&seconds);
    date = string(ctime(&seconds));
    while (bStop == kFALSE)
    {
        getline(file, line);
        if (line.find("Job run at ") != string::npos)
        {
            idate = 0;
            //verify that the date are the same actually and in the file
            while(date[idate] == line[idate+11])
            {
                idate++;
            }
            if (idate == 24)
                test[0]=1;
        }
        if (line.find("NIPS Iteration Summary") != string::npos)
            test[1]=1;
        if (line.find("    0       0.9535            0") != string::npos)
            test[2]=1;
        if (line.find("    1    3.506e-31            0           1    3.506e-31    3   0      1e+02") != string::npos)
            test[3]=1;
        if (line.find("Optimization method       = Nonlinear Interior-Point Method") != string::npos)
            test[4]=1;
        if (line.find("Return code               = 2 (OptNIPSLike: Algorithm converged)") != string::npos)
            test[5]=1;
        if (line.find("1           5") != string::npos)
            test[6]=1;
        if (line.find("2           5") != string::npos)
            test[7]=1;
        if (line.find("3           5") != string::npos)
            test[8]=1;
        if (line.find("Function Value     =    3.506e-31") != string::npos)
            test[9]=1;
        if (line.find("Norm of gradient   =    5.583e-16") != string::npos)
            test[10]=1;
        
        if (file.eof())
            bStop = kTRUE;
    }

    delete topths65;
    
    CPPUNIT_ASSERT_EQUAL(1, test[0]);
    CPPUNIT_ASSERT_EQUAL(1, test[1]);
    CPPUNIT_ASSERT_EQUAL(1, test[2]);
    CPPUNIT_ASSERT_EQUAL(1, test[3]);
    CPPUNIT_ASSERT_EQUAL(1, test[4]);
    CPPUNIT_ASSERT_EQUAL(1, test[5]);
    CPPUNIT_ASSERT_EQUAL(1, test[6]);
    CPPUNIT_ASSERT_EQUAL(1, test[7]);
    CPPUNIT_ASSERT_EQUAL(1, test[8]);
    CPPUNIT_ASSERT_EQUAL(1, test[9]);
    CPPUNIT_ASSERT_EQUAL(1, test[10]);
    
#else    
// THIS TEST HAS BEEN DISABLED
// TODO: fix me
    URANIE_INBUILDING_TEST()
    ;
#endif	
}
void TOptimizerOptTest::testOptimizerOptgetMethodCall()
{
    // THIS TEST HAS BEEN DISABLED
    // TODO: fix me
    URANIE_INBUILDING_TEST()
    ;
}
void TOptimizerOptTest::testOptimizerOptgetIteration()
{
  cout << endl;
  TString libraryPath = "../../teST/liBRARY/hs65/";
#ifndef WIN32
  TString libraryName = "libdllOptpphs65.so";
#else
  TString libraryName = "Debug/libdllOptpphs65.dll";
#endif
  libraryPath += libraryName;
#ifndef WIN32

    TOptimizerOpt * topths65 = new TOptimizerOpt(_tds, gSystem->UnixPathName(libraryPath), "hs65_2", "init_hs65");
    topths65->unsetLog();
    
    CPPUNIT_ASSERT_EQUAL(0, topths65->getIteration());
    topths65->incrementIteration();
    CPPUNIT_ASSERT_EQUAL(1, topths65->getIteration());
    
    delete topths65;
#else
// THIS TEST HAS BEEN DISABLED
// TODO: fix me
    URANIE_INBUILDING_TEST()
    ;
#endif
    
    
}

void TOptimizerOptTest::testTOptimizerOptSetLog()
{
  cout << endl;
  TString libraryPath = "../../teST/liBRARY/hs65/";
#ifndef WIN32
  TString libraryName = "libdllOptpphs65.so";
#else
  TString libraryName = "Debug/libdllOptpphs65.dll";
#endif
  libraryPath += libraryName;

  TOptimizerOpt * topths65 = new TOptimizerOpt(_tds, gSystem->UnixPathName(libraryPath), "hs65_2", "init_hs65");
    topths65->unsetLog();
    
    CPPUNIT_ASSERT(!topths65->getLog());
    
    topths65->setLog();    
    CPPUNIT_ASSERT(topths65->getLog());
    
    topths65->changeLog();    
    CPPUNIT_ASSERT(!topths65->getLog());
    
    topths65->changeLog();    
    CPPUNIT_ASSERT(topths65->getLog());

    delete topths65;
}

////////////////////////////////
///                          ///                            
///     throwexception       ///
///                          ///
////////////////////////////////
void TOptimizerOptTest::throwExceptionOptimizerOptConstructorTdsNull()
{
    // THIS TEST HAS BEEN DISABLED
    // TODO: fix me
    URANIE_INBUILDING_TEST()
    ;
}
void TOptimizerOptTest::throwExceptionOptimizerOptConstructorUnknowFunction()
{
    // THIS TEST HAS BEEN DISABLED
    // TODO: fix me
    URANIE_INBUILDING_TEST()
    ;
}
void TOptimizerOptTest::throwExceptionOptimizerOptConstructorUnknowLibrary()
{
    // THIS TEST HAS BEEN DISABLED
    // TODO: fix me
    URANIE_INBUILDING_TEST()
    ;
}
