//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TVIZIR_H
#define CPPUNITEST_TVIZIR_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TVizir.h"
#include "TDataServer.h"
#include "TLauncher.h"
#include "TAttribute.h"
#include "TOutputFileDataServer.h"
#include "TOutputFileKey.h"
#include "UExceptions.h"

class TVizirTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TVizirTest);

    CPPUNIT_TEST (testTVizirConstructor);
    CPPUNIT_TEST (testTVizirsetPrecision);
    CPPUNIT_TEST (testTVizirsetSurvivalRate);
    CPPUNIT_TEST (testTVizirsetHomozygoteRate);
    CPPUNIT_TEST (testTVizirsetMutationRate);
    CPPUNIT_TEST (testTVizirsetMaxEvaluation);
    CPPUNIT_TEST (testTVizirGetLastGenerationIndex);
    CPPUNIT_TEST (testTVizirSetDecode);
    CPPUNIT_TEST (testTVizirSetHardConstraint);
    CPPUNIT_TEST (testTVizirSetCost);
    CPPUNIT_TEST (testTVizirSetConstraintAndCost);
    CPPUNIT_TEST (testTVizirSetNbGenerationToDraw);
    CPPUNIT_TEST (testTVizirSetVarDraw);
    CPPUNIT_TEST (testTVizirSetLog);
    CPPUNIT_TEST (testTVizirEvaluer);
    CPPUNIT_TEST (testTVizirfitnessChanged);
    CPPUNIT_TEST (testTVizirOptimizeMonoCriterionWithoutConstraint);
    CPPUNIT_TEST (testTVizirOptimizeMonoCriterionWithConstraint);
    CPPUNIT_TEST (testTVizirOptimizeMultiCriterionWithoutConstraint);
    CPPUNIT_TEST (testTVizirOptimizeMultiCriterionWithConstraint);
    CPPUNIT_TEST (testTVizirDecode);
    CPPUNIT_TEST (testTVizirEvaluerCode);
    CPPUNIT_TEST (throwExceptionTVizirConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionTVizirConstructorFalseNpop);
    CPPUNIT_TEST (throwExceptionTVizirSetDecodeFalseFun);
    CPPUNIT_TEST (throwExceptionTVizirsetHardConstraintFalseFun);
    CPPUNIT_TEST (throwExceptionTVizirsetConstraintAndCostFalseFun);
    CPPUNIT_TEST (throwExceptionTVizirsetConstraintAndCostFalseInput);
    CPPUNIT_TEST (throwExceptionTVizirsetConstraintAndCostFalseNHidden);
    CPPUNIT_TEST (throwExceptionTVizirsetConstraintAndCostFalseNbConst);
    CPPUNIT_TEST (throwExceptionTVizirsetConstraintAndCostFalseTCode);
    CPPUNIT_TEST (throwExceptionTVizirevaluerDecodNull);
    CPPUNIT_TEST (throwExceptionTVizirevaluerHardNull);
    CPPUNIT_TEST (throwExceptionTVizirevaluerSoftNull);
    CPPUNIT_TEST (throwExceptionTVizirevaluerTCodeNull);
    CPPUNIT_TEST (throwExceptionTVizirSetNbGenerationToDraw);
    CPPUNIT_TEST (throwExceptionTVizirSetVarDraw);
    CPPUNIT_TEST (throwExceptionTViziroptimizeFalseOption);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TVizirTest();

    //! Default destructor
    virtual ~TVizirTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    void testTVizirConstructor();
    void testTVizirsetPrecision();
    void testTVizirsetSurvivalRate();
    void testTVizirsetHomozygoteRate();
    void testTVizirsetMutationRate();
    void testTVizirsetMaxEvaluation();
    void testTVizirGetLastGenerationIndex();
    void testTVizirSetDecode();
    void testTVizirSetHardConstraint();
    void testTVizirSetCost();
    void testTVizirSetConstraintAndCost();
    void testTVizirSetNbGenerationToDraw();
    void testTVizirSetVarDraw();
    void testTVizirSetLog();
    void testTVizirEvaluer();
    void testTVizirfitnessChanged();
    void testTVizirOptimizeMonoCriterionWithoutConstraint();
    void testTVizirOptimizeMonoCriterionWithConstraint();
    void testTVizirOptimizeMultiCriterionWithoutConstraint();
    void testTVizirOptimizeMultiCriterionWithConstraint();
    void testTVizirDecode();
    void testTVizirEvaluerCode();
    void throwExceptionTVizirConstructorTdsNull();
    void throwExceptionTVizirConstructorFalseNpop();
    void throwExceptionTVizirSetDecodeFalseFun();
    void throwExceptionTVizirsetHardConstraintFalseFun();
    void throwExceptionTVizirsetConstraintAndCostFalseFun();
    void throwExceptionTVizirsetConstraintAndCostFalseInput();
    void throwExceptionTVizirsetConstraintAndCostFalseNHidden();
    void throwExceptionTVizirsetConstraintAndCostFalseNbConst();
    void throwExceptionTVizirsetConstraintAndCostFalseTCode();
    void throwExceptionTVizirevaluerDecodNull();
    void throwExceptionTVizirevaluerHardNull();
    void throwExceptionTVizirevaluerSoftNull();
    void throwExceptionTVizirevaluerTCodeNull();
    void throwExceptionTVizirSetNbGenerationToDraw();
    void throwExceptionTVizirSetVarDraw();
    void throwExceptionTViziroptimizeFalseOption();

    //@}

private:
    URANIE::DataServer::TDataServer *_tds;
    URANIE::DataServer::TDataServer *_tds1;
    URANIE::DataServer::TDataServer *_tds2;
    URANIE::DataServer::TDataServer *_tdsRosenbrock;
    URANIE::Optimizer::TVizir *_vzr;
    URANIE::Launcher::TOutputFileDataServer *_foutDataServer;
    URANIE::Launcher::TOutputFileKey * _fOutputFile;
    URANIE::Launcher::TCode *_tcode;
    TString _workingTestPath;

};

#endif  // CPPUNITEST_TVIZIR_H
