{
    // Definition of the reference data server
    TDataServer *tdsRef = new TDataServer("tdsRef", "Objectives values");
    tdsRef->fileDataRead("rosenbrokReferences.dat");

    TString sinputfile ="input_rosenbrock_with_keys.dat";
    tdsRef->getAttribute("x1")->setFileKey(sinputfile, "x");
    tdsRef->getAttribute("x2")->setFileKey(sinputfile, "y");

    // Definition of the output file of the code
    TOutputFileRow *fout = new TOutputFileRow("_output_rosenbrock_with_values_rows_.dat");
    fout->addAttribute(new TAttribute("yhat"), 3);

    // Definition of the code
    TCode *mycode = new TCode(tdsRef, "rosenbrock -k >> /dev/null");
    mycode->addOutputFile(fout);

    // Launcher on the TDS Ref
    TLauncher * tl = new TLauncher(tdsRef, mycode);
    tl->setWorkingDirectory("URANIE_1");
    tl->setDrawProgressBar(kFALSE);
    tl->run();

    // Export the reference values for the (a,b) parameters
    tdsRef->exportData("_output_rosenbrock_references_.dat","x1:x2:yhat:y");

    // Compute the L1, L2 and LMax norms for the (a,b) parameters
    tdsRef->addAttribute("normL1", "TMath::Abs(yhat-y)");
    tdsRef->addAttribute("normL2", "(yhat-y)*(yhat-y)");
    tdsRef->computeStatistic("normL1:normL2");

    Double_t dnormL1 = tdsRef->getAttribute("normL1")->getMean();
    Double_t dnormL2 = tdsRef->getAttribute("normL2")->getMean();
    Double_t dnormLMax = tdsRef->getAttribute("normL1")->getMaximum();

    ofstream sortie("_output_rosenbrock_criteria_.dat",ios::out);
    sortie << "normL1  = " << dnormL1 << " ; " << endl;
    sortie << "normL2  = " << dnormL2 << " ; " << endl;
    sortie << "normLMax  = " << dnormLMax << " ; " << endl;
    sortie << endl;
    sortie.close();

}
