#include <iostream>
using namespace URANIE::DataServer;
using namespace URANIE::Launcher;
using namespace URANIE::Optimizer;

void rosenbrock(Double_t *x, Double_t *y)
{
    Double_t dx1 = x[0], dx2 = x[1];
    Double_t da = 10.0, db = 1.0;

    y[0] = da * (dx2 - dx1 * dx1) * (dx2 - dx1 * dx1)
            + db * (1.0 - dx1) * (1.0 - dx1);

}

void testRosenbrockFunctionCombined()
{
    // Attributes of the study
    TAttribute *x = new TAttribute("x", -1.5, 1.5);
    x->setDefaultValue(-1.2);
    x->setStepValue(0.01);

    TAttribute *y = new TAttribute("y", -1.5, 1.5);
    y->setDefaultValue(1.);
    y->setStepValue(0.01);

    // Collection of attributes
    TDataServer *tdsRosenbrock = new TDataServer("tdsRosenbrock",
            "Optimize Code externe Rosenbrock via TDataServer");
    tdsRosenbrock->addAttribute(x);
    tdsRosenbrock->addAttribute(y);

    // Optimizer process
    TOptimizer * topt = new TOptimizer(tdsRosenbrock, rosenbrock, "", "out");
    topt->setMethod(TOptimizer::kCombined);
    topt->setTolerance(0.0001);
    topt->setMaxIterations(300);
    topt->optimize("same");

    // Store informations in an output file
    Double_t dval;
    ofstream sortie;
    sortie.open("testRosenbrockFunction.out", ios::out);
    sortie.precision(12);
    sortie.setf(ios::scientific);
    x->getDefaultValue(dval);
    sortie << dval << "\n";
    y->getDefaultValue(dval);
    sortie << dval << "\n";
    sortie.close();

}
