//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   vzr_alloc.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2003-12-10.14.05.33

 \brief macro d'allocation de vecteur, matrice, 3DMatrice
 */

#ifndef __VZRALLOC__
#define __VZRALLOC__

#include "vzr_struct.h"

#define NEWvecteur(vec,nb) vec = (double *) calloc((size_t) nb, sizeof(double))
#define NEWvecteurI(vec,nb) vec = (int *) calloc((size_t) nb, sizeof(int))
#define NEWmatrice(mat,nl,nc) \
    {\
	mat = (double **) calloc((size_t) nl, sizeof(double*));\
        mat[0] = (double *) calloc((size_t) (nc)*(nl), sizeof(double));\
	if (mat[0])\
	    for(i=1;i<(nl);i++) mat[i]=mat[i-1] + (nc);\
	else {\
	    free(mat);\
	    mat = (double **) NULL;\
	}\
    }
#define NEWmatriceG(mat,nl,nc) \
    {\
	mat = (vzrgene_t **) calloc((size_t) nl, sizeof(vzrgene_t*));\
        mat[0] = (vzrgene_t *) calloc((size_t) (nc)*(nl), sizeof(vzrgene_t));\
	if (mat[0])\
	    for(i=1;i<(nl);i++) mat[i]=mat[i-1] + (nc);\
	else {\
	    free(mat);\
	    mat = (vzrgene_t **) NULL;\
	}\
    }
#define NEWmatriceG3D(mat,nl,nc,n) \
{\
    mat = (vzrgene_t ***) calloc((size_t) nl, sizeof(vzrgene_t**));\
    mat[0] = (vzrgene_t **) calloc((size_t) (nc*nl), sizeof(vzrgene_t*));\
    mat[0][0] = (vzrgene_t *) calloc((size_t) (nc*nl*n), sizeof(vzrgene_t));\
    if (mat[0])\
	for(i=1;i<nl;i++) mat[i]= mat[i-1] + nc;\
    else {\
	free(mat);\
	mat = (vzrgene_t ***) NULL;\
    }\
    if(mat[0][0]) {\
        for(j=1;j<nc;j++) mat[0][j] = mat[0][j-1]+n;\
        for(i=1;i<nl;i++) {\
	    mat[i][0] = mat[i-1][nc-1]+n;\
            for(j=1;j<nc;j++) mat[i][j] = mat[i][j-1]+n;\
        }\
    }\
    else {\
        free(mat[0]);\
	free(mat);\
        mat = (vzrgene_t ***) NULL;\
    }\
}

#define DELvecteur(vec) free(vec)	    
#define DELmatrice(mat) {free(mat[0]); free(mat);}
#define DELmatrice3D(mat) {free(mat[0][0]); free(mat[0]); free(mat);}

#endif

