//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   vzr_survivre.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2008-03-19.09.17.44

 \brief selection des individus
 */

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#include "vzr_util.h"

/* fonction d'evaluation std */
struct vzrfitnessstd
{
    void *extra;
    int (*fun)(vzrgene_t*, double*, void*);
};

static int vzrfunstd_fun(void *info, vzrgene_t *in, double *out)
{
    struct vzrfitnessstd *std = (struct vzrfitnessstd *) info;

    return (*std->fun)(in, out, std->extra);
}

static void vzrfunstd_free(struct vzrfitness *fun)
{
    free(fun);
}

static struct vzrfitness *vzrfunstd_objet(void *info)
{
    struct vzrfitness *ret;

    ret = (struct vzrfitness *) malloc(sizeof(struct vzrfitness));
    ret->data = info;
    ret->compute = &vzrfunstd_fun;
    ret->dealloc = &vzrfunstd_free;

    return ret;
}

static void vzrfunstd_clean(struct vzrfitnessbuild *self)
{
    free(self->data);
    free(self);
}

struct vzrfitnessbuild *vizir1_vzrfunstd_create(int (*fct)(vzrgene_t*, double*, void*),
        void *extra)
{
    struct vzrfitnessstd *dt;
    struct vzrfitnessbuild *af;

    dt = malloc(sizeof(struct vzrfitnessstd));
    dt->extra = extra;
    dt->fun = fct;

    af = (struct vzrfitnessbuild*) malloc(sizeof(struct vzrfitnessbuild));
    af->data = (void *) dt;
    af->create = &vzrfunstd_objet;
    af->dealloc = &vzrfunstd_clean;

    return af;
}

/*--------------------------------------------------------------------------*/
/* survie */
void vizir1_tridicho(struct vizir *self) /*tri suivant l' ordre croissant de r : */
{ /*  r lui meme et pop_val et population */
    int h, u, v, i, k;
    double temp;
    vzrgene_t tempg;

    for (h = self->N / 2; h > 0; h /= 2)
        for (u = h; u < self->N; u++)
            for (v = u - h;
                    v >= 0
                            && self->pop_val[v][self->rangdomi]
                                    > self->pop_val[v + h][self->rangdomi]; v -=
                            h)
            {
                for (k = 0; k < self->nbcrit; k++)
                {
                    temp = self->pop_val[v][k];
                    self->pop_val[v][k] = self->pop_val[v + h][k];
                    self->pop_val[v + h][k] = temp;
                }
                for (i = 0; i < 2; i++)
                    for (k = 0; k < self->nbgene; k++)
                    {
                        tempg = self->population[v][i][k];
                        self->population[v][i][k] =
                                self->population[v + h][i][k];
                        self->population[v + h][i][k] = tempg;
                    }
            }

}

int vizir1_parents(struct vizir *self)
{
    int ret;

    /* classement de la population */
    PRO_CALL(self->darwin, rerank, self);
    pthread_mutex_lock(&(self->mutpop));
    vizir1_tridicho(self); /*modifie pop-val,population */

    /* test d'arret */
    self->nbelite = PRO_CALL(self->darwin, elite_size, self);
    if (self->pop_val[self->itaille - 1][self->rangdomi]
            - self->pop_val[0][self->rangdomi] <= self->prec /* la population est optimale*/
    || self->nbelite >= self->itaille /* */
    )
    {
        self->bgstate |= CONV;
        ret = 1; /* arret des generations */
    }
    else
    {
        /* on doit construire une nouvelle generation */
        self->nbsurv = self->nbelite +
                (int) floor(self->alpha * (self->N - self->nbelite)); /* nb de survivant pour la generation */
        self->fitseuil = self->pop_val[self->nbsurv - 1][self->rangdomi];
        ret = 0;
    }
    pthread_mutex_unlock(&(self->mutpop));
    return ret;
}

