#include "TAttribute.h"
#include "TNormalDistribution.h"
#include "TLogNormalDistribution.h"
#include "TSimpleTransform.h"
#include "TCJitEval.h"
#include "TFormEval.h"
#include "TThreadedRun.h"
#include "TGradientEstimationRun.h"
#include "TDataServer.h"
#include "TOptimFit.h"
#include "TNloptCobyla.h"
#include "TNloptDirect.h"
#include "TNlopt.h"
#include "TSorm.h"

/* possible solver
 * gradient
 #define SOLV TNloptMMA
 #define SOLV TNloptSLSQP
 * lagragien gradient
 #define SOLV TNloptLBFGS
 #define SOLV TNloptNewton
 #define SOLV TNloptVariableMetric
 *
 * */
#define SOLV TNloptMMA

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;
using namespace URANIE::Reliability;

/********** User Function *****************/
static void fselect(double *in, double *res)
{
    double x, fe, M;

    x = in[0];
    fe = in[1];
    M = in[2];
    /* critere */
    res[0] = M - x * fe;
}

/********** Optimization  *****************/
void test_TNloptGrad()
{
    /* inputs */
    TNormalDistribution x("x", 40, 5), fe("x2", 50, 2.5), M("M", 1000, 200);
    /* outputs */
    TAttribute cont("seuil");
    /* starting point */
    double start[] =
    { -1., -1., 1. };

    /* code */
    TSimpleTransform fobj;
    fobj.setParameters(3, &x, &fe, &M);

    TCJitEval fcont(&fselect);
    fcont.setInputs(3, &x, &fe, &M);
    fcont.setOutputs(1, &cont);

    TFormEval code(&fobj, &fcont);
    TGreaterFit it(0.0);
    code.addConstraint(&cont, &it);

    TThreadedRun trun(&code, 5);
    TGradientEstimationRun run(&trun);
    run.startSlave();
    if (run.onMaster())
    {
        /* tds */
        TDataServer tds("toto", "tds for vizir test");
        code.addAllInputs(&tds);

        /*** FORM ***/
        /* optimizer */
        SOLV solv;
        TNlopt nlo(&tds, &run, &solv);
        code.addObjective(&nlo);

        /* resolution */
        nlo.setStartingPoint(3,start);
        nlo.solverLoop();

        /*** SORM ***/
        TSorm sorm(&tds, &run);

        sorm.solverLoop();

        /* resultats */
        tds.getTuple()->Scan("*");

        /* cleanup */
        run.stopSlave();
    }

}
