//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TNloptCobyla.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-03-20.16.05.36

 \brief   Cobyla algorithm
 */

#include "TNloptCobyla.h"

#include "nlopt.h"

ClassImp (URANIE::Reoptimizer::TNloptCobyla);
ClassImp (URANIE::Reoptimizer::TNloptSLSQP);
ClassImp (URANIE::Reoptimizer::TNloptMMA);

namespace URANIE
{
namespace Reoptimizer
{

//TNloptCobyla
TNloptCobyla::TNloptCobyla()
{
}
TNloptCobyla::~TNloptCobyla()
{
}

nlopt_opt TNloptCobyla::build(int var, int ineq, int equ, double prec,
        nlopt_opt *sub)
{
    UNUSED(ineq,equ);
    nlopt_opt ret;

    *sub = NULL;
    ret = nlopt_create(NLOPT_LN_COBYLA, var);
    nlopt_set_ftol_abs(ret, prec);

    return ret;
}

//TNloptSLSQP
TNloptSLSQP::TNloptSLSQP()
{
}
TNloptSLSQP::~TNloptSLSQP()
{
}

nlopt_opt TNloptSLSQP::build(int var, int ineq, int equ, double prec,
        nlopt_opt *sub)
{
    UNUSED(ineq,equ);
    nlopt_opt ret;

    *sub = NULL;
    ret = nlopt_create(NLOPT_LD_SLSQP, var);
    nlopt_set_ftol_abs(ret, prec);

    return ret;
}

//TNloptMMA
TNloptMMA::TNloptMMA()
{
}
TNloptMMA::~TNloptMMA()
{
}

nlopt_opt TNloptMMA::build(int var, int ineq, int equ, double prec,
        nlopt_opt *subsolv)
{
    UNUSED(ineq);
    nlopt_opt ret;

    if (equ)
    {
        // contrainte, utilisation du lagragien
        *subsolv = nlopt_create(NLOPT_LD_MMA, var);
        nlopt_set_ftol_abs(*subsolv, prec);
        ret = nlopt_create(NLOPT_AUGLAG_EQ, var);
        nlopt_set_local_optimizer(ret, *subsolv);
    }
    else
    {
        // pas de contrainte
        *subsolv = NULL;
        ret = nlopt_create(NLOPT_LD_MMA, var);
    }
    nlopt_set_ftol_abs(ret, prec);

    return ret;
}

}
} //namespace

