//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TNloptDirect.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-03-22.10.49.43

 \brief algorithme local direct (sans gradient) de NLopt
 */

#ifndef __NLOPT_DIRECT__
#define __NLOPT_DIRECT__

#include "TNlopt.h"

namespace URANIE
{
namespace Reoptimizer
{

// Direct Algorithm
class TNloptNelderMead: public TNloptSolver
{
protected:
    virtual nlopt_opt solver(int);

public:
    TNloptNelderMead()
    {
    }
    virtual ~TNloptNelderMead()
    {
    }

    virtual nlopt_opt build(int, int, int, double, nlopt_opt*);

    ClassDef(URANIE::Reoptimizer::TNloptNelderMead, ID_REOPTIMIZER)
};

class TNloptBobyqa: public TNloptNelderMead
{
protected:
    virtual nlopt_opt solver(int);

public:
    TNloptBobyqa()
    {
    }
    virtual ~TNloptBobyqa()
    {
    }

    ClassDef(URANIE::Reoptimizer::TNloptBobyqa, ID_REOPTIMIZER)
};

class TNloptPraxis: public TNloptNelderMead
{
protected:
    virtual nlopt_opt solver(int);

public:
    TNloptPraxis()
    {
    }
    virtual ~TNloptPraxis()
    {
    }

    ClassDef(URANIE::Reoptimizer::TNloptPraxis, ID_REOPTIMIZER)
};

class TNloptSubplexe: public TNloptNelderMead
{
protected:
    virtual nlopt_opt solver(int);

public:
    TNloptSubplexe()
    {
    }
    virtual ~TNloptSubplexe()
    {
    }

    ClassDef(URANIE::Reoptimizer::TNloptSubplexe, ID_REOPTIMIZER)
};

// Gradient algorithm
class TNloptLBFGS: public TNloptNelderMead
{
protected:
    virtual nlopt_opt solver(int);

public:
    TNloptLBFGS()
    {
    }
    virtual ~TNloptLBFGS()
    {
    }

    ClassDef(URANIE::Reoptimizer::TNloptLBFGS, ID_REOPTIMIZER)
};

class TNloptNewton: public TNloptNelderMead
{
protected:
    virtual nlopt_opt solver(int);

public:
    TNloptNewton()
    {
    }
    virtual ~TNloptNewton()
    {
    }

    ClassDef(URANIE::Reoptimizer::TNloptNewton, ID_REOPTIMIZER)
};

class TNloptVariableMetric: public TNloptNelderMead
{
protected:
    virtual nlopt_opt solver(int);

public:
    TNloptVariableMetric()
    {
    }
    virtual ~TNloptVariableMetric()
    {
    }

    ClassDef(URANIE::Reoptimizer::TNloptVariableMetric, ID_REOPTIMIZER)
};

}
} //namespace

#endif
