//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TOptimFit.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-11-28.15.27.31

 \brief   objectifs et contraintes standards
 */

#include "TOptimFit.h"

#include "math.h"

ClassImp (URANIE::Reoptimizer::TOptimFit);
ClassImp (URANIE::Reoptimizer::TMaximizeFit);
ClassImp (URANIE::Reoptimizer::TTargetFit);
ClassImp (URANIE::Reoptimizer::TLesserFit);
ClassImp (URANIE::Reoptimizer::TGreaterFit);
ClassImp (URANIE::Reoptimizer::TInsideFit);

namespace URANIE
{
namespace Reoptimizer
{

// TMaximizeFit
double TMaximizeFit::operator()(int size, double in, double *grad, double *ret)
{
    int i;
    for (i = 0; i < size; i++)
        ret[i] = -grad[i];
    return -in;
}

// TTargetFit
double TTargetFit::operator()(double in)
{
    //return fabs(in - _ref);
    return (in - _ref) * (in - _ref);
}

double TTargetFit::operator()(int size, double in, double *grad, double *ret)
{
    int i;
    for (i = 0; i < size; i++)
        ret[i] = 2 * grad[i] * (in - _ref);
    return (in - _ref) * (in - _ref);
}

//TLesserFit
double TLesserFit::operator()(int size, double in, double *grad, double *ret)
{
    int i;
    for (i = 0; i < size; i++)
        ret[i] = grad[i];
    return in - _ref;
}

// TGreaterFit
double TGreaterFit::operator()(int size, double in, double *grad, double *ret)
{
    int i;
    for (i = 0; i < size; i++)
        ret[i] = -grad[i];
    return _ref - in;
}

// TInsideFit
TInsideFit::TInsideFit(double low, double top) :
        _low(low), _top(top)
{
}

double TInsideFit::operator()(double in)
{
    //if (in < _low) return _low - in ;
    //else if (in > _top) return in - _top ;
    //else return 0.0;
    return (in - _low) * (in - _top) / (_top - _low);
}

double TInsideFit::operator()(int size, double in, double *grad, double *ret)
{
    int i;
    for (i = 0; i < size; i++)
        ret[i] = grad[i] * (2 * in - _top - _low) / (_top - _low);
    return (in - _low) * (in - _top) / (_top - _low);
}

TMaximizeFit kMazimizeFit;

}
} // namespace

