//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TOptimShare.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-03-20.11.12.36

 \brief   methodes communes aux algo doptimisation
 */

#include "TOptimShare.h"

#include "TFitGetter.h"

//#include <typeinfo>
#define PREC 0.00001

ClassImp (URANIE::Reoptimizer::TOptimShare);

namespace URANIE
{
namespace Reoptimizer
{

//TOptimShare
TOptimShare::TOptimShare(URANIE::DataServer::TDataServer *tds,
        URANIE::Relauncher::TRun *run) :
        TMaster(tds, run)
{
    create();
    try{
        if(_Naux!=0)
        {
            TString list="";
            TObject *var;
            TIter next(&_param);
            while ((var = next()))
            {
                URANIE::Relauncher::TAuxiVar *tmp = dynamic_cast<URANIE::Relauncher::TAuxiVar *>(var);
                if( tmp !=0)
                    list+=TString(tmp->getRef()->GetName())+TString(" ");
            }
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                                                       Form("TOptimShare There are [%d] input attributes defined in the dataserver but not used in the code.\n The list of corresponding inputs is [%s]. This should not be doable when considering optmisation problems.", _Naux, list.Data()));
        }   
    }catch(URANIE::Exceptions::UErrorExceptions &ue)
    {
       	ue.printMessage();
	throw ue;        
    }

}

TOptimShare::TOptimShare(URANIE::DataServer::TDataServer *tds,
        URANIE::Relauncher::TEval *fun) :
        TMaster(tds, fun)
{
    create();
    try{
        if(_Naux!=0)
        {
            TString list="";
            TObject *var;
            TIter next(&_param);
            while ((var = next()))
            {
                URANIE::Relauncher::TAuxiVar *tmp = dynamic_cast<URANIE::Relauncher::TAuxiVar *>(var);
                if( tmp !=0)
                    list+=TString(tmp->getRef()->GetName())+TString(" ");
            }
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                                                       Form("TOptimShare There are [%d] input attributes defined in the dataserver but not used in the code.\n The list of corresponding inputs is [%s]. This should not be doable when considering optmisation problems.", _Naux, list.Data()));
        }   
    }catch(URANIE::Exceptions::UErrorExceptions &ue)
    {
       	ue.printMessage();
	throw ue;        
    }
}

TOptimShare::~TOptimShare()
{
    TFitGetter *one;
    TIter nextc(&_const);

    while ((one = (TFitGetter *) nextc()))
        delete one;
}

/* constructeur helpers */
void TOptimShare::create()
{
    /* defaut */
    _blog = kFALSE;
    _prec = PREC;
    /* init */
    _idtrc = 0;
    _tdstrc = NULL;
    /* compile data */
    //std::cout << typeid(this).name() << " TOptimShare::create\n" ;
}

/* objective and constraints */
void TOptimShare::addConstraint(URANIE::DataServer::TAttribute *cnt,
        TOptimFit *fit)
{
    TFitGetter *one;
    one = new TFitGetter(cnt, _fun->getOutputList(), fit);
    _const.Add(one);
}

void TOptimShare::initTrace()
{
    TList *allattr = _tds->getListOfAttributes("*");
    TIter next(allattr);
    URANIE::DataServer::TAttribute *attr;
    while ((attr = (URANIE::DataServer::TAttribute *) next())) _tdstrc->addAttribute(attr);
    _tdstrc->createTuple();
    delete allattr;
}

}
} //namespace
