//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TVizir2.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-01-25.04.38.47

 \brief methodes communes aux algo de Vizir
 */

#ifndef __VIZIR2__
#define __VIZIR2__

//ROOT
#include "Rtypes.h"
#include "TNamed.h"

// Uranie
#include "Reoptimizer.h"
#include "TVizirShare.h"

#include "TDataServer.h"
#include "TEval.h"

namespace URANIE
{
namespace Reoptimizer
{

class TVizir2: public TVizirShare
{
protected:
    struct vzragent *_agent;

    void run(int cpus);

public:
    TVizir2(URANIE::DataServer::TDataServer *tds, URANIE::Relauncher::TRun *fun,
            TVizirSolver *algo);
    TVizir2(URANIE::DataServer::TDataServer *tds,
            URANIE::Relauncher::TEval *fun, TVizirSolver *algo);
    virtual ~TVizir2();

    /** @name TMaster protocole **/
    virtual void solverLoop();
    /** extra **/
    bool isConverged();
    void extend(int more);
    void fitnessChanged();

    int item(int id, double *zone, double *front, int *rank, int *age, int *info, double *isol);

    ClassDef(URANIE::Reoptimizer::TVizir2, ID_REOPTIMIZER)
};

}
} //namespace
#endif
