// --- C++ ---
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2024 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TVizirAnts.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2024-05-06
 
 \brief  ants colony optimisation algorithm
*/

#include "TVizirAnts.h"

// vizir
#include "ants/aco.h"
#include "vizir/vzr_macro.h"

ClassImp(URANIE::Reoptimizer::TVizirPheromone);
ClassImp(URANIE::Reoptimizer::TVizirAnts);

namespace URANIE
{
namespace Reoptimizer
{

//TVizirAnts
/* constructeur destructeur */
TVizirAnts::TVizirAnts() : TVizirSolverShare()
{
    defaultShare();
    _phero = NULL;
    //_coded = kTRUE;
}

TVizirAnts::~TVizirAnts() {}

/* intialisation */
struct vzragent *TVizirAnts::build(int gene, int obj, int cont, double prec)
{
    struct aco *aco;
    
    aco = aco_init(gene, obj, cont, prec);
    buildShare(&(aco->base));
    
    if (_phero) aco->work = _phero->pheromone();
    else return NULL;
    
    if (_doresume) {
        aco_resume(aco, _pop, _resume);
        vzrpop_set_again(&(aco->base.pop));
    }
    else aco_prepare(aco, _step, _pop, _eval);
    
    return vzragent_aco(aco);
}

}} //namespace

