/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2024 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TVizirAnts.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2024-05-06
 
 \brief  ants colony optimisation algorithm
*/

#ifndef __TVIZIRANTS__
#define __TVIZIRANTS__

//ROOT
#include "Rtypes.h"
#include "TObject.h"

//Uranie
#include "Reoptimizer.h"
#include "TVizirSolver.h"

struct acorun;

namespace URANIE
{
namespace Reoptimizer
{

class TVizirPheromone
{
public:
    virtual ~TVizirPheromone() {}
    virtual struct acorun *pheromone() = 0;
    
    ClassDef(URANIE::Reoptimizer::TVizirPheromone, ID_REOPTIMIZER)
};

class TVizirAnts: public TVizirSolverShare
{
protected:
    TVizirPheromone *_phero;
    
public:
    TVizirAnts();
    virtual ~TVizirAnts();
    
    // set specific parameter
    void setPheromone(TVizirPheromone *phero)
    {
        _phero = phero;
    }
    
    // protocole
    virtual struct vzragent *build(int, int, int, double);
    virtual Bool_t isCoded() {return kFALSE;}
    
    ClassDef(URANIE::Reoptimizer::TVizirAnts, ID_REOPTIMIZER)
};

}} // namespace

#endif //__TVIZIRANTS__
