//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TVizirEval.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-01-11.15.51.32

 \brief   utilisation des vzrfitnessbuild de vizir
 */

#include "TVizirEval.h"

#include "vizir/vzrfitness.h"
#include "vizir/vzr_macro.h"

ClassImp (URANIE::Reoptimizer::TVizirEval);

namespace URANIE
{
namespace Reoptimizer
{

TVizirEval::TVizirEval(struct vzrfitnessbuild *fbuild) :
    TDoubleEval()
{
    _build = fbuild;
    _fit = (struct vzrfitness *) NULL;
}

TVizirEval::~TVizirEval()
{
    if (_fit)
        PRO_DEALLOC(_fit);
    else
        PRO_DEALLOC(_build);
}

void TVizirEval::buildVizir()
{
    _fit = PRO_CREATE(_build);
}

URANIE::Relauncher::TEval *TVizirEval::rebuild(int rank, Bool_t chef)
{
    UNUSED(rank);
    if (chef)
    {
        TVizirEval *ret = new TVizirEval(*this);
        ret->buildVizir();
        return (TEval *) ret;
    }
    else
        return (TEval *) this;
}

int TVizirEval::eval(Double_t *in, Double_t *out, int ncalc)
{
    UNUSED(ncalc);
    return PRO_CALL(_fit, compute, in, out);
}


}
} //namespace
