//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TVizirGenetic.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-03-31.10.57.52

 \brief   optimisation par Vizir
 */

#include "TVizirGenetic.h"

#include <iostream>
#include <stdio.h>

//ROOT
//#include "TRegexp.h"
//#include "TPad.h"
//#include "TH1F.h"

//URANIE
//#include "UExceptions.h"
//#include "TRun.h"

#include "genetic/vizir.h"
//#include "vizir/vzragent_api.h"
//#include "vizir/vzr_macro.h"

ClassImp (URANIE::Reoptimizer::TVizirCrossover);
ClassImp (URANIE::Reoptimizer::TVizirGenetic);

namespace URANIE
{
namespace Reoptimizer
{

// TVizirGenetic
/* constructeur destructeur */
TVizirGenetic::TVizirGenetic()
{
    /* specific defaults */
    defaultShare();
    _cross = NULL;
    _mutation = _zygotie = _survie = -1.0;
    _coded = kTRUE;
}

TVizirGenetic::~TVizirGenetic()
{
}

/* initialisation */
struct vzragent * TVizirGenetic::build(int gene, int obj, int cont, double prec)
{
    struct vizir *viz;
	
    viz = vzr_init(gene, obj, cont, prec);
    buildShare(&(viz->base));
    
    if (_survie > 0.0)
        vzr_setSurvivalRate(viz, _survie);
    if (_mutation >= 0.0)
        vzr_setMutationRate(viz, _mutation);
    if (_zygotie >= 0.0)
        vzr_setHomozygoteRate(viz, _zygotie);
    if (_cross)
        vzr_setCrossover(viz, _cross->crossover());
    if (_doresume) {
        vzr_resume(viz, _pop, _resume);
        vzr_reevaluate(viz);
    }
    else vzr_prepare(viz, _step, _pop, _eval);

    return vzragent_vzr(viz);
}

/* parameters */
void TVizirGenetic::setCrossover(TVizirCrossover *val, Bool_t coded)
{
    _cross = val;
    _coded = coded;
}

/* solve */
void TVizirGenetic::fitnessChanged(struct vzragent *ag)
{
    vzr_reevaluate((struct vizir *) ag->data);
}

/* debug helpers */
void TVizirGenetic::printLog(Option_t *option)
{
    std::cout << " ** TVizirGenetic::printLog[" << option << "]" << std::endl;
    std::cout << " ** Name[" << GetName() << "] Title[" << GetTitle() << "]"
            << std::endl;
    std::cout << " ** population[" << _pop << "] duree[" << _eval << "]"
            << std::endl;
    //std::cout << " ** gene[" << _NinDat << "] contrainte[" << _valid << "] cost[" << _obj << "]" << std::endl;
    std::cout << " ** survie[" << _survie << "] zygotie[" << _zygotie
            << "] mutation[" << _mutation << "]" << std::endl;
    std::cout << " ** End Of TVizirGenetic::printLog[" << option << "]"
            << std::endl;

}

}
} // closing namespace
