//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TVizirGenetic.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-03-31.10.57.39

 \brief optimisation par Vizir
 */

#ifndef __TVIZIR__
#define __TVIZIR__

//ROOT
#include "Rtypes.h"
#include "TNamed.h"

// Uranie
#include "Reoptimizer.h"
#include "TVizirSolver.h"

//struct vizir;
struct vzrcrossoverbuild;

namespace URANIE
{
namespace Reoptimizer
{

class TVizirCrossover
{
public:
    virtual ~TVizirCrossover()
    {
    }
    virtual struct vzrcrossoverbuild *crossover() = 0;

    ClassDef(URANIE::Reoptimizer::TVizirCrossover, ID_REOPTIMIZER)
};

class TVizirGenetic: public TVizirSolverShare
{
protected:
    //struct vizir *_viz;
    TVizirCrossover *_cross;
    double _survie, _zygotie, _mutation;
    Bool_t _blog, _coded;

public:
    TVizirGenetic();
    virtual ~TVizirGenetic();

    // set specific parameter
    //! Set the survival rate
    void setSurvivalRate(double val)
    {
        _survie = val;
    }
    //! Set the proba of homozigotie
    void setHomozygoteRate(double val)
    {
        _zygotie = val;
    }
    //! Set the probability of mutation
    void setMutationRate(double val)
    {
        _mutation = val;
    }
    //! Set crossover
    void setCrossover(TVizirCrossover *val, Bool_t coded=kFALSE);

    // protocole
    virtual struct vzragent *build(int, int, int, double);
    virtual Bool_t isCoded() {return _coded;}

    void fitnessChanged(struct vzragent *);

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    //@}
    //! print log
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Reoptimizer::TVizirGenetic, ID_REOPTIMIZER)
};

}
} //namespace
#endif
