//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TVizirShare.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-01-25.04.38.47

 \brief methodes communes aux algo de Vizir
 */

#ifndef __VIZIR_SHARE__
#define __VIZIR_SHARE__

//ROOT
#include "Rtypes.h"
#include "TNamed.h"

// Uranie
#include "Reoptimizer.h"
#include "TOptimShare.h"

#include "TDataServer.h"
#include "TEval.h"
#include "TOptimFit.h"
#include "TVizirSolver.h"

namespace URANIE
{
namespace Reoptimizer
{

class TVizirShare: public TOptimShare
{
protected:
    int _obj, _valid;
    TMatrixD _minmax;
    TObjArray _crit;
    TVizirSolver *_solv;

    //void buildDomain();
    void build();

    void nodecode(Double_t *code, Double_t *phys);
    void decode(Double_t *code, Double_t *phys);
    void fillChoice(vector<URANIE::DataServer::UEntry*> *all, double *out);
    vector<URANIE::DataServer::UEntry *> * fillData( vector<URANIE::DataServer::UEntry*> *in, 
        vector<URANIE::DataServer::UEntry*> *out);
    void fillTrace(vector<URANIE::DataServer::UEntry*> *in, 
        vector<URANIE::DataServer::UEntry*> *out,
        vector<URANIE::DataServer::UEntry*> *all);
    //virtual void run(TRun *call, int cpus) = 0;
    void fill(struct vzragent *ag);
    void fill(struct vzragent *ag, int &decal);

public:
    TVizirShare(URANIE::DataServer::TDataServer *tds,
            URANIE::Relauncher::TRun *run, TVizirSolver *algo);
    TVizirShare(URANIE::DataServer::TDataServer *tds,
            URANIE::Relauncher::TEval *fun, TVizirSolver *algo);
    virtual ~TVizirShare();

    // initialisation
    void setFixedSeed();
    virtual void addObjective(URANIE::DataServer::TAttribute *, TOptimFit *f =
            NULL);

    ClassDef(URANIE::Reoptimizer::TVizirShare, ID_REOPTIMIZER)
};

}
} //namespace
#endif
