////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#ifndef __TESTTNLOPT__
#define __TESTTNLOPT__

// CppUnit
#include <cppunit/extensions/HelperMacros.h>
// #include <cppunit/Test.h>

// URANIE
#include "TDataServer.h"
#include "TNlopt.h"

class TNloptTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TNloptTest);

    CPPUNIT_TEST (roseBobyqa);
    CPPUNIT_TEST (roseSimplex);
    CPPUNIT_TEST (rosePraxis);
    CPPUNIT_TEST (roseLBFGS);
    CPPUNIT_TEST (roseNewton);

    CPPUNIT_TEST (constCobyla);
    CPPUNIT_TEST (constMMA);
    CPPUNIT_TEST (constSLSQP);
    CPPUNIT_TEST (constSubplexe);
    CPPUNIT_TEST (constVariableMetric);

    CPPUNIT_TEST (constMMADiff);
    CPPUNIT_TEST (constSLSQPDiff);
    CPPUNIT_TEST (constVariableMetricDiff);

#ifdef WITH_MPI    
    CPPUNIT_TEST (barMPIMMAGradDCJit);
    CPPUNIT_TEST (barMPIMMAGradDCInt);
    CPPUNIT_TEST (barMPIMMAGradDCodeLine);
    CPPUNIT_TEST (barMPIMMAGradDCodeFlat);
    CPPUNIT_TEST (barMPIMMAGradDCodeKey);

    CPPUNIT_TEST (barMPISubplexeGradDCJit);
    CPPUNIT_TEST (barMPISubplexeGradDCInt);
    CPPUNIT_TEST (barMPISubplexeGradDCodeLine);
    CPPUNIT_TEST (barMPISubplexeGradDCodeFlat);
    CPPUNIT_TEST (barMPISubplexeGradDCodeKey);

    CPPUNIT_TEST (barMPISubplexeNoGradDCJit);
    CPPUNIT_TEST (barMPISubplexeNoGradDCInt);
    CPPUNIT_TEST (barMPISubplexeNoGradDCodeLine);
    CPPUNIT_TEST (barMPISubplexeNoGradDCodeFlat);
    CPPUNIT_TEST (barMPISubplexeNoGradDCodeKey);

    CPPUNIT_TEST (barMPISubplexeInitInConstCJit);
    CPPUNIT_TEST (barMPISubplexeInitInConstCInt);
    CPPUNIT_TEST (barMPISubplexeInitInConstCodeLine);
    CPPUNIT_TEST (barMPISubplexeInitInConstCodeFlat);
    CPPUNIT_TEST (barMPISubplexeInitInConstCodeKey);
#endif
    
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     *      **/
    //@{
    //! Default constructor
    TNloptTest();

    //! Default destructor
    virtual ~TNloptTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    void tearDown();
    //@}
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void roseBobyqa();
    void roseSimplex();
    void rosePraxis();
    void roseLBFGS();
    void roseNewton();
    void constCobyla();
    void constMMA();
    void constSLSQP();
    void constSubplexe();
    void constVariableMetric();
    void constMMADiff();
    void constSLSQPDiff();
    void constVariableMetricDiff();
#ifdef WITH_MPI    
    void barMPIMMAGradDCJit();
    void barMPIMMAGradDCInt();
    void barMPIMMAGradDCodeLine();
    void barMPIMMAGradDCodeFlat();
    void barMPIMMAGradDCodeKey();

    void barMPISubplexeGradDCJit();
    void barMPISubplexeGradDCInt();
    void barMPISubplexeGradDCodeLine();
    void barMPISubplexeGradDCodeFlat();
    void barMPISubplexeGradDCodeKey();

    void barMPISubplexeNoGradDCJit();
    void barMPISubplexeNoGradDCInt();
    void barMPISubplexeNoGradDCodeLine();
    void barMPISubplexeNoGradDCodeFlat();
    void barMPISubplexeNoGradDCodeKey();

    void barMPISubplexeInitInConstCJit();
    void barMPISubplexeInitInConstCInt();
    void barMPISubplexeInitInConstCodeLine();
    void barMPISubplexeInitInConstCodeFlat();
    void barMPISubplexeInitInConstCodeKey();
#endif
    //@}

private:
    void *_p2fRosenbrock;

    void rosenbrock(URANIE::Reoptimizer::TNloptSolver &solv);
    void rosenbrockGrad(URANIE::Reoptimizer::TNloptSolver &solv);
    void rosenbrockVec(URANIE::Reoptimizer::TNloptSolver &solv);
    void validateRosenbrock(URANIE::DataServer::TDataServer &tds, int sz);
    void rosenbrock2(URANIE::Reoptimizer::TNloptSolver &solv);
    void validateRosenbrock2(URANIE::DataServer::TDataServer &tds);
    void constraint(URANIE::Reoptimizer::TNloptSolver &solv);
    void constraintGrad(URANIE::Reoptimizer::TNloptSolver &solv);
    void constraintPlus(URANIE::Reoptimizer::TNloptSolver &solv);
    void validateConstraint(URANIE::DataServer::TDataServer &tds, int natt);
    void validateBar();

};

#endif
