//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#ifndef __TESTVIZIR__
#define __TESTVIZIR__

// CppUnit
#include <cppunit/extensions/HelperMacros.h>
// #include <cppunit/Test.h>

// URANIE
#include "TDataServer.h"
#include "TAttribute.h"
#include "TCIntEval.h"
#include "TCJitEval.h"
#include "TVizirSolver.h"

class TVizirTest: public CPPUNIT_NS::TestFixture
{
public:
    /** @name Constructor and Destructor
     *      **/
    //@{
    //! Default constructor
    TVizirTest();

    //! Default destructor
    virtual ~TVizirTest();
    //@}

    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    void tearDown();
    //@}

    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void mono();
    void cmaes();
    void multi();
    void many_pair();
    void many_ibea();
    void many_knee();
    void many_moeadd();
    virtual void monoConstrained();
    virtual void multiConstrained();
    //@}

protected:
    URANIE::DataServer::TAttribute *_x, *_y, *_z;
    URANIE::DataServer::TAttribute *_obj1, *_obj2, *_obj12, *_cont1, *_cont2,
            *_cont3, *_cont;
    URANIE::Relauncher::TEval *_fun;
    TList _todel;

    void many(URANIE::Reoptimizer::TVizirSolver &);

    void validate(URANIE::DataServer::TDataServer &tds);
    void validateCmaes(URANIE::DataServer::TDataServer &tds);
    void validateMono(URANIE::DataServer::TDataServer &tds);
    void validateMonoC(URANIE::DataServer::TDataServer &tds);
    void validateMonoCVec(URANIE::DataServer::TDataServer &tds);
    void validateMulti(URANIE::DataServer::TDataServer &tds);
    void validateMany(URANIE::DataServer::TDataServer &tds);
    void validateMultiC(URANIE::DataServer::TDataServer &tds);

};

class TVizirJitTest: public TVizirTest
{
    CPPUNIT_TEST_SUITE (TVizirJitTest);

    CPPUNIT_TEST (mono);
    CPPUNIT_TEST (cmaes);
    CPPUNIT_TEST (multi);
    CPPUNIT_TEST (many_pair);
    CPPUNIT_TEST (many_ibea);
    CPPUNIT_TEST (many_moeadd);
    CPPUNIT_TEST (monoConstrained);
    CPPUNIT_TEST (multiConstrained);

    CPPUNIT_TEST_SUITE_END();

public:
    /** @name Constructor and Destructor
     *      **/
    //@{
    //! Default constructor
    TVizirJitTest();

    //! Default destructor
    virtual ~TVizirJitTest();
    //@}

    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    //@}
};

class TVizirJitTest2: public TVizirTest
{
    CPPUNIT_TEST_SUITE (TVizirJitTest2);

    CPPUNIT_TEST (monoConstrained);

    CPPUNIT_TEST_SUITE_END();

public:
    /** @name Constructor and Destructor
     *      **/
    //@{
    //! Default constructor
    TVizirJitTest2();

    //! Default destructor
    virtual ~TVizirJitTest2();
    //@}

    void monoConstrained();

    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    //@}
};

class TVizirIntTest: public TVizirTest
{
    CPPUNIT_TEST_SUITE (TVizirIntTest);

    CPPUNIT_TEST (mono);
    CPPUNIT_TEST (multi);
    CPPUNIT_TEST (many_knee);
#ifdef ROOT5
    CPPUNIT_TEST( monoConstrained ); //Threaded
    CPPUNIT_TEST( multiConstrained ); //Threaded
#endif

    CPPUNIT_TEST_SUITE_END();

public:
    /** @name Constructor and Destructor
     *      **/
    //@{
    //! Default constructor
    TVizirIntTest();

    //! Default destructor
    virtual ~TVizirIntTest();
    //@}

#ifdef ROOT5
    void monoConstrained();
    void multiConstrained();
#endif
    
    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();    
    //@}

protected:
    void *_p2fSelect;
};

class TVizirExeTest: public TVizirTest
{
    CPPUNIT_TEST_SUITE (TVizirExeTest);

    CPPUNIT_TEST (monoConstrained);
    CPPUNIT_TEST (multiConstrained); //Threaded

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     *      **/
    //@{
    //! Default constructor
    TVizirExeTest();

    //! Default destructor
    virtual ~TVizirExeTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    void tearDown();
    //@}
};

#endif

