/* --- Ansi C --- */
/* --- coding: utf-8 ---

    File
      creation : 2009-10-22.15.20.18
      revision : $Id$

    Copyright © 2009 Commissariat à l'Energie Atomique
      par Gilles ARNAUD (DM2S/SFME/LETR)
        C.E. Saclay; Bat 454; 91191 GIF/YVETTE CEDEX; France
        Tel: 01 69 08 38 86; Fax : 33 1 69 08 85 68 
        Gilles.Arnaud@cea.fr
 
    Object
      operateur specifique au plan de rechargement
 
___________________________________________________________________*/

#include <stdlib.h>

#include "ants/aco_struct.h"

#include "vizir/vzr_aleas.h"
#include "vizir/vzr_alloc.h"
#include "vizir/vzr_macro.h"

//#define TRACE

#define ASSM 1
#define POSM 2

#define EXACT_UPDATE 1
#define COLOR_UPDATE 0.8
#define TOTO_UPDATE  0.6

#define COLOR(x) x
#define TOTO(x) x

struct localdata {
    int zone, color;
    double **pherom;
};

/* initialisation de l'env
 * */
static void prepare(void *info, struct aco *self)
{
    int ic, iz;
    struct localdata *run = (struct localdata *) info;
    
    for EACH(iz, run->zone) {
        for EACH(ic, run->color) run->pherom[iz][ic] = self->tmax;
    }
}

/* génération d'un chemin
 * */
static int build_solution(void *info, struct aco *self, double *sol)
{
    int iz, ic;
    double cum;
    struct localdata *run = (struct localdata *) info;
    VZRTAB(double, prob, run->color);

    for EACH(iz, run->zone) {
        cum = 0.;
        for EACH(ic, run->color) {
            prob[ic] = run->pherom[iz][ic];
            cum += prob[ic];
        }
        cum *= mt_real2();
        for (ic=0; cum>0; ic++) cum -= prob[ic] ;
        ic--;
        sol[iz] = ic*255/run->color;
    }
    VZRTAB_OUT(prob);
    return 1;

}

/* ajout de pheromone sur un chemin
 * */
static void update(void *info, struct aco *self, double *ind, double rg)
{
    int iz, ic, id;
    double cum;
    struct localdata *run = (struct localdata *) info;

    for EACH(iz, run->zone) {
        id = ind[iz]*run->color/255;
        for EACH(ic, run->color) {
            cum =  (id == ic)? COLOR_UPDATE : 0.;
            cum += run->pherom[iz][ic] * self->evap;
            if (cum > self->tmax) cum = self->tmax ;
            else if (cum < self->tmin) cum = self->tmin ;
            run->pherom[iz][ic] = cum;
        }
    }
}

/* dealloc
 * */
static void dealloc(struct acorun *self)
{
    struct localdata *run = (struct localdata *) self->data;

    DELmatrice(run->pherom);
    free(run);
    free(self);
}

/* creation
 * */
struct acorun *acoplan_create(int zone, int color)
{
    int i;
    struct acorun *ret;
    struct localdata *dat;

    dat = (struct localdata *) malloc(sizeof(struct localdata));
    dat->zone = zone;
    dat->color = color;
    NEWmatrice(double, dat->pherom, zone, color);

    ret = (struct acorun *) malloc(sizeof(struct acorun));
    ret->data = (void *) dat;
    ret->prepare = &prepare;
    ret->build = &build_solution;
    ret->depose = &update;
    ret->dealloc = &dealloc;
    
    return ret;
}

