//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifdef JIT
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TAttribute.h"
#include "TTangramEval.h"
#include "TMpiRun.h"
#include "TDataServer.h"
#include "TOptimFit.h"
#include "TVizirSwarm.h"
#include "TVizirGenetic.h"
#include "TVizir2.h"

#endif

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::MpiRelauncher;
using namespace URANIE::Reoptimizer;

/* user defined */

#define TOLERANCE 0.001
#define NBmaxEVAL 50000
#define TAILLEPOP 256 
//#define TAILLEPOP 10 

#define NBGENE 7 // nombre de paramètres

/* solver */
void testVizirMpi()
{
    /* init */
    TAttribute z1("zone1", 0., 255.), z2("zone2", 0., 255.), z3("zone3", 0.,
            255.), z4("zone4", 0., 255.), z5("zone5", 0., 255.), z6("zone6", 0.,
            255.), z7("zone7", 0., 255.), ct("contrast");

    TTangramEval code;
    code.addInput(&z1);
    code.addInput(&z2);
    code.addInput(&z3);
    code.addInput(&z4);
    code.addInput(&z5);
    code.addInput(&z6);
    code.addInput(&z7);
    code.addOutput(&ct);

    TMpiRun run(&code);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tdsvzr("tdsvzr", "tds for vizir test");
        code.addAllInputs(&tdsvzr);

        TVizirGenetic solv;
        solv.setSurvivalRate(0.5);
        solv.setSize(TAILLEPOP, NBmaxEVAL);
        TVizirIsland vzr(&tdsvzr, &run, &solv);
        vzr.setTolerance(TOLERANCE);
        TMaximizeFit maxim;
        vzr.addObjective(&ct, &maxim);

        /* resolution */
        vzr.solverLoop();

        tdsvzr.exportData("_tangram_mpi_.dat");
        run.stopSlave();
    }

}
