
import math
import ROOT
from ROOT.URANIE import DataServer, Relauncher, Reoptimizer, MpiRelauncher

# low level eval
def lowfun(x, y, z, aa, bb, aaa, bbb, zzz, r1, r2, r3) :
    d1 = x*x + y*y + z*z
    d2 = (x-aa)*(x-aa) + (y-bb)*(y-bb) + z*z
    d3 = (x-bb)*(x-bb) + (y-aa)*(y-aa) + z*z
    d4 = (x-aaa)*(x-aaa) + (y-bbb)*(y-bbb) + (z-zzz)*(z-zzz)
    d5 = (x-bbb)*(x-bbb) + (y-aaa)*(y-aaa) + (z-zzz)*(z-zzz)
    
    out = []
    out.append(math.sqrt(d1) - r1)
    out.append(math.sqrt(d2) - r2)
    out.append(math.sqrt(d3) - r3)
    out.append(math.sqrt(d4) + math.sqrt(d5))
    return out

# resume from doe
def tds_resume(tds, att) :
    siz = tds.getTuple().GetEntries()
    leaves = [ tds.GetTuple().GetLeaf(at.GetName()) for at in att]
    
    obj = 0.0
    cont = 0.0
    for i in range(siz) :
        # get value
        tds.GetTuple().GetEntry(i)
        tmp = []
        for leaf in leaves :
            tmp.append( leaf.GetValue(0))
        # 
        if tmp[-1] > obj :
            obj = tmp[-1]
        tag = 0
        for k in tmp[:-1] :
            if k < 0 :
                tag = 1
        if tag :
            cont += 100./siz
    return [obj, cont]


# hight level eval
    

def optimS() :

    # inputs
    x = DataServer.TAttribute("x", -5.0, 5.0)
    y = DataServer.TAttribute("y", -5.0, 5.0)
    z = DataServer.TAttribute("z", -5.0, 5.0)
    funih = [x, y, z]
    r1 = DataServer.TAttribute("r1")
    r2 = DataServer.TAttribute("r2")
    zz = DataServer.TAttribute("zz")
    funil = [r1, r2, zz]
    # low output
    c1 = DataServer.TAttribute("c1")
    c2 = DataServer.TAttribute("c2")
    c3 = DataServer.TAttribute("c3")
    obj = DataServer.TAttribute("obj")
    funo = [c1, c2, c3, obj]
    # high output
    dist = DataServer.TAttribute("dist")
    cont = DataServer.TAttribute("cont")
    
    # fonction low
    lowlambfun = lambda x, y, z, r2, r3, zz : lowfun(x, y, z, 1.5, 0.5, 1.0, 0.5, zz, 1.0, r2, r3) 
    lfun = Relauncher.TPythonEval(lowlambfun)
    for h in funih:
        lfun.addInput(h)
    for l in funil:
        lfun.addInput(l)
    for o in funo :
        lfun.addOutput(o)

    # fonction high
    def doeTest(vx, vy, vz) :
        # runner
        #lrun = Relauncher.TSequentialRun(lfun)
        #lrun = Relauncher.TThreadedRun(lfun, 3)
        lrun = MpiRelauncher.TSubMpiRun(lfun)
        lrun.startSlave()
        if lrun.onMaster() :
            ltds = DataServer.TDataServer("doe", "tds4doe")
            ltds.keepFinalTuple(False)
            for i in funil :
                ltds.addAttribute(i)
            ltds.fileDataRead("doe.dat", False, True, "quiet")
        
            launch = Relauncher.TLauncher2(ltds, lrun)
            launch.addConstantValue(x, vx)
            launch.addConstantValue(y, vy)
            launch.addConstantValue(z, vz)

            # run doe
            launch.solverLoop()
        
            # get critere
            out = tds_resume(ltds, funo)
        
            lrun.stopSlave()
            ROOT.SetOwnership(lrun, True)
            return out
        else :
            # to avoid "no return" message
            # output size should be coherent
            return [0., 0.]
    #
    doefun = Relauncher.TPythonEval(doeTest)
    for h in funih :
        doefun.addInput(h)
    doefun.addOutput(dist)
    doefun.addOutput(cont)
    
    # runner
    #run = Relauncher.TSequentialRun(doefun)
    #run = Relauncher.TThreadedRun(doefun, 4)
    #run = MpiRelauncher.TMpiRun(doefun)
    run = MpiRelauncher.TBiMpiRun(doefun, 3)
    run.startSlave()
    if run.onMaster() :
        tds = DataServer.TDataServer("tdsvzr", "tds4optim")
        doefun.addAllInputs(tds)
        
        #
        gene = Reoptimizer.TVizirGenetic()
        gene.setSize(200, 20000, 50)
        #gene.setStrictConstraintPolicy()
        viz = Reoptimizer.TVizir2(tds, run, gene)
        viz.setTolerance(0.001)
        viz.addObjective(dist)
        dixb = Reoptimizer.TLesserFit(10.)
        viz.addConstraint(cont, dixb)
        
        viz.solverLoop()
        
        tds.exportData("_mpi_robust_py_.dat")

        run.stopSlave()
        ROOT.SetOwnership(run, True)


#ROOT.EnableThreadSafety()
optimS()
