
#define AA 1.5
#define BB 0.5
#define AAA 1.0
#define BBB 0.5

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::MpiRelauncher;
using namespace URANIE::Reoptimizer;


void lowfun(double *in, double *out)
{
    double x,y,z,r3,r2,zz;
    double d1, d2, d3, d4, d5;
    
    x = in[0]; y = in[1]; z = in[2];
    r2 = in[3]; r3 = in[4], zz = in[5];
    
    d1 = x*x + y*y + z*z;
    d2 = (x-AA)*(x-AA) + (y-BB)*(y-BB) + z*z;
    d3 = (x-BB)*(x-BB) + (y-AA)*(y-AA) + z*z;
    d4 = (x-AAA)*(x-AAA) + (y-BBB)*(y-BBB) + (z-zz)*(z-zz);
    d5 = (x-BBB)*(x-BBB) + (y-AAA)*(y-AAA) + (z-zz)*(z-zz);
    
    out[0] = sqrt(d1) - 1.0;
    out[1] = sqrt(d2) - r2;
    out[2] = sqrt(d3) - r3;
    out[3] = sqrt(d4) + sqrt(d5);
}

void tds_resume(TDataServer *tds, TAttribute **att, double *res)
{
    TList leaves;
    TLeaf *leaf;
    int i, j, k, siz, tag;
    double obj, cont;
    std::vector<double> tmp;
    
    siz = tds->getTuple()->GetEntries();
    
    for (i=0; att[i]; i++) {
        leaves.Add(tds->GetTuple()->GetLeaf(att[i]->GetName()));
    }
    tmp.resize(i);
    
    obj = 0;
    cont = 0;
    TIter nextl(&leaves);
    for (i=0; i<siz; i++, nextl.Reset()) {
        tds->GetTuple()->GetEntry(i);
        for (j=0; (leaf = (TLeaf *) nextl() ); j++) {
            tmp[j] = leaf->GetValue(0);
        }
        if (tmp[j-1] > obj) obj = tmp[j-1];
        tag = 0;
        for (k=0; k<j-1; k++) {
            if (tmp[k] < 0) tag = 1;
        }
        if (tag) cont += 100./siz;
    }
    res[0] = obj;
    res[1] = cont;
}
    

void testVizirRobust2()
{
    //ROOT::EnableThreadSafety();
    int i;

    // inputs
    TAttribute x("x", -5.0, 5.0);
    TAttribute y("y", -5.0, 5.0);
    TAttribute z("z", -5.0, 5.0);
    TAttribute r1("r1");
    TAttribute r2("r2");
    TAttribute zz("zz");
    // output low
    TAttribute c1("c1");
    TAttribute c2("c2");
    TAttribute c3("c3");
    TAttribute obj("obj");
    // output high
    TAttribute dist("dist");
    TAttribute cont("cont");
    
    // fonction low
    TAttribute *funil[] = {&r1, &r2, &zz, NULL};
    TAttribute *funih[] = {&x, &y, &z, NULL};
    TAttribute *funo[] = {&c1, &c2, &c3, &obj, NULL};
    
    TCJitEval lfun(lowfun);
    for (i=0; funih[i]; i++) lfun.addInput(funih[i]);
    for (i=0; funil[i]; i++) lfun.addInput(funil[i]);
    for (i=0; funo[i]; i++) lfun.addOutput(funo[i]);
    
    // fonction high
    std::function<int(double*,double*)> lambfun = [&lfun,&funih,&funil,&funo](double *in, double *out) {
        int i;
        
        // runner
        //TThreadedRun run(&lfun, 3);
        //TSequentialRun run(&lfun);
        TSubMpiRun run(&lfun);
        run.startSlave();
        if (run.onMaster()) {
            TDataServer tds("doe", "tds4doe");
            tds.keepFinalTuple(kFALSE);
            for (i=0; funil[i]; i++) tds.addAttribute(funil[i]);
            tds.fileDataRead("doe.dat", kFALSE, kTRUE, "quiet");
        
            TLauncher2 launch(&tds, &run);
            for (i=0; funih[i]; i++) launch.addConstantValue(funih[i], in[i]);

            // run doe
            launch.solverLoop();
        
            // get critere
            tds_resume(&tds, funo, out);
        
            run.stopSlave();
        }
        return 1;
    };
    TCJitEval doefun(&lambfun);
    for (i=0; funih[i]; i++) doefun.addInput(funih[i]);
    doefun.addOutput(&dist);
    doefun.addOutput(&cont);
    
    // runner
    //TThreadedRun run(&doefun, 4);
    //TSequentialRun run(&doefun);
    TBiMpiRun run(&doefun, 3);
    run.startSlave();
    if (run.onMaster()) {
        TDataServer tds("tdsvzr", "tds4optim");
        doefun.addAllInputs(&tds);
        
        //
        TVizirGenetic gene;
        gene.setSize(200, 20000, 50);
        //gene.setStrictConstraintPolicy();
        TVizir2 viz(&tds, &run, &gene);
        viz.setTolerance(0.001);
        viz.addObjective(&dist);
        TLesserFit dix(10.);
        viz.addConstraint(&cont, &dix);
        
        viz.solverLoop();
        
        run.stopSlave();
        
        tds.exportData("_mpi_robust2_.dat");
        
    }
    
}

