
#ifdef COMPIL
#include "TLeaf.h"

#include "TDataServer.h"
#include "TAttribute.h"
#include "TCJitEval.h"
#include "TSequentialRun.h"
#include "TThreadedRun.h"
#include "TLauncher2.h"
#include "TVizirGenetic.h"
#include "TVizir2.h"
#endif

/* user defined */
#define TOLERANCE 0.0001
#define POP 256
#define EVALmax 50000

#define NBPREC 100000

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;
using namespace URANIE::MpiRelauncher;

// pre declaration

int mpifun(double *in, double *out);


void testVizirUserMpi()
{
    /* input var */
    TAttribute pl("plus", 0.3, 3.);
    TAttribute mn("moins", 0.3, 3.);
    /* output var */
    TAttribute ln("ln2_dist");
    /* code */
    TCJitEval code(mpifun);
    code.addInput(&pl);
    code.addInput(&mn);
    code.addOutput(&ln);

    TBiMpiRun run(&code, 3);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tdsvzr("tdsvzr", "tds for vizir test");
        tdsvzr.addAttribute(&pl);
        tdsvzr.addAttribute(&mn);

        TVizirGenetic solv;
        solv.setSize(POP, EVALmax);
        TVizir2 vzr(&tdsvzr, &run, &solv);
        vzr.addObjective(&ln);
        vzr.setTolerance(TOLERANCE);

        /* resolution */
        vzr.solverLoop();

        /* sauvegarde */
        tdsvzr.exportData("_user_mpi_.dat");
        run.stopSlave();
    }
}

/* function is placed after optimMpiFun
 * otherwise cling doesnt't know about MPI_Comm type
 * retrieve it from TBiMpiRun declaration  
 ****/

int mpifun(double *in, double *out)
{
    double sum, res;
    int i, id, size;
    MPI_Comm comm;
    
    comm = URANIE::MpiRelauncher::TBiMpiRun::getCalculMpiComm();
    MPI_Comm_rank(comm, &id);
    MPI_Comm_size(comm, &size);
    
    /* local */
    sum = 0.;
    for (i = id; i < NBPREC; i += size) {
        if (i % 2) sum -= 1. / (i + in[0]);
        else sum += 1. / (i + in[1]);
    }
    /* global */
    MPI_Reduce(&sum, &res, 1, MPI_DOUBLE, MPI_SUM, 0, comm);
    if (!id) {
        res -= 0.69314718055994529;
        out[0] = res * res;
    }
    return 1;
    
}


#ifdef COMPIL
int main()
{
    optimMpiFun();
    return 0;
}
#endif
