/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   aco_agent.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-02-18
 
 \brief  agent island pour les fourmis
*/

#include <stdlib.h>

#include "aco_agent.h"

#include "vizir/vzritem.h"
#include "vizir/vzrfifo_api.h"

#include "aco_query.h"
#include "aco_core.h"

#define FIRST 3
#define STEP 100

static int aco_emigrate(struct aco *self, vzrgene_t *mig)
{
    int i, j, ret;
    struct vzrpop *pop;

    ret = 0;
    pop = &self->base.pop;
    pthread_mutex_lock(&(self->base.lock));
    for (i=0; i<pop->elite; i++) {
        if (!(pop->item[pop->rg[i]].flag & MIGRED)) {
            ret = 1;
            for (j=0; j<pop->gene; j++) mig[j] = pop->item[pop->rg[i]].zone[j];
            pop->item[i].flag |= MIGRED ;
            break;
        }
    }
    pthread_mutex_unlock(&(self->base.lock));
    return ret;
}


struct vzragent *vzragent_aco(struct aco *self)
{
    struct vzragent  *ret;

    /* agent */
    ret = (struct vzragent *) malloc(sizeof(struct vzragent));
    ret->data = (void *) self;
    ret->first = FIRST * self->base.pop.stop;
    ret->step = STEP;
    ret->immigr = ret->emigr = 0;
    vzrfifo_init(&(ret->migr), self->base.pop.gene);
    vzrfifo_build(&(ret->migr), self->base.pop.step);
    ret->core = (struct vzrcore *(*)(void*)) vzrcore_aco;
    ret->emigrate = (int(*)(void*, vzrgene_t*)) aco_emigrate ;
    ret->dealloc = (void(*)(void*)) aco_dealloc ;
    //ret->save = NULL ;

    self->base.migr = &(ret->migr);

    return ret;
}


