/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   nms_agent.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-06-09
 
 \brief  communicationt par ilot
*/

#include <stdlib.h>

#include "nms_agent.h"

#include "vizir/vzr_macro.h"
#include "vizir/vzrfifo_api.h"
#include "vizir/vzritem.h"
#include "nms_query.h"
#include "nms_core.h"

#define FIRST 3
#define STEP 3

static int nms_emigrate(struct nms *self, vzrgene_t *mig)
{
    int ret, i, j;
    struct vzrpop *pop = &self->base.pop;

    ret = 0;
    pthread_mutex_lock(&(self->base.lock));
    for EACH(i, pop->cur) {
        if (!(pop->item[pop->rg[i]].flag & MIGRED)) {
            for EACH(j, pop->gene) mig[j] = pop->item[pop->rg[i]].zone[j];
            pop->item[pop->rg[i]].flag |= MIGRED;
            //self->base.emigr++;
            ret = 1;
            break;
        }
    }
    pthread_mutex_unlock(&(self->base.lock));
    return ret;
}

struct vzragent *vzragent_nms(struct nms *self)
{
    struct vzragent  *ret;

    /* agent */
    ret = (struct vzragent *) malloc(sizeof(struct vzragent));
    ret->data = (void *) self;
    ret->first = FIRST * self->base.pop.stop;
    ret->step = STEP * self->base.pop.step;
    ret->immigr = ret->emigr = 0;
    vzrfifo_init(&(ret->migr), self->base.pop.gene);
    vzrfifo_build(&(ret->migr), self->base.pop.step);
    ret->core = (struct vzrcore *(*)(void*)) vzrcore_nms;
    ret->emigrate = (int(*)(void*,vzrgene_t*)) nms_emigrate ;
    ret->dealloc = (void(*)(void*)) nms_dealloc ;

    self->base.migr = &(ret->migr);

    return ret;
}


