/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   pso_query.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-03-04
 
 \brief  initilisation et requete resultat pour les essaims
*/

#include <pthread.h>
#include <stdlib.h>
#include <stdio.h>

#include "swarm/pso_query.h"

#include "swarm/pso_movestd.h"
#include "swarm/pso_guide.h"

#include "vizir/vzr_aleas.h"
#include "vizir/vzrbase_api.h"
#include "vizir/vzrpop_api.h"
#include "vizir/vzrfifo_api.h"
#include "vizir/vzr_alloc.h"
#include "vizir/vzr_macro.h"
#include "vizir/vzritem.h"

#define STEPRATIO 2

struct psoswarm *pso_init(int nbin, int nbout, int nbcont, double prec)
{
    struct psoswarm *ret;

    ret = (struct psoswarm *) malloc(sizeof(struct psoswarm));
    vzrpop_init(&(ret->base.pop), nbin, nbcont, nbout, prec);
    /* working value */
    mt_init(0);
    vzrbase_init(&(ret->base));
    ret->nbpart = ret->partc = 0;
    ret->move = NULL; ret->base.migr = NULL;
    ret->mate = NULL;

    return ret;
}

void pso_particle(struct psoswarm *self, int nbpart)
{
    self->nbpart = nbpart;
}

static int pso_build_common(struct psoswarm *self) 
{
    struct psopart *part;
    int i;
    
    /* default */
    if (!self->nbpart) self->nbpart = self->base.pop.step/STEPRATIO;
    if (!self->move) self->move = pso_movestd(self->base.pop.gene, 0, 2);
    if (self->mate) {
        if (self->mate->genre != None && self->mate->genre != self->base.pop.genre) {
            printf("parental selection dismatch env selection info\n");
            return 0;
        }
    }
    else self->mate = &psomate_std;
    /* build */
    NEWvecteur(struct psopart, self->swarm, self->nbpart);
    for EACH(i, self->nbpart) {
        part = &(self->swarm[i]);
        part->id = i;
        part->part = self->nbpart;
        //vzrpop_init(&(part->local), self->base.pop.gene, self->base.pop.valid, self->base.pop.cost, self->base.pop.precis);
        //part->local.hiden = self->base.pop.hiden;
        //part->local.policy = VZR_MOGA;
        //vzrpop_aploid_build(&(part->local), 24, 8);
        part->swarm = self;
        part->move = PRO_CREATE(self->move);
    }
    
    return 1;
}

int pso_prepare(struct psoswarm *self, int step, int stop, int nbeval)
{
    /* working value */
    vzrbase_build(&(self->base), nbeval);
    vzrpop_aploid_build(&(self->base.pop), stop, step);

    return pso_build_common(self);    
}

int pso_build(struct psoswarm *ret, int ant, int cyc, int stop)
{
    printf("pso_build is deprecated, use pso_prepare instead\n");
    ret->nbpart = ant;
    return pso_prepare(ret, ant*STEPRATIO, stop, cyc);
}

int pso_resume(struct psoswarm *self, int nbeval, char *nfile)
{
    /* working value */
    vzrbase_build(&(self->base), nbeval);
    vzrpop_resume(&(self->base.pop), APLOID, nfile);
    
    return pso_build_common(self);
}

void pso_dump(struct psoswarm *self, char *nfile)
{
    vzrpop_dump(&(self->base.pop), nfile);
}

void pso_dealloc(struct psoswarm *self)
{
    struct psopart *part;
    int i;

    vzrbase_dealloc(&(self->base));
    vzrpop_aploid_dealloc(&(self->base.pop));
    for EACH(i, self->nbpart) {
        part = &(self->swarm[i]);
        //vzrpop_aploid_dealloc(&(part->local));
        PRO_DEALLOC(part->move);
    }
    DELvecteur(self->swarm);
    PRO_DEALLOC(self->move);
    free(self);
}
