/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   pso_struct.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-03-03
 
 \brief  structure pour les essaims de particules
*/

#ifndef __PSO_STRUCT__
#define __PSO_STRUCT__

#include "vizir/vzrbase_api.h"

struct psopart;
struct psoswarm;

/* protocole */
#ifdef HIDE
struct psoglobal {
    void *data;
    int (*add)(void*, vzrgene_t*, double *, int*);
    int (*candidate)(void*, vzrgene_t*);
    int (*best)(void*, int, vzrgene_t*);
    int (*get)(void*, int, vzrgene_t**, double**);
    void (*dealloc)(struct psoglobal*);
};

struct psoglobalbuild {
    void *data;
    struct psoglobal *(*create)(void *, struct psoswarm *);
    void (*dealloc)(struct psoglobalbuild *);
};

struct psolocal {
    void *data;
    int (*add)(void*, vzrgene_t*, double *, int*);
    int (*best)(void*, vzrgene_t*, vzrgene_t*);
    void (*dealloc)(struct psolocal*);
};

struct psolocalbuild {
    void *data;
    struct psolocal *(*create)(void *, struct psoswarm *);
    void (*dealloc)(struct psolocalbuild *);
};
#endif

struct psomate {
    enum popinfo genre;
    int (*local) (struct psopart *, struct vzrpop *, vzrgene_t*);
    void (*social) (struct psopart *, struct vzrpop *, vzrgene_t*, vzrgene_t*);
};

struct psomove {
    void *data;
    void (*sample) (void*, vzrgene_t*);
    void (*goodmove)(void*, vzrgene_t*, vzrgene_t*, vzrgene_t*);
    void (*badmove)(void*, vzrgene_t*, vzrgene_t*, vzrgene_t*);
    void (*dealloc)(struct psomove*);
};

struct psomovebuild {
    void *data;
    struct psomove *(*create)(void*);
    void (*dealloc)(struct psomovebuild *);
};

/* structure */
struct psopart {
    short state, last;
    int id, part, memo;
    //struct vzrpop local;
    struct psoswarm *swarm;
    struct psomove *move;
};

struct psoswarm {
    struct vzrbase base;
    /* read only */
    int nbpart;
    struct psopart *swarm;
    /* working data */
    int partc;
    /* comportement */
    //struct psoglobalbuild *memg;
    //struct psolocalbuild *meml;
    struct psomovebuild *move;
    struct psomate *mate;
};

#endif
