/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzr_heapsort.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2013-09-13
 
 \brief  heap sort kernel
*/

#ifndef __VZR_HEAPSORT__
#define __VZR_HEAPSORT__

static INLINE void hs_insert(int *idx, int mem, int low, int siz, struct heap_comp *hs)
{
    int i, j;

    i = low;
    j = ((low+1)<<1)-1; 
    while (j < siz) {
        if (j+1 < siz && hs_comp(hs, idx[j], idx[j+1])) j++;
        if (hs_comp(hs, mem, idx[j])) {
            idx[i] = idx[j];
            i = j;
            j = ((j+1)<<1)-1;
        }
        else break; /* j = up+1; */
    }
    idx[i] = mem;
}

static INLINE int hs_rank(int *idx, int size, int top, struct heap_comp *comp)
{
    int i, mem;

    /* first phase */
    for (i = (size >> 1)-1; i >= 0; i--) {
        mem = idx[i];
        hs_insert(idx, mem, i, size, comp);
    }
    /* second phase */
    for (i = size - 1; i >= top; i--) {
        mem = idx[i];
        idx[i] = idx[0];
        hs_insert(idx, mem, 0, i, comp);
    }
    return i;
}

#endif
