/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2014-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzr_hypervol.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2014-05-15
 
 \brief  calcul de l'indicateur d'hyper-volume
*/


#include "vizir/vzr_hypervol.h"

#include "vizir/vzr_macro.h"

#ifdef TRACE
#include <stdio.h>
#endif

/* local sort */
struct heap_comp {
    int one, two;
    struct vzritem *item;
};

static INLINE int hs_comp(struct heap_comp *hs, int i, int j)
{
    return (hs->item[i].front[hs->one] < hs->item[j].front[hs->one] ||
        (   
            hs->item[i].front[hs->one] == hs->item[j].front[hs->one] &&
            hs->item[i].front[hs->two] < hs->item[j].front[hs->two] 
        )
    );
}

#include "vizir/vzr_heapsort.h"

static int vzr_rank_by_obj(int size, int *idx, int objone, int objtwo, struct vzritem *item)
{
    struct heap_comp comp = {objone, objtwo , item};
    return hs_rank(idx, size, 0, &comp);
}

/* 2D indicator */
double hyper_surface(struct vzrpop *pop, struct ref2D *ref)
{
    int i;
    double indic, top;
    VZRTAB(int, id, pop->level);
    
    /* premier classement */
    for EACH(i, pop->level) id[i] = pop->rg[i];
    vzr_rank_by_obj(pop->level, id, ref->id1, ref->id2, pop->item);

    /* */
    indic = 0.0;
    top = ref->f2;
    for EACH(i, pop->level) {
        /* filtrage domaine */
        if (pop->item[id[i]].front[ref->id1] > ref->f1) continue;
        //if (pop->item[id[i]].front[ref->id2] > ref->f2) continue;
        /* filtrage pareto */
        if (pop->item[id[i]].front[ref->id2] > top ) continue;
        /* calcul indic */
        indic += (ref->f1 - pop->item[id[i]].front[ref->id1]) *
                (top - pop->item[id[i]].front[ref->id2]);
        /* next */
        top = pop->item[id[i]].front[ref->id2];

    }
    VZRTAB_OUT(id);
    return indic;
}

