/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzragent_api.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-12-02
 
 \brief  gestion des agents autonomes
*/

#include <stdio.h>
#include <stdlib.h>

#include "vizir/vzragent_api.h"

//#include "vizir/vzrshare.h"
#include "vizir/vzr_macro.h"

#include "vizir/vzrbase_api.h"
#include "vizir/vzrfifo_api.h"
#include "vizir/vzrcore_api.h"

void vzragent_reset(struct vzragent *self)
{
    struct vzrbase *base = (struct vzrbase*) self->data;
    
    self->immigr = self->emigr = 0;
    self->next = self->first;
    base->slave = -1;
}

struct vzrcore *vzragent_core(struct vzragent *self)
{
    return PRO_ASK(self, core);
}

void vzragent_run(struct vzragent *self)
{
    vzrcore_run(PRO_ASK(self, core));
}

void vzragent_start(struct vzragent *self)
{
    vzrcore_start(PRO_ASK(self, core));
}

int vzragent_emigrate(struct vzragent *self, vzrgene_t *emig)
{
    int ret;
    struct vzrbase *base = (struct vzrbase*) self->data;

    ret = (base->eval > self->next && PRO_CALL(self, emigrate, emig));
    if (ret) {
        self->next = base->eval + self->step;
        self->emigr++;
    }
    return ret;
}

int vzragent_immigrate(struct vzragent *self, vzrgene_t *mig)
{
    int ret;
    if (!(ret = vzrfifo_put(&(self->migr), mig)))
        printf ("    file saturée \n");
    self->immigr++;  
    return ret;
}

/*
int vzragent_immigration(struct vzragent *self, vzrgene_t *ind)
{
    int ret;
    if ((ret = vzrfifo_get(&(self->migr), ind)) ) self->algo->act.immigr++  ;
    return ret;
}
*/

int vzragent_booting(struct vzragent *self)
{
    struct vzrbase *base = (struct vzrbase*) self->data;
    
    return (base->eval <= self->first);
}

int vzragent_running(struct vzragent *self)
{
    return vzrbase_running((struct vzrbase *) self->data);
}

void vzragent_wait(struct vzragent *self)
{
    vzrbase_wait((struct vzrbase *) self->data);
}

int vzragent_converged(struct vzragent *self)
{
    struct vzrbase *base = (struct vzrbase *) self->data;
    
    return (base->pop.elite >= base->pop.stop);
}

void vzragent_extend(struct vzragent *self, int eval)
{
    struct vzrbase *base = (struct vzrbase *) self->data;
    
    base->maxeval = base->eval + eval ;
}

void vzragent_fitness_changed(struct vzragent *self)
{
    struct vzrbase *base = (struct vzrbase *) self->data;
    
    if (base->slave < 1) vzrpop_set_again(&(base->pop));
    else printf("reevaluate needs solver to be stopped\n");
}

int vzragent_item(struct vzragent *self, int id, vzrgene_t **zone, double **front,
        void **data, int *rang, int *age, int *info, double *isol)
{
    struct vzrbase *base = (struct vzrbase*) self->data;
    struct vzritem *it;
    
    if (id < base->pop.level)
    {
        it = &base->pop.item[base->pop.rg[id]];
        if (zone) *zone = it->zone;
        if (front) *front = it->front;
        if (data) *data = it->data;
        if (rang) *rang = it->rank;
        if (age) *age = it->cycle;
        if (info) *info = it->info;
        if (isol) *isol = it->isol;
        return 1;
    }
    else
        return 0;
    
}

void vzragent_dump(struct vzragent *self, char *nfile)
{
    vzrbase_dump((struct vzrbase *) self->data, nfile);
}

void vzragent_dealloc(struct vzragent *self)
{
    vzrfifo_dealloc(&(self->migr));
    self->dealloc(self->data);
    free(self);
}

