/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzrbase_api.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-12-02
 
 \brief  gestion des activités des solvers
*/

#ifndef __VZRBASE__
#define __VZRBASE__

#include "vizir/vzrpop_api.h"

/* structure */
typedef struct vzrbase {
    char *id; // identifiant (trace)
    int maxeval, eval;
    short slave;
    char stop, quiet;
    pthread_mutex_t lock;
    struct vzrpop pop;
    struct vzrfitnessbuild *fit;
    struct vzrfifo *migr;
} vzrbase_t;

/* private */
static INLINE int vzrbase_end(struct vzrbase *self) 
{
    return (
        (self->stop) ||
        (self->eval >= self->maxeval - self->slave +1)
    );
}

void vzrbase_init(struct vzrbase *self);
void vzrbase_build(struct vzrbase *self, int max);
int vzrbase_first(struct vzrbase *self);
void vzrbase_wait(struct vzrbase *self);
int vzrbase_running(struct vzrbase *self);
void vzrbase_stop(struct vzrbase *self);
void vzrbase_dump(struct vzrbase *self, char *nfile);
void vzrbase_dealloc(struct vzrbase *self);

#endif
