//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   hollowbar.cxx
 \author Jean-Baptiste Blanchard <jean-baptiste.blanchard@©ea.fr>
 \date   lun nov 23 11:08:44 2009
 
 \brief  "Pseudo" Code \b created to play with hollowbar with reoptimizer

 $Revision$
 $State$
 $Date$

 \par Revision
 \li JBB 2017-10-24
 Creation
 \endverbatim
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>

#include <iostream>     // for I/O
#include <fstream>      // for file I/O
#include <sstream>   
#include <cstring>

//---------------------------------------------
/** @name Methods to construct hollowbar executable.
 **/
//@{
/*!
 Returns the list of options available with hollowbar.
 -h,-?: prints help message.  
 */
void usage(const char *scmd)
{
    printf("\n");
    printf("Usage: %s line | flat | key \n", scmd);
    printf("\n");
    printf("  Compute variable for the hollow bar problem.\n");
    printf("\t It needs an input file [hollowbar_input.txt] (can be recreated on the flight) only containing two numbers, outer and inner width of the bar.\n");
    printf("\t It dumps an output file whose format depends on the flag requested: \n");
    printf("\t   line         : TLineResult format   [_hollowbar_output_line_.txt]\n");
    printf("\t   flat         : TFlatResult format   [_hollowbar_output_flat_.txt]\n");
    printf("\t   key          : TKeyResult  format   [_hollowbar_output_key_.txt]\n");

}

void barAllCost(double *param, double *res)
{
  double x = param[0];
  double y = param[1];
  /* thickness constraint */
  res[0] = x-y; 
  /* section criterion */
  //res[1] = 10 * (x*x - y*y);
  res[1] = x*x - y*y;
  /* deformation criterion */
  res[2] = 1. / (  x*x*x*x - y*y*y*y);
}

void hollow(std::string type)
{
  
    // //Read input file
    std::string inname="hollowbar_input.txt";
    std::ifstream fin(inname.c_str(), std::ios::in); std::stringstream sstr; sstr.precision(13); sstr.str(""); 
    if (!fin)
    {
	std::cout << "Can not open" << inname << std::endl;
	return ;
    }

    double input[2];
    fin >> input[0] >> input[1];
    fin.close();
    
    double output[3];
    barAllCost(input, output);

    // //Write Output File
    std::ofstream out; sstr.str("");
    if(type=="line")
    {
	//DataServer Output
	out.open("_hollowbar_output_line_.txt");
	sstr << output[0] << "\n" << output[1] << "\n" << output[2] << "\n";
	out << sstr.str(); sstr.str("");
	out.close();
    }
    if(type=="flat")
    {
	//DataServer Output
	out.open("_hollowbar_output_flat_.txt");
	sstr << output[0] << " " << output[1] << " " << output[2] << "\n";
	out << sstr.str(); sstr.str("");
	out.close();
    }
    if(type=="key")
    {
	//DataServer Output
	out.open("_hollowbar_output_key_.txt");
	sstr << "thick="<<output[0] << "\n" << "sect="<<output[1] << "\n" << "def=" << output[2] << "\n";
	out << sstr.str(); sstr.str("");
	out.close();
    }

}

int main(int argc, char *argv[])
{
    
    for (int i = 1; i < argc; i++)
    {
        
        if (!strcmp(argv[i], "line") )
	{
	    hollow("line");
	    return 0;
	}

        else if (!strcmp(argv[i], "flat") )
	{
	    hollow("flat");
	    return 0;
	}

        else if (!strcmp(argv[i], "key") )
	{
	    hollow("key");
	    return 0;
	}

        else
	{
	    usage("hollowbar");
	    return 0;
	}

        
    }

    // Default
    hollow("line");
    return 0;
}
