
#ifdef JIT
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TDataServer.h"
#include "TCJitEval.h"
#include "TThreadedRun.h"
#include "TOptimFit.h"
#include "TNloptCobyla.h"
#include "TNloptDirect.h"
#include "TNlopt.h"

#endif /* JIT */

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;

/* user defined */
/* critere
 * dur (MAXIM) ou plus soft (defaut)
 #define MAXIM
 * */

/* possible solver
 * direct
 #define SOLV TNloptCobyla
 #define SOLV TNloptBobyqa
 #define SOLV TNloptPraxis
 #define SOLV TNloptNelderMead
 #define SOLV TNloptSubplexe
 */
#define SOLV TNloptCobyla

//#define SLOW 500
#define TOLERANCE 0.001
#define NBmaxEVAL 50000
#define TAILLEPOP 400 
//#define TAILLEPOP 10 

/* constantes */
#define NIV 255.0 //niveau de gris
#define TAB 8 // nombre de régions
#define NBGENE 7 // nombre de paramètres

/* modèle */
static const char tgr[TAB * TAB] =
{
/*  0  1  2  3  4  5  6  7 */
0, 1, 1, 1, 0, 0, 1, 1, // 0
        1, 0, 1, 1, 1, 0, 0, 0, // 1
        1, 1, 0, 0, 1, 0, 0, 0, // 2
        1, 1, 0, 0, 0, 1, 1, 0, // 3
        0, 1, 1, 0, 0, 1, 0, 1, // 4
        0, 0, 0, 1, 1, 0, 1, 0, // 5
        1, 0, 0, 1, 0, 1, 0, 1, // 6
        1, 0, 0, 0, 1, 0, 1, 0  // 7
        };

static const char *tangram[TAB] =
{ &tgr[0], &tgr[TAB], &tgr[2 * TAB], &tgr[3 * TAB], &tgr[4 * TAB],
        &tgr[5 * TAB], &tgr[6 * TAB], &tgr[7 * TAB] };

void my_contraste(int size, double *sol, const char **connexion, double *out)
{
    int i, j;
    double min, diff;
    double cum;

    min = NIV;
    cum = 0.;
#ifdef SLOW
    for (int k=0; k<SLOW; k++)
#endif
    for (i = 0; i < size; i++)
    {
        for (j = i + 1; j < size; j++)
        {
            if (connexion[i][j])
            {
                diff = fabs(sol[i] - sol[j]);
                /* min contraste */
                if (diff < min)
                    min = diff;
                /* audze-eglais inspired contrast */
                cum += 1. / (1e-4 + diff * diff);
            }
        }
    }
    out[0] = min;
    out[1] = cum;
}

/* wrapper */
void my_decode(int size, double *intern, double *user)
{
    int i;

    user[0] = NIV; /* bord blanc imposé */
    for (i = 1; i < size; i++)
        user[i] = intern[i - 1];
}

void my_fitness(double *in, double *out)
{
    double all[TAB];
    my_decode(TAB, in, all);
    my_contraste(TAB, all, tangram, out);
}

/* solver */
void test_TNloptMono()
{
    /* entree */
    TAttribute z1("zone1", 0., 255.), z2("zone2", 0., 255.), z3("zone3", 0.,
            255.), z4("zone4", 0., 255.), z5("zone5", 0., 255.), z6("zone6", 0.,
            255.), z7("zone7", 0., 255.);
    /* sorties */
    TAttribute c1("maxmin"), c2("audze");
    /* starting points */
    double start[][7] =
    {
    { 120, 120, 120, 120, 120, 120, 120 },
    { 110, 120, 130, 105, 125, 135, 115 },
    { 30, 60, 90, 120, 150, 180, 210 },
    { 150, 60, 30, 210, 120, 180, 90 },
    { 150, 60, 30, 210, 120, 180, 90 } };
    /* evaluation */
    TCJitEval code(&my_fitness);
    code.setInputs(NBGENE, &z1, &z2, &z3, &z4, &z5, &z6, &z7);
    code.setOutputs(2, &c1, &c2);
    /* run */
#ifdef JIT
    TThreadedRun run(&code, 3);
#else
    TSequentialRun run(&code);
#endif
    run.startSlave();
    if (run.onMaster())
    {
        /* tds */
        TDataServer tds("tdsnlopt", "tds for result");
        code.addAllInputs(&tds);
        TDataServer trc("tdstrc", "tds for trace");
        
        /* optimizer */
        SOLV solv;
        TNlopt vzr(&tds, &run, &solv);
#ifdef MAXIM
        TMaximizeFit maximize;
        vzr.addObjective(&c1, &maximize);
#else
        vzr.addObjective(&c2);
#endif
        vzr.setTolerance(TOLERANCE);
        vzr.setTrace(&trc);

        /* resolution */
        int i;
        for (i = 0; i < 5; i++)
        {
            vzr.setStartingPoint(7,start[i]);
        }
        vzr.solverLoop();

        /* solution */
        //tds.exportData("_int_.dat");
        tds.getTuple()->Scan("*");
        trc.getTuple()->Scan("*");
        run.stopSlave();
    }
}
