#ifdef JIT
#include "TAttribute.h"
#include "TFlatScript.h"
#include "TKeyScript.h"
#include "TFlatResult.h"
#include "TCodeEval.h"
#include "TComposeEval.h"
#include "TDataServer.h"
#include "TNloptDirect.h"
#include "TNloptCobyla.h"
#include "TNlopt.h"
#endif

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;

// user defined 
#define TOLERANCE 0.001
#define NBmaxEVAL 10000

/* possible solver
 * direct
 #define SOLV TNloptCobyla
 #define SOLV TNloptBobyqa
 #define SOLV TNloptPraxis
 #define SOLV TNloptNelderMead
 #define SOLV TNloptSubplexe
 * gradient
 #define SOLV TNloptMMA
 #define SOLV TNloptSLSQP
 #define SOLV TNloptLBFGS
 #define SOLV TNloptNewton
 #define SOLV TNloptVariableMetric
 * */
#define SOLV TNloptMMA

void test_TNloptMpiFlowrate()
{
    /*** variables ***/
    // Inputs
    TAttribute rw("rw", 0.05, 0.15), r("r", 100.0, 50000.0), tu("tu", 63070.0,
            115600.0), tl("tl", 63.1, 116.0), hu("hu", 990.0, 1110.0), hl("hl",
            700.0, 820.0), l("l", 1120.0, 1680.0), kw("kw", 9855.0, 12045.0);
    // Outputs
    TAttribute yh("yhat"), d("d");
    // Starting point
    double start[][8] =
    {
    { 0.06, 500, 65000, 65, 1000, 720, 1150, 10000 },
    { 0.1, 25000, 90000, 90, 1050, 760, 1400, 11000 },
    { 0.14, 45000, 115000, 110, 1100, 800, 1650, 12000 } };
    /*** Composed code ***/
    // working dir
    TString dir(
            gSystem->Getenv("HOME") + TString("/tmp/LanceurUranie/flowrate"));
    // premier code
    TKeyScript fin1("flowrate1.py");
    fin1.setInputs(8, &rw, "Rw", &r, "R", &tu, "Tu", &tl, "Tl", &hu, "Hu", &hl,
            "Hl", &l, "L", &kw, "Kw");
    TFlatResult fout1("flowrate.res");
    fout1.setOutputs(1, &yh);
#ifdef PYTHON3
    TCodeEval code1("python2 flowrate1.py > flowrate.res");
#else
    TCodeEval code1("python flowrate1.py > flowrate.res");
#endif
    code1.setWorkingDir(dir);
    code1.addInputFile(&fin1);
    code1.addOutputFile(&fout1);

    // second code
    TKeyScript fin2("flowrate2.py");
    fin2.setInputs(2, &hu, "Hu", &hl, "Hl");
    TFlatResult fout2("flowrate2.res");
    fout2.setOutputs(1, &d);
#ifdef PYTHON3
    TCodeEval code2("python2 flowrate2.py > flowrate2.res");
#else
    TCodeEval code2("python flowrate2.py > flowrate2.res");
#endif
    code2.setWorkingDir(dir);
    code2.addInputFile(&fin2);
    code2.addOutputFile(&fout2);

    // compose code
    TComposeEval mycode;
    mycode.setEvals(2, &code1, &code2);

    TMpiRun tmprun(&mycode);
    TGradientEstimationRun run(&tmprun);
    run.startSlave();
    if (run.onMaster())
    {
        int i;
        // DataServer
        TDataServer tds("tdsFlowrate",
                "OATMinMax Design of experiments for Flowrate");
        tds.addAttribute(&rw);
        tds.addAttribute(&r);
        tds.addAttribute(&tu);
        tds.addAttribute(&tl);
        tds.addAttribute(&hu);
        tds.addAttribute(&hl);
        tds.addAttribute(&l);
        tds.addAttribute(&kw);

        /*** solver ***/
        // optimizer
        SOLV solv;
        TNlopt vzr(&tds, &run, &solv);
        vzr.addObjective(&yh);
        vzr.setTolerance(TOLERANCE);
        vzr.setMaximumEval(NBmaxEVAL);

        /*** resolution ***/
        for (i = 0; i < 3; i++)
            vzr.setStartingPoint(8,start[i]);
        vzr.solverLoop();

        /*** solution ***/
        tds.getTuple()->Scan("*");
        //tds.exportData("_flow_.dat");
        run.stopSlave();
    }
}

