// --- C++ ---
// --- coding: utf-8 ---
//
//    File
//      creation : 2011-08-24.09.50.39
//      revision : $Id$
//
//    Copyright © 2011 Commissariat à l'Energie Atomique
//      par Gilles ARNAUD (DM2S/SFME/LETR)
//        C.E. Saclay; Bat 454; 91191 GIF/YVETTE CEDEX; France
//        Tel: 01 69 08 38 86; Fax : 33 1 69 08 85 68 
//        Gilles.Arnaud@cea.fr
// 
//    Object
//      two level MPI example
// 
//___________________________________________________________________

#ifdef JIT
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TAttribute.h"
#include "TMpiLn2Eval.h"
#include "TBiMpiRun.h"
#include "TDataServer.h"
#include "TOptimFit.h"
#include "TVizirGenetic.h"
#include "TVizir2.h"

#endif

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::MpiRelauncher;
using namespace URANIE::Reoptimizer;

/* user defined */
#define TOLERANCE 0.0001
#define POP 600
#define EVALmax 50000

/* solver */
void test_TVizirMpiBi()
{
    /* init */
    TAttribute pl("plus", 0.3, 3.), mn("moins", 0.3, 3.), ln("ln2 dist");

    TMpiLn2Eval code;
    code.addInput(&pl);
    code.addInput(&mn);
    code.addOutput(&ln);

    TBiMpiRun run(&code);
    run.calculSize(2);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tdsvzr("tdsvzr", "tds for vizir test");
        tdsvzr.addAttribute(&pl);
        tdsvzr.addAttribute(&mn);

        TVizirGenetic solv;
        solv.setSize(POP, EVALmax);
        TVizir2 vzr(&tdsvzr, &run, &solv);
        vzr.addObjective(&ln);
        vzr.setTolerance(TOLERANCE);

        /* resolution */
        vzr.solverLoop();

        /* sauvegarde */
        tdsvzr.exportData("_bimpi_.dat");
        run.stopSlave();
    }
}

