//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TFrankCopula.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mer oct 17 18:08:40 CEST 2012
 
 \brief Interface of the class URANIE::Sampler::TFrankCopula
 */
/**
 @class URANIE::Sampler::TFrankCopula
 
 @brief Description of the class TFrankCopula.
 
 The \b Frank \b copula for bivariate is defined by the formula
 \f[
 C(u_1, u_2 ; \theta) \: = \: - \theta^{-1}\log{ \left( 1 + \frac{\left( e^{-\theta u_1} - 1 \right) \left( e^{-\theta u_2} - 1 \right) }{e^{-\theta} - 1} \right)}
 \f]
 with \f$\theta \in [ - \infty, \infty]\f$.
 
 */
#ifndef TFRANKCOPULA_H
#define TFRANKCOPULA_H

////// URANIE
#include "TArchimedianCopula.h"
#include "UExceptions.h"

namespace URANIE
{
namespace Sampler
{
class TFrankCopula: public TArchimedianCopula
{
private:
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TFrankCopula(URANIE::DataServer::TDataServer *tds, Double_t dparam,
            Int_t nCalcul);
    //! Default destructor
    virtual ~TFrankCopula();
    //@}

    //---------------------------------------------
    /** @name Generation of the sample
     **/
    //@{
    //! Generates the matrix of U
    /*!
     \param option (Option_t *) The option to pass.
     */
    void generateUMatrix(Option_t *option);
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sampler::TFrankCopula, ID_SAMPLER)
    // TFrankCopula Class
};

} // Fin du namespace Sampler
} // Fin du namespace URANIE
#endif
