//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef TMCMC_H
#define TMCMC_H

#ifndef ROOT_TNtupleD
#include "TNtupleD.h"
#endif

#ifndef ROOT_TNtuple
#include "TNtuple.h"
#endif

#ifndef ROOT_TFile
#include "TFile.h"
#endif

#ifndef ROOT_TVectorD
#include "TVectorD.h"
#endif

#ifndef ROOT_TMatrixD
#include "TMatrixD.h"
#endif

#ifndef ROOT_TRandom3
#include "TRandom3.h"
#endif

#ifndef ROOT_TMath
#include "TMath.h"
#endif

#ifndef ROOT_TCanvas
#include "TCanvas.h"
#endif

#ifndef ROOT_TString
#include "TString.h"
#endif

#ifndef ROOT_TEllipse
#include "TEllipse.h"
#endif

#ifndef ROOT_TPaveText
#include "TPaveText.h"
#endif

#ifndef ROOT_TMelange
#include "TMelange.h"
#endif

#include "TDistribution.h"
#include <iostream>
#include "Rtypes.h"
#include "TNormale.h"
#include "TSampler.h"

class TList;
class TMCMC
{
    // Associations
    // Attributes
public:
    /*! \brief Sample size*/
    Int_t _size;
    /*! \brief Variable representing a normal law which is used in all intermediary calculations*/
    TNormale* q;
    Double_t _score;
    /*! \brief Generator of random number*/
    TRandom3* _rdm;
    /** Association d'un canvas pour la visu */
    TCanvas* cvisu;
    /** Tuple de base ayant servi � mod�liser */
    TNtupleD *_dataorg;
    /*! \brief Generator of random number*/

    TString _sdraw;
    // TNtupleD* _vect;
    // TFile* _rootFile;
    //TNtuple* _tuple;
    // Operations

public:
    TMCMC();
    virtual ~TMCMC();
    void setSize(Int_t n)
    {
        _size = n;
    }
    Int_t getSize()
    {
        return _size;
    }
    TNtupleD* varTimeMetropolisSampling(TNormale* f, TVectorD X);
    TNtupleD* randomWalkMetropolisSampling(TNormale* f, TVectorD X);
    TNtupleD* gibbsSampling(TNormale* f, TVectorD X);
    TVectorD getObs(TNormale* f, TVectorD X);
    TNtupleD* gibbsSampling(TMelange* mel, TVectorD X);
    TNtupleD* randomWalkMetropolisSampling(TMelange* mel, TVectorD X);
    TNtupleD* varTimeMetropolisSampling(TMelange* mel, TVectorD X);
    void Draw1(TMelange* mel, TNtupleD* ntd, Option_t *option);
    void Draw(TMelange* mel, TNtupleD* ntd, Option_t *option);
    Double_t getScore()
    {
        return _score;
    }
    Int_t selectComponentState();
    TNtupleD* NKCSampling(TMelange* mel, TVectorD* X);
    TNtupleD* NKCSampling2(TMelange* mel, TVectorD* X);
    /** Association d'un canvas */
    void SetCanvas(TCanvas * canvas);
    /** No descriptions */
    void SetTuple(TNtupleD *tuple);

    void setVarDraw(TString str)
    {
        _sdraw = str;
    }

    ClassDef(TMCMC, ID_SAMPLER)
    //Classe
};

#endif
