//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
// TSobolSequence Wed Jul 26 20:20:06 CEST 2006
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TSobolSequence.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   Wed Jul 26 20:20:06 CEST 2006
 
 \brief Interface of the class URANIE::Sampler::TSobolSequence
 
 */
/*!
 
 @class URANIE::Sampler::TSobolSequence

 @brief Description of the class TSobolSequence.

 The dimension must be lower than 40.

 \section  sobol_description Description
 (source Numerical Recipes - Chapitre 7-7 page 310-315)
 \par

 \section halton_implementation Implementation
 the code is given in Gnu Scintific Library (GSL) and in the numerical recipes.

 \par See
 \li [Bratley+Fox, TOMS 14, 88 (1988)]
 \li [Antonov+Saleev, USSR Comput. Maths. Math. Phys. 19, 252 (1980)]

 */
#ifndef TSOBOLSEQUENCE_H
#define TSOBOLSEQUENCE_H
#include "TSequence.h"

#ifndef ROOT_TMatrixD
#include "TMatrixD.h"
#endif
#ifndef ROOT_TVectorD
#include "TVectorD.h"
#endif

#include <iostream>
using std::cout;
using std::endl;

/* maximum allowed space dimension */
#define SOBOL_MAX_DIMENSION 40

/* bit count; assumes sizeof(int) >= 32-bit */
#define SOBOL_BIT_COUNT 30

namespace URANIE
{
namespace Sampler
{
class TSobolSequence: public TSequence
{

    // Attributes
private:
    TMatrixD v_direction;                 //! Matrix of directions
    TVectorD _last_numerator_vec;         //! Vector 
    Double_t _last_denominator_inv;
    Int_t _nCounter;

    // Operations
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TSobolSequence(ULong_t ndim);
    //! Default destructor
    virtual ~TSobolSequence();
    //@}

    void setInstance(const ULong_t n);
    void init();

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    //! Prints the log
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sampler::TSobolSequence, ID_SAMPLER)
    //Classe de
};

} // Fin du namespace Sampler
} // Fin du namespace URANIE
#endif
// fin du fichier $RCSfile$.
