//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestMain.cxx
 \author Cedric Forgues <uranietm@cea.fr>
 \date   fri may 29 15:56:01 2009
 
 \brief Principal program for Unary Tests of Sampler library.
 
 */

#include "cppunit/CompilerOutputter.h"
#include "cppunit/TestResult.h"
#include "cppunit/TestResultCollector.h"
#include "cppunit/TextTestProgressListener.h"
#include "cppunit/BriefTestProgressListener.h"
#include "cppunit/extensions/TestFactoryRegistry.h"
#include "cppunit/TestRunner.h"
#include "cppunit/XmlOutputter.h"

#include <stdexcept>

#include <iostream>
#include <fstream>

//uranie
#include "UVersion.h"
// Liste des Tests Unitaires d'uranie
#include "TestTSampling.h"
#include "TestTMaxiMinLHS.h"
#include "TestTConstrLHS.h"
#include "TestTBasicSampling.h"
#include "TestTQMC.h"
#include "TestTSamplerField.h"
#include "TestTPetras.h"
#include "TestTSpaceFilling.h"
#include "TestTNeuralGas.h"
#include "TestTGaussianSampling.h"
#include "TestTImportanceSampling.h"
#include "TestTOA.h"
#include "TestTCopula.h"
#include "TestTOATSampling.h"
//#include "TestTOATSamplingMultiNominal.h"
#include "TestTOATDesign.h"
#include "TestTSamplingMixture.h"

CPPUNIT_TEST_SUITE_REGISTRATION (TSamplingTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TMaxiMinLHSTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TConstrLHSTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TBasicSamplingTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TQMCTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TSamplerFieldTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TPetrasTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TSpaceFillingTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TOATest);
CPPUNIT_TEST_SUITE_REGISTRATION (TCopulaTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TGaussianSamplingTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TImportanceSamplingTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TNeuralGasTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TOATSamplingTest);
//CPPUNIT_TEST_SUITE_REGISTRATION( TOATSamplingMultiNominalTest );
CPPUNIT_TEST_SUITE_REGISTRATION (TOATDesignTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TSamplingMixtureTest);

bool uranie_test_suite(const char *module, const char *xml, const char *xsl);

int main()
{

    bool wasSuccessful;
#ifdef WIN32
    wasSuccessful = uranie_test_suite("Sampler",
                                      "testSamplerResults.xml", "..\\..\\..\\..\\testReport.xsl");
#else
    wasSuccessful = uranie_test_suite("Sampler",
                                      "testSamplerResults.xml", "../../../../testReport.xsl");
#endif


    // ---  Return error code 1 if the one of test failed.

    gDirectory->Delete("htemp;*");
    gDirectory->Delete("tdsMem__data__tree__;*");
    // List the gDirectory (must be empty)
    gDirectory->ls();
    
    return wasSuccessful ? 0 : 1;
}
