//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TCONSTRLHS_H
#define CPPUNITEST_TCONSTRLHS_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TConstrLHS.h"

class TConstrLHSTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TConstrLHSTest);

    CPPUNIT_TEST (test1constr4var);
    CPPUNIT_TEST (test1constr4varInvert);
    CPPUNIT_TEST (test1constr4varNoSeed);
    CPPUNIT_TEST (test1constr4varFilledTDS);
    CPPUNIT_TEST (test2constr4var);
    CPPUNIT_TEST (test2constr4varInvert);    
    CPPUNIT_TEST (test2constr4varNoSeed);
    CPPUNIT_TEST (test2constr4varFilledTDS);

    CPPUNIT_TEST (throwExceptionConstrutorTdsNull);
    CPPUNIT_TEST (throwExceptionWrongNumberOfSample);
    CPPUNIT_TEST (throwExceptionWrongNumberOfParameter);
    CPPUNIT_TEST (throwExceptionWrongParameterValue);
    CPPUNIT_TEST (throwExceptionSameVariableChangedInMultiConstraints);
    CPPUNIT_TEST (throwExceptionNoConstrainProvided);
    CPPUNIT_TEST (throwExceptionProvidedSampleImpossibleConstraint);
    CPPUNIT_TEST (throwExceptionImpossibleConstraint);
    CPPUNIT_TEST (throwExceptionFilledTDSWrongNS);
    
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TConstrLHSTest();

    //! Default destructor
    virtual ~TConstrLHSTest();

    void setUp();
    void tearDown();
    //@}

    // //---------------------------------------------
    // /** @name Constrain functions
    //  **/
    // //@{
    // //! Only one constraint
    // static void OneConstraint(double *param, double *res);
    // //! Only two constraints
    // static void TwoConstraints(double *param, double *res);
    // //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //!
    void test1constr4var();
    //!
    void test1constr4varInvert();
    //!
    void test1constr4varNoSeed();
    //!
    void test1constr4varFilledTDS();
    //!
    void test2constr4var();
    //!
    void test2constr4varInvert();    
    //!
    void test2constr4varNoSeed();
    //!
    void test2constr4varFilledTDS();
    //@}

    //---------------------------------------------
    /** @name throwException methods
     **/
    //@{
    //!
    void throwExceptionConstrutorTdsNull();
    //!
    void throwExceptionWrongNumberOfSample();
    //!
    void throwExceptionWrongNumberOfParameter();
    //!
    void throwExceptionWrongParameterValue();
    //!
    void throwExceptionSameVariableChangedInMultiConstraints();
    //!
    void throwExceptionNoConstrainProvided();
    //!
    void throwExceptionProvidedSampleImpossibleConstraint();
    //!
    void throwExceptionImpossibleConstraint();
    //!
    void throwExceptionFilledTDSWrongNS();
    //@}

private:
    TString _workingTestPath;

    URANIE::DataServer::TDataServer *_tds;
    URANIE::Sampler::TConstrLHS *_sampler;
    Int_t _nS, _nseed;
};

#endif  // CPPUNITEST_TSAMPLING_H

