//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestTOA.cxx
 \author Maud Salvoldi (OCC/Areva group)
 \date   $Date$
 
 \brief  Test File of URANIE::Sampler::TOA class
 
 Cr�e le : mer nov 02 17:00:27 2011
 
 */

// Uranie
#include "TestTOA.h"
#include "TAttribute.h"
#include "TUniformDistribution.h"
#include "TNormalDistribution.h"
#include "TWeibullDistribution.h"
#include "TUniformByPartsDistribution.h"
#include "TBetaDistribution.h"
#include "TCauchyDistribution.h"
#include "UDisableTest.h"
// ROOT
#include "TString.h"

#include <iostream>
#include <math.h>

#define DBL_PRECISION_COMPARE 1.e-10
#define EPSILON 0.03

TOATest::TOATest() :
        _tds(NULL), _fsam(NULL), _nSize(3721)
{
}

TOATest::~TOATest()
{
}
void TOATest::setUp()
{
    // Create a TDataServer
    _tds = new URANIE::DataServer::TDataServer();
    // Fill the DataServer with the three attributes of the study
    _tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("y1", 1., 5.));
    _tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("y2", 2., 3.));
    _tds->addAttribute(
            new URANIE::DataServer::TNormalDistribution("y3", 2., 3.));
    _tds->getAttribute("y1")->delShare();
    _tds->getAttribute("y2")->delShare();
    _tds->getAttribute("y3")->delShare();
}

void TOATest::tearDown()
{
    delete _fsam;
    _fsam = NULL;
    delete _tds;
    _tds = NULL;
}
void TOATest::testTOAgeneratesample()
{
    _fsam = new URANIE::Sampler::TOA(_tds, "", _nSize);
//   _fsam->setLog();
    _tds->computeStatistic("*");

    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.999, _tds->getAttribute("y1")->getMean(),
            EPSILON);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.154, _tds->getAttribute("y1")->getStd(),
            EPSILON);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.499, _tds->getAttribute("y2")->getMean(),
            EPSILON);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.288, _tds->getAttribute("y2")->getStd(),
            EPSILON);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.002, _tds->getAttribute("y3")->getMean(),
            EPSILON);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.999, _tds->getAttribute("y3")->getStd(),
            EPSILON);

}
void TOATest::throwExceptionTOAConstructorTdsNull()
{
  URANIE::DataServer::TDataServer * tdsnull = NULL;
    CPPUNIT_ASSERT_THROW(new URANIE::Sampler::TOA(tdsnull, "", _nSize),
            URANIE::Exceptions::UErrorExceptions);
}
void TOATest::throwExceptionTOAConstructorSmallSize()
{
    CPPUNIT_ASSERT_THROW(new URANIE::Sampler::TOA(_tds, "", 2),
            URANIE::Exceptions::UErrorExceptions);
}
