//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TSAMPLERFIELD_H
#define CPPUNITEST_TSAMPLERFIELD_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSampler.h"
#include "TSamplerField.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TSamplerFieldTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TSamplerFieldTest);
    CPPUNIT_TEST (testSamplerFieldConstructor);
    CPPUNIT_TEST (testSamplerFieldcreateListOfAttributes);
    CPPUNIT_TEST (testSamplerFieldsetMethod);
    CPPUNIT_TEST (testSamplerFieldsetVariogram);
    CPPUNIT_TEST (testSamplerFieldSetRandomFunction);
    CPPUNIT_TEST (testSamplerFieldSetVariance);
    CPPUNIT_TEST (testSamplerFieldSetIsotropy);
    CPPUNIT_TEST (testSamplerFieldSetScaleFactor);
    CPPUNIT_TEST (testSamplerFieldgenerateSampleGauss2D);
    CPPUNIT_TEST (testSamplerFieldgenerateSampleSinus3D);
    CPPUNIT_TEST (testSamplerFieldgenerateSampleGauss4D);
    CPPUNIT_TEST (testSamplerFieldgenerateManySampleGauss1D);
    CPPUNIT_TEST (testSamplerFieldgenerateManySampleGauss2D);
    
    CPPUNIT_TEST (throwExceptionTSamplerFieldConstrutorTdsNull);
    CPPUNIT_TEST (throwExceptionTSamplerFieldConstrutorTdsNullFalseNumberArgument);
    /* CPPUNIT_TEST (throwExceptionTSamplerFieldConstrutorToManyAttribut); */
    /* CPPUNIT_TEST (throwExceptionTSamplerFieldConstrutorOnAttribut); */
    CPPUNIT_TEST (throwExceptionTSamplerFieldsetMethod);
    CPPUNIT_TEST (throwExceptionTSamplerFieldsetVariogram);
//  CPPUNIT_TEST( throwExceptionTSamplerFieldSetRandomFunction );
    /* CPPUNIT_TEST (throwExceptionTSamplerFieldSetIsotropy); */
    CPPUNIT_TEST (throwExceptionTSamplerFieldSetVariance);
    CPPUNIT_TEST (throwExceptionTSamplerFieldSetScaleFactor);
    CPPUNIT_TEST (throwExceptionSamplerFieldgenerateSampleNoVarisogram);
    CPPUNIT_TEST (throwExceptionSamplerFieldgenerateSampleNoMethod);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TSamplerFieldTest();

    //! Default destructor
    virtual ~TSamplerFieldTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! 
    void testSamplerFieldConstructor();
    //! 
    void testSamplerFieldcreateListOfAttributes();
    //! 
    void testSamplerFieldsetMethod();
    //! 
    void testSamplerFieldsetVariogram();
    //! 
    void testSamplerFieldSetRandomFunction();
    //! 
    void testSamplerFieldSetIsotropy();
    //! 
    void testSamplerFieldSetVariance();
    //! 
    void testSamplerFieldSetScaleFactor();
    //! 
    void testSamplerFieldgenerateSampleGauss2D();
    //! 
    void testSamplerFieldgenerateSampleSinus3D();
    //! 
    void testSamplerFieldgenerateSampleGauss4D();
    //! 
    void testSamplerFieldgenerateManySampleGauss1D();
    //! 
    void testSamplerFieldgenerateManySampleGauss2D();
    //@}

    //---------------------------------------------
    /** @name throwException methods
     **/
    //@{
    //! 
    void throwExceptionTSamplerFieldConstrutorTdsNull();
    //!
    void throwExceptionTSamplerFieldConstrutorTdsNullFalseNumberArgument();
    void throwExceptionTSamplerFieldsetMethod();
    //! 
    void throwExceptionTSamplerFieldsetVariogram();
    //! 
    void throwExceptionTSamplerFieldSetRandomFunction();
    //! 
    void throwExceptionTSamplerFieldSetVariance();
    //! 
    void throwExceptionTSamplerFieldSetScaleFactor();
    //!
    void throwExceptionSamplerFieldgenerateSampleNoVarisogram();
    //!
    void throwExceptionSamplerFieldgenerateSampleNoMethod();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds;
    URANIE::Sampler::TSamplerField *_splf;

};

#endif  // CPPUNITEST_TSAMPLERFIELD_H
