//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TSAMPLINGMIXTURE_H
#define CPPUNITEST_TSAMPLINGMIXTURE_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSampler.h"
#include "TSamplingMixture.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TSamplingMixtureTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TSamplingMixtureTest);

    CPPUNIT_TEST (testSamplingMixtureWithoutCandidates);
    CPPUNIT_TEST (testSamplingMixtureWithCandidates);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TSamplingMixtureTest();

    //! Default destructor
    virtual ~TSamplingMixtureTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Auxiliary method
     **/
    //@{
    //! Add Constraints
    void addConstraints();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test of the mixtures sampling without all candidates
    void testSamplingMixtureWithoutCandidates();
    //! Test of the mixtures sampling with all candidates
    void testSamplingMixtureWithCandidates();
    //@}

    //@}

private:
    URANIE::DataServer::TDataServer *_tds1;
    URANIE::Sampler::TSamplingMixture * _tmix;
    Int_t _nS;
};

#endif  // CPPUNITEST_TSAMPLINGMIXTURE_H

