//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
// TScreeming mer sep 13 12:36:50 CEST 2006
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TScreeming.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   wen sep 13 12:36:50 CEST 2006
 
 \brief Interface of class URANIE::Sensitivity::TScreeming
 
 */
/**
 
 * @class URANIE::Sensitivity::TScreeming
 *
 * @brief Description of the class TScreeming.
 *
 * To be written by the developper.
 *
 */

// ROOT
#ifdef ROOT5
#include "Api.h"
#endif
#include "TLegend.h"
#include "TPie.h"

#ifndef TSCREEMING_H
#define TSCREEMING_H
#include "TSensitivity.h"

#include <iostream>
using std::cout;
using std::endl;

namespace URANIE
{
namespace Sensitivity
{
class TScreeming: public TSensitivity
{
private:
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
	//! Default constructor with the name of a function
    TScreeming(URANIE::DataServer::TDataServer *tds, const char *fcn, Int_t ns = -1,
            const char *varexpinput = "", const char *varexpoutput = "") :
            TSensitivity(tds, fcn, ns, varexpinput, varexpoutput)
    {
    }
    ;
#ifdef ROOT5
    //! Default constructor with a function
    TScreeming(URANIE::DataServer::TDataServer *tds, void *fcn, Int_t ns = -1,
            const char *varexpinput = "", const char *varexpoutput = "") :
            TSensitivity(tds, fcn, ns, varexpinput, varexpoutput)
    {
    }
    ;
#else
    TScreeming(URANIE::DataServer::TDataServer *tds, 
            void (*fcn)(Double_t*, Double_t*),
            const char *varexpinput, const char *varexpoutput, Int_t ns = -1) :
        TSensitivity(tds, fcn, varexpinput, varexpoutput, ns)
    { };
#endif
    //! Default constructor with a TDataServer filling
    TScreeming(URANIE::DataServer::TDataServer *tds, const char *varexpinput,
            const char *varexpoutput, Option_t * option = "") :
            TSensitivity(tds, varexpinput, varexpoutput, option)
    {
    }
    ;
    //! Default constructor with TCode arg
    TScreeming(URANIE::DataServer::TDataServer *tds,
            URANIE::Launcher::TCode *code, Int_t ns = 100) :
            TSensitivity(tds, code, ns)
    {
    }
    ;
    //! Default constructor with TRun arg
    TScreeming(URANIE::DataServer::TDataServer *tds,
            URANIE::Relauncher::TRun *code, Int_t ns = 100) :
        TSensitivity(tds, code, ns)
    { };
    //! Default destructor
    virtual ~TScreeming();
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sensitivity::TScreeming, ID_SENSITIVITY)
    //Classe de
};

} // Fin du namespace Sensitivity
} // Fin du namespace URANIE
#endif
