////////////////////////////////////////////////////////////// 
// Copyright (C) 2013-2024 CEA/DES 
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
\file   TSobolRank.h
\author Rudy Chocat <rudy.chocat@cea.fr>
\date   2024-03-01 15:59:38

\brief Interface of class URANIE::Sensitivity::TSobolRank

*/

#ifndef TSOBOLRANK_H
#define TSOBOLRANK_H

/// ROOT
#include "TVector.h"
#include "TMatrixDSymfwd.h"

/// Uranie
#include "TSensitivity.h"

namespace URANIE {
namespace Sensitivity {
class TSobolRank: public TSensitivity {
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    /*!
     * \brief Default constructor
     */
    TSobolRank();

    /*!
     * \brief TSobolRank constructor with data contined in the TDS and the inputs and outputs attributes
     *
     * \param tds
     * \param sInputs
     * \param sOutputs
     * \param option (Option_t *) option to pass [""]
     */
    TSobolRank(URANIE::DataServer::TDataServer *tds, const char *sInputs, const char *sOutputs, Option_t * option = "");

    /*!
     * \brief Default destructor
     */
    virtual ~TSobolRank();
    //@}

    //---------------------------------------------
    /** @name Parse the option
     **/
    //@{
    //! Read option specific to TJohnsonRW
    /*!
      Possible options specific to this methods are
     \li "regressors": the computation is done on regressors instead of covariance matrix
    */
    // void parseOption(Option_t *option = "");
    //@}

    // //---------------------------------------------
    // /** @name Generation of the sample
    //     In this case, there is noting to do
    // **/
    // //@{
void generateSample(Option_t * option = "")
    {
    TString soption = option;
    soption.ToLower();
    }
    // //@}

    // //---------------------------------------------
    /** @name Computes the indexes
     **/
    //@{
    void evaluateIndexes(Option_t * option = "");
    // //!PreTreatment for every output
    // /*!
    //   Doing a pre-treatment when running over outputs.
    // */
    // void preTreatment();
     //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    // virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sensitivity::TSobolRank, ID_SENSITIVITY)
    //Classe de
};

}
}
#endif
