//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestMain.cxx
 \author Cedric Forgues <uranietm@cea.fr>
 \date   lun mai 25 9:00:01 2009
 
 
 \brief Programme principal pour les tests unitaires sur la librairie Sensitivity.
 
 */
#include "cppunit/CompilerOutputter.h"
#include "cppunit/TestResult.h"
#include "cppunit/TestResultCollector.h"
#include "cppunit/TextTestProgressListener.h"
#include "cppunit/BriefTestProgressListener.h"
#include "cppunit/extensions/TestFactoryRegistry.h"
#include "cppunit/TestRunner.h"
#include "cppunit/XmlOutputter.h"

#include <stdexcept>

#include <iostream>
#include <fstream>

//uranie
//#include "UVersion.h"

// Liste des Tests Unitaires d'uranie
#include "TestTRegression.h"
#include "TestTSobol.h"
#include "TestTShapley.h"
#include "TestTMorris.h"
#include "TestTFiniteDifferences.h"
#include "TestTCMN.h"
#include "TestTDGSM.h"
#include "TestTHSIC.h"
#include "TestTJohnsonRW.h"
#include "TestTSobolRank.h"
#ifdef WITH_FFTW
#include "TestTFast.h"
#include "TestTRBD.h"
#endif
#include "uranie_commands.h"

CPPUNIT_TEST_SUITE_REGISTRATION (TRegressionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TSobolTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TMorrisTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TFiniteDifferencesTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TDGSMTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TCMNTest);
CPPUNIT_TEST_SUITE_REGISTRATION (THSICTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TJohnsonRWTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TSobolRankTest);
#ifdef WITH_FFTW
CPPUNIT_TEST_SUITE_REGISTRATION( TFastTest );
CPPUNIT_TEST_SUITE_REGISTRATION( TRBDTest );
#endif
CPPUNIT_TEST_SUITE_REGISTRATION (TShapleyTest);


bool uranie_test_suite(const char *module, const char *xml, const char *xsl);

int main()
{
    
    bool wasSuccessful;
#ifdef WIN32
    wasSuccessful = uranie_test_suite("Sensitivity",
                                      "testSensitivityResults.xml", "..\\..\\..\\..\\testReport.xsl");
#else
    wasSuccessful = uranie_test_suite("Sensitivity",
                                      "testSensitivityResults.xml", "../../../../testReport.xsl");
#endif

    ////////////////
    // Clean
    ////////////////
    UranieTests::removeFolder(gSystem->pwd(),"tmpLanceurUranie");
    UranieTests::removeFolder(gSystem->pwd(),"URANIE");
    UranieTests::removeFile(gSystem->pwd(), "_dataserver_.root");
    UranieTests::removeFile(gSystem->pwd(), "_oat_.dat");
    UranieTests::removeFile(gSystem->pwd(), "_oat_.root");

    // Delete temporary object in gDirectory
    gDirectory->Delete("tmpsobol;*");
    gDirectory->Delete("htemp;*");
    gDirectory->Delete("tmpOATSampling__data__tree__;*");
    gDirectory->Delete("Sampling_Sampling_;*");
    gDirectory->Delete("Sampling_Sampling__copied;*");
    // List the gDirectory (must be empty)
    gDirectory->ls();

    // ---  Return error code 1 if the one of test failed.
    return wasSuccessful ? 0 : 1;
}
