//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TRBD_H
#define CPPUNITEST_TRBD_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSensitivity.h"
#include "TOutputFileRow.h"
#include "TFast.h"
#include "TRBD.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TRBDTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TRBDTest);

    CPPUNIT_TEST (testTRBDwithFunction);
    CPPUNIT_TEST (testTRBDwithCode);
#ifdef WITH_LEVELE
    CPPUNIT_TEST (testLevelERBD);
    CPPUNIT_TEST (testLevelERBDOutputRestrOption);
#endif
    CPPUNIT_TEST (testTRBDsetFrequencies);
    CPPUNIT_TEST (throwExceptionTRBDsetFrequencies);

    CPPUNIT_TEST (testIshigamiRBDComputeIndexesWithTLauncherFunction);
    CPPUNIT_TEST (testIshigamiRBDComputeIndexesGenerateSampleWithTLauncherFunction);
    CPPUNIT_TEST (testIshigamiRBDComputeIndexesGenerateSampleCreateLauncherWithTLauncherFunction);

    CPPUNIT_TEST (testFlowrateRBDComputeIndexesWithTLauncherCode);
    CPPUNIT_TEST (testFlowrateRBDComputeIndexesGenerateSampleWithTLauncherCode);

    CPPUNIT_TEST (testFlowrateRBDComputeIndexesGenerateSampleCreateLauncherWithTLauncherCode);
    CPPUNIT_TEST (testFlowrateRBDComputeIndexesWithTCodeEval);

    CPPUNIT_TEST (testIshigamiRBDComputeIndexesWithThreadedandNot);
    CPPUNIT_TEST (testIshigamiRBDComputeIndexesWithTLauncher2);
    
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputFunction);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputCode);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputRun);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TRBDTest();

    //! Default destructor
    virtual ~TRBDTest();

    void setUp();
    void setUp1();
    void setUp2();
    void tearDown();
    //@}

    //---------------------------------------------

    /** @name Validation of the sample values for Ishigami
     **/
    //@{
    void validateValuesIshigami();
    //@}

    /** @name Validation of the sample values for Flowrate
     **/
    //@{
    void validateValuesFlowrate();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    void testTRBDwithFunction();
    void testTRBDwithCode();
#ifdef WITH_LEVELE
    void setUpLevelE();
    //! Test of the RBD for both SRC and SRRC using LevelE
    void testLevelERBD();
    //! Test the Sobol method for the Levele function restraining output vector in option
    void testLevelERBDOutputRestrOption();
    //! Check the results of the levelE sobol estrimation
    void checkLevelEResults(TTree *ntresu, vector<int> *elmts = NULL);
#endif
    void testTRBDsetFrequencies();
    void throwExceptionTRBDsetFrequencies();

    void testIshigamiRBDComputeIndexesWithTLauncherFunction();
    void testIshigamiRBDComputeIndexesGenerateSampleWithTLauncherFunction();
    void testIshigamiRBDComputeIndexesGenerateSampleCreateLauncherWithTLauncherFunction();
    void testFlowrateRBDComputeIndexesWithTLauncherCode();
    void testFlowrateRBDComputeIndexesWithTCodeEval();
    void testFlowrateRBDComputeIndexesGenerateSampleWithTLauncherCode();
    void testFlowrateRBDComputeIndexesGenerateSampleCreateLauncherWithTLauncherCode();

    void testIshigamiRBDComputeIndexesWithThreadedandNot();
    void testIshigamiRBDComputeIndexesWithTLauncher2();
    
    void throwExceptionConstructorTooLongOutputFunction();
    void throwExceptionConstructorTooLongOutputCode();
    void throwExceptionConstructorTooLongOutputRun();

    //@}

private:
    URANIE::DataServer::TDataServer *_tds, *_tds2, *_tds3;
    URANIE::Sensitivity::TRBD * _trbd;
    Int_t _ns;
    URANIE::Launcher::TCode *_mycode;
    void * _p2fIshigami;
    URANIE::Launcher::TOutputFileRow *_fout;
};

#endif  // CPPUNITEST_TRBD_H
