//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TREGRESSION_H
#define CPPUNITEST_TREGRESSION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSensitivity.h"
#include "TRegression.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TRegressionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TRegressionTest);

    CPPUNIT_TEST (testSRCRegression);
    CPPUNIT_TEST (testSRRCRegression);
    CPPUNIT_TEST (testIshigamiRegression);
    CPPUNIT_TEST (testIshigamiRegressionWithDrawings);
#ifdef WITH_LEVELE
    CPPUNIT_TEST (testLevelERegression);
    CPPUNIT_TEST (testLevelERegressionOutputRestrVarexp);
    CPPUNIT_TEST (testLevelERegressionOutputRestrOption);
    CPPUNIT_TEST (testLevelERegressionOutputRestrBoth);
#endif
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputFunctionFirst);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputFunctionSecond);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputCode);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputRun);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputFromData);
    CPPUNIT_TEST (throwExceptionTRegressionConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionTRegressionConstructorFalseInput);
    CPPUNIT_TEST (throwExceptionTRegressionConstructorFalseOutput);
    CPPUNIT_TEST (throwExceptionTRegressionConstructorFalseOption);
    CPPUNIT_TEST (testTRegressionsetMethod);
    CPPUNIT_TEST (throwExceptionTRegressioncomputeIndexesMoreOneOutput);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TRegressionTest();

    //! Default destructor
    virtual ~TRegressionTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test of the SRC Regression
    void testSRCRegression();
    //! Test of the SRRC Regression
    void testSRRCRegression();

#ifdef WITH_LEVELE
    //! Test of the Regression for both SRC and SRRC using LevelE
    void testLevelERegression();
    //! Test the Sobol method for the Levele function restraining output vector in constructor
    void testLevelERegressionOutputRestrVarexp();
    //! Test the Sobol method for the Levele function restraining output vector in option
    void testLevelERegressionOutputRestrOption();
    //! Test the Sobol method for the Levele function restraining output vector in both
    void testLevelERegressionOutputRestrBoth();
    //! Check the results of the levelE sobol estrimation
    void checkLevelEResults(TTree *ntresu, vector<int> *elmts = NULL);
#endif
    //! Test the SRC/SRRC Regression for the Ishigami function
    void testIshigamiRegression();
    void testIshigamiRegressionWithDrawings();
    void throwExceptionConstructorTooLongOutputFunctionFirst();
    void throwExceptionConstructorTooLongOutputFunctionSecond();
    void throwExceptionConstructorTooLongOutputCode();
    void throwExceptionConstructorTooLongOutputRun();
    void throwExceptionConstructorTooLongOutputFromData();
    void throwExceptionTRegressionConstructorTdsNull();
    void throwExceptionTRegressionConstructorFalseInput();
    void throwExceptionTRegressionConstructorFalseOutput();
    void throwExceptionTRegressionConstructorFalseOption();
    void testTRegressionsetMethod();
    void throwExceptionTRegressioncomputeIndexesMoreOneOutput();
    //@}

private:
    URANIE::DataServer::TDataServer *tds1, *tds;
    URANIE::Sensitivity::TRegression *treg;
    URANIE::Sensitivity::TRegression *tss1;
};

#endif  // CPPUNITEST_TREGRESSION_H
