//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTSensitivity.cxx
 \author Cedric Forgues <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::Sensitivity::TSensitivity
 
 Cr�e le : mer nov 21 07:59:24 2007

 \par Modifications
 \li M. Salvoldi (Areva/OCC) 2011/09/28
 - Authorize the access to testIshigamiFAST() only if FFTW libary is found
 
 */

// Uranie
#include "TestTSensitivity.h"

void TestTSensitivity::add_flowrate_attributes(std::vector<URANIE::DataServer::TUniformDistribution>&
    inputs, bool default_values) {

    std::vector<std::pair<std::string, std::array<double, 3>>> attributes;
    attributes.push_back(make_pair("rw", std::array<double, 3> {{0.05, 0.15, 0.075}}));
    attributes.push_back(make_pair("r", std::array<double, 3> {{100.0, 50000.0, 25000.0}}));
    attributes.push_back(make_pair("tu", std::array<double, 3> {{63070.0, 115600.0, 90000.0}}));
    attributes.push_back(make_pair("tl", std::array<double, 3> {{63.1, 116.0, 90.0}}));
    attributes.push_back(make_pair("hu", std::array<double, 3> {{990.0, 1110.0, 1050.0}}));
    attributes.push_back(make_pair("hl", std::array<double, 3> {{700.0, 820.0, 760.0}}));
    attributes.push_back(make_pair("l", std::array<double, 3> {{1120.0, 1680.0, 1400.0}}));
    attributes.push_back(make_pair("kw", std::array<double, 3>   {{9855.0, 12045.0, 10500.0}}));

    for (const auto& attribute: attributes) {
        inputs.push_back(URANIE::DataServer::TUniformDistribution(attribute.first,
            attribute.second[0], attribute.second[1]));
        if (default_values)
            inputs.back().setDefaultValue(attribute.second[2]);
    }
}

std::string toupper_first(std::string name){
    name[0] = toupper(name[0]);
    return name;
}

void TestTSensitivity::set_file_key_to_upper_first(URANIE::DataServer::TDataServer& tds,
    TString& input_file) {
    TList *list=tds.getListOfAttributes();
    for (const auto& attribute: *list) {
        const std::string name = toupper_first(attribute->GetName());
        tds.getAttribute(attribute->GetName())->setFileKey(input_file, name.data());
    }
    delete list;
}
