//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestUserFunctions.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   2012-09-14 10:45:11

 \brief Inteface for the rosenbrock function
 */

void flowrateModel(Double_t *x, Double_t *y);

void Ishigami(double *paramt, Double_t *res);

void FonctionAffine(double *paramt, Double_t *res);

/*!
 * \brief CounterExemple3Function
 * This function comes from the Sobol and Kucherenko article (2009) "Derivate based global sensitivity measures and their link with global sensitivity inidces".
 * It is the Exemple 3 in the section 7 "Counterexample" where \f$ x_{i} \in {\cal U}[0., 1.]\f$ and the function \f$ f\f$ have the following ANOVA decomposition
 * \f[
 * f(x) \: = \: \sum_{i=1}^{4} c_{i} times \left( x_{i} - \frac{1}{2} \right) \: c_{12} \times \left(  x_{1} - \frac{1}{2} \right)  \times \left(  x_{2} - \frac{1}{2} \right)^{5}
 * \f]
 * where $\f c_{i} = 1.0 \f$ et $\f c_{12} = 50.0 \f$.
 * In this case, the analytical indices are respectively \f$ ( 1.22, 3.26, 1.0, 1.0) \f$.
 *
 * \param x (double *) the input vector (dim=4)
 * \param y (double *) the output vector (dim=1)
 */
void CounterExemple3Function(double *x, Double_t *y);

void IshigamiDecaled(double *paramt, Double_t *res);

void badPrototypeFunction();

// End Of File
