#include <iostream>
using namespace TMath;
using namespace URANIE::DataServer;
using namespace URANIE::Sensitivity;

void waterborehole(double *x, Double_t *y)
{
    Double_t drw = x[0], dr = x[1];
    Double_t dtu = x[2], dtl = x[3];
    Double_t dhu = x[4], dhl = x[5];
    Double_t dl = x[6], dkw = x[7];

    Double_t dnum = 2.0 * TMath::Pi() * dtu * (dhu - dhl);
    Double_t dlnronrw = TMath::Log(dr / drw);
    Double_t dden =
            dlnronrw
                    * (1.0 + (2.0 * dl * dtu) / (dlnronrw * drw * drw * dkw)
                            + dtu / dtl);

    y[0] = dnum / dden;
}

void testFast()

{
    TDataServer *tds = new TDataServer("tdsflowreate", "DataBase flowreate");
    tds->addAttribute(new TUniformDistribution("rw", 0.05, 0.15));
    tds->addAttribute(new TUniformDistribution("r", 100.0, 50000.0));
    tds->addAttribute(new TUniformDistribution("tu", 63070.0, 115600.0));
    tds->addAttribute(new TUniformDistribution("tl", 63.1, 116.0));
    tds->addAttribute(new TUniformDistribution("hu", 990.0, 1110.0));
    tds->addAttribute(new TUniformDistribution("hl", 700.0, 820.0));
    tds->addAttribute(new TUniformDistribution("l", 1120.0, 1680.0));
    tds->addAttribute(new TUniformDistribution("kw", 9855.0, 12045.0));

    Int_t ns = 500;
    TFast * tfast = new TFast(tds, waterborehole, "", "waterborehole", ns);

    ofstream sortie;
    sortie.open("testFast.out", ios::out);
    sortie << tfast->getTransformFunction() << "\n";
    tfast->setTransformFunction(TFast::kCukier);
    sortie << tfast->getTransformFunction() << "\n";
    tfast->setTransformFunction(TFast::kSaltelliA);
    sortie << tfast->getTransformFunction() << "\n";
    tfast->setTransformFunction(TFast::kSaltelliB);
    sortie << tfast->getTransformFunction() << "\n";

    sortie << tfast->getM() << "\n";
    tfast->setM(8);
    sortie << tfast->getM() << "\n";

    sortie.close();

    delete tfast;
    delete tds;

}
