#include <iostream>
using namespace TMath;
using namespace URANIE::DataServer;
using namespace URANIE::Sensitivity;
using namespace URANIE::Relauncher;

void Ishigami(double *paramt, Double_t *res)
{
    Double_t a = 7.0;
    Double_t b = 0.1;

    Double_t x1 = paramt[0];
    Double_t x2 = paramt[1];
    Double_t x3 = paramt[2];

    res[0] = Sin(x1);
    res[0] += a * Sin(x2) * Sin(x2);
    res[0] += b * x3 * x3 * x3 * x3 * Sin(x1);
}

void testFiniteIshigamiRe(Int_t nS = 100000)
{
    // Attributes of the study
    TUniformDistribution x1("x_{1}", -1.0 * Pi(), Pi());
    TUniformDistribution x2("x_{2}", -1.0 * Pi(), Pi());
    TUniformDistribution x3("x_{3}", -1.0 * Pi(), Pi());
    x1.setDefaultValue(0.);
    x2.setDefaultValue(0.);
    x3.setDefaultValue(0.);
    
    TAttribute ishi("Ishigami");
    
    // function
    TCJitEval eval(&Ishigami);
    eval.setInputs(3, &x1, &x2, &x3);
    eval.addOutput(&ishi);
    
    TSequentialRun run(&eval);
    run.startSlave();
    if (run.onMaster()) {
        TDataServer * tds = new TDataServer("tdsTSI",
                "TDS for Ishigami function test");
        tds->addAttribute(&x1);
        tds->addAttribute(&x2);
        tds->addAttribute(&x3);

        TFiniteDifferences *tsa = 
                //new TFiniteDifferences(tds, &run, "x_{1}:x_{2}:x_{3}", "minmax");
                new TFiniteDifferences(tds, &run);
        tsa->setLog();
        tsa->setDrawProgressBar(kFALSE);
        tsa->setSeed(123);
        tsa->computeIndexes();
/*
        Double_t sobol;
        TTree *tree = tsa->getResultTuple();
        tree->SetBranchAddress("Value", &sobol);

        // Store informations in an output file
        ofstream sortie;
        sortie.open("testFiniteIshigamiRe.out", ios::out);
        sortie.precision(12);
        sortie.setf(ios::scientific);

        // First Order
        tree->Draw(">>tel_first", "Algorithm==\"--first--\"");
        TEventList *telfirst = (TEventList*) gDirectory->Get("tel_first");

        // Total Order
        tree->Draw(">>tel_total", "Algorithm==\"--total--\"");
        TEventList *teltotal = (TEventList*) gDirectory->Get("tel_total");

        tree->GetEntry(telfirst->GetEntry(0));
        sortie << sobol << "\n";
        tree->GetEntry(teltotal->GetEntry(0));
        sortie << sobol << "\n";
        tree->GetEntry(telfirst->GetEntry(1));
        sortie << sobol << "\n";
        tree->GetEntry(teltotal->GetEntry(1));
        sortie << sobol << "\n";
        tree->GetEntry(telfirst->GetEntry(2));
        sortie << sobol << "\n";
        tree->GetEntry(teltotal->GetEntry(2));
        sortie << sobol << "\n";
*/    }
}
