#include <iostream>
using namespace TMath;
using namespace URANIE::DataServer;
using namespace URANIE::Sensitivity;

void Ishigami(double *paramt, Double_t *res)
{
    Double_t a = 7.0;
    Double_t b = 0.1;

    Double_t x1 = paramt[0];
    Double_t x2 = paramt[1];
    Double_t x3 = paramt[2];

    res[0] = Sin(x1);
    res[0] += a * Sin(x2) * Sin(x2);
    res[0] += b * x3 * x3 * x3 * x3 * Sin(x1);
}

void testMorrisIshigami(Int_t nS = 100000)
{
    // Attributes of the study
    TDataServer * tds = new TDataServer("tds", "TDS for Ishigami function");
    tds->addAttribute(new TUniformDistribution("x_{1}", -1.0 * Pi(), Pi()));
    tds->addAttribute(new TUniformDistribution("x_{2}", -1.0 * Pi(), Pi()));
    tds->addAttribute(new TUniformDistribution("x_{3}", -1.0 * Pi(), Pi()));

    TMorris * scmo = new TMorris(tds, "Ishigami", 3, 10);
    scmo->setDrawProgressBar(kFALSE);
    scmo->computeIndexes();

    Double_t sobol;
    TTree *tree = scmo->getResultTuple();
    tree->SetBranchAddress("Value", &sobol);

    // Store informations in an output file
    ofstream sortie;
    sortie.open("testMorrisIshigami.out", ios::out);
    sortie.precision(12);
    sortie.setf(ios::scientific);

    Int_t npat = (Int_t)(tree->GetEntries());
    sortie << npat << "\n";
    for (Int_t i = 0; i < npat; i++)
    {
        tree->GetEntry(i);
        sortie << sobol << "\n";
    }
}
