//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TComposeEval.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-09-26.10.09.15

 \brief composition de fonction
 */

#ifndef __TCOMPOSEEVAL__
#define __TCOMPOSEEVAL__

//ROOT
#include "TList.h"
#include "TObjArray.h"

//URANIE
#include "Relauncher.h"
#include "TShareEval.h"

namespace URANIE
{
namespace Relauncher
{

class TSubEval: public TObject
{
protected:
    TEval *_eval;
    int _dec;
    std::vector<int> _select;
    std::vector<int> _selectout;

public:
    TSubEval(TEval*);
    virtual ~TSubEval();

    TEval *getEval();
    int freeze(Int_t, TObjArray &, TObjArray &, TObjArray &);
    void match(TObjArray &);
    void _rebuild(int, Bool_t);
    TSubEval *rebuild(int, Bool_t);
    void unbuild();
    int eval(vector<URANIE::DataServer::UEntry*>*, vector<URANIE::DataServer::UEntry*>*, int =0 );

};

class TComposeEval: public TShareEval
{
protected:
    TList *_eval;
    TObjArray _const;
    std::vector<double> _val;
    int _allsize;

public:
    TComposeEval();
    virtual ~TComposeEval();

    void addEval(TEval *);
    void setEvals(Int_t, TEval *, ...);
    void addConstantValue(URANIE::DataServer::TAttribute *, double);

    virtual void freeze();
    void _rebuild(int, Bool_t);
    virtual TEval *rebuild(int rank, Bool_t chef = kTRUE);
    virtual Bool_t unbuild();
    virtual int eval(vector<URANIE::DataServer::UEntry*>*, vector<URANIE::DataServer::UEntry*>*, int =0 );

    virtual bool isThreadSafe();
    
    ClassDef(URANIE::Relauncher::TComposeEval, ID_RELAUNCHER)
};

}
} //namespace

#endif
