//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TComposeMulti.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-11-24.15.38.13

 \brief composition de fonctions avec une fonction multiple
 */

#ifndef __TCOMPOSEMULTI__
#define __TCOMPOSEMULTI__

//ROOT
#include "TList.h"
#include "TObjArray.h"

//URANIE
#include "Relauncher.h"
#include "TComposeEval.h"

namespace URANIE
{
namespace Relauncher
{

class TComposeMulti: public TShareMulti
{
protected:
    TList *_after, *_before;
    TSubMulti *_multi;
    Int_t _mfirst, _msize, _allsize;
    TObjArray _const;
    std::vector<double> _val;

public:
    TComposeMulti(TEval*);
    virtual ~TComposeMulti();

    void addBeforeEval(TEval *);
    void addAfterEval(TEval *);
    void setBeforeEvals(Int_t, TEval *, ...);
    void setAfterEvals(Int_t, TEval *, ...);
    void addConstantValue(URANIE::DataServer::TAttribute *, double);

    virtual void freeze();
    void _rebuild(int, Bool_t);
    virtual TEval *rebuild(int rank, Bool_t chef = kTRUE);
    virtual Bool_t unbuild();
    virtual int meval(double*, double**);

    ClassDef(URANIE::Relauncher::TComposeMulti, ID_RELAUNCHER)
};

}
} //namespace

#endif
