//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.1 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TCoroutineRun.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-12-18.11.51.18

 \brief   execution recherche dans une coroutine
 */

#include "TCoroutineRun.h"

ClassImp (URANIE::Relauncher::TCoroutineRun);

namespace URANIE
{
namespace Relauncher
{

TCoroutineRun::TCoroutineRun(TEval *user, long nbnod, 
    coroutine<void>::push_type *yield) :
        TSimpleRun(user)
{
    _ncpus = nbnod;
    _yield = yield;
}

TCoroutineRun::TCoroutineRun(TEval *user, long nbnod, void *yield) :
        TSimpleRun(user)
{
    _ncpus = nbnod;
    _yield = (boost::coroutines2::coroutine<void>::push_type *) yield;
    //std::cout << "yield " << yield << std::endl;
}

TCoroutineRun::~TCoroutineRun(void)
{
}

void TCoroutineRun::startSlave(void)
{
}

void TCoroutineRun::stopSlave(void)
{
    put(-1, (vector<URANIE::DataServer::UEntry*>*) NULL, 
        (vector<URANIE::DataServer::UEntry*>*) NULL, 0);
    (*_yield)();
}

bool TCoroutineRun::empty()
{
    return _qin.empty();
}

struct element TCoroutineRun::serviceGet()
{
    struct element tupl;

    tupl = _qin.front();
    _qin.pop();
    //std::cout << tupl.id << "Coroutine get\n";
    return tupl;
}

void TCoroutineRun::servicePut(long id, int ret)
{
    _next.id = id;
    _next.stat = ret;
}

int TCoroutineRun::get(long &id)
{
    (*_yield)();
    id = _next.id;
    return _next.stat;
}

}
} // namespace

