//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.4 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TEval.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-03-30.10.48.04

 \brief evaluation dun jeu de parametres
 */

#ifndef __TEval__
#define __TEval__

//ROOT
#include "Rtypes.h"
#include "TObject.h"
#include "TObjArray.h"

// Uranie
#include "Relauncher.h"
#include "TDataServer.h"

namespace URANIE
{
namespace Relauncher
{

class TEval: public TObject
{
public:
    
    class ResultTypeMismatch
    {
    public:
	ResultTypeMismatch(){std::cout<<"[91m <ERROR>"<<std::endl<<" <ERROR> This eval is not suited for the requested method"<<std::endl<<" <ERROR> [0m"<<std::endl;}
    };
    class FreezedEval
    {
    public:
	FreezedEval(){std::cout<<"[91m <ERROR>"<<std::endl<<" <ERROR> This eval is frozen"<<std::endl<<" <ERROR> [0m"<<std::endl;}
    };
    class CanNotDealWithNotDouble
    {
    public:
	CanNotDealWithNotDouble(const char *name){
	    std::cout<<"[91m <ERROR>"<<std::endl<<" <ERROR> The eval "<<name<<" is not suited to deal with Strings and Vectors"<<std::endl<<" <ERROR> [0m"<<std::endl;
	}
    };
    
    class EvalIsNotThreadable
    {
    public:
	EvalIsNotThreadable(){
	    std::cout<<"[91m <ERROR>"<<std::endl<<" <ERROR> This eval is known to be not thread-safe"<<std::endl<<" <ERROR> [0m"<<std::endl;
	}
    };
    
    virtual ~TEval()
	{
	}

    virtual TObjArray *getInputList() = 0;
    virtual TObjArray *getOutputList() = 0;
    virtual void addAllInputs(URANIE::DataServer::TDataServer *) = 0;

    virtual void provideSingleResult() = 0;

    virtual void freeze(void) = 0;
    virtual TEval *rebuild(int rank, Bool_t chef = kTRUE) = 0;
    virtual Bool_t unbuild() = 0;
    virtual bool isThreadSafe() = 0;
    virtual bool containsStringType() = 0;

    virtual void clean() = 0;
    virtual void cleanOutputs() = 0;

    virtual int eval(vector<URANIE::DataServer::UEntry*>*, vector<URANIE::DataServer::UEntry*>*, int =0 ) = 0;

    virtual int getOutDimension(vector<URANIE::DataServer::UEntry*>*) = 0;
    
    ClassDef(URANIE::Relauncher::TEval, ID_RELAUNCHER)
};


}} // namespace

#endif
