//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TFlagScript.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-05-29.11.18.00

 \brief gestion des fichiers entree de code utilisant les flags
 */

#ifndef __TFLAGSCRIPT__
#define __TFLAGSCRIPT__

#include <fstream>
//ROOT
#include "TString.h"
#include "TList.h"
#include "TObjArray.h"
#include "TRegexp.h"

//Uranie
#include "Relauncher.h"
#include "TAttribute.h"
#include "TFlatScript.h"

namespace URANIE
{
namespace Relauncher
{

class TFlagParser: public TNamed
{
protected:
    Ssiz_t _istart, _istop;

public:
    URANIE::DataServer::TAttribute *_att;
    Ssiz_t _line, _index;

    TFlagParser(const char *name, URANIE::DataServer::TAttribute *att);
    virtual ~TFlagParser();

    virtual Int_t Compare(const TObject *) const;
    void build(int id, std::istream &fi, TObjArray *v);
    TString &replace(TString *line, Double_t val);
    TString &replace(TString *line, TString val);

};

class TFlagScript: public TShareScript
{
protected:
    TObjArray _vars, _occur;
    
public:
    TFlagScript(const char *path);
    virtual ~TFlagScript(void);

    void addInput(URANIE::DataServer::TAttribute *, const char *);
    void setInputs(Int_t size, URANIE::DataServer::TAttribute *, const char *, ...);

    /* protocole */
    virtual void build(TString &dir, TObjArray *in);
    virtual void put(TString &ref, TString &tmp, Double_t*);

    void put(TString &ref, TString &work, vector<URANIE::DataServer::UEntry*> *in);

    ClassDef(URANIE::Relauncher::TFlagScript, ID_RELAUNCHER)

};

}
} //namespace

#endif

