//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TInformMaster.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-07-24.14.59.23

 \brief master qui complete un tds existant
 */

#ifndef __INFORM_MASTER__
#define __INFORM_MASTER__

//ROOT
#include "Rtypes.h"

// Uranie
#include "Relauncher.h"
#include "TMaster.h"

#include "TDataServer.h"
#include "TRun.h"
#include "TEval.h"

#include "TDSNtupleD.h"

namespace URANIE
{
namespace Relauncher
{

class TInformMaster: public TMaster
{
protected:
    URANIE::DataServer::TDSNtupleD *_sampl;
    string _samplename;
    TList _inleaves;

    void getSample();
    void build();
    void allocSamplEntries(vector<URANIE::DataServer::UEntry*> &vec);

    //Entry part for vectors
    //void allocDataEntries(int siz, vector<vector<URANIE::DataServer::UEntry*>> &);
    void getCurrentPattern(Int_t ipati, vector<URANIE::DataServer::UEntry*> *,
        vector<URANIE::DataServer::UEntry*> *);
    void putCurrentPattern(vector<URANIE::DataServer::UEntry*> *in, vector<URANIE::DataServer::UEntry*> *out, vector<URANIE::DataServer::UEntry*> *evt);
    void putCurrentAsError(Int_t i, vector<URANIE::DataServer::UEntry*> *in, 
        URANIE::DataServer::TDataServer *tds);

 public:
    TInformMaster(URANIE::DataServer::TDataServer *tds, TRun *run);
    TInformMaster(URANIE::DataServer::TDataServer *tds, TEval *fun);
    TInformMaster(URANIE::DataServer::TDataServer *tds, TStandardEval *fun, const char *in, const char *out);
    virtual ~TInformMaster();

    ClassDef(URANIE::Relauncher::TInformMaster, ID_RELAUNCHER)

};

}
} //namespace
#endif

