//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TKeyScript.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-09-09.14.50.12

 \brief jeux de données du TCodeEval
 */

#ifndef __TKEYSCRIPT__
#define __TKEYSCRIPT__

#include <fstream>
//ROOT
#include "TString.h"
#include "TList.h"
#include "TObjArray.h"
#include "TRegexp.h"

//Uranie
#include "Relauncher.h"
#include "TAttribute.h"
#include "TCodeEval.h"

namespace URANIE
{
namespace Relauncher
{

class TKeyParser: public TNamed
{


public:
    URANIE::DataServer::TAttribute *_att;
    Ssiz_t _line, _istart, _istop, _index;
    TString _svecbeg, _svecend, _svecdel, _sstrbeg, _sstrend; 
    TRegexp _re_equ, _re_fin, _re_full;
    
    class KeyNotFound{
    public:
	KeyNotFound(TString key){
	    std::cout<<"[91m <ERROR> \n <ERROR> The key "<<key.Data()<<" is not found in the provided input file.\n <ERROR>[0m"<<std::endl;
	}
    };
    std::stringstream buffer;
    
    TKeyParser(const char *name, URANIE::DataServer::TAttribute *att);
    virtual ~TKeyParser();

    //virtual Bool_t IsSortable();
    virtual Int_t Compare(const TObject *) const;
    void build(int id, std::istream &fi);
    TString &replace(TString *line, Double_t val);
    TString &replace(TString *line, URANIE::DataServer::UEntry *val, TString format);
    void setVectorProperties( TString beg, TString delim, TString end);
    void setStringProperties( TString beg, TString end);
};

class TKeyScript: public TCodeScript
{
protected:
    TString _path;
    TObjArray _vars;
    TString _format;
    
public:

    class FileNotFound{
    public:
	FileNotFound(TString file){
	    std::cout<<"[91m <ERROR> \n <ERROR> The file "<<file<<" is not found. Cannot proceed to search for keys.\n <ERROR>[0m"<<std::endl;
	}
    };
    TKeyScript(const char *path);
    virtual ~TKeyScript(void);

    void addInput(URANIE::DataServer::TAttribute *, const char *);
    void setInputs(Int_t size, URANIE::DataServer::TAttribute *, const char *,
            ...);
    void setVectorProperties( TString beg, TString delim, TString end);
    void setStringProperties( TString beg, TString end);
    void setFormat(TString form) {_format=form;}
    
    /* protocole */
    virtual void build(TString &dir, TObjArray *in);
    virtual void put(TString &ref, TString &tmp, Double_t*);
    virtual void put(TString &ref, TString &tmp, vector<URANIE::DataServer::UEntry*> *in);

    ClassDef(URANIE::Relauncher::TKeyScript, ID_RELAUNCHER)

};

}
} //namespace

#endif

