//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TLineScript.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-09-19.15.01.02

 \brief   entree sur une ligne
 */

#include "TLineScript.h"

#include <iostream>
#include <fstream>

ClassImp (URANIE::Relauncher::TLineScript);

namespace URANIE
{
namespace Relauncher
{

TLineScript::TLineScript(const char *name) :
    TVoidScript(name)
{
    _svecbeg=TString(""); _svecend=TString(""); _svecdel=TString(" ");
    _sstrbeg=TString("");  _sstrend=TString("");  
}

TLineScript::~TLineScript() {}


void TLineScript::put(TString &ref, TString &work, Double_t *in)
{
    UNUSED(ref); //Only used for flag where a reference file is needed
    char* copy = new char[work.Length() + _path.Length() + 3];
#ifdef WIN32
    sprintf(copy, "%s\\%s", work.Data(), _path.Data());
#else
    sprintf(copy, "%s/%s", work.Data(), _path.Data());
#endif

    std::ofstream fout(copy, std::ios::out);
    TIter next(&_vars);
    TNoParser *np;

    fout.precision(SSTR_PREC);
    while ((np = (TNoParser *) next()))
        fout << in[np->_index] << "\n";
    fout.close();
	
    delete[] copy;
}

void TLineScript::put(TString &ref, TString &work, vector<URANIE::DataServer::UEntry*> *in)
{
    UNUSED(ref); //Only used for flag where a reference file is needed
    char* copy = new char[work.Length() + _path.Length() + 3];
#ifdef WIN32
    sprintf(copy, "%s\\%s", work.Data(), _path.Data());
#else
    sprintf(copy, "%s/%s", work.Data(), _path.Data());
#endif

    std::ofstream fout(copy, std::ios::out);
    TIter next(&_vars);
    TNoParser *np;

    fout.precision(SSTR_PREC);
    while ((np = (TNoParser *) next()))
    {
         if(in->at(np->_index)->tag==URANIE::DataServer::UEntry::Vector)
             fout << _svecbeg << in->at(np->_index)->dumpStr( _svecdel,"","",_format ) << _svecend << endl;
        else
            fout << ((in->at(np->_index)->tag==URANIE::DataServer::UEntry::String) ? _sstrbeg : "")
                 << in->at(np->_index)->dumpStr( "","","",_format )
                 << ((in->at(np->_index)->tag==URANIE::DataServer::UEntry::String) ? _sstrend : "") << endl;;
    }
    fout << std::endl;
    fout.close();
	
    delete[] copy;
}

}
} //namespace

