//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TMpiLn2Eval.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-08-23.15.30.26

 \brief   fonction utilisateur mpi
 */

#include "TMpiLn2Eval.h"

//#include "mpi.h"
#include "TBiMpiRun.h"

#define N 100000

ClassImp (URANIE::MpiRelauncher::TMpiLn2Eval);

namespace URANIE
{
namespace MpiRelauncher
{

TMpiLn2Eval::TMpiLn2Eval() :
    URANIE::Relauncher::TSimpleEval()
{
    //if (_ninput != 2) throw InputError();
    //if (_noutput != 1) throw OutputError();
}

TMpiLn2Eval::~TMpiLn2Eval()
{
}

int TMpiLn2Eval::eval(vector<URANIE::DataServer::UEntry*> *in, 
    vector<URANIE::DataServer::UEntry*> *out, int icalc)
{
    UNUSED(icalc);
    double sum, res, din[2];
    int i, id, size;
    MPI_Comm comm;
    
    comm = TBiMpiRun::getCalculMpiComm();
    MPI_Comm_rank(comm, &id);
    MPI_Comm_size(comm, &size);
    //MPI_Comm *comm = reinterpret_cast<MPI_Comm *>(com);
    
    din[0] = *in->at(0)->val.dou;
    din[1] = *in->at(1)->val.dou;
    sum = 0.;
    for (i = id; i < N; i += size)
        if (i % 2)
            sum -= 1. / (i + din[0]);
        else
            sum += 1. / (i + din[1]);
    MPI_Reduce(&sum, &res, 1, MPI_DOUBLE, MPI_SUM, 0, comm);
    if (!id)
    {
        res -= 0.69314718055994529;
        out->at(0)->pushEntry(res * res);
    }
    return 1;
}

} } //namespace

