/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TSocketRun.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2017-11-23
 
 \brief  independant process communicating by socket
*/

#ifndef __TMPIRUN__
#define __TMPIRUN__

#include "TSeparateRun.h"

//#include <queue>
//#include <vector>

#define MPICH_IGNORE_CXX_SEEK
#define OMPI_SKIP_MPICXX
#include <mpi.h>


namespace URANIE
{
namespace MpiRelauncher
{

class TMpiRun : public URANIE::Relauncher::TSeparateRun
{
protected:
    static int _count;
    int _id;
    
    MPI_Comm _comm;

    virtual int wait_service();
    virtual int send_header(int canal, int *head, int size);
    virtual int send_value(int canal, double *mesg, int size);
    virtual int receive_header(int canal, int *head, int size);
    virtual int receive_value(int canal, double *mesg, int size);
    
public:
    
    TMpiRun(URANIE::Relauncher::TEval *r, int *argc, char ***argv);
    TMpiRun(URANIE::Relauncher::TEval *r, Bool_t init=kTRUE);
    TMpiRun(URANIE::Relauncher::TEval *r, MPI_Comm comm);
    virtual ~TMpiRun();
    
    virtual void startSlave();
    virtual Bool_t onMaster();
    virtual Bool_t readyToGet();
    
    ClassDef(URANIE::MpiRelauncher::TMpiRun, ID_RELAUNCHER)
};


}} //namespace
#endif
