/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TSocketRun.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2017-11-23
 
 \brief  independant process communicating by socket
*/

#ifndef __TSOCKETRUN__
#define __TSOCKETRUN__

#include "TSeparateRun.h"

#include <queue>
#include <vector>

#include "TSocket.h"
#include "TServerSocket.h"
#include "TMonitor.h"

namespace URANIE
{
namespace Relauncher
{

class TSocketRun : public TSeparateRun
{
protected:
    
    // server side
    TServerSocket *_sserv;
    TMonitor *_moni;
    std::vector<TSocket *> _sclient;
    // client side
    const char *_conn;
    int _port;
    //TSocket *_sock;
    

    virtual int wait_service();
    virtual int send_header(int canal, int *head, int size);
    virtual int send_value(int canal, double *mesg, int size);
    virtual int receive_header(int canal, int *head, int size);
    virtual int receive_value(int canal, double *mesg, int size);
    
public:
    
    TSocketRun(URANIE::Relauncher::TEval *r, int p=5432);
    virtual ~TSocketRun();
    
    virtual void startSlave();
    virtual Bool_t onMaster();
    virtual Bool_t readyToGet();
    
    ClassDef(URANIE::Relauncher::TSocketRun, ID_RELAUNCHER)
};


}} //namespace
#endif
