/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TStandardEval.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2014-07-07
 
 \brief  simple eval shared class
*/

#ifndef __TStandardEval__
#define __TStandardEval__

//ROOT
#include "Rtypes.h"
#include "TObjArray.h"

// Uranie
#include "Relauncher.h"
#include "TEval.h"
#include "TBaseEval.h"
#include "TDataServer.h"

namespace URANIE
{
namespace Relauncher
{

class TStandardEval : public TBaseEval
{
public:
    TStandardEval() : TBaseEval() {}
    virtual void addInput(URANIE::DataServer::TAttribute *);
    virtual void addOutput(URANIE::DataServer::TAttribute *);
    virtual void setInputs(Int_t, URANIE::DataServer::TAttribute *, ...);
    virtual void setOutputs(Int_t, URANIE::DataServer::TAttribute *, ...);

    ClassDef(URANIE::Relauncher::TStandardEval, ID_RELAUNCHER)

};


class TSimpleEval: public TStandardEval
{
public:
    TSimpleEval() : TStandardEval() {}
    virtual void provideSingleResult() { } ;
    virtual int eval(std::vector<URANIE::DataServer::UEntry*>*, std::vector<URANIE::DataServer::UEntry*>*, int =0 ){ return 0; }
    
    ClassDef(URANIE::Relauncher::TSimpleEval, ID_RELAUNCHER)
};


class TDoubleEval: public TSimpleEval
{
public:
    TDoubleEval() : TSimpleEval() {}
    virtual int eval(double*, double*, int =0 ) { return 0; } // = 0 ; //
    virtual int eval(std::vector<URANIE::DataServer::UEntry*>*, std::vector<URANIE::DataServer::UEntry*>*, int =0 );
    
    ClassDef(URANIE::Relauncher::TDoubleEval, ID_RELAUNCHER)
};

}} //namespace

#endif
