//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TTangramEval.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-04-07.12.21.07

 \brief   fonction utilisateur
 */

#include "TTangramEval.h"

#include <math.h>

/* user */
#define SLOW 3000

/* constantes */
#define NIV 255.0 //niveau de gris
#define TAB 8 // nombre de régions
#define NBGENE 7 // nombre de paramètres

/* modèle */
static const char tgr[TAB * TAB] =
{
/*  0  1  2  3  4  5  6  7 */
0, 1, 1, 1, 0, 0, 1, 1, // 0
        1, 0, 1, 1, 1, 0, 0, 0, // 1
        1, 1, 0, 0, 1, 0, 0, 0, // 2
        1, 1, 0, 0, 0, 1, 1, 0, // 3
        0, 1, 1, 0, 0, 1, 0, 1, // 4
        0, 0, 0, 1, 1, 0, 1, 0, // 5
        1, 0, 0, 1, 0, 1, 0, 1, // 6
        1, 0, 0, 0, 1, 0, 1, 0  // 7
        };

static const char *tangram[TAB] =
{ &tgr[0], &tgr[TAB], &tgr[2 * TAB], &tgr[3 * TAB], &tgr[4 * TAB],
        &tgr[5 * TAB], &tgr[6 * TAB], &tgr[7 * TAB] };

double contraste(int size, double *sol, const char **connexion)
{
    int i, j;
    double min, diff;

    min = NIV;
#ifdef SLOW
    for (int k = 0; k < SLOW; k++)
#endif
        for (i = 0; i < size; i++)
        {
            for (j = i + 1; j < size; j++)
            {
                if (connexion[i][j])
                {
                    diff = fabs(sol[i] - sol[j]);
                    if (diff < min)
                        min = diff;
                }
            }
        }
    return min;
}

/* wrapper */
void mydecode(int size, double *intern, double *user)
{
    int i;

    user[0] = NIV; /* bord blanc imposé */
    for (i = 1; i < size; i++)
        user[i] = intern[i - 1];
}

/* TTangram */

ClassImp (URANIE::Relauncher::TTangramEval);

namespace URANIE
{
namespace Relauncher
{

TTangramEval::TTangramEval():
    TDoubleEval()
{
    //_ninput = NBGENE;
    //_noutput = 1;
}

int TTangramEval::eval(double *in, double *out, int ncalc)
{
    UNUSED(ncalc);
    double all[TAB];
    mydecode(TAB, in, all);
    out[0] = contraste(TAB, all, tangram);
    return 1;
}


//namespace
}
}

