//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.4 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TThreadedRun.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2008-08-22.15.45.31

 \brief   execution multithreader de la recherche
 */

#ifndef __TTHREADEDRUN__
#define __TTHREADEDRUN__

#include <pthread.h>
#include <queue>

#include "TShareRun.h"

namespace URANIE
{
namespace Relauncher
{

class TThreadedRun;

struct job
{
    int id;
    TThreadedRun *run;
    pthread_t thr;
};

class TThreadedRun: public TShareRun
{
protected:
    long _nbnode;
    int _runcore;

    pthread_mutex_t _mutex_in, _mutex_out;
    pthread_cond_t _invide, _outvide;
    std::queue<struct element> _qin;
    std::queue<struct relement> _qout;
    std::vector<struct job> _job;

    struct element serviceGet();
    void servicePut(long, int);

public:
    TThreadedRun(TEval *, long);
    virtual ~TThreadedRun(void);

    virtual void startSlave(void);
    virtual Bool_t onMaster(void);
    virtual void stopSlave(void);

    void calcul_loop(int);
    virtual void put(long int, std::vector<URANIE::DataServer::UEntry*>*, std::vector<URANIE::DataServer::UEntry*>*, int =0 );
    virtual Bool_t readyToGet();
    virtual int get(long &);

    ClassDef(URANIE::Relauncher::TThreadedRun, ID_RELAUNCHER)
};

}
}

#endif
