using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::MpiRelauncher;

void Ishigami(Double_t *paramt, Double_t *res)
{
    Double_t a = 7.0;
    Double_t b = 0.1;

    Double_t x1 = paramt[0];
    Double_t x2 = paramt[1];
    Double_t x3 = paramt[2];
    res[0] = sin(x1);
    res[0] += a * sin(x2) * sin(x2);
    res[0] += b * x3 * x3 * x3 * x3 * sin(x1);
}

void testIshigamiMPI()
{
    TAttribute ct("Ishigami");
    TAttribute *x1 = new TAttribute("x_{1}");
    TAttribute *x2 = new TAttribute("x_{2}");
    TAttribute *x3 = new TAttribute("x_{3}");

    TCJitEval code(Ishigami);
    code.addInput(x1);
    code.addInput(x2);
    code.addInput(x3);
    code.addOutput(&ct);

    TMpiRun run(&code);
    run.startSlave();
    if (run.onMaster())
    {
	TDataServer *tds = new URANIE::DataServer::TDataServer("tds","Design of experiments for Ishigami");
	tds->addAttribute(x1);
	tds->addAttribute(x2);
	tds->addAttribute(x3);
	tds->fileDataRead("Ishigami.dat", kFALSE, kTRUE);

	TLauncher2 launch(tds, &run);

        // resolution
        launch.solverLoop();

        tds->exportData("_output_testIshigamiMPI_.dat");

        run.stopSlave();
        delete tds;
    }
}

