#!/usr/bin/env python

import ROOT
import math

def ishigami(x1, x2, x3) :
    """ Ishigami function """
    a = 7.0
    b = 0.1
    
    res = math.sin(x1)
    res += a * math.sin(x2) * math.sin(x2)
    res += b * x3 * x3 * x3 * x3 * math.sin(x1)

    return [res,]

# ------------------------------------------------------------------------------

# output attribute
ishi = ROOT.URANIE.DataServer.TAttribute("Ishigami")

# tds
tds = ROOT.URANIE.DataServer.TDataServer("tdsishigami", "tds for Python Ishigami")
tds.fileDataRead("Ishigami.dat")

x1 = tds.getAttribute("x_{1}")
x2 = tds.getAttribute("x_{2}")
x3 = tds.getAttribute("x_{3}")

# code
code = ROOT.URANIE.Relauncher.TPythonEval(ishigami)
code.addInput(x1)
code.addInput(x2)
code.addInput(x3)
code.addOutput(ishi)

# run
run = ROOT.URANIE.Relauncher.TSequentialRun(code)
run.startSlave()
if (run.onMaster()) :
    # launcher
    launch = ROOT.URANIE.Relauncher.TLauncher2(tds, run)
    launch.solverLoop()

    tds.exportData("_output_testIshigamiSequentialPython_.dat");

    run.stopSlave()
