#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TAttribute.h"
#include "TUniformDistribution.h"
#include "TCJitEval.h"
#include "TThreadedRun.h"
#include "TDataServer.h"
#include "TSampling.h"
#include "TLauncher2.h"
#include "TCanvas.h"

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;

void Ishigami(Double_t *paramt, Double_t *res)
{
    Double_t a = 7.0;
    Double_t b = 0.1;

    Double_t x1 = paramt[0];
    Double_t x2 = paramt[1];
    Double_t x3 = paramt[2];

    res[0] = sin(x1);
    res[0] += a * sin(x2) * sin(x2);
    res[0] += b * x3 * x3 * x3 * x3 * sin(x1);
}

void testIshigamiSo()
{
    TAttribute ct("Ishigami");

    TDataServer *tds = new URANIE::DataServer::TDataServer("tdsishigami",
            "Design of experiments for Ishigami");
    tds->fileDataRead("Ishigami.dat");
    TAttribute *x1 = tds->getAttribute("x_{1}");
    TAttribute *x2 = tds->getAttribute("x_{2}");
    TAttribute *x3 = tds->getAttribute("x_{3}");

    TCJitEval code(&Ishigami);
    code.setInputs(3, x1, x2, x3);
    code.setOutputs(1, &ct);

    TThreadedRun run(&code, 14);
    run.startSlave();
    if (run.onMaster())
    {
        TLauncher2 launch(tds, &run);

        // resolution
        launch.solverLoop();

        tds->exportData("_output_testIshigamiSo_.dat");

        run.stopSlave();
    }

}
