#!/usr/bin/env python

import ROOT
import math

def ReadIt(seed) :

    ref=["zero","one","two","three","four","five","six","seven","eight","nine"]
    rand=ROOT.TRandom3();
    rand.SetSeed(int(seed));

    w1=ref[rand.Integer(10)]; w2=ref[rand.Integer(10)];
    sizemax=15;
    size1 = rand.Integer(sizemax);
    while size1 == 0 : size1 = rand.Integer(sizemax);
    size2 = rand.Integer(sizemax);
    while size2 == 0 : size2 = rand.Integer(sizemax);
    
    v1=[rand.Gaus(0,1) for i in range(size1) ]
    v2=[rand.Gaus(10,3) for i in range(size2) ]
    f1 = rand.Gaus(50,10.2);

    return [v1, v2, w1, w2, f1]

# ------------------------------------------------------------------------------

# output attribute
v1 = ROOT.URANIE.DataServer.TAttribute("v1", ROOT.URANIE.DataServer.TAttribute.kVector)
v2 = ROOT.URANIE.DataServer.TAttribute("v2", ROOT.URANIE.DataServer.TAttribute.kVector)
w1 = ROOT.URANIE.DataServer.TAttribute("w1", ROOT.URANIE.DataServer.TAttribute.kString)
w2 = ROOT.URANIE.DataServer.TAttribute("w2", ROOT.URANIE.DataServer.TAttribute.kString)
f1 = ROOT.URANIE.DataServer.TAttribute("f1")


# tds
tds = ROOT.URANIE.DataServer.TDataServer("tdsreadmt", "tds for Python multitype function")
tds.fileDataRead("multitype_sampling.dat")

# code
code = ROOT.URANIE.Relauncher.TPythonEval(ReadIt)
code.addInput(tds.getAttribute("seed"))
code.addOutput(v1)
code.addOutput(v2)
code.addOutput(w1)
code.addOutput(w2)
code.addOutput(f1)

# run
run = ROOT.URANIE.Relauncher.TSequentialRun(code)
run.startSlave()
if (run.onMaster()) :
    # launcher
    launch = ROOT.URANIE.Relauncher.TLauncher2(tds, run)
    launch.solverLoop()

    tds.exportData("_output_testmtSequentialPython_.dat");

    run.stopSlave()
