#!/usr/bin/env python

import ROOT
import math

def ishigami(x1, x2, x3) :
    a = 7.0
    b = 0.1
    
    res = math.sin(x1)
    res += a * math.sin(x2) * math.sin(x2)
    res += b * x3 * x3 * x3 * x3 * math.sin(x1)

    return [res,]


# patch(2) for a clean mpi exit
#ROOT.SetMemoryPolicy(ROOT.kMemoryStrict)

# variables
inputs = [
    ROOT.URANIE.DataServer.TUniformDistribution("x_{1}", -math.pi, math.pi),
    ROOT.URANIE.DataServer.TUniformDistribution("x_{2}", -math.pi, math.pi),
    ROOT.URANIE.DataServer.TUniformDistribution("x_{3}", -math.pi, math.pi),
]

ishi = ROOT.URANIE.DataServer.TAttribute("Ishigami")

# code
code = ROOT.URANIE.Relauncher.TPythonEval(ishigami)
for a in inputs :
    code.addInput(a)
code.addOutput(ishi)

# run
run = ROOT.URANIE.Relauncher.TMpiRun(code)
run.startSlave()
if (run.onMaster()) :
    # tds
    tds = ROOT.URANIE.DataServer.TDataServer("tds", "tds for Python Ishigami")
    for a in inputs :
        tds.addAttribute(a)

    # sampler
    sampl = ROOT.URANIE.Sampler.TSampling(tds)
    sampl.generateSample()

    # launcher
    launch = ROOT.URANIE.Relauncher.TLauncher2(tds, run)
    launch.solverLoop()

    # save result
    tds.exportData("ishiMpiPy.dat")
    if False :
        canvas = ROOT.TCanvas("c1", "c1",15,36,713,934)
        canvas.Clear()
        canvas.Divide(1,3)
        canvas.cd(1)
        tds.Draw("Ishigami:x_{1}")
        canvas.cd(2)
        tds.Draw("Ishigami:x_{2}")
        canvas.cd(3)
        tds.Draw("Ishigami:x_{3}")
    
    run.stopSlave()
    # patch(1) for a clean MPI exit
    ROOT.SetOwnership(run, True)
    #run.IsA().Destructor(run)
    # patch(2) for a clean MPI exit
    #del tds


