using namespace URANIE::DataServer;
using namespace URANIE::Sensitivity;
using namespace URANIE::Relauncher;

#define NS 60000
//#define NS 600

void flowrateModel(double *x, double *y)
{
    Double_t drw = x[0], dr = x[1];
    Double_t dtu = x[2], dtl = x[3];
    Double_t dhu = x[4], dhl = x[5];
    Double_t dl = x[6], dkw = x[7];

    Double_t dnum = 2.0 * TMath::Pi() * dtu * (dhu - dhl);
    Double_t dlnronrw = TMath::Log(dr / drw);
    Double_t dden =
            dlnronrw
                    * (1.0 + (2.0 * dl * dtu) / (dlnronrw * drw * drw * dkw)
                            + dtu / dtl);

    y[0] = dnum / dden;
}

void test_TLauncherSobol()
{
    // problem variables
    TUniformDistribution rw("rw", 0.05, 0.15);
    TUniformDistribution r("r", 100.0, 50000.0);
    TUniformDistribution tu("tu", 63070.0, 115600.0);
    TUniformDistribution tl("tl", 63.1, 116.0);
    TUniformDistribution hu("hu", 990.0, 1110.0);
    TUniformDistribution hl("hl", 700.0, 820.0);
    TUniformDistribution l("l", 1120.0, 1680.0);
    TUniformDistribution kw("kw", 9855.0, 12045.0);

    TAttribute yhat("flowrateModel"); // bug if name don't match function

    // code
    TCJitEval code(flowrateModel);
    code.setInputs(8, &rw, &r, &tu, &tl, &hu, &hl, &l, &kw);
    code.addOutput(&yhat);

    // Define the DataServer
    TDataServer tds("tdsflowreate", "DataBase flowreate");
    tds.addAttribute(&rw);
    tds.addAttribute(&r);
    tds.addAttribute(&tu);
    tds.addAttribute(&tl);
    tds.addAttribute(&hu);
    tds.addAttribute(&hl);
    tds.addAttribute(&l);
    tds.addAttribute(&kw);

    // sensitivity sample
    TSobol tsobol(&tds, flowrateModel, "flowrateModel", NS);
    tsobol.generateSample();

    // launcher
    TLauncher2 launch(&tds, &code);
    launch.solverLoop();

    // sensitivity analyse
    tsobol.computeIndexes();

    // visual
    if (0)
    {
        TCanvas cc("canhist", "histgramme");
        tsobol.drawIndexes("Flowrate", "", "nonewcanv,hist,all");
        cc.Print("appliUranieFlowrateSobol60000Histogram.png");

        TCanvas ccc("canpie", "TPie");
        tsobol.drawIndexes("Flowrate", "", "nonewcanv,pie,first");
        ccc.Print("appliUranieFlowrateSobol60000Pie.png");

        //tds.StartViewer();
    }

    tds.exportData("_sobol_launching_.dat");

    tsobol.getResultTuple()->Scan("*", "Algo==\"--first--\"");
    tsobol.getResultTuple()->Scan("*", "Algo==\"--total--\"");

}
