#ifndef RESOURCELISTENER_H
#define RESOURCELISTENER_H

#include <cppunit/TestListener.h>

class ResourceModel;


/// TestListener that prints a flatten or hierarchical view of the test tree.
class ResourceListener : public CPPUNIT_NS::TestListener
{
public:
  ResourceListener( ResourceModel *model,
                   bool text );

  virtual ~ResourceListener();

  void startTestRun( CPPUNIT_NS::Test *test, 
                     CPPUNIT_NS::TestResult *eventManager );

  void endTestRun( CPPUNIT_NS::Test *test, 
                   CPPUNIT_NS::TestResult *eventManager );

  void startTest( CPPUNIT_NS::Test *test );

  void endTest( CPPUNIT_NS::Test *test );

  void startSuite( CPPUNIT_NS::Test *suite );

  void endSuite( CPPUNIT_NS::Test *suite );

private:
  void printStatistics() const;

  void printTest( int testIndex,
                  const std::string &indentString ) const;

  void printTestIndent( const std::string &indent,
                        const int indentLength ) const;

  void printTime( double time ) const;

  void printMemory( double time ) const;
    
  void printTimeMemory( double time, double memory ) const;

  /// Prevents the use of the copy constructor.
  ResourceListener( const ResourceListener &other );

  /// Prevents the use of the copy operator.
  void operator =( const ResourceListener &other );

private:
  ResourceModel *m_model;
  bool m_text;
};



// Inlines methods for ResourceListener:
// -----------------------------------



#endif  // RESOURCELISTENER_H
