//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestShare.cxx
 \author Gilles Arnaud <gilles.arnaud@cea.fr>
 \date   mar mai 21 2013
 
 $Revision$
 $Date$
 
 \brief Programme principal pour les tests unitaires
 
 */
#include "cppunit/CompilerOutputter.h"
#include "cppunit/TestResult.h"
#include "cppunit/TestResultCollector.h"
#include "cppunit/TextTestProgressListener.h"
#include "cppunit/BriefTestProgressListener.h"
#include "cppunit/extensions/TestFactoryRegistry.h"
#include "cppunit/TestRunner.h"
#include "cppunit/XmlOutputter.h"

#include <stdexcept>

#include <iostream>
#include <fstream>

//uranie
#include "UVersion.h"

//uranie_hooks header
#include "TestXmlOutputterHook.h"
#include "ResourceListener.h"
#include "ResourceModel.h"
#include "ResourceXmlHook.h"

bool uranie_test_suite(const char *module, const char *xml, const char *xsl)
{
    // --- Create the event manager and test controller
    CPPUNIT_NS::TestResult controller;

    // ---  Add a listener that colllects test result
    CPPUNIT_NS::TestResultCollector result;
    controller.addListener(&result);

    CPPUNIT_NS::BriefTestProgressListener progress;
    controller.addListener(&progress);

    ResourceModel clockerModel;
    ResourceListener clocker(&clockerModel,true);
    controller.addListener( &clocker ) ;
    ResourceXmlHook xmlHook( &clockerModel );    

    // ---  Get the top level suite from the registry

    CPPUNIT_NS::Test *suite =
        CPPUNIT_NS::TestFactoryRegistry::getRegistry().makeTest();

    // ---  Adds the test to the list of test to run

    CPPUNIT_NS::TestRunner runner;
    runner.addTest(suite);

    std::ofstream outputFileXml(xml);
    CPPUNIT_NS::XmlOutputter outputter(&result, outputFileXml);

    TestXmlOutputterHook hook("Ur@nie", "Fabrice Gaudier");
    hook.setModule(module, URANIE_RELEASE);

    runner.run(controller);

    outputter.addHook(&hook);
    outputter.addHook(&xmlHook);    
    outputter.setStyleSheet(xsl);
    outputter.write();

    // ---  Run the tests.

    bool wasSuccessful = result.wasSuccessful();
    outputFileXml.close();

    return wasSuccessful;
}

