#!/bin/csh -f

#UNSET TEMPORARY VARIABLES

# OPT++
if($?URANIEOPTSYS) then
    unset URANIEOPTSYS
endif

# JSONCPP
if ($?URANIEJSONCPPSYS) then
    unset URANIEJSONCPPSYS
endif

setenv ROOTSYS @ROOTSYS@
setenv URANIESYS @CMAKE_INSTALL_PREFIX@
setenv URANIEOPTSYS @OPT_LIBRARY@
set URANIEJSONCPPSYS = @JsonCpp_LIBDIR@
set MPILIBRARY = @MPI_LIBRARY_PATH@
set URANIENLOPTSYS = @NLOPT_LIBS@

set URANIEPYTLIB = @Python_LIBRARIES@

#------------------------------------------------
#--- PATH & LD_LIBRARY_PATH 
#------------------------------------------------
if ( -d $ROOTSYS/lib/root ) then
    set ROOTSYSLIB = ${ROOTSYS}/lib/root
else
    set ROOTSYSLIB = ${ROOTSYS}/lib
endif

setenv PATH ${URANIESYS}/bin:${ROOTSYS}/bin:$PATH

## PYTHONPATH
if($?PYTHONPATH) then
    setenv PYTHONPATH ${URANIESYS}/lib:${ROOTSYSLIB}:$PYTHONPATH
else
    setenv PYTHONPATH ${URANIESYS}/lib:${ROOTSYSLIB}
endif


if($?LD_LIBRARY_PATH) then
    setenv LD_LIBRARY_PATH ${URANIESYS}/lib:${ROOTSYSLIB}:$LD_LIBRARY_PATH
else
    setenv LD_LIBRARY_PATH ${URANIESYS}/lib:${ROOTSYSLIB}
endif

echo "**************************"
echo "**"
echo "**  Environment variables for Uranie v@URANIE_VERSION@" 
echo "**"
echo "**  URANIESYS : ${URANIESYS}"
echo "**   with"
echo "**      ROOT  : v"`root-config --version`


#IF OPT++ is available
if($URANIEOPTSYS != "") then
    setenv LD_LIBRARY_PATH ${URANIEOPTSYS}:$LD_LIBRARY_PATH
    echo "**      OPT++ libs  : ${URANIEOPTSYS}"
endif

#IF JSONCPP is available
if($URANIEJSONCPPSYS != "") then
    setenv LD_LIBRARY_PATH ${URANIEJSONCPPSYS}:$LD_LIBRARY_PATH
    echo "**      JSONCPP libs  : ${URANIEJSONCPPSYS}"
endif


#IF MPI is available
if($MPILIBRARY != "") then
    setenv PATH @MPIEXEC_PATH@:$PATH
    setenv LD_LIBRARY_PATH ${MPILIBRARY}:$LD_LIBRARY_PATH
    echo "**      MPI libs    : ${MPILIBRARY}"
endif

#IF NLopt is available
if($URANIENLOPTSYS != "") then
  
  if( -f $URANIENLOPTSYS ) then      
    set URANIENLOPTSYS = `dirname "${URANIENLOPTSYS}"`;
  else
    set URANIENLOPTSYS = ${URANIENLOPTSYS}/lib;
  endif
    setenv LD_LIBRARY_PATH ${URANIENLOPTSYS}:$LD_LIBRARY_PATH
    echo "**      NLOPT lib   : ${URANIENLOPTSYS}"
endif

#IF python is available
if($URANIEPYTLIB != "") then
    echo "**      PYTHON lib  : ${URANIEPYTLIB}"
endif

echo "**************************"


if(-f ${URANIESYS}/share/uranie/index.html) then
    echo ""
    echo "Documentation can be seen with a web browser at: "
    echo "file://${URANIESYS}/share/uranie/index.html"
endif

if(-d ${URANIESYS}/share/uranie/macros) then
    echo ""
    echo "Use-case macros can be found here: "
    echo "${URANIESYS}/share/uranie/macros"
endif


#------------------------------------------------
#--- COMPILATION SHORTCUT 
#------------------------------------------------
setenv URANIECPPFLAG "`root-config --cflags` -I${URANIESYS}/include"
set ULD = "-lUranie"
setenv URANIELDFLAG " `root-config --libs` -L${URANIESYS}/lib ${ULD}DataServer ${ULD}Sampler ${ULD}Launcher ${ULD}Relauncher ${ULD}RelauncherMPI  ${ULD}Modeler ${ULD}Sensitivity ${ULD}Reoptimizer ${ULD}Optimizer ${ULD}Reliability ${ULD}XMLProblem "
