//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2024 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TBaseModel.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   fri jul 13 10:11:28 CEST 2007
 
 $Revision$
 $Date$
 
 \brief Implementation of the class URANIE::DataServer::TBaseModel
 
 */
#include <iostream>
using std::cout;
using std::endl;

#include "TBaseModel.h"

ClassImp (URANIE::XMLProblem::TBaseModel)

namespace URANIE
{
  namespace XMLProblem
  {
    TBaseModel::TBaseModel() :
      _sinput(""), _shidden(""), _soutput(""),
      _nlanguage(kEMPTY), _nNormType(kCR),
      _sfileNameExport(""), _sfunctionNameExport(""),
      _niteration(1), _ninitialization(1),
      _dlearn(0.8), _dtolerance(DEFAULTTOLERANCE), _nseed(0)
    {
      SetNameTitle("DefaultUranieModel", "Model in the uranie tool");
    }

    TBaseModel::TBaseModel(TString sinput, TString soutput) :
      _sinput(sinput), _shidden(""), _soutput(soutput),
      _nlanguage(kEMPTY), _nNormType(kCR),
      _sfileNameExport(""), _sfunctionNameExport(""),
      _niteration(1), _ninitialization(1),
      _dlearn(0.8), _dtolerance(DEFAULTTOLERANCE), _nseed(0)
    {
      SetNameTitle("DefaultUranieModel",
                   Form("Model in the uranie tool input[%s] output[%s]",
                        _sinput.Data(), _soutput.Data()));
    }

    TBaseModel::TBaseModel(TString sname, TString sinput, TString shidden, TString soutput) :
      _sinput(sinput), _shidden(shidden), _soutput(soutput),
      _nlanguage(kEMPTY), _nNormType(kCR),
      _sfileNameExport(sname), _sfunctionNameExport(sname),
      _niteration(1), _ninitialization(1),
      _dlearn(0.8), _dtolerance(DEFAULTTOLERANCE), _nseed(0)
    {
      SetNameTitle(sname,
                   Form("Model [%s] in the uranie tool input[%s] hidden[%s] output[%s]",
                        sname.Data(), sinput.Data(), shidden.Data(),
                        soutput.Data()));
    }
    TBaseModel::~TBaseModel()
    {
    }

    void TBaseModel::initExport()
    {
      // Default value of File Name is the name of Model
      if (_sfileNameExport == "")
        _sfileNameExport = GetName();
      // Default value of Function Name is the name of Model
      if (_sfunctionNameExport == "")
        _sfunctionNameExport = GetName();

      switch (_nlanguage)
        {
        case kC:
          _sfileNameExport += ".C";
          break;
        case kCXX:
          _sfileNameExport += ".cxx";
          break;
        case kFORTRAN:
          _sfileNameExport += ".f";
          break;
        case kPYTHON:
          _sfileNameExport += ".py";
          break;
        case kPMML:
          _sfileNameExport += ".pmml";
          break;
        default:
          break;
        }
    }

    void TBaseModel::printLog(Option_t *option)
    {
      cout << endl << "************************************" << endl;
      cout << "*** TBaseModel::printLog[" << option << "]" << endl;
      cout << " ** Name[" << GetName() << "] Title[" << GetTitle() << "]" << endl;
      cout << " ** _niteration[" << _niteration << "]" << endl;
      cout << " ** _ninitialization[" << _ninitialization << "]" << endl;
      cout << " ** _nseed[" << _nseed << "]" << endl;
      cout << " ** _dlearn[" << _dlearn << "] _dtolerance[" << _dtolerance << "]"
           << endl;
      cout << " ** _sinput[" << _sinput << "] _shidden[" << _shidden
           << "] : _soutput[" << _soutput << "]" << endl;
      cout << " ** _sfileNameExport[" << _sfileNameExport
           << "] : _sfunctionNameExport[" << _sfunctionNameExport << "]"
           << endl;
      switch (_nlanguage)
        {
        case kEMPTY:
          cout << " **   Language[kEMPTY]" << endl;
          break;
        case kC:
          cout << " **   Language[kC]" << endl;
          break;
        case kCXX:
          cout << " **   Language[kCXX]" << endl;
          break;
        case kPYTHON:
          cout << " **   Language[kPYTHON]" << endl;
          break;
        case kFORTRAN:
          cout << " **   Language[kFORTRAN]" << endl;
          break;
        default:
          break;
        }
      switch (_nNormType)
        {
        case kNO:
          cout << " ** _nNormType[NO]" << endl;
          break;
        case kCR:
          cout << " ** _nNormType[CenterReduct]" << endl;
          break;
        case kMinusOneOne:
          cout << " ** _nNormType[kMinusOneOne]" << endl;
          break;
        case kZeroOne:
          cout << " ** _nNormType[kZeroOne]" << endl;
          break;
        }
      cout << "*** End Of TBaseModel::printLog[" << option << "]" << endl;
      cout << "************************************" << endl;
    }

  } // Fin du namespace XMLProblem
} // Fin du namespace URANIE

